/*
 * Folder.cpp
 *
 *  Created on: Jun 14, 2010
 *      Author: João V. Gomes
 */

#include "Folder.h"

Folder::Folder() {

}

Folder::Folder(const std::string &Path) {
	this->CreateFolder(Path);
}

Folder::~Folder() {
	// TODO Auto-generated destructor stub
}

void Folder::InitFolder(const std::string &Path) {
	this->CreateFolder(Path);
}

void Folder::CreateFolder(const std::string &Path) {

	this->FolderPath = this->CleanLastBar(Path);

    if (!this->FolderPath.empty())
    {
        std::ofstream logFile;
        const int isdir = this->IsDir(this->FolderPath);

        if (isdir == 0)
        {
        	this->CreateSubDir(this->FolderPath);

            if (mkdir(this->FolderPath.c_str() , DIR_PERMISSIONS) == -1)  // Create the directory
                throw Fail_Creating_Dir(strerror(errno));
        }
        else if (isdir == -1)
            throw Dir_Path_Is_File(this->FolderPath);
        else if (isdir == -2)
            throw Fail_Access_Dir(this->FolderPath);
    }
    else
        throw Path_String_Empty();
}

void Folder::CreateSubDir(const std::string &Path)
{
	std::string SubDirPath = this->CleanLastBar(Path);

	size_t last;
	last = SubDirPath.rfind('/');

	if (last >= 0)
		SubDirPath.erase(last);

	if (!SubDirPath.empty())
    {
        std::ofstream logFile;
        const int isdir = this->IsDir(SubDirPath);

        if (isdir == 0)
        {
        	this->CreateSubDir(SubDirPath);

            if (mkdir(SubDirPath.c_str() , DIR_PERMISSIONS) == -1)  // Create the directory
                throw Fail_Creating_Dir(strerror(errno));
        }
        else if (isdir == -1)
            throw Dir_Path_Is_File(SubDirPath);
        else if (isdir == -2)
            throw Fail_Access_Dir(SubDirPath);
    }

}

// It returns:
//  1 - If the directory path already exists.
//  0 - If the directory path does not exist.
// -1 - If the directory path refers to a file.
// -2 - If an error has occurred.
const int Folder::IsDir(const std::string &Path) const
{
    if ( access(Path.c_str(), 0) == 0 )
    {
        struct stat status;
        stat( Path.c_str(), &status );

        if ( status.st_mode & S_IFDIR )
            return 1; // The directory exists.
        else
            return -1; // It is a file.
    }
    else
        return 0; // The directory does not exist.

    return -2;
}

void Folder::WriteNewFile(const std::string &FileName, const std::string &Msg) const
{
	this->WriteFile(FileName, Msg, std::ios::out);
}

void Folder::WriteExistentFile(const std::string &FileName, const std::string &Msg) const
{
	this->WriteFile(FileName, Msg, std::ios::app);
}

void Folder::WriteFile(const std::string &FileName, const std::string &Msg, const std::_Ios_Openmode &flag) const
{
    std::ofstream logFile;

    std::string FilePath = this->FolderPath + "/" + FileName;

    logFile.open(FilePath.c_str(), flag);

    if( logFile.fail() )
        throw Fail_Read_File(FilePath);
    else
        logFile << Msg;
    logFile.close();
}

const std::string Folder::Path() const
{
	return this->FolderPath;
}

void Folder::ListFiles(std::vector<std::string> * Files) const
{
    DIR *dp;
    struct dirent *dirp;

    if((dp  = opendir(this->FolderPath.c_str())) == NULL)
        throw Fail_Access_Dir(this->FolderPath);

    while ((dirp = readdir(dp)) != NULL)
    	if (!this->IsDir(dirp->d_name))
        	Files->push_back(std::string(dirp->d_name));

    closedir(dp);
}

void Folder::ListFilesPath(std::vector<std::string> * Files) const
{
    DIR *dp;
    struct dirent *dirp;

    if((dp  = opendir(this->FolderPath.c_str())) == NULL)
        throw Fail_Access_Dir(this->FolderPath);

    while ((dirp = readdir(dp)) != NULL)
    	if (!this->IsDir(dirp->d_name))
        	Files->push_back(this->FolderPath + "/" + (std::string(dirp->d_name)));

    closedir(dp);
}

const std::string Folder::CleanLastBar(const std::string &Path) const
{
	std::string PathStr = Path;

	while ( *(PathStr.end() - 1) == ('/') )
		PathStr.erase(PathStr.end() - 1);

	return PathStr;
}
