/*
 * direxception.h
 *
 *  Created on: Jun 14, 2010
 *      Author: João V. Gomes
 */

#ifndef DIREXCEPTION_H_INCLUDED
#define DIREXCEPTION_H_INCLUDED

#include <stdexcept>
#include <string>

///
/// Generic Directory Exception.
///	Derives from std::runtime_error.
///
class DirException : public std::runtime_error
{
	public:

		DirException(std::string const& err_msg)
		:	std::runtime_error(err_msg)
		{ }

};

class Fail_Creating_Dir : public DirException
{
    public:

        Fail_Creating_Dir(std::string const& err_msg)
        :   DirException("Failed to create directory: " + err_msg)
        { }
};

class Fail_Access_Dir : public DirException
{
    public:

        Fail_Access_Dir(std::string const& err_msg)
        :   DirException("Failed accessing to the directory: " + err_msg)
        { }
};

class Dir_Path_Is_File : public DirException
{
    public:

        Dir_Path_Is_File(std::string const& err_msg)
        :   DirException("The directory path is a file: " + err_msg)
        { }
};

class Path_String_Empty : public DirException
{
    public:

        Path_String_Empty()
        :   DirException("The directory path provided is an empty string.")
        { }
};

class Fail_Read_File : public DirException
{
    public:

        Fail_Read_File(std::string const& err_msg)
        :   DirException("Failed to open or read file: " + err_msg)
        { }
};

#endif // DIREXCEPTION_H_INCLUDED
