/*
 * \author João V. Gomes
 *
 *  packetprocessor.cpp
 *
 *  Created on: Jul 5, 2011
 *
 * \brief TODO
 * TODO
 */

#include "packetprocessor.h"

PacketProcessor::PacketProcessor(PacketSource *PacketSrc)
{
    this->init(PacketSrc, false, "255.255.255.255", "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength)
{
    this->init(PacketSrc, OnlyPayloadLength, "255.255.255.255", "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const std::string &IPaddr)
{
    this->init(PacketSrc, false, IPaddr, "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddr)
{
    this->init(PacketSrc, OnlyPayloadLength, IPaddr, "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const std::string &IPaddr, const std::string &Mask)
{
    this->init(PacketSrc, false, IPaddr, Mask);
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddr, const std::string &Mask)
{
    this->init(PacketSrc, OnlyPayloadLength, IPaddr, Mask);
}

PacketProcessor::~PacketProcessor()
{
	this->dispose();
}

void PacketProcessor::dispose() {

}

void PacketProcessor::init(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddr, const std::string &Mask)
{
    this->ResetPacketSource(PacketSrc);

    this->PacketSizeProp = false;
    this->InterArrivalTimeProp = false;
    this->IPPortProp = false;

    this->AllTraffic = false;
    this->IncomingTraffic = false;
    this->OutgoingTraffic = false;

    this->FilterAck = false;
    this->FilterSyn = false;
    this->FilterFull = false;
    this->FilterTCPnoPayload = false;
    this->FilterNoTCP = false;
    this->FilterNoLess60 = false;
    this->FilterNoLess65 = false;
    this->FilterTCPPayloadLess5 = false;
    this->FilterPayloadLess5 = false;

    this->IPaddress = inet_addr(IPaddr.c_str());
    this->NetMask = inet_addr(Mask.c_str());

    this->OnlyPayloadLen = OnlyPayloadLength;
    this->Segmentation = false;
    this->SegmentationBase = 1;
}

void PacketProcessor::SetSegmentation(const bool &Seg, const int &Base)
{
	this->Segmentation = Seg;
	this->SegmentationBase = Base;
}

void PacketProcessor::SetPropertiesToAnalyze(const int &Prop)
{
	if (Prop == PACKET_SIZE)
		this->PacketSizeProp = true;
	else if (Prop == INTER_ARRIVAL_TIME)
		this->InterArrivalTimeProp = true;
	else if (Prop == IP_PORT)
		this->IPPortProp = true;
}

void PacketProcessor::UnsetPropertiesToAnalyze(const int &Prop)
{
	if (Prop == PACKET_SIZE)
		this->PacketSizeProp = false;
	else if (Prop == INTER_ARRIVAL_TIME)
		this->InterArrivalTimeProp = false;
	else if (Prop == IP_PORT)
		this->IPPortProp = false;
}

void PacketProcessor::SetDirections(const int &Direction)
{
	if (Direction == DIR_OUT)
		this->OutgoingTraffic = true;
	else if (Direction == DIR_IN)
		this->IncomingTraffic = true;
	else if (Direction == DIR_ALL)
		this->AllTraffic = true;
}

void PacketProcessor::UnsetDirections(const int &Direction)
{
	if (Direction == DIR_OUT)
		this->OutgoingTraffic = false;
	else if (Direction == DIR_IN)
		this->IncomingTraffic = false;
	else if (Direction == DIR_ALL)
		this->AllTraffic = false;
}

void PacketProcessor::SetFilters(const int &Filter)
{
	if (Filter == ACK_PCKT)
		this->FilterAck = true;
	else if (Filter == SYN_PCKT)
		this->FilterSyn = true;
	else if (Filter == FULL_PCKT)
		this->FilterFull = true;
	else if (Filter == NO_PAYLOAD)
		this->FilterTCPnoPayload = true;
	else if (Filter == NO_TCP)
		this->FilterNoTCP = true;
	else if (Filter == NO_LESS_60)
		this->FilterNoLess60 = true;
	else if (Filter == NO_LESS_65)
		this->FilterNoLess65 = true;
	else if (Filter == NO_TCP_PL_LESS_5)
		this->FilterTCPPayloadLess5 = true;
	else if (Filter == NO_PL_LESS_5)
		this->FilterPayloadLess5 = true;
}

void PacketProcessor::UnsetFilters(const int &Filter)
{
	if (Filter == ACK_PCKT)
		this->FilterAck = false;
	else if (Filter == SYN_PCKT)
		this->FilterSyn = false;
	else if (Filter == FULL_PCKT)
		this->FilterFull = false;
	else if (Filter == NO_PAYLOAD)
		this->FilterTCPnoPayload = false;
	else if (Filter == NO_TCP)
		this->FilterNoTCP = false;
	else if (Filter == NO_LESS_60)
		this->FilterNoLess60 = false;
	else if (Filter == NO_LESS_65)
		this->FilterNoLess65 = false;
	else if (Filter == NO_TCP_PL_LESS_5)
		this->FilterTCPPayloadLess5 = false;
	else if (Filter == NO_PL_LESS_5)
		this->FilterPayloadLess5 = false;
}

void PacketProcessor::ExcludeIP(const std::string &IPaddr)
{
	this->ExcludedIPs.insert(IPaddr);
}

void PacketProcessor::AllowExcludedIP(const std::string &IPaddr)
{
	this->ExcludedIPs.erase(IPaddr);
}

void PacketProcessor::ResetPacketSource(PacketSource *PacketSrc)
{
    if (PacketSrc->IsOpen())
        this->PcktSrc = PacketSrc;
    else
        throw PSrc_Not_Open();
}

const bool PacketProcessor::SelectHost(const std::string &IPaddr) const
{
	std::set<std::string>::iterator it;

    for(it = this->ExcludedIPs.begin(); it != this->ExcludedIPs.end(); it++)
    	if (inet_addr((*it).c_str()) == inet_addr(IPaddr.c_str()))
    			return false;

	if (inet_addr("255.255.255.255") == this->IPaddress)
		return true;

	if ((this->NetMask & inet_addr(IPaddr.c_str())) == this->IPaddress)
		return true;

	return false;
}

bool const PacketProcessor::FilterPacket(const Packet * Pckt) const
{
	if (this->FilterNoTCP)
		if (Pckt->Protocol() == PKT_PTC_TCP)
			return true;

	if ( this->FilterFull )
		if (Pckt->Len() == 1514)
			return true;

	if ( this->FilterNoLess60 )
		if (Pckt->Len() <= 60)
			return true;

	if ( this->FilterNoLess65 )
		if (Pckt->Len() <= 65)
			return true;

	if ( Pckt->Protocol() == PKT_PTC_UDP )
	{
		const UDPPacket * UDPP = reinterpret_cast<UDPPacket const *>(Pckt);

		if ( this->FilterPayloadLess5 && (UDPP->UdpPayloadLen() <= 5) )
			return true;
	}

	if ( Pckt->Protocol() == PKT_PTC_TCP )
	{
		const TCPPacket * TCPP = reinterpret_cast<TCPPacket const *>(Pckt);

		if ( (this->FilterTCPPayloadLess5 || this->FilterPayloadLess5) &&
			 (TCPP->TcpPayloadLen() <= 5) )
			return true;

		if ( this->FilterSyn && TCPP->Syn() )
			return true;

		if ( this->FilterTCPnoPayload && (TCPP->TcpPayloadLen() == 0) )
			return true;

		if ( this->FilterAck &&
			 TCPP->Ack()     &&
			 (TCPP->TcpPayloadLen() == 0) )
//					if ( !(TCPP->Syn()) 	  &&
//						 !(TCPP->Rst())       &&
//						 !(TCPP->Fin())       &&
//						 !(TCPP->Urg())       &&
//						 !(TCPP->Psh())          )
			return true;
	}

	return false;
}

//const int PacketProcessor::GetHostClassification(const std::string &HostPort)
//{
//	if ( this->EntropyALL.at(HostPort) > 3 && this->EntropyOUT.at(HostPort) < 1 ) return 0;
//
//	if ( this->EntropyOUT.at(HostPort) == this->EntropyOUTR1.at(HostPort) && (this->EntropyALLR2.at(HostPort) > 0 || this->EntropyALLR3.at(HostPort) > 0) ) return 0;
//
//	if ( this->EntropyALL.at(HostPort) > 2 ) { if ( this->EntropyALLR1.at(HostPort) > 2 || this->EntropyOUT.at(HostPort) > 2 || (this->TotalPacketsR2.at(HostPort) > this->TotalPacketsR3.at(HostPort) && this->EntropyALLR3.at(HostPort) > 0)) return 0; else return 1; }
//
//	if ( this->EntropyALL.at(HostPort)   > 1 && this->EntropyOUT.at(HostPort)   < 0.5 &&
//		(this->EntropyALLR2.at(HostPort) > 0 || this->EntropyALLR3.at(HostPort) > 0)  &&
//		(this->EntropyOUTR2.at(HostPort) > 0 || this->EntropyOUTR3.at(HostPort) > 0)    )
//		return 1;
//
//	if ( this->TotalPacketsR2.at(HostPort) < this->TotalPacketsR3.at(HostPort) )
//	{
//		if ( this->EntropyALLR3.at(HostPort) > 1 ) return 1;
//
//		if ( this->EntropyALLR1.at(HostPort) > 0.5 &&
//			(this->EntropyALLR2.at(HostPort) > 0 || this->EntropyALLR3.at(HostPort) > 0) &&
//			(this->EntropyOUTR2.at(HostPort) > 0 || this->EntropyOUTR3.at(HostPort) > 0)    )
//			return 1;
//	}
//	else
//	{
////		if ( this->EntropyALLR2.at(HostPort) < 0.1 && (this->EntropyALLR2.at(HostPort) > 0 || this->EntropyALLR3.at(HostPort) > 0) ) return 1;
//	}
//
//	if ( this->EntropyOUT.at(HostPort) > 1 && this->EntropyOUTR3.at(HostPort) > 0 ) return 1;
//
//	if ( this->EntropyALLR1.at(HostPort) > 1 && (this->TotalPacketsR2.at(HostPort) >= this->WinSize || this->TotalPacketsR3.at(HostPort) >= this->WinSize) ) return 1;
//
////	if ( this->EntropyALL.at(HostPort)   > 0.5 && this->EntropyALL.at(HostPort)   < 1   &&
////		// this->EntropyALLR1.at(HostPort) > 0   && this->EntropyALLR1.at(HostPort) < 0.1 &&
////		 this->EntropyALLR3.at(HostPort) > 0   && this->EntropyALLR3.at(HostPort) < 0.1 &&
////		 this->EntropyOUT.at(HostPort)   > 0.1											   )
////		return 1;
//
//	if ( this->EntropyALL.at(HostPort)   <  1 &&
//		 this->EntropyALLR1.at(HostPort) >  0 && this->EntropyALLR1.at(HostPort) < 1 &&
//		 this->EntropyOUTR1.at(HostPort) >  0 && this->EntropyOUTR1.at(HostPort) < 1 &&
//		 this->EntropyALLR3.at(HostPort) >  0 && this->EntropyALLR3.at(HostPort) < 1 &&
//		 this->EntropyOUTR3.at(HostPort) >  0 && this->EntropyOUTR3.at(HostPort) < 1 &&
//		 this->EntropyALLR3.at(HostPort)      != this->EntropyOUTR3.at(HostPort)        )
//		return 1;
//
//	if ( this->EntropyALLR1.at(HostPort) > 0   && this->EntropyALLR1.at(HostPort) < 1 &&
//		 this->EntropyALLR3.at(HostPort) > 0.5 && this->EntropyALLR3.at(HostPort) < 1 &&
//		(this->EntropyOUTR2.at(HostPort) > 0   || this->EntropyOUTR3.at(HostPort) > 0)   )
//		return 1;
//
////	if ( this->EntropyALLR2.at(HostPort) > 0 && this->EntropyALLR2.at(HostPort) == this->EntropyOUTR2.at(HostPort) &&
////		 this->EntropyALLR3.at(HostPort) > 0 && this->EntropyALLR3.at(HostPort) == this->EntropyOUTR3.at(HostPort)    )
////		return 1;
//
//	return 0;
//
//}

const int PacketProcessor::GetHostClassification(const std::string &HostPort)
{
	if ( this->EntropyIN.at(HostPort)  > 1.5										&&
		 this->EntropyOUT.at(HostPort) > 0.5										&&
		(this->EntropyOUT.at(HostPort) - this->EntropyOUTR1.at(HostPort)) < 0.01		)
		return -6;

	if ( this->EntropyOUT.at(HostPort) > 2.5	&&
		 this->EntropyIN.at(HostPort)  > 2.5		)
		return 25;

	if ( this->EntropyOUT.at(HostPort)   > 0.5	&&
//		 this->EntropyIN.at(HostPort)    > 0.5	&&
		 this->EntropyALLR3.at(HostPort) > 0		)
		return 1;

	if ( this->EntropyHPALL.at(HostPort) > 0.5 )
		return 2;

	if ( this->EntropyOUT.at(HostPort) == this->EntropyOUTR1.at(HostPort)	&&
		 this->EntropyALLR3.at(HostPort) > 0 								&&
		 this->EntropyIAT.at(HostPort)   < 0.5 									)
		return -2;

	if ( this->EntropyALLR3.at(HostPort) > 1 )
		return 13;

	if ( this->EntropyOUT.at(HostPort)   > 1	&&
		 this->EntropyALLR3.at(HostPort) > 0		)
		return 22;

	if ( this->EntropyIAT.at(HostPort) > 1 		&&
		 this->EntropyALL.at(HostPort) < 1.5		)
		return  5;

	if ( this->EntropyALL.at(HostPort)   >  1	&&
		 this->EntropyALLR1.at(HostPort) == 0	&&
		 this->EntropyIAT.at(HostPort)   >  0		)
		return 8;

	if ( this->EntropyALLR1.at(HostPort) < 0.5 	&&
		 this->EntropyALLR3.at(HostPort) > 0.5  &&
		 this->EntropyIN.at(HostPort)    < 2	&&
		 this->EntropyIAT.at(HostPort)   > 0.1		)
		return 6;

	if ( this->EntropyALL.at(HostPort) > 1 		&&
		 this->EntropyOUT.at(HostPort) < 0.5	&&
		 this->EntropyIN.at(HostPort)  < 1			)
	   {
		   if ( this->EntropyALLR1.at(HostPort) < 0.1																			&&
				this->EntropyALLR3.at(HostPort) < 1																				&&
			  ((this->EntropyOUTR1.at(HostPort) * 2) >= this->EntropyOUT.at(HostPort) || this->EntropyOUTR1.at(HostPort) == 0 )		)
			   return -4;
		   else
			   return 11;
	   }

	if ( this->EntropyOUT.at(HostPort) > 0.001	&&
		 this->EntropyALL.at(HostPort) < 1.5	&&
		 this->EntropyIAT.at(HostPort) > 0.1		)
		return 26;

	return 0;

}

void PacketProcessor::PrintClassification(Folder * LogsFolder, const long &TSsec, const long &TSusec, const int &Length, const std::string &HostPort)
{
    std::string fileName = HostPort + ".log";
    double dTS = TSsec + TSusec * 0.000001;
    std::stringstream ss (std::stringstream::out);

    if (this->FirstTS.at(HostPort) == dTS)
    {
        std::stringstream ss_hdr (std::stringstream::out);

        time_t rawtime;
        struct tm * timeinfo;

        time ( &rawtime );
        timeinfo = localtime ( &rawtime );

        ss_hdr << "#\n"
    	   << "# Log for host " << HostPort << "\n"
    	   << "#\n"
    	   << "# Created on " << asctime (timeinfo)
    	   << "#\n"
    	   << "# Window of " << this->WinSize << " packets\n"
    	   << "#\n"
		   << "#\n"

		   << "# <Timestamp>	<Time elapsed>	<Length>	<Total bytes>	<Entropy ALL>	<Classification>	<Classification Avg>	<Entropy ALL R1>	<Entropy ALL R2>	<Entropy ALL R3>	<Entropy OUT>	<Entropy OUT R1>	<Entropy OUT R2>	<Entropy OUT R3>	<Entropy HostPort>	<Entropy IATimes>	<Entropy TTL>	<Entropy IN>\n"
		   << "#\n"
		   << "# <Timestamp>		<Elapsed>	<Len>	<Bytes>	<Ent ALL>	<Class>	<Classification Avg>	<Ent ALL R1>	<Ent ALL R2>	<Ent ALL R3>		<Ent OUT>		<Ent OUT R1>	<Ent OUT R2>	<Ent OUT R3>		<Entropy HostPort>	<Entropy IATimes>	<Entropy TTL>		<Entropy IN>\n"
    	   << "#" << std::endl;

    	LogsFolder->WriteNewFile(fileName, ss_hdr.str());
    }

    if (this->TotalPacketsClassified.count(HostPort) == 0)
    {
    	this->TotalPacketsClassified.insert(std::pair<std::string, long>(HostPort, 0));
    	this->ClassAvg.insert(std::pair<std::string, double>(HostPort, 0));
    }

    double EntALL = 0, EntALLR1 = 0, EntALLR2 = 0, EntALLR3 = 0, EntIN = 0, EntOUT = 0, EntOUTR1 = 0, EntOUTR2 = 0, EntOUTR3 = 0, EntHPALL = 0, EntIAT = 0, EntTTL = 0;

//    if (this->EntropyTCPNoPayload.count(Host) > 0)		EntTCPnoPL = this->EntropyTCPNoPayload.at(Host);
//    if (this->EntropyOutgoing.count(Host) > 0)			EntOut = this->EntropyOutgoing.at(Host);
//    if (this->EntropyOutgoingSegmented.count(Host) > 0)	EntOutSeg = this->EntropyOutgoingSegmented.at(Host);
    if (this->EntropyALL.count(HostPort)   > 0)	EntALL   = this->EntropyALL.at(HostPort);
    if (this->EntropyALLR1.count(HostPort) > 0)	EntALLR1 = this->EntropyALLR1.at(HostPort);
//    if (this->EntropyALLR2.count(HostPort) > 0)	EntALLR2 = this->EntropyALLR2.at(HostPort);
    if (this->EntropyALLR3.count(HostPort) > 0)	EntALLR3 = this->EntropyALLR3.at(HostPort);
    if (this->EntropyIN.count(HostPort) > 0)	EntIN	 = this->EntropyIN.at(HostPort);
    if (this->EntropyOUT.count(HostPort) > 0)	EntOUT	 = this->EntropyOUT.at(HostPort);
    if (this->EntropyOUTR1.count(HostPort) > 0)	EntOUTR1 = this->EntropyOUTR1.at(HostPort);
//    if (this->EntropyOUTR2.count(HostPort) > 0)	EntOUTR2 = this->EntropyOUTR2.at(HostPort);
//    if (this->EntropyOUTR3.count(HostPort) > 0)	EntOUTR3 = this->EntropyOUTR3.at(HostPort);
    if (this->EntropyHPALL.count(HostPort) > 0)	EntHPALL = this->EntropyHPALL.at(HostPort);
    if (this->EntropyIAT.count(HostPort) > 0)	EntIAT   = this->EntropyIAT.at(HostPort);
//    if (this->EntropyTTL.count(HostPort) > 0)	EntTTL	 = this->EntropyTTL.at(HostPort);

    int ClassRes = 0;
    if (this->TotalPacketsR1.at(HostPort) >= this->WinSize || this->TotalPacketsR2.at(HostPort) >= this->WinSize || this->TotalPacketsR3.at(HostPort) >= this->WinSize)
    {
    	ClassRes = this->GetHostClassification(HostPort);
    	int ClassBin; if (ClassRes > 0 ) ClassBin = 1; else ClassBin = 0;
    	this->ClassAvg.at(HostPort) = double ( (this->ClassAvg.at(HostPort) * this->TotalPacketsClassified.at(HostPort) ) + ClassBin ) / double ( this->TotalPacketsClassified.at(HostPort) + 1 );
    	this->TotalPacketsClassified.at(HostPort) ++;
    }

    ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << (dTS - this->FirstTS.at(HostPort)) << "\t" << Length << "\t" << this->TotalBytes.at(HostPort) << "\t" << EntALL << "\t" << ClassRes << "\t" << this->ClassAvg.at(HostPort) << "\t\t" << EntALLR1 << "\t" << EntALLR2 << "\t" << EntALLR3 << "\t\t" << EntOUT << "\t\t" << EntOUTR1 << "\t" << EntOUTR2 << "\t" << EntOUTR3 << "\t\t" << EntHPALL << "\t\t" << EntIAT << "\t\t" << EntTTL << "\t\t" << EntIN << std::endl;

    LogsFolder->WriteExistentFile(fileName, ss.str());
}

void PacketProcessor::ClassifyHostsPorts(const int &Win, const std::string &LogsPath, const bool &PrintLogs)
{
	this->ClassifyHostsPorts(Win, LogsPath, PrintLogs, false, 0, 0, 0, 0);
}

void PacketProcessor::ClassifyHostsPorts(const int &Win, const std::string &LogsPath, const bool &PrintLogs, const bool &considerTProto,
		                                 const long &maxDurPckts, const long &maxDurSecs,
		                                 const long &timeoutInterval, const long &cleanFrequency)
{
    std::stringstream ss (std::stringstream::out);
	const int iEntIndex = 1;

	std::string Path = this->CleanLastBar(LogsPath);
	ss << Path << "/EntIndex_" << iEntIndex;

	Folder * LogsFolder = new Folder(ss.str());

	this->WinSize = Win;
	this->LastClean = 0;

	HostPortAnalyzer<int>         * HPAnEntALL   = new HostPortAnalyzer<int>        (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
	HostPortAnalyzer<int>         * HPAnEntALLR1 = new HostPortAnalyzer<int>        (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
//	HostPortAnalyzer<int>         * HPAnEntALLR2 = new HostPortAnalyzer<int>        (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
	HostPortAnalyzer<int>         * HPAnEntALLR3 = new HostPortAnalyzer<int>        (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
	HostPortAnalyzer<int>         * HPAnEntIN    = new HostPortAnalyzer<int>        (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
	HostPortAnalyzer<int>         * HPAnEntOUT   = new HostPortAnalyzer<int>        (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
	HostPortAnalyzer<int>         * HPAnEntOUTR1 = new HostPortAnalyzer<int>        (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
//	HostPortAnalyzer<int>         * HPAnEntOUTR2 = new HostPortAnalyzer<int>        (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
//	HostPortAnalyzer<int>         * HPAnEntOUTR3 = new HostPortAnalyzer<int>        (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
	HostPortAnalyzer<std::string> * HPAnEntHPALL = new HostPortAnalyzer<std::string>(WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
	HostPortAnalyzer<double>      * HPAnEntIAT   = new HostPortAnalyzer<double>     (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
//	HostPortAnalyzer<int>         * HPAnEntTTL   = new HostPortAnalyzer<int>        (WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);

    bool end = false;

    while(!end)
        try
        {
            Packet *Pckt;

            Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();

            // Analyze only TCP and UDP packets
            if ( (Proto == PKT_PTC_TCP) || (Proto == PKT_PTC_UDP) ) //&& !(this->FilterPacket(Pckt)) )
            {
            	int sPort, dPort;

            	std::string TProto;

                IPPacket *IPP = static_cast<IPPacket *>(Pckt);

                std::string const *src = IPP->SrcIPaddr(), *dst = IPP->DstIPaddr();
                long TSsec = IPP->TSseconds(), TSusec = IPP->TSmicrosec();

                double dTS = TSsec + TSusec * 0.000001;

				int len = IPP->Len();
				int frLen = len;

            	if (Proto == PKT_PTC_TCP)
            	{
            		TProto = "TCP";

            		TCPPacket *TCPP = static_cast<TCPPacket *>(Pckt);

            		sPort = TCPP->SrcPort();
            		dPort = TCPP->DstPort();

            		if(this->OnlyPayloadLen) len = TCPP->TcpPayloadLen();
            	}

            	if (Proto == PKT_PTC_UDP)
            	{
            		TProto = "UDP";

            		UDPPacket *UDPP = static_cast<UDPPacket *>(Pckt);

            		sPort = UDPP->SrcPort();
            		dPort = UDPP->DstPort();

            		if(this->OnlyPayloadLen) len = UDPP->UdpPayloadLen();
            	}

				double * EntSrc, * EntSrcR1, * EntSrcR2, * EntSrcR3, * EntDst, * EntDstR1, * EntDstR2, * EntDstR3, * EntIn, * EntOut, * EntOutR1,  * EntOutR2, * EntOutR3, * EntIAT, * EntTTL, * EntHPALL;

			    if (this->SelectHost(*src))
				{
				    std::stringstream ss_hp (std::stringstream::out);
			    	ss_hp << *src << "." << sPort;
			    	if (considerTProto) ss_hp << "." << TProto;
					std::string HostPort = ss_hp.str();

				    std::stringstream ss_remotehp (std::stringstream::out);
				    ss_remotehp << *dst << "." << dPort;
					std::string remoteHostPort = ss_remotehp.str();

					if (this->TotalBytes.count(HostPort) == 0)
					{
						this->TotalBytes.insert(std::pair<std::string, long long>(HostPort, 0));
						this->TotalPacketsR1.insert(std::pair<std::string, long>(HostPort, 0));
						this->TotalPacketsR2.insert(std::pair<std::string, long>(HostPort, 0));
						this->TotalPacketsR3.insert(std::pair<std::string, long>(HostPort, 0));
					}
					this->TotalBytes.at(HostPort) += frLen;

					if (this->FirstTS.count(HostPort) == 0)
					{
						this->FirstTS.insert(std::pair<std::string, double>(HostPort, dTS));
						this->LastTS.insert(std::pair<std::string, double>(HostPort, dTS));
						this->numPckts.insert(std::pair<std::string, long>(HostPort, 0));
					}
					this->numPckts.at(HostPort) ++;

					if ((maxDurPckts == 0 || this->numPckts.at(HostPort) <= maxDurPckts) && (maxDurSecs == 0 || (dTS - this->FirstTS.at(HostPort) <= maxDurSecs)) )
					{
						double IATime = this->Round(dTS - this->LastTS.at(HostPort), PP_IAT_PRECISION);
						this->LastTS.at(HostPort) = dTS;

						if (!(this->FilterPacket(Pckt)))
						{
							// TODO: considerar o TCPnoPL
						}

						EntSrc = HPAnEntALL->AddValue(TSsec, TSusec, *src, sPort, len);
						EntOut = HPAnEntOUT->AddValue(TSsec, TSusec, *src, sPort, len);
						EntHPALL = HPAnEntHPALL->AddValue(TSsec, TSusec, *src, sPort, remoteHostPort);
//						EntTTL = HPAnEntTTL->AddValue(TSsec, TSusec, *src, sPort, IPP->TTL());
						if (this->EntropyALL.count(HostPort) == 0) {
							this->EntropyALL.insert(std::pair<std::string, double>(HostPort, EntSrc[iEntIndex]));
							this->EntropyALLR1.insert(std::pair<std::string, double>(HostPort, 0));
//							this->EntropyALLR2.insert(std::pair<std::string, double>(HostPort, 0));
							this->EntropyALLR3.insert(std::pair<std::string, double>(HostPort, 0));
							this->EntropyIN.insert(std::pair<std::string, double>(HostPort, 0));
							this->EntropyOUT.insert(std::pair<std::string, double>(HostPort, EntOut[iEntIndex]));
							this->EntropyOUTR1.insert(std::pair<std::string, double>(HostPort, 0));
//							this->EntropyOUTR2.insert(std::pair<std::string, double>(HostPort, 0));
//							this->EntropyOUTR3.insert(std::pair<std::string, double>(HostPort, 0));
							this->EntropyHPALL.insert(std::pair<std::string, double>(HostPort, EntHPALL[iEntIndex]));
							this->EntropyIAT.insert(std::pair<std::string, double>(HostPort, 0));
//							this->EntropyTTL.insert(std::pair<std::string, double>(HostPort, EntTTL[iEntIndex]));
						}
						else
						{
							EntIAT = HPAnEntIAT->AddValue(TSsec, TSusec, *src, sPort, IATime);

							this->EntropyALL.at(HostPort) = EntSrc[iEntIndex];
							this->EntropyOUT.at(HostPort) = EntOut[iEntIndex];
							this->EntropyHPALL.at(HostPort) = EntHPALL[iEntIndex];
							this->EntropyIAT.at(HostPort) = EntIAT[iEntIndex];
//							this->EntropyTTL.at(HostPort) = EntTTL[iEntIndex];
						}

						if (len <= RANGES_MAX_1)
						{
							EntSrcR1 = HPAnEntALLR1->AddValue(TSsec, TSusec, *src, sPort, len);
							EntOutR1 = HPAnEntOUTR1->AddValue(TSsec, TSusec, *src, sPort, len);
							this->EntropyALLR1.at(HostPort) = EntSrcR1[iEntIndex];
							this->EntropyOUTR1.at(HostPort) = EntOutR1[iEntIndex];
							this->TotalPacketsR1.at(HostPort) ++;
						}

						else if (len <= RANGES_MAX_2)
						{
//							EntSrcR2 = HPAnEntALLR2->AddValue(TSsec, TSusec, *src, sPort, len);
//							EntOutR2 = HPAnEntOUTR2->AddValue(TSsec, TSusec, *src, sPort, len);
//							this->EntropyALLR2.at(HostPort) = EntSrcR2[iEntIndex];
//							this->EntropyOUTR2.at(HostPort) = EntOutR2[iEntIndex];
							this->TotalPacketsR2.at(HostPort) ++;
						}

						else if (len <= RANGES_MAX_3)
						{
							EntSrcR3 = HPAnEntALLR3->AddValue(TSsec, TSusec, *src, sPort, len);
//							EntOutR3 = HPAnEntOUTR3->AddValue(TSsec, TSusec, *src, sPort, len);
							this->EntropyALLR3.at(HostPort) = EntSrcR3[iEntIndex];
//							this->EntropyOUTR3.at(HostPort) = EntOutR3[iEntIndex];
							this->TotalPacketsR3.at(HostPort) ++;
						}

						this->PrintClassification(LogsFolder, TSsec, TSusec, len, HostPort);
					}
				}

				if (this->SelectHost(*dst))
				{
				    std::stringstream ss_hp (std::stringstream::out);
					ss_hp << *dst << "." << dPort;
			    	if (considerTProto) ss_hp << "." << TProto;
					std::string HostPort = ss_hp.str();

				    std::stringstream ss_remotehp (std::stringstream::out);
				    ss_remotehp << *src << "." << sPort;
					std::string remoteHostPort = ss_remotehp.str();

					if (this->TotalBytes.count(HostPort) == 0)
					{
						this->TotalBytes.insert(std::pair<std::string, long long>(HostPort, 0));
						this->TotalPacketsR1.insert(std::pair<std::string, long>(HostPort, 0));
						this->TotalPacketsR2.insert(std::pair<std::string, long>(HostPort, 0));
						this->TotalPacketsR3.insert(std::pair<std::string, long>(HostPort, 0));
					}
					this->TotalBytes.at(HostPort) += frLen;

					if (this->FirstTS.count(HostPort) == 0)
					{
						this->FirstTS.insert(std::pair<std::string, double>(HostPort, dTS));
						this->LastTS.insert(std::pair<std::string, double>(HostPort, dTS));
						this->numPckts.insert(std::pair<std::string, long>(HostPort, 0));
					}
					this->numPckts.at(HostPort) ++;

					if ((maxDurPckts == 0 || this->numPckts.at(HostPort) <= maxDurPckts) && (maxDurSecs == 0 || (dTS - this->FirstTS.at(HostPort) <= maxDurSecs)) )
					{
						double IATime = this->Round(dTS - this->LastTS.at(HostPort), PP_IAT_PRECISION);
						this->LastTS.at(HostPort) = dTS;

						if (!(this->FilterPacket(Pckt)))
						{
							// TODO: considerar o TCPnoPL
						}

						EntDst = HPAnEntALL->AddValue(TSsec, TSusec, *dst, dPort, len);
						EntIn = HPAnEntIN->AddValue(TSsec, TSusec, *dst, dPort, len);
						EntHPALL = HPAnEntHPALL->AddValue(TSsec, TSusec, *dst, dPort, remoteHostPort);
//						EntTTL = HPAnEntTTL->AddValue(TSsec, TSusec, *dst, dPort, IPP->TTL());
						if (this->EntropyALL.count(HostPort) == 0) {
							this->EntropyALL.insert(std::pair<std::string, double>(HostPort, EntDst[iEntIndex]));
							this->EntropyALLR1.insert(std::pair<std::string, double>(HostPort, 0));
//							this->EntropyALLR2.insert(std::pair<std::string, double>(HostPort, 0));
							this->EntropyALLR3.insert(std::pair<std::string, double>(HostPort, 0));
							this->EntropyIN.insert(std::pair<std::string, double>(HostPort, EntIn[iEntIndex]));
							this->EntropyOUT.insert(std::pair<std::string, double>(HostPort, 0));
							this->EntropyOUTR1.insert(std::pair<std::string, double>(HostPort, 0));
//							this->EntropyOUTR2.insert(std::pair<std::string, double>(HostPort, 0));
//							this->EntropyOUTR3.insert(std::pair<std::string, double>(HostPort, 0));
							this->EntropyHPALL.insert(std::pair<std::string, double>(HostPort, EntHPALL[iEntIndex]));
							this->EntropyIAT.insert(std::pair<std::string, double>(HostPort, 0));
//							this->EntropyTTL.insert(std::pair<std::string, double>(HostPort, EntTTL[iEntIndex]));
						}
						else
						{
							EntIAT = HPAnEntIAT->AddValue(TSsec, TSusec, *dst, dPort, IATime);

							this->EntropyALL.at(HostPort) = EntDst[iEntIndex];
							this->EntropyIN.at(HostPort) = EntIn[iEntIndex];
							this->EntropyHPALL.at(HostPort) = EntHPALL[iEntIndex];
							this->EntropyIAT.at(HostPort) = EntIAT[iEntIndex];
//							this->EntropyTTL.at(HostPort) = EntTTL[iEntIndex];
						}


						if (len <= RANGES_MAX_1)
						{
							EntDstR1 = HPAnEntALLR1->AddValue(TSsec, TSusec, *dst, dPort, len);
							this->EntropyALLR1.at(HostPort) = EntDstR1[iEntIndex];
							this->TotalPacketsR1.at(HostPort) ++;
						}

						else if (len <= RANGES_MAX_2)
						{
//							EntDstR2 = HPAnEntALLR2->AddValue(TSsec, TSusec, *dst, dPort, len);
//							this->EntropyALLR2.at(HostPort) = EntDstR2[iEntIndex];
							this->TotalPacketsR2.at(HostPort) ++;
						}

						else if (len <= RANGES_MAX_3)
						{
							EntDstR3 = HPAnEntALLR3->AddValue(TSsec, TSusec, *dst, dPort, len);
							this->EntropyALLR3.at(HostPort) = EntDstR3[iEntIndex];
							this->TotalPacketsR3.at(HostPort) ++;
						}

						this->PrintClassification(LogsFolder, TSsec, TSusec, len, HostPort);
					}
				}

				if ( (cleanFrequency > 0) && ((this->LastClean + cleanFrequency) < dTS) )
				{
					if (this->LastClean != 0)
					{
						typename std::map<std::string, double>::iterator it;

						for (it = this->LastTS.begin(); it != this->LastTS.end(); it++)
							if ( (this->LastTS.at(it->first) + timeoutInterval) < dTS )
							{

								const int portpos = it->first.find_last_of('.');
								const std::string Host = it->first.substr(0, portpos);
								const std::string Port = it->first.substr(portpos + 1);

								//std::cout << Host << "\t" << Port << std::endl;
								HPAnEntALL->ClearHostPort(Host, atoi(Port.c_str()));
								HPAnEntALLR1->ClearHostPort(Host, atoi(Port.c_str()));
//								HPAnEntALLR2->ClearHostPort(Host, atoi(Port.c_str()));
								HPAnEntALLR3->ClearHostPort(Host, atoi(Port.c_str()));
								HPAnEntIN->ClearHostPort(Host, atoi(Port.c_str()));
								HPAnEntOUT->ClearHostPort(Host, atoi(Port.c_str()));
								HPAnEntOUTR1->ClearHostPort(Host, atoi(Port.c_str()));
//								HPAnEntOUTR2->ClearHostPort(Host, atoi(Port.c_str()));
//								HPAnEntOUTR3->ClearHostPort(Host, atoi(Port.c_str()));
								HPAnEntHPALL->ClearHostPort(Host, atoi(Port.c_str()));
								HPAnEntIAT->ClearHostPort(Host, atoi(Port.c_str()));
//								HPAnEntTTL->ClearHostPort(Host, atoi(Port.c_str()));

						        this->EntropyALL.erase(it->first);
						        this->EntropyALLR1.erase(it->first);
						        this->EntropyALLR2.erase(it->first);
						        this->EntropyALLR3.erase(it->first);
						        this->EntropyIN.erase(it->first);
						        this->EntropyOUT.erase(it->first);
						        this->EntropyOUTR1.erase(it->first);
						        this->EntropyOUTR2.erase(it->first);
						        this->EntropyOUTR3.erase(it->first);
						        this->EntropyHPALL.erase(it->first);
						        this->EntropyIAT.erase(it->first);
						        this->EntropyTTL.erase(it->first);
						        this->TotalBytes.erase(it->first);
						        this->TotalPacketsClassified.erase(it->first);
						        this->TotalPacketsR1.erase(it->first);
						        this->TotalPacketsR2.erase(it->first);
						        this->TotalPacketsR3.erase(it->first);
						        this->ClassAvg.erase(it->first);

						        this->numPckts.erase(it->first);
						        this->FirstTS.erase(it->first);
						        this->LastTS.erase(it->first);
							}
					}

					this->LastClean = dTS;
				}
            }

            delete Pckt;
        }
        catch (No_More_Packets e1)
        {
            end = true;
        }
        catch (Timeout_Expired e2)
        {
            std::cout << "Timeout: " << e2.what() << "\n";
            end = true;
        }
        catch (Packet_Reading_Error e3)
        {
            std::cout << "Failed: " << e3.what() << "\n";
            end = true;
        }

	delete HPAnEntALL;
	delete HPAnEntALLR1;
//	delete HPAnEntALLR2;
	delete HPAnEntALLR3;
	delete HPAnEntIN;
	delete HPAnEntOUT;
	delete HPAnEntOUTR1;
//	delete HPAnEntOUTR2;
//	delete HPAnEntOUTR3;
	delete HPAnEntHPALL;
	delete HPAnEntIAT;
//	delete HPAnEntTTL;

	delete LogsFolder;
}

const std::string PacketProcessor::CleanLastBar(const std::string &Path) const
{
	std::string PathStr = Path;

	while ( *(PathStr.end() - 1) == ('/') )
		PathStr.erase(PathStr.end() - 1);

	return PathStr;
}

const double PacketProcessor::Round(const double &number, const int &precision) const {
	double Aux, Result;
	int precFactor, intRes;

	if (precision == 0)
		return number;
	else if (precision > 0) {
		precFactor = 10;
		for (int i = 1; i < precision; i++)
			precFactor *= 10;
	} else
		return number;

	Aux = number * (double) precFactor;
	intRes = (int) Aux;

	Result = intRes / (double) precFactor;

	return Result;
}
