#include "ethpacket.h"

EthPacket::EthPacket()
{
    this->init();
}

EthPacket::EthPacket(const pcap_pkthdr *&Pkt_Hdr, const u_char *&Pkt_Data)
{
    this->init(Pkt_Hdr, Pkt_Data);
}

EthPacket::EthPacket(const long &TSsec, const long &TSmsec, const long &CapLength, const long &Length, const std::string &SrcMACaddr, const std::string &DstMACaddr)
{
    this->init(TSsec, TSmsec, CapLength, Length, SrcMACaddr, DstMACaddr);
}

EthPacket::~EthPacket()
{
    //dtor
}

void EthPacket::init()
{
    Packet::init();
    this->setProto(PKT_PTC_ETHERNET);
}

void EthPacket::init(const pcap_pkthdr *&Pkt_Hdr, const u_char *&Pkt_Data)
{
    Packet::init(Pkt_Hdr, Pkt_Data);
    this->setProto(PKT_PTC_ETHERNET);

    struct ether_header * eth_hdr = (struct ether_header *) (Pkt_Data);

    this->SourceMACaddr = static_cast<std::string> (ether_ntoa(reinterpret_cast<struct ether_addr *>(eth_hdr->ether_shost)));
    this->DestinationMACaddr = static_cast<std::string> (ether_ntoa(reinterpret_cast<struct ether_addr *>(eth_hdr->ether_dhost)));
}

void EthPacket::init(const long &TSsec, const long &TSmsec, const long &CapLength, const long &Length, const std::string &SrcMACaddr, const std::string &DstMACaddr)
{
    Packet::init(TSsec, TSmsec, CapLength, Length);
    Packet::setProto(PKT_PTC_ETHERNET);
    this->SourceMACaddr = SrcMACaddr;
    this->DestinationMACaddr = DstMACaddr;
}

std::string const * EthPacket::SrcMACaddr() const
{
    return &(this->SourceMACaddr);
}

std::string const * EthPacket::DstMACaddr() const
{
    return &(this->DestinationMACaddr);
}
