#include "ippacket.h"

// constructor
IPPacket::IPPacket()
{
    this->init();
}

IPPacket::IPPacket(const pcap_pkthdr *&Pkt_Hdr, const u_char *&Pkt_Data)
{
    this->init(Pkt_Hdr, Pkt_Data);
}

// destructor
IPPacket::~IPPacket()
{

}

void IPPacket::init()
{
    EthPacket::init();
    this->setProto(PKT_PTC_IP);
}

void IPPacket::init(const pcap_pkthdr *&Pkt_Hdr, const u_char *&Pkt_Data)
{
    EthPacket::init(Pkt_Hdr, Pkt_Data);
    this->setProto(PKT_PTC_IP);

    struct ip * ip_hdr = (struct ip *) (Pkt_Data + ETHER_HDRLEN);

    this->SourceIPaddr = static_cast<std::string>(inet_ntoa(ip_hdr->ip_src));
    this->DestinationIPaddr = static_cast<std::string>(inet_ntoa(ip_hdr->ip_dst));

    //ip_hl Internet Header Length which is the length of the header in multiples of 4 bytes (eg. 5 means 20 bytes)
    this->IPheaderLen = (ip_hdr->ip_hl) * 4;

    this->IPlength = ntohs(ip_hdr->ip_len);

    this->TTLvalue = (int) ip_hdr->ip_ttl;
}

std::string const * IPPacket::SrcIPaddr() const
{
    return &(this->SourceIPaddr);
}

std::string const * IPPacket::DstIPaddr() const
{
    return &(this->DestinationIPaddr);
}

int const IPPacket::IPhdrLen() const
{
    return this->IPheaderLen;
}

int const IPPacket::IPlen() const
{
    return this->IPlength;
}

int const IPPacket::TTL() const
{
    return this->TTLvalue;
}
