#ifndef PCAPEXCEPTION_H_INCLUDED
#define PCAPEXCEPTION_H_INCLUDED

#include <stdexcept>
#include <string>

///
/// Generic PCap Exception.
///	Derives from std::runtime_error.
///
class PCapException : public std::runtime_error
{
	public:

		PCapException(std::string const& err_msg)
		:	std::runtime_error(err_msg)
		{ }

};

class Timeout_Expired : public PCapException
{
    public:

        Timeout_Expired()
        :   PCapException("Timeout expired.")
        { }
};

class Packet_Reading_Error : public PCapException
{
    public:
        Packet_Reading_Error(std::string const& err_msg)
        :   PCapException(err_msg)
        { }
};

class No_More_Packets : public PCapException
{
    public:

        No_More_Packets()
        :   PCapException("No more packets to read.")
        { }
};

class Dev_Open_Offline_Fail : public PCapException
{
    public:

        Dev_Open_Offline_Fail(std::string const& err_msg)
        :   PCapException(err_msg)
        { }
};

class Dev_Open_Live_Fail : public PCapException
{
    public:

        Dev_Open_Live_Fail(std::string const& err_msg)
        :   PCapException(err_msg)
        { }
};

class Dev_Open_Live_Warning : public PCapException
{
    public:

        Dev_Open_Live_Warning(std::string const& err_msg)
        :   PCapException(err_msg)
        { }
};

class Src_Already_Open : public PCapException
{
    public:

        Src_Already_Open()
        :   PCapException("Object PacketSource is already open.")
        { }
};

class Src_Not_Open : public PCapException
{
    public:

        Src_Not_Open()
        :   PCapException("PacketSource is not open.")
        { }
};

class Not_Ethernet_Source : public PCapException
{
    public:

        Not_Ethernet_Source()
        :   PCapException("The source is not an Ethernet link layer.")
        { }
};

class Dev_Find_Fail : public PCapException
{
    public:

        Dev_Find_Fail(std::string const& err_msg)
        :   PCapException(err_msg)
        { }
};


#endif // PCAPEXCEPTION_H_INCLUDED
