/*
 * \author João V. Gomes
 *
 *  packetprocessor.cpp
 *
 *  Created on: Jun 4, 2011
 *
 * \brief TODO
 * TODO
 */

#include "packetprocessor.h"

PacketProcessor::PacketProcessor(PacketSource *PacketSrc)
{
    this->init(PacketSrc, false, "255.255.255.255", "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength)
{
    this->init(PacketSrc, OnlyPayloadLength, "255.255.255.255", "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const std::string &IPaddr)
{
    this->init(PacketSrc, false, IPaddr, "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddr)
{
    this->init(PacketSrc, OnlyPayloadLength, IPaddr, "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const std::string &IPaddr, const std::string &Mask)
{
    this->init(PacketSrc, false, IPaddr, Mask);
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddr, const std::string &Mask)
{
    this->init(PacketSrc, OnlyPayloadLength, IPaddr, Mask);
}

PacketProcessor::~PacketProcessor()
{
    //dtor
}

void PacketProcessor::init(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddr, const std::string &Mask)
{
    this->ResetPacketSource(PacketSrc);

    this->PacketSizeProp = false;
    this->InterArrivalTimeProp = false;
    this->IPPortProp = false;

    this->AllTraffic = false;
    this->IncomingTraffic = false;
    this->OutgoingTraffic = false;

    this->FilterAck = false;
    this->FilterSyn = false;
    this->FilterFull = false;
    this->FilterTCPnoPayload = false;
    this->FilterNoTCP = false;
    this->FilterNoLess60 = false;
    this->FilterNoLess65 = false;
    this->FilterTCPPayloadLess5 = false;
    this->FilterPayloadLess5 = false;

    this->IPaddress = inet_addr(IPaddr.c_str());
    this->NetMask = inet_addr(Mask.c_str());

    this->OnlyPayloadLen = OnlyPayloadLength;
    this->Segmentation = false;
    this->SegmentationBase = 1;
}

void PacketProcessor::SetSegmentation(const bool &Seg, const int &Base)
{
	this->Segmentation = Seg;
	this->SegmentationBase = Base;
}

void PacketProcessor::SetPropertiesToAnalyze(const int &Prop)
{
	if (Prop == PACKET_SIZE)
		this->PacketSizeProp = true;
	else if (Prop == INTER_ARRIVAL_TIME)
		this->InterArrivalTimeProp = true;
	else if (Prop == IP_PORT)
		this->IPPortProp = true;
}

void PacketProcessor::UnsetPropertiesToAnalyze(const int &Prop)
{
	if (Prop == PACKET_SIZE)
		this->PacketSizeProp = false;
	else if (Prop == INTER_ARRIVAL_TIME)
		this->InterArrivalTimeProp = false;
	else if (Prop == IP_PORT)
		this->IPPortProp = false;
}

void PacketProcessor::SetDirections(const int &Direction)
{
	if (Direction == DIR_OUT)
		this->OutgoingTraffic = true;
	else if (Direction == DIR_IN)
		this->IncomingTraffic = true;
	else if (Direction == DIR_ALL)
		this->AllTraffic = true;
}

void PacketProcessor::UnsetDirections(const int &Direction)
{
	if (Direction == DIR_OUT)
		this->OutgoingTraffic = false;
	else if (Direction == DIR_IN)
		this->IncomingTraffic = false;
	else if (Direction == DIR_ALL)
		this->AllTraffic = false;
}

void PacketProcessor::SetFilters(const int &Filter)
{
	if (Filter == ACK_PCKT)
		this->FilterAck = true;
	else if (Filter == SYN_PCKT)
		this->FilterSyn = true;
	else if (Filter == FULL_PCKT)
		this->FilterFull = true;
	else if (Filter == NO_PAYLOAD)
		this->FilterTCPnoPayload = true;
	else if (Filter == NO_TCP)
		this->FilterNoTCP = true;
	else if (Filter == NO_LESS_60)
		this->FilterNoLess60 = true;
	else if (Filter == NO_LESS_65)
		this->FilterNoLess65 = true;
	else if (Filter == NO_TCP_PL_LESS_5)
		this->FilterTCPPayloadLess5 = true;
	else if (Filter == NO_PL_LESS_5)
		this->FilterPayloadLess5 = true;
}

void PacketProcessor::UnsetFilters(const int &Filter)
{
	if (Filter == ACK_PCKT)
		this->FilterAck = false;
	else if (Filter == SYN_PCKT)
		this->FilterSyn = false;
	else if (Filter == FULL_PCKT)
		this->FilterFull = false;
	else if (Filter == NO_PAYLOAD)
		this->FilterTCPnoPayload = false;
	else if (Filter == NO_TCP)
		this->FilterNoTCP = false;
	else if (Filter == NO_LESS_60)
		this->FilterNoLess60 = false;
	else if (Filter == NO_LESS_65)
		this->FilterNoLess65 = false;
	else if (Filter == NO_TCP_PL_LESS_5)
		this->FilterTCPPayloadLess5 = false;
	else if (Filter == NO_PL_LESS_5)
		this->FilterPayloadLess5 = false;
}

void PacketProcessor::ExcludeIP(const std::string &IPaddr)
{
	this->ExcludedIPs.insert(IPaddr);
}

void PacketProcessor::AllowExcludedIP(const std::string &IPaddr)
{
	this->ExcludedIPs.erase(IPaddr);
}

void PacketProcessor::ResetPacketSource(PacketSource *PacketSrc)
{
    if (PacketSrc->IsOpen())
        this->PcktSrc = PacketSrc;
    else
        throw PSrc_Not_Open();
}

const bool PacketProcessor::SelectHost(const std::string &IPaddr) const
{
	std::set<std::string>::iterator it;

    for(it = this->ExcludedIPs.begin(); it != this->ExcludedIPs.end(); it++)
    	if (inet_addr((*it).c_str()) == inet_addr(IPaddr.c_str()))
    			return false;

	if (inet_addr("255.255.255.255") == this->IPaddress)
		return true;

	if ((this->NetMask & inet_addr(IPaddr.c_str())) == this->IPaddress)
		return true;

	return false;
}

bool const PacketProcessor::FilterPacket(const Packet * Pckt) const
{
	if (this->FilterNoTCP)
		if (Pckt->Protocol() == PKT_PTC_TCP)
			return true;

	if ( this->FilterFull )
		if (Pckt->Len() == 1514)
			return true;

	if ( this->FilterNoLess60 )
		if (Pckt->Len() <= 60)
			return true;

	if ( this->FilterNoLess65 )
		if (Pckt->Len() <= 65)
			return true;

	if ( Pckt->Protocol() == PKT_PTC_UDP )
	{
		const UDPPacket * UDPP = reinterpret_cast<UDPPacket const *>(Pckt);

		if ( this->FilterPayloadLess5 && (UDPP->UdpPayloadLen() <= 5) )
			return true;
	}

	if ( Pckt->Protocol() == PKT_PTC_TCP )
	{
		const TCPPacket * TCPP = reinterpret_cast<TCPPacket const *>(Pckt);

		if ( (this->FilterTCPPayloadLess5 || this->FilterPayloadLess5) &&
			 (TCPP->TcpPayloadLen() <= 5) )
			return true;

		if ( this->FilterSyn && TCPP->Syn() )
			return true;

		if ( this->FilterTCPnoPayload && (TCPP->TcpPayloadLen() == 0) )
			return true;

		if ( this->FilterAck &&
			 TCPP->Ack()     &&
			 (TCPP->TcpPayloadLen() == 0) )
//					if ( !(TCPP->Syn()) 	  &&
//						 !(TCPP->Rst())       &&
//						 !(TCPP->Fin())       &&
//						 !(TCPP->Urg())       &&
//						 !(TCPP->Psh())          )
			return true;
	}

	return false;
}

const int PacketProcessor::GetHostClassification(const std::string &Host)
{
	if (this->EntropyTCPNoPayload.count(Host) == 0 || this->EntropyOutgoing.count(Host) == 0 || this->EntropyOutgoingSegmented.count(Host) == 0)
		return 0;

	if (this->EntropyTCPNoPayload.at(Host) > 3.0)
		return 1;

	if (this->EntropyTCPNoPayload.at(Host) < 1.0)
		return 0;

//	if (this->EntropyOutgoing.at(Host) > 1.0)
//		return 1;

	if (this->EntropyOutgoing.at(Host) < 0.5)
		return 0;

	if (this->EntropyOutgoingSegmented.at(Host) < 0.1)
		return 0;

	return 1;
}

void PacketProcessor::PrintClassification(Folder * LogsFolder, const long &TSsec, const long &TSusec, const int &Length, const std::string &Host)
{
    std::string fileName = Host + ".log";
    double dTS = TSsec + TSusec * 0.000001;
    std::stringstream ss (std::stringstream::out);

    if (this->FirstTS.at(Host) == dTS)
    {
        std::stringstream ss_hdr (std::stringstream::out);

        time_t rawtime;
        struct tm * timeinfo;

        time ( &rawtime );
        timeinfo = localtime ( &rawtime );

        ss_hdr << "#\n"
    	   << "# Log for host " << Host << "\n"
    	   << "#\n"
    	   << "# Created on " << asctime (timeinfo)
    	   << "#\n"
    	   << "# Window of " << this->WinSize << " packets\n"
    	   << "#\n"
		   << "#\n"
    	   << "# <Timestamp>	<Time elapsed>	<Length>	<Entropy excluding packets with no TCP payload>	<Entropy for outgoing traffic>	<Entropy for outgoing traffic with segments of " << this->SegmentationBase << " bytes>	Classification\n"
    	   << "#" << std::endl;

    	LogsFolder->WriteNewFile(fileName, ss_hdr.str());
    }

    double EntTCPnoPL = 0, EntOut = 0, EntOutSeg = 0;

    if (this->EntropyTCPNoPayload.count(Host) > 0)		EntTCPnoPL = this->EntropyTCPNoPayload.at(Host);
    if (this->EntropyOutgoing.count(Host) > 0)			EntOut = this->EntropyOutgoing.at(Host);
    if (this->EntropyOutgoingSegmented.count(Host) > 0)	EntOutSeg = this->EntropyOutgoingSegmented.at(Host);

    ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << (dTS - this->FirstTS.at(Host)) << "\t" << Length << "\t" << EntTCPnoPL << "\t" << EntOut << "\t" << EntOutSeg << "\t" << this->GetHostClassification(Host) << std::endl;

    LogsFolder->WriteExistentFile(fileName, ss.str());
}

void PacketProcessor::ClassifyHosts(const int &Win, const std::string &LogsPath, const bool &PrintLogs)
{
    std::stringstream ss (std::stringstream::out);
	const int iEntIndex = 1;

	std::string Path = this->CleanLastBar(LogsPath);
	ss << Path << "/EntIndex_" << iEntIndex;
	Folder * LogsFolder = new Folder(ss.str());

	this->WinSize = Win;

	HostAnalyzer<int> * HAnEntTCPNoPayload = new HostAnalyzer<int>(WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
	HostAnalyzer<int> * HAnEntOut          = new HostAnalyzer<int>(WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);
	HostAnalyzer<int> * HAnEntOutSeg       = new HostAnalyzer<int>(WinSize, Path + PS_FOLDER + ALL_FOLDER, PrintLogs);

    bool end = false;

    while(!end)
        try
        {
            Packet *Pckt;

            Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();

            // Analyze only TCP and UDP packets
            if ( (Proto == PKT_PTC_TCP) || (Proto == PKT_PTC_UDP) ) //&& !(this->FilterPacket(Pckt)) )
            {
                IPPacket *IPP = static_cast<IPPacket *>(Pckt);

                std::string const *src = IPP->SrcIPaddr(), *dst = IPP->DstIPaddr();
                long TSsec = IPP->TSseconds(), TSusec = IPP->TSmicrosec();

                double dTS = TSsec + TSusec * 0.000001;

				int len = IPP->Len();

				double * EntSrc, * EntDst;

				if (this->SelectHost(*dst) && !(this->FilterPacket(Pckt)))
				{
					if (this->FirstTS.count(*dst) == 0)
						this->FirstTS.insert(std::pair<std::string, double>(*dst, dTS));

					EntDst = HAnEntTCPNoPayload->AddValue(TSsec, TSusec, *dst, len);

					if (this->EntropyTCPNoPayload.count(*dst) == 0)
						this->EntropyTCPNoPayload.insert(std::pair<std::string, double>(*dst, EntDst[iEntIndex]) );

					this->EntropyTCPNoPayload.at(*dst) = EntDst[iEntIndex];

					this->PrintClassification(LogsFolder, TSsec, TSusec, len, *dst);
				}

				if (this->SelectHost(*src))
				{
					if (this->FirstTS.count(*src) == 0)
						this->FirstTS.insert(std::pair<std::string, double>(*src, dTS));

					if ( !(this->FilterPacket(Pckt)) )
					{
						EntSrc = HAnEntTCPNoPayload->AddValue(TSsec, TSusec, *src, len);

						if (this->EntropyTCPNoPayload.count(*src) == 0)
							this->EntropyTCPNoPayload.insert(std::pair<std::string, double>(*src, EntSrc[iEntIndex]));

						this->EntropyTCPNoPayload.at(*src) = EntSrc[iEntIndex];
					}

					EntSrc = HAnEntOut->AddValue(TSsec, TSusec, *src, len);

					if (this->EntropyOutgoing.count(*src) == 0)
						this->EntropyOutgoing.insert(std::pair<std::string, double>(*src, EntSrc[iEntIndex]) );
					else
						this->EntropyOutgoing.at(*src) = EntSrc[iEntIndex];


					EntSrc = HAnEntOutSeg->AddValue(TSsec, TSusec, *src, len / this->SegmentationBase);
					if (this->EntropyOutgoingSegmented.count(*src) == 0)
						this->EntropyOutgoingSegmented.insert(std::pair<std::string, double>(*src, EntSrc[iEntIndex]) );
					else
						this->EntropyOutgoingSegmented.at(*src) = EntSrc[iEntIndex];

					this->PrintClassification(LogsFolder, TSsec, TSusec, len, *src);
				}
            }

            delete Pckt;
        }
        catch (No_More_Packets e1)
        {
            end = true;
        }
        catch (Timeout_Expired e2)
        {
            std::cout << "Timeout: " << e2.what() << "\n";
            end = true;
        }
        catch (Packet_Reading_Error e3)
        {
            std::cout << "Failed: " << e3.what() << "\n";
            end = true;
        }

	delete HAnEntTCPNoPayload;
	delete HAnEntOut;
	delete HAnEntOutSeg;
}

const std::string PacketProcessor::CleanLastBar(const std::string &Path) const
{
	std::string PathStr = Path;

	while ( *(PathStr.end() - 1) == ('/') )
		PathStr.erase(PathStr.end() - 1);

	return PathStr;
}
