/*
 * \author João V. Gomes
 *
 *  packetprocessor.h
 *
 *  Created on: Jun 4, 2011
 *
 * \brief TODO
 * TODO
 */

#ifndef PACKETPROCESSOR_H
#define PACKETPROCESSOR_H

#include <statisticalanalyzer.h>
#include <pcapwrap.h>
#include <ppexception.h>
#include <Folder.h>
#include <string>
#include <iostream>
#include <set>

#ifdef __cplusplus
extern "C"
{
#endif
#include <arpa/inet.h>
#include <time.h>
#ifdef __cplusplus
}
#endif

#ifndef STREAMS
#define STREAMS
#define DIR_OUT 0
#define DIR_IN 1
#define DIR_ALL 2
#endif // STREAMS

#ifndef PROPERTIESSET
#define PROPERTIESSET
#define PACKET_SIZE 0
#define INTER_ARRIVAL_TIME 1
#define IP_PORT 2
#endif // PROPERTIESSET

#ifndef FOLDERPATHS
#define FOLDERPATHS
#define	EX_FOLDER "/Extended"
#define	PS_FOLDER "/PacketSize"
#define	IA_FOLDER "/InterArrivalTime"
#define	IPP_FOLDER "/IPPort"
#define OUT_FOLDER "/OUT"
#define IN_FOLDER "/IN"
#define ALL_FOLDER "/ALL"
#endif // FOLDERPATHS

#ifndef FILTERS
#define FILTERS
#define ACK_PCKT 0
#define SYN_PCKT 1
#define FULL_PCKT 2
#define NO_PAYLOAD 3
#define NO_TCP 4
#define NO_LESS_60 5
#define NO_LESS_65 6
#define NO_TCP_PL_LESS_5 7
#define NO_PL_LESS_5 8
#endif // FILTERS

#ifndef PP_IAT_PRECISION
#define PP_IAT_PRECISION 6
#endif // PP_IAT_PRECISION

class PacketProcessor
{
    public:
		PacketProcessor(PacketSource *PacketSrc);
		PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength);
		PacketProcessor(PacketSource *PacketSrc, const std::string &IPaddress);
		PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddress);
		PacketProcessor(PacketSource *PacketSrc, const std::string &IPaddress, const std::string &NetMask);
		PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddress, const std::string &NetMask);
        virtual ~PacketProcessor();

        void ClassifyHosts(const int &WinSize, const std::string &LogsPath, const bool &PrintLogs);

		void AllowExcludedIP(const std::string &IPaddr);
		void ExcludeIP(const std::string &IPaddr);
        void ResetPacketSource(PacketSource *PacketSrc);
        void SetPropertiesToAnalyze(const int &Prop);
        void UnsetPropertiesToAnalyze(const int &Prop);
        void SetDirections(const int &Direction);
        void UnsetDirections(const int &Direction);
        void SetFilters(const int &Filter);
        void UnsetFilters(const int &Filter);
        void SetSegmentation(const bool &Seg, const int &Base);

    protected:
    private:
        PacketSource *PcktSrc;
        bool PacketSizeProp;
        bool InterArrivalTimeProp;
        bool IPPortProp;

        bool OutgoingTraffic;
        bool IncomingTraffic;
        bool AllTraffic;

        int WinSize;

        std::map<std::string, double> FirstTS;

        bool FilterAck;
        bool FilterSyn;
        bool FilterFull;
        bool FilterTCPnoPayload;
        bool FilterNoTCP;
        bool FilterNoLess60;
        bool FilterNoLess65;
        bool FilterTCPPayloadLess5;
        bool FilterPayloadLess5;

        bool OnlyPayloadLen;
        bool Segmentation;

        int SegmentationBase;

        std::set<std::string> ExcludedIPs;
        std::map<std::string, double> EntropyTCPNoPayload;
        std::map<std::string, double> EntropyOutgoing;
        std::map<std::string, double> EntropyOutgoingSegmented;

        in_addr_t IPaddress;	// analyze only the hosts with this address (if NetMask is not defined) or
        in_addr_t NetMask;		// the host whose IPs belong to the subnet defined by IPaddress and NetMask

        void init(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddress, const std::string &NetMask);

        const std::string CleanLastBar(const std::string &Path) const;
        const bool FilterPacket(const Packet * Pckt) const;
        const bool SelectHost(const std::string &IPaddr) const;
        const int GetHostClassification(const std::string &Host);
        void PrintClassification(Folder * LogsFolder, const long &TSsec, const long &TSusec, const int &Length, const std::string &Host);
};

#endif // PACKETPROCESSOR_H

