#ifndef ENTROPYCONTAINER_H
#define ENTROPYCONTAINER_H

#ifdef __cplusplus
extern "C"
{
#endif
#include <math.h>
#ifdef __cplusplus
}
#endif

#include <slidingwindow.h>
#include <meanvariancedump.h>

#ifndef OUTPUT_LEVEL
#define OUTPUT_LEVEL
#define OUT_ENTROPY_VAL 0
#define OUT_ENTROPY 1
#define OUT_VALUES 2
#define OUT_WINDOW 3
#endif

#include <string>

template<class V> class EntropyContainer
{
    public:
		EntropyContainer();
		EntropyContainer(const int &WindowSize);
        virtual ~EntropyContainer();

        // Entropy
        const double EntropyVal() const;
        const double EntropyMean() const;
        const double EntropyVariance() const;
        const double EntropyMax() const;
        const double EntropyMin() const;

        // Values
        const double ValueMean() const;
        const double ValueExpMean() const;
        const double ValueVariance() const;

        // Limits in the window
        const double WinMean() const;
        const V Max() const;
        const double MaxMean() const;
        const double MaxVariance() const;
        const V Min() const;
        const double MinMean() const;
        const double MinVariance() const;

        void AddValue(const V &value);
        const long TotalAnalyzedItems() const;
        double * GetStats();
        double * GetStatsClassification();
        void Clear();
        const int WindowSize() const;
    protected:
    private:
        void init(const int &WindowSize);
        void UpdateMean();
        void UpdateExpMean(const V &Val);
        void UpdateLimits();
        void dispose();

        MeanVarianceDump * EntropyDump;
        MeanVarianceDump * MaxDump;
        MeanVarianceDump * MinDump;
        MeanVarianceDump * ValueDump;
        SlidingWindow<V> *Win;
        double CurrentEntropy;
        double MaxEntropy;
        double MinEntropy;
        double ExpMean;
        long Total;

        double *aStats;
};

#endif // ENTROPYCONTAINER_H
