/*
 * \author João V. Gomes
 *
 *  flowanalyzerextended.h
 *
 *  Created on: Apr 12, 2011
 *
 * \brief TODO
 * TODO
 */

#ifndef FLOWANALYZEREXTENDED_H_
#define FLOWANALYZEREXTENDED_H_

#ifdef __cplusplus
extern "C"
{
#endif
#include <time.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include <string.h>
#ifdef __cplusplus
}
#endif

#include <entropycontainer.h>
#include <direxception.h>
#include <Folder.h>
#include <map>
#include <string>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <algorithm>
#include <pcapwrap.h>
#include <slidingwindow.h>

#ifndef MAX_TIMEOUT
#define MAX_TIMEOUT
#define MAX_UDP_TIMEOUT 64  // (in seconds) based on Claffy et al., "A parameterizable methodology for Internet traffic flow profiling"
#define MSL 120 			// (in seconds) Maximum Segment Life for TCP: 2 minutes based on RFC 793
#endif

#ifndef DIRECTIONS
#define DIRECTIONS
#define CS 1	// client-server direction (client is usually the one who starts the connection)
#define SC 2
#endif

#ifndef TCP_STATE
#define TCP_STATE
#define CLOSED 0
#define RST 1
#define FIN 2
#define INITIAL 3
#endif

#ifndef RANGES_MAX
#define RANGES_MAX
#define RANGES_MAX_1 100
#define RANGES_MAX_2 1000
#define RANGES_MAX_3 1600
#endif // RANGES_MAX

#ifndef DEFAULT_PRECISION
#define DEFAULT_PRECISION 6
#endif // DEFAULT_PRECISION

class FlowAnalyzerExtended
{
    public:
		FlowAnalyzerExtended(const int &WindowSize);
		FlowAnalyzerExtended(const int &WindowSize, const std::string &LogsBase, const std::string &LogsComplement);
		FlowAnalyzerExtended(const int &WindowSize, const std::string &LogsBase, const std::string &LogsComplement, const bool &PrintSummary, const bool &PrintLogs);
		FlowAnalyzerExtended(const int &WindowSize, const int &precision);
		FlowAnalyzerExtended(const int &WindowSize, const int &precision, const std::string &LogsBase, const std::string &LogsComplement);
		FlowAnalyzerExtended(const int &WindowSize, const int &precision, const std::string &LogsBase, const std::string &LogsComplement, const bool &PrintSummary, const bool &PrintLogs);
        virtual ~FlowAnalyzerExtended();
        void AddPacket(const Packet *pPkt, const bool &OnlyPayloadLen);
        unsigned long TotTCPflows() const;
        unsigned long TotTCPpackets() const;
        unsigned long TotUDPflows() const;
        unsigned long TotUDPpackets() const;
        void Clear();
    protected:
    private:
        // Flow, Entropy container
        std::map<std::string, EntropyContainer<int> *> Lengths;
        std::map<std::string, EntropyContainer<int> *> LengthsRange1;
        std::map<std::string, EntropyContainer<int> *> LengthsRange2;
        std::map<std::string, EntropyContainer<int> *> LengthsRange3;
        std::map<std::string, EntropyContainer<double> *> IAtimes;
        std::map<std::string, EntropyContainer<double> *> IAtimesRange1;
        std::map<std::string, EntropyContainer<double> *> IAtimesRange2;
        std::map<std::string, EntropyContainer<double> *> IAtimesRange3;
        std::map<std::string, EntropyContainer<int> *> TTLs;
        std::map<std::string, EntropyContainer<int> *> TTLsRange1;
        std::map<std::string, EntropyContainer<int> *> TTLsRange2;
        std::map<std::string, EntropyContainer<int> *> TTLsRange3;

        std::map<std::string, int> HalfClosedCSflows;		// < Flow , state >
        std::map<std::string, int> HalfClosedSCflows;		// < Flow , state >
        std::map<std::string, double> LastTS;				// < Flow , timestamp >
        std::map<std::string, long> NumPackets;       		// < Flow , Number of Packets >
        std::map<std::string, long long> TotalBytes;       	// < Flow , Number of Packets >
        std::map<std::string, long long> TotalBytesR1;      // < Flow , Number of Packets >
        std::map<std::string, long long> TotalBytesR2;      // < Flow , Number of Packets >
        std::map<std::string, long long> TotalBytesR3;      // < Flow , Number of Packets >
        std::map<std::string, int> NumFlows;				// < Flow , Number of Flows with the same Key >
        std::map<std::string, double> FirstTS;
        unsigned long TotalTCPflows;
        unsigned long TotalTCPpackets;
        unsigned long TotalUDPflows;
        unsigned long TotalUDPpackets;
        int WinSize;
        int precision;
        Folder LogsFolder;
    	Folder LogsSummary;
    	std::string SummaryFileName;
    	bool PrintSummary;
    	bool PrintLogs;

        void init(const int &WindowSize, const int &precision, const std::string &LogsBase, const std::string &LogsComplement, const bool &PrintSummary, const bool &PrintLogs);
        void dispose();
        void WriteEntropy(const std::string &Key, const double &TStamp,
						  const int &Length, const double &IAtime, const int &TTL,
						  EntropyContainer<int> * EntropyLengths, EntropyContainer<int> * EntropyLengthsRange1, EntropyContainer<int> * EntropyLengthsRange2, EntropyContainer<int> * EntropyLengthsRange3,
						  EntropyContainer<double> * EntropyIAtimes, EntropyContainer<double> * EntropyIAtimesRange1, EntropyContainer<double> * EntropyIAtimesRange2, EntropyContainer<double> * EntropyIAtimesRange3,
						  EntropyContainer<int> * EntropyTTLs, EntropyContainer<int> * EntropyTTLsRange1, EntropyContainer<int> * EntropyTTLsRange2, EntropyContainer<int> * EntropyTTLsRange3);
    	void WriteSummary();
        void WriteNewLog(const std::string &Key, const std::string &Msg);
        void WriteExistentLog(const std::string &Key, const std::string &Msg);
        void NewFlow(const std::string &Key, const double &TStamp, const std::string &srcIP, const int &sPort, const std::string &dstIP, const int dPort, const std::string &Proto);
        void CloseConnection(const std::string &Key);
        void UpdateHalfClosedFlows(const std::string &Key, const int &Direction, const bool &tagFIN, const bool &tagRST, const bool &tagACK);
        const double Round(const double &number, const int &precision) const;
        long long Max(long long R1Val, long long R2Val, long long R3Val) const;
    	const double Max(const double &val1, const double &val2, const double &val3) const;
};

#endif /* FLOWANALYZEREXTENDED_H_ */
