#ifndef HOSTANALYZER_H
#define HOSTANALYZER_H

#ifdef __cplusplus
extern "C"
{
#endif
#include <time.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include <string.h>
#ifdef __cplusplus
}
#endif

#include <entropycontainer.h>
#include <Folder.h>
#include <thresholdsset.h>
#include <slidingwindow.h>

#include <map>
#include <string>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>

template<class V>
class HostAnalyzer
{
    public:
		HostAnalyzer(const int &WindowSize);
        HostAnalyzer(const int &WindowSize, const std::string &LogsPath);
        HostAnalyzer(const int &WindowSize, const std::string &LogsPath, ThresholdsSet * TSet);
        HostAnalyzer(const int &WindowSize, const std::string &LogsPath, const bool &PrintLogs);
        HostAnalyzer(const int &WindowSize, const std::string &LogsPath, ThresholdsSet * TSet, const bool &PrintLogs);
        virtual ~HostAnalyzer();
        double * AddValue(const std::string &HostIP, const V &Value);
        double * AddValue(const long &TSsec, const long &TSusec, const std::string &HostIP, const V &Value);
        template <class D> double * AddValueDif(const std::string &HostIP, const D &Value, int precision);
        template <class D> double * AddValueDif(const long &TSsec, const long &TSusec, const std::string &HostIP, const D &Value, int precision);

        void SetPrintLogs(const bool &PrintL);
        void SetLogsFolter(const std::string &PrintL);
        unsigned long TotHosts() const;
        unsigned long TotValues() const;
        void Clear();
    protected:
        virtual void WriteEntropy(const std::string &HostIP, const long &TSsec, const long &TSusec, const V &Value, EntropyContainer<V>* Entropy);
        void init(const int &WindowSize, const std::string &Path, ThresholdsSet * TSet, const bool &PrintLogs);
        void WriteExistentLog(const std::string &HostIP, const std::string &Msg) const;
    private:
        std::map<std::string,EntropyContainer<V> *> Hosts;
        unsigned long TotalHosts;
        unsigned long TotalValues;
        int WinSize;
        Folder LogsFolder;
        bool PrintLogs;
        ThresholdsSet * Thresholds;
        std::map<std::string, V> LastValue;
        std::map<std::string, double> FirstTS;

        void dispose();
        void WriteNewLog(const std::string &HostIP, const std::string &Msg) const;
        template <class D> const D Round(const D &number, const int &precision) const;
};

#endif // HOSTANALYZER_H
