/*
 * \author João V. Gomes
 *
 *  hostanalyzerextended.cpp
 *
 *  Created on: Apr 12, 2011
 *
 * \brief TODO
 * TODO
 */

#include "hostanalyzerextended.h"

HostAnalyzerExtended::HostAnalyzerExtended() {

}

HostAnalyzerExtended::HostAnalyzerExtended(const int &WindowSize)
{
    init(WindowSize, DEFAULT_PRECISION, std::string());
}

HostAnalyzerExtended::HostAnalyzerExtended(const int &WindowSize, const std::string &LogsPath)
{
    init(WindowSize, DEFAULT_PRECISION, LogsPath);
}

HostAnalyzerExtended::HostAnalyzerExtended(const int &WindowSize, const int &Precision)
{
    init(WindowSize, Precision, std::string());
}

HostAnalyzerExtended::HostAnalyzerExtended(const int &WindowSize, const int &Precision, const std::string &LogsPath)
{
    init(WindowSize, Precision, LogsPath);
}

HostAnalyzerExtended::~HostAnalyzerExtended()
{
    this->dispose();
}

void HostAnalyzerExtended::Clear()
{
    this->dispose();

    this->Lengths.clear();
    this->LengthsRange1.clear();
    this->LengthsRange2.clear();
    this->LengthsRange3.clear();

    this->TTLs.clear();
    this->TTLsRange1.clear();
    this->TTLsRange2.clear();
    this->TTLsRange3.clear();

    this->IAtimes.clear();
    this->IAtimesRange1.clear();
    this->IAtimesRange2.clear();
    this->IAtimesRange3.clear();

    this->HostPorts.clear();
    this->HostPortsRange1.clear();
    this->HostPortsRange2.clear();
    this->HostPortsRange3.clear();

    this->TotalHosts = 0;
    this->TotalPackets = 0;
	this->Precision = DEFAULT_PRECISION;

    this->WinSize = 0;
}

void HostAnalyzerExtended::dispose()
{
	std::map<std::string, EntropyContainer<int> *>::iterator it;

    for(it = this->Lengths.begin(); it != this->Lengths.end(); it++)
    {
        delete (it->second);
		delete this->LengthsRange1.at(it->first);
		delete this->LengthsRange2.at(it->first);
		delete this->LengthsRange3.at(it->first);

        delete this->IAtimes.at(it->first);
        delete this->IAtimesRange1.at(it->first);
        delete this->IAtimesRange2.at(it->first);
        delete this->IAtimesRange3.at(it->first);

        delete this->TTLs.at(it->first);
        delete this->TTLsRange1.at(it->first);
        delete this->TTLsRange2.at(it->first);
        delete this->TTLsRange3.at(it->first);

        delete this->HostPorts.at(it->first);
        delete this->HostPortsRange1.at(it->first);
        delete this->HostPortsRange2.at(it->first);
        delete this->HostPortsRange3.at(it->first);
    }
}

void HostAnalyzerExtended::init(const int &WindowSize, const int &Precision, const std::string &Path)
{
    this->TotalHosts = 0;
    this->TotalPackets = 0;
    this->WinSize = WindowSize;
	this->Precision = DEFAULT_PRECISION;

    this->LogsFolder.InitFolder(Path);
}

void HostAnalyzerExtended::WriteEntropy(const std::string &SrcIP,
										const long &TSsec, const long &TSusec,
										const int &Length, const double &IAtime, const int &TTL, const std::string &HostPort,
										EntropyContainer<int> * EntropyLengths, EntropyContainer<int> * EntropyLengthsRange1, EntropyContainer<int> * EntropyLengthsRange2, EntropyContainer<int> * EntropyLengthsRange3,
										EntropyContainer<double> * EntropyIAtimes, EntropyContainer<double> * EntropyIAtimesRange1, EntropyContainer<double> * EntropyIAtimesRange2, EntropyContainer<double> * EntropyIAtimesRange3,
										EntropyContainer<int> * EntropyTTLs, EntropyContainer<int> * EntropyTTLsRange1, EntropyContainer<int> * EntropyTTLsRange2, EntropyContainer<int> * EntropyTTLsRange3,
										EntropyContainer<std::string> * EntropyHostPorts, EntropyContainer<std::string> * EntropyHostPortsRange1, EntropyContainer<std::string> * EntropyHostPortsRange2, EntropyContainer<std::string> * EntropyHostPortsRange3)
{
    std::stringstream ss (std::stringstream::out);

	const double dTS = TSsec + ( TSusec * 0.000001 );
	double dTimeElapsed;

    if (this->FirstTS.count(SrcIP) == 0)
		this->FirstTS.insert(std::pair<std::string, double>(SrcIP, dTS));

	dTimeElapsed = dTS - (this->FirstTS[SrcIP]);

	ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << dTimeElapsed << "\t" <<

		Length << "\t" <<
		EntropyLengths->EntropyVal() << "\t" << EntropyLengthsRange1->EntropyVal() << "\t" << EntropyLengthsRange2->EntropyVal() << "\t" << EntropyLengthsRange3->EntropyVal() << "\t" <<
		EntropyLengths->EntropyMean() << "\t" << EntropyLengthsRange1->EntropyMean() << "\t" << EntropyLengthsRange2->EntropyMean() << "\t" << EntropyLengthsRange3->EntropyMean() << "\t" <<
		EntropyLengths->EntropyVariance() << "\t" << EntropyLengthsRange1->EntropyVariance() << "\t" << EntropyLengthsRange2->EntropyVariance() << "\t" << EntropyLengthsRange3->EntropyVariance() << "\t" <<
		EntropyLengths->EntropyMin() << "\t" << EntropyLengthsRange1->EntropyMin() << "\t" << EntropyLengthsRange2->EntropyMin() << "\t" << EntropyLengthsRange3->EntropyMin() << "\t" <<
		EntropyLengths->EntropyMax() << "\t" << EntropyLengthsRange1->EntropyMax() << "\t" << EntropyLengthsRange2->EntropyMax() << "\t" << EntropyLengthsRange3->EntropyMax() << "\t" <<
		EntropyLengths->ValueMean() << "\t" << EntropyLengthsRange1->ValueMean() << "\t" << EntropyLengthsRange2->ValueMean() << "\t" << EntropyLengthsRange3->ValueMean() << "\t" <<
		EntropyLengths->ValueVariance() << "\t" << EntropyLengthsRange1->ValueVariance() << "\t" << EntropyLengthsRange2->ValueVariance() << "\t" << EntropyLengthsRange3->ValueVariance() << "\t" <<
		EntropyLengths->Min() << "\t" << EntropyLengthsRange1->Min() << "\t" << EntropyLengthsRange2->Min() << "\t" << EntropyLengthsRange3->Min() << "\t" <<
		EntropyLengths->Max() << "\t" << EntropyLengthsRange1->Max() << "\t" << EntropyLengthsRange2->Max() << "\t" << EntropyLengthsRange3->Max() << "\t" <<
		EntropyLengths->MinMean() << "\t" << EntropyLengthsRange1->MinMean() << "\t" << EntropyLengthsRange2->MinMean() << "\t" << EntropyLengthsRange3->MinMean() << "\t" <<
		EntropyLengths->MaxMean() << "\t" << EntropyLengthsRange1->MaxMean() << "\t" << EntropyLengthsRange2->MaxMean() << "\t" << EntropyLengthsRange3->MaxMean() << "\t" <<
		EntropyLengths->MinVariance() << "\t" << EntropyLengthsRange1->MinVariance() << "\t" << EntropyLengthsRange2->MinVariance() << "\t" << EntropyLengthsRange3->MinVariance() << "\t" <<
		EntropyLengths->MaxVariance() << "\t" << EntropyLengthsRange1->MaxVariance() << "\t" << EntropyLengthsRange2->MaxVariance() << "\t" << EntropyLengthsRange3->MaxVariance() << "\t" <<

		IAtime << "\t" <<
		EntropyIAtimes->EntropyVal() << "\t" << EntropyIAtimesRange1->EntropyVal() << "\t" << EntropyIAtimesRange2->EntropyVal() << "\t" << EntropyIAtimesRange3->EntropyVal() << "\t" <<
		EntropyIAtimes->EntropyMean() << "\t" << EntropyIAtimesRange1->EntropyMean() << "\t" << EntropyIAtimesRange2->EntropyMean() << "\t" << EntropyIAtimesRange3->EntropyMean() << "\t" <<
		EntropyIAtimes->EntropyVariance() << "\t" << EntropyIAtimesRange1->EntropyVariance() << "\t" << EntropyIAtimesRange2->EntropyVariance() << "\t" << EntropyIAtimesRange3->EntropyVariance() << "\t" <<
		EntropyIAtimes->EntropyMin() << "\t" << EntropyIAtimesRange1->EntropyMin() << "\t" << EntropyIAtimesRange2->EntropyMin() << "\t" << EntropyIAtimesRange3->EntropyMin() << "\t" <<
		EntropyIAtimes->EntropyMax() << "\t" << EntropyIAtimesRange1->EntropyMax() << "\t" << EntropyIAtimesRange2->EntropyMax() << "\t" << EntropyIAtimesRange3->EntropyMax() << "\t" <<
		EntropyIAtimes->ValueMean() << "\t" << EntropyIAtimesRange1->ValueMean() << "\t" << EntropyIAtimesRange2->ValueMean() << "\t" << EntropyIAtimesRange3->ValueMean() << "\t" <<
		EntropyIAtimes->ValueVariance() << "\t" << EntropyIAtimesRange1->ValueVariance() << "\t" << EntropyIAtimesRange2->ValueVariance() << "\t" << EntropyIAtimesRange3->ValueVariance() << "\t" <<
		EntropyIAtimes->Min() << "\t" << EntropyIAtimesRange1->Min() << "\t" << EntropyIAtimesRange2->Min() << "\t" << EntropyIAtimesRange3->Min() << "\t" <<
		EntropyIAtimes->Max() << "\t" << EntropyIAtimesRange1->Max() << "\t" << EntropyIAtimesRange2->Max() << "\t" << EntropyIAtimesRange3->Max() << "\t" <<
		EntropyIAtimes->MinMean() << "\t" << EntropyIAtimesRange1->MinMean() << "\t" << EntropyIAtimesRange2->MinMean() << "\t" << EntropyIAtimesRange3->MinMean() << "\t" <<
		EntropyIAtimes->MaxMean() << "\t" << EntropyIAtimesRange1->MaxMean() << "\t" << EntropyIAtimesRange2->MaxMean() << "\t" << EntropyIAtimesRange3->MaxMean() << "\t" <<
		EntropyIAtimes->MinVariance() << "\t" << EntropyIAtimesRange1->MinVariance() << "\t" << EntropyIAtimesRange2->MinVariance() << "\t" << EntropyIAtimesRange3->MinVariance() << "\t" <<
		EntropyIAtimes->MaxVariance() << "\t" << EntropyIAtimesRange1->MaxVariance() << "\t" << EntropyIAtimesRange2->MaxVariance() << "\t" << EntropyIAtimesRange3->MaxVariance() << "\t" <<

		TTL << "\t" <<
		EntropyTTLs->EntropyVal() << "\t" << EntropyTTLsRange1->EntropyVal() << "\t" << EntropyTTLsRange2->EntropyVal() << "\t" << EntropyTTLsRange3->EntropyVal() << "\t" <<
		EntropyTTLs->EntropyMean() << "\t" << EntropyTTLsRange1->EntropyMean() << "\t" << EntropyTTLsRange2->EntropyMean() << "\t" << EntropyTTLsRange3->EntropyMean() << "\t" <<
		EntropyTTLs->EntropyVariance() << "\t" << EntropyTTLsRange1->EntropyVariance() << "\t" << EntropyTTLsRange2->EntropyVariance() << "\t" << EntropyTTLsRange3->EntropyVariance() << "\t" <<
		EntropyTTLs->EntropyMin() << "\t" << EntropyTTLsRange1->EntropyMin() << "\t" << EntropyTTLsRange2->EntropyMin() << "\t" << EntropyTTLsRange3->EntropyMin() << "\t" <<
		EntropyTTLs->EntropyMax() << "\t" << EntropyTTLsRange1->EntropyMax() << "\t" << EntropyTTLsRange2->EntropyMax() << "\t" << EntropyTTLsRange3->EntropyMax() << "\t" <<
		EntropyTTLs->ValueMean() << "\t" << EntropyTTLsRange1->ValueMean() << "\t" << EntropyTTLsRange2->ValueMean() << "\t" << EntropyTTLsRange3->ValueMean() << "\t" <<
		EntropyTTLs->ValueVariance() << "\t" << EntropyTTLsRange1->ValueVariance() << "\t" << EntropyTTLsRange2->ValueVariance() << "\t" << EntropyTTLsRange3->ValueVariance() << "\t" <<
		EntropyTTLs->Min() << "\t" << EntropyTTLsRange1->Min() << "\t" << EntropyTTLsRange2->Min() << "\t" << EntropyTTLsRange3->Min() << "\t" <<
		EntropyTTLs->Max() << "\t" << EntropyTTLsRange1->Max() << "\t" << EntropyTTLsRange2->Max() << "\t" << EntropyTTLsRange3->Max() << "\t" <<
		EntropyTTLs->MinMean() << "\t" << EntropyTTLsRange1->MinMean() << "\t" << EntropyTTLsRange2->MinMean() << "\t" << EntropyTTLsRange3->MinMean() << "\t" <<
		EntropyTTLs->MaxMean() << "\t" << EntropyTTLsRange1->MaxMean() << "\t" << EntropyTTLsRange2->MaxMean() << "\t" << EntropyTTLsRange3->MaxMean() << "\t" <<
		EntropyTTLs->MinVariance() << "\t" << EntropyTTLsRange1->MinVariance() << "\t" << EntropyTTLsRange2->MinVariance() << "\t" << EntropyTTLsRange3->MinVariance() << "\t" <<
		EntropyTTLs->MaxVariance() << "\t" << EntropyTTLsRange1->MaxVariance() << "\t" << EntropyTTLsRange2->MaxVariance() << "\t" << EntropyTTLsRange3->MaxVariance() << "\t" <<

		HostPort << "\t" <<
		EntropyHostPorts->EntropyVal() << "\t" << EntropyHostPortsRange1->EntropyVal() << "\t" << EntropyHostPortsRange2->EntropyVal() << "\t" << EntropyHostPortsRange3->EntropyVal() << "\t" <<
		EntropyHostPorts->EntropyMean() << "\t" << EntropyHostPortsRange1->EntropyMean() << "\t" << EntropyHostPortsRange2->EntropyMean() << "\t" << EntropyHostPortsRange3->EntropyMean() << "\t" <<
		EntropyHostPorts->EntropyVariance() << "\t" << EntropyHostPortsRange1->EntropyVariance() << "\t" << EntropyHostPortsRange2->EntropyVariance() << "\t" << EntropyHostPortsRange3->EntropyVariance() << "\t" <<
		EntropyHostPorts->EntropyMin() << "\t" << EntropyHostPortsRange1->EntropyMin() << "\t" << EntropyHostPortsRange2->EntropyMin() << "\t" << EntropyHostPortsRange3->EntropyMin() << "\t" <<
		EntropyHostPorts->EntropyMax() << "\t" << EntropyHostPortsRange1->EntropyMax() << "\t" << EntropyHostPortsRange2->EntropyMax() << "\t" << EntropyHostPortsRange3->EntropyMax() << "\t" <<

		std::endl;

    this->WriteExistentLog(SrcIP, ss.str());
}

void HostAnalyzerExtended::WriteNewLog(const std::string &HostIP, const std::string &Msg) const
{
    std::string fileName = HostIP + ".log";

    this->LogsFolder.WriteNewFile(fileName, Msg);
}

void HostAnalyzerExtended::WriteExistentLog(const std::string &HostIP, const std::string &Msg) const
{
    std::string fileName = HostIP + ".log";

    this->LogsFolder.WriteExistentFile(fileName, Msg);
}

void HostAnalyzerExtended::AddPacket(const std::string &SrcIP, const std::string &DstIP, const int &DstPort, const std::string &Proto,
									 const int &Length, const int &TTL, const long &TSsec, const long &TSusec)
{
    const double dTS = TSsec + ( TSusec * 0.000001 );

    if (this->Lengths.count(SrcIP) == 0)
    {
        this->Lengths.insert(std::pair<std::string, EntropyContainer<int> *>(SrcIP, new EntropyContainer<int>(this->WinSize)));
        this->LengthsRange1.insert(std::pair<std::string, EntropyContainer<int> *>(SrcIP, new EntropyContainer<int>(this->WinSize)));
        this->LengthsRange2.insert(std::pair<std::string, EntropyContainer<int> *>(SrcIP, new EntropyContainer<int>(this->WinSize)));
        this->LengthsRange3.insert(std::pair<std::string, EntropyContainer<int> *>(SrcIP, new EntropyContainer<int>(this->WinSize)));
        this->IAtimes.insert(std::pair<std::string, EntropyContainer<double> *>(SrcIP, new EntropyContainer<double>(this->WinSize)));
        this->IAtimesRange1.insert(std::pair<std::string, EntropyContainer<double> *>(SrcIP, new EntropyContainer<double>(this->WinSize)));
        this->IAtimesRange2.insert(std::pair<std::string, EntropyContainer<double> *>(SrcIP, new EntropyContainer<double>(this->WinSize)));
        this->IAtimesRange3.insert(std::pair<std::string, EntropyContainer<double> *>(SrcIP, new EntropyContainer<double>(this->WinSize)));
        this->TTLs.insert(std::pair<std::string, EntropyContainer<int> *>(SrcIP, new EntropyContainer<int>(this->WinSize)));
        this->TTLsRange1.insert(std::pair<std::string, EntropyContainer<int> *>(SrcIP, new EntropyContainer<int>(this->WinSize)));
        this->TTLsRange2.insert(std::pair<std::string, EntropyContainer<int> *>(SrcIP, new EntropyContainer<int>(this->WinSize)));
        this->TTLsRange3.insert(std::pair<std::string, EntropyContainer<int> *>(SrcIP, new EntropyContainer<int>(this->WinSize)));
		this->HostPorts.insert(std::pair<std::string, EntropyContainer<std::string> *>(SrcIP, new EntropyContainer<std::string>(this->WinSize)));
		this->HostPortsRange1.insert(std::pair<std::string, EntropyContainer<std::string> *>(SrcIP, new EntropyContainer<std::string>(this->WinSize)));
		this->HostPortsRange2.insert(std::pair<std::string, EntropyContainer<std::string> *>(SrcIP, new EntropyContainer<std::string>(this->WinSize)));
		this->HostPortsRange3.insert(std::pair<std::string, EntropyContainer<std::string> *>(SrcIP, new EntropyContainer<std::string>(this->WinSize)));
        (this->TotalHosts)++;

        time_t rawtime;
        struct tm * timeinfo;

        time ( &rawtime );
        timeinfo = localtime ( &rawtime );

        std::stringstream ss (std::stringstream::out);
        ss << "#\n# Log for host " << SrcIP <<
              "\n#\n# Created on " << asctime (timeinfo) << "#\n" <<
        	  "# Window of " << this->WinSize << " packets\n#\n" <<
        	  "# Range 1: 0-" << RANGES_MAX_1 << " bytes\n" <<
        	  "# Range 2: " << RANGES_MAX_1 << "-"<< RANGES_MAX_2 << " bytes\n" <<
        	  "# Range 3: " << RANGES_MAX_2 << "-" << RANGES_MAX_3 << " bytes\n#\n" <<
        	  "# <Timestamp>	<Time elapsed>	" <<

        	  "<Length>	" <<
        	  "<Entropy [lengths]>	<Entropy [lengths Range1]>	<Entropy [lengths Range2]>	<Entropy [lengths Range3]>	" <<
        	  "<Mean of the entropy [lengths]>	<Mean of the entropy [lengths Range1]>	<Mean of the entropy [lengths Range2]>	<Mean of the entropy [lengths Range3]>	" <<
        	  "<Variance of the entropy [lengths]>	<Variance of the entropy [lengths Range1]>	<Variance of the entropy [lengths Range2]>	<Variance of the entropy [lengths Range3]>	" <<
        	  "<MIN of the entropy [lengths]>	<MIN of the entropy [lengths Range1]>	<MIN of the entropy [lengths Range2]>	<MIN of the entropy [lengths Range3]>	" <<
        	  "<MAX of the entropy [lengths]>	<MAX of the entropy [lengths Range1]>	<MAX of the entropy [lengths Range2]>	<MAX of the entropy [lengths Range3]>	" <<
        	  "<Mean [lengths]>	<Mean [lengths Range1]>	<Mean [lengths Range2]>	<Mean [lengths Range3]>	" <<
        	  "<Variance [lengths]>	<Variance [lengths Range1]>	<Variance [lengths Range2]>	<Variance [lengths Range3]>	" <<
        	  "<MIN in the window [lengths]>	<MIN in the window [lengths Range1]>	<MIN in the window [lengths Range2]>	<MIN in the window [lengths Range3]>	" <<
        	  "<MAX in the window [lengths]>	<MAX in the window [lengths Range1]>	<MAX in the window [lengths Range2]>	<MAX in the window [lengths Range3]>	" <<
        	  "<Mean of the MIN [lengths]>	<Mean of the MIN [lengths Range1]>	<Mean of the MIN [lengths Range2]>	<Mean of the MIN [lengths Range3]>	" <<
        	  "<Mean of the MAX [lengths]>	<Mean of the MAX [lengths Range1]>	<Mean of the MAX [lengths Range2]>	<Mean of the MAX [lengths Range3]>	" <<
        	  "<Variance of the MIN [lengths]>	<Variance of the MIN [lengths Range1]>	<Variance of the MIN [lengths Range2]>	<Variance of the MIN [lengths Range3]>	" <<
        	  "<Variance of the MAX [lengths]>	<Variance of the MAX [lengths Range1]>	<Variance of the MAX [lengths Range2]>	<Variance of the MAX [lengths Range3]>	" <<

        	  "<IATime>	" <<
        	  "<Entropy [IAtimes]>	<Entropy [IAtimes Range1]>	<Entropy [IAtimes Range2]>	<Entropy [IAtimes Range3]>	" <<
        	  "<Mean of the entropy [IAtimes]>	<Mean of the entropy [IAtimes Range1]>	<Mean of the entropy [IAtimes Range2]>	<Mean of the entropy [IAtimes Range3]>	" <<
        	  "<Variance of the entropy [IAtimes]>	<Variance of the entropy [IAtimes Range1]>	<Variance of the entropy [IAtimes Range2]>	<Variance of the entropy [IAtimes Range3]>	" <<
        	  "<MIN of the entropy [IAtimes]>	<MIN of the entropy [IAtimes Range1]>	<MIN of the entropy [IAtimes Range2]>	<MIN of the entropy [IAtimes Range3]>	" <<
        	  "<MAX of the entropy [IAtimes]>	<MAX of the entropy [IAtimes Range1]>	<MAX of the entropy [IAtimes Range2]>	<MAX of the entropy [IAtimes Range3]>	" <<
        	  "<Mean [IAtimes]>	<Mean [IAtimes Range1]>	<Mean [IAtimes Range2]>	<Mean [IAtimes Range3]>	" <<
        	  "<Variance [IAtimes]>	<Variance [IAtimes Range1]>	<Variance [IAtimes Range2]>	<Variance [IAtimes Range3]>	" <<
        	  "<MIN in the window [IAtimes]>	<MIN in the window [IAtimes Range1]>	<MIN in the window [IAtimes Range2]>	<MIN in the window [IAtimes Range3]>	" <<
        	  "<MAX in the window [IAtimes]>	<MAX in the window [IAtimes Range1]>	<MAX in the window [IAtimes Range2]>	<MAX in the window [IAtimes Range3]>	" <<
        	  "<Mean of the MIN [IAtimes]>	<Mean of the MIN [IAtimes Range1]>	<Mean of the MIN [IAtimes Range2]>	<Mean of the MIN [IAtimes Range3]>	" <<
        	  "<Mean of the MAX [IAtimes]>	<Mean of the MAX [IAtimes Range1]>	<Mean of the MAX [IAtimes Range2]>	<Mean of the MAX [IAtimes Range3]>	" <<
        	  "<Variance of the MIN [IAtimes]>	<Variance of the MIN [IAtimes Range1]>	<Variance of the MIN [IAtimes Range2]>	<Variance of the MIN [IAtimes Range3]>	" <<
        	  "<Variance of the MAX [IAtimes]>	<Variance of the MAX [IAtimes Range1]>	<Variance of the MAX [IAtimes Range2]>	<Variance of the MAX [IAtimes Range3]>	" <<

        	  "<TTL>	" <<
        	  "<Entropy [TTLs]>	<Entropy [TTLs Range1]>	<Entropy [TTLs Range2]>	<Entropy [TTLs Range3]>	" <<
        	  "<Mean of the entropy [TTLs]>	<Mean of the entropy [TTLs Range1]>	<Mean of the entropy [TTLs Range2]>	<Mean of the entropy [TTLs Range3]>	" <<
        	  "<Variance of the entropy [TTLs]>	<Variance of the entropy [TTLs Range1]>	<Variance of the entropy [TTLs Range2]>	<Variance of the entropy [TTLs Range3]>	" <<
        	  "<MIN of the entropy [TTLs]>	<MIN of the entropy [TTLs Range1]>	<MIN of the entropy [TTLs Range2]>	<MIN of the entropy [TTLs Range3]>	" <<
        	  "<MAX of the entropy [TTLs]>	<MAX of the entropy [TTLs Range1]>	<MAX of the entropy [TTLs Range2]>	<MAX of the entropy [TTLs Range3]>	" <<
        	  "<Mean [TTLs]>	<Mean [TTLs Range1]>	<Mean [TTLs Range2]>	<Mean [TTLs Range3]>	" <<
        	  "<Variance [TTLs]>	<Variance [TTLs Range1]>	<Variance [TTLs Range2]>	<Variance [TTLs Range3]>	" <<
        	  "<MIN in the window [TTLs]>	<MIN in the window [TTLs Range1]>	<MIN in the window [TTLs Range2]>	<MIN in the window [TTLs Range3]>	" <<
        	  "<MAX in the window [TTLs]>	<MAX in the window [TTLs Range1]>	<MAX in the window [TTLs Range2]>	<MAX in the window [TTLs Range3]>	" <<
        	  "<Mean of the MIN [TTLs]>	<Mean of the MIN [TTLs Range1]>	<Mean of the MIN [TTLs Range2]>	<Mean of the MIN [TTLs Range3]>	" <<
        	  "<Mean of the MAX [TTLs]>	<Mean of the MAX [TTLs Range1]>	<Mean of the MAX [TTLs Range2]>	<Mean of the MAX [TTLs Range3]>	" <<
        	  "<Variance of the MIN [TTLs]>	<Variance of the MIN [TTLs Range1]>	<Variance of the MIN [TTLs Range2]>	<Variance of the MIN [TTLs Range3]>	" <<
        	  "<Variance of the MAX [TTLs]>	<Variance of the MAX [TTLs Range1]>	<Variance of the MAX [TTLs Range2]>	<Variance of the MAX [TTLs Range3]>	" <<

        	  "<HostPort>	" <<
        	  "<Entropy [HostPorts]>	<Entropy [HostPorts Range1]>	<Entropy [HostPorts Range2]>	<Entropy [HostPorts Range3]>	" <<
        	  "<Mean of the entropy [HostPorts]>	<Mean of the entropy [HostPorts Range1]>	<Mean of the entropy [HostPorts Range2]>	<Mean of the entropy [HostPorts Range3]>	" <<
        	  "<Variance of the entropy [HostPorts]>	<Variance of the entropy [HostPorts Range1]>	<Variance of the entropy [HostPorts Range2]>	<Variance of the entropy [HostPorts Range3]>	" <<
        	  "<MIN of the entropy [HostPorts]>	<MIN of the entropy [HostPorts Range1]>	<MIN of the entropy [HostPorts Range2]>	<MIN of the entropy [HostPorts Range3]>	" <<
        	  "<MAX of the entropy [HostPorts]>	<MAX of the entropy [HostPorts Range1]>	<MAX of the entropy [HostPorts Range2]>	<MAX of the entropy [HostPorts Range3]>	" <<

        	  "\n#\n" <<

          	  "# Timestamp		Elapsed		Length	Entropy [L]	Entropy [LR1]	Entropy [LR2]	Entropy [LR3]	Mean Ent [L]	Mean Ent [LR1]	Mean Ent [LR2]	Mean Ent [LR3]	Var Ent [L]	Var Ent [LR1]	Var Ent [LR2]	Var Ent [LR3]	MIN Ent [L]	MIN Ent [LR1]	MIN Ent [LR2]	MIN Ent [LR3]	MAX Ent [L]	MAX Ent [LR1]	MAX Ent [LR2]	MAX Ent [LR3]	Mean [L]	Mean [LR1]	Mean [LR2]	Mean [LR3]	Var [L]		Var [LR1]	Var [LR2]	Var [LR3]	MN W[L]	MN W[L1]MN W[L2]MN W[L3]MX W[L]	MX W[L1]MX W[L2]MX W[L3]Mean of MN [L]	Mean of MN [L1]	Mean of MN [L2]	Mean of MN [L3]	Mean of MX [L]	Mean of MX [L1]	Mean of MX [L2]	Mean of MX [L3]	Var of MN [LR]	Var of MN [LR1]	Var of MN [LR2]	Var of MN [LR3]	Var of MX [LR]	Var of MX [LR1]	Var of MX [LR2]	Var of MX [LR3]	IATime		Entropy [I]	Entropy [IR1]	Entropy [IR2]	Entropy [IR3]	Mean Ent [I]	Mean Ent [IR1]	Mean Ent [IR2]	Mean Ent [IR3]	Var Ent [I]	Var Ent [IR1]	Var Ent [IR2]	Var Ent [IR3]	MIN Ent [I]	MIN Ent [IR1]	MIN Ent [IR2]	MIN Ent [IR3]	MAX Ent [I]	MAX Ent [IR1]	MAX Ent [IR2]	MAX Ent [IR3]	Mean [I]	Mean [IR1]	Mean [IR2]	Mean [IR3]	Var [I]		Var [IR1]	Var [IR2]	Var [IR3]	MN W[I]		MN W[I1]	MN W[I2]	MN W[I3]	MX W[I]		MX W[I1]	MX W[I2]	MX W[I3]	Mean of MN [I]	Mean of MN [I1]	Mean of MN [I2]	Mean of MN [I3]	Mean of MX [I]	Mean of MX [I1]	Mean of MX [I2]	Mean of MX [I3]	Var of MN [IR]	Var of MN [IR1]	Var of MN [IR2]	Var of MN [IR3]	Var of MX [IR]	Var of MX [IR1]	Var of MX [IR2]	Var of MX [IR3]	TTL	Entropy [T]	Entropy [TR1]	Entropy [TR2]	Entropy [TR3]	Mean Ent [T]	Mean Ent [TR1]	Mean Ent [TR2]	Mean Ent [TR3]	Var Ent [T]	Var Ent [TR1]	Var Ent [TR2]	Var Ent [TR3]	MIN Ent [T]	MIN Ent [TR1]	MIN Ent [TR2]	MIN Ent [TR3]	MAX Ent [T]	MAX Ent [TR1]	MAX Ent [TR2]	MAX Ent [TR3]	Mean [T]	Mean [TR1]	Mean [TR2]	Mean [TR3]	Var [T]		Var [TR1]	Var [TR2]	Var [TR3]	MN W[T]	MN W[T1]MN W[T2]MN W[T3]MX W[T]	MX W[T1]MX W[T2]MX W[T3]Mean of MN [T]	Mean of MN [T1]	Mean of MN [T2]	Mean of MN [T3]	Mean of MX [T]	Mean of MX [T1]	Mean of MX [T2]	Mean of MX [T3]	Var of MN [TR]	Var of MN [TR1]	Var of MN [TR2]	Var of MN [TR3]	Var of MX [TR]	Var of MX [TR1]	Var of MX [TR2]	Var of MX [TR3]	HostPort		Entropy [HP]	Entropy [HPR1]	Entropy [HPR2]	Entropy [HPR3]	Mean Ent [HP]	Mean Ent [HPR1]	Mean Ent [HPR2]	Mean Ent [HPR3]	Var Ent [HP]	Var Ent [HPR1]	Var Ent [HPR2]	Var Ent [HPR3]	MIN Ent [HP]	MIN Ent [HPR1]	MIN Ent [HPR2]	MIN Ent [HPR3]	MAX Ent [HP]	MAX Ent [HPR1]	MAX Ent [HPR2]	MAX Ent [HPR3]	\n#\n";


        this->WriteNewLog(SrcIP, ss.str());

    	this->LastTS.insert(std::pair<std::string, double>(SrcIP,dTS));
    }

    EntropyContainer<int> * EntropyLengths = this->Lengths.at(SrcIP);
    EntropyContainer<int> * EntropyLengthsRange1 = this->LengthsRange1.at(SrcIP);
    EntropyContainer<int> * EntropyLengthsRange2 = this->LengthsRange2.at(SrcIP);
    EntropyContainer<int> * EntropyLengthsRange3 = this->LengthsRange3.at(SrcIP);
    EntropyContainer<double> * EntropyIAtimes = this->IAtimes.at(SrcIP);
    EntropyContainer<double> * EntropyIAtimesRange1 = this->IAtimesRange1.at(SrcIP);
    EntropyContainer<double> * EntropyIAtimesRange2 = this->IAtimesRange2.at(SrcIP);
    EntropyContainer<double> * EntropyIAtimesRange3 = this->IAtimesRange3.at(SrcIP);
    EntropyContainer<int> * EntropyTTLs = this->TTLs.at(SrcIP);
    EntropyContainer<int> * EntropyTTLsRange1 = this->TTLsRange1.at(SrcIP);
    EntropyContainer<int> * EntropyTTLsRange2 = this->TTLsRange2.at(SrcIP);
    EntropyContainer<int> * EntropyTTLsRange3 = this->TTLsRange3.at(SrcIP);
    EntropyContainer<std::string> * EntropyHostPorts = this->HostPorts.at(SrcIP);
    EntropyContainer<std::string> * EntropyHostPortsRange1 = this->HostPortsRange1.at(SrcIP);
    EntropyContainer<std::string> * EntropyHostPortsRange2 = this->HostPortsRange2.at(SrcIP);
    EntropyContainer<std::string> * EntropyHostPortsRange3 = this->HostPortsRange3.at(SrcIP);

	std::stringstream ssHP(std::stringstream::out);
	ssHP << DstIP << ":" << DstPort << ":" << Proto;
    const std::string HostPort = ssHP.str();

    double IAtime = this->Round(dTS - this->LastTS.at(SrcIP), this->Precision);
    this->LastTS.at(SrcIP) = dTS;

    EntropyLengths->AddValue(Length);
	EntropyTTLs->AddValue(TTL);
	EntropyIAtimes->AddValue(IAtime);
	EntropyHostPorts->AddValue(HostPort);

    if (Length <= RANGES_MAX_1)
    {
		EntropyLengthsRange1->AddValue(Length);
		EntropyTTLsRange1->AddValue(TTL);
		EntropyIAtimesRange1->AddValue(IAtime);
		EntropyHostPortsRange1->AddValue(HostPort);
    }
    else if (Length <= RANGES_MAX_2)
    {
		EntropyLengthsRange2->AddValue(Length);
		EntropyTTLsRange2->AddValue(TTL);
		EntropyIAtimesRange2->AddValue(IAtime);
		EntropyHostPortsRange2->AddValue(HostPort);
    }
    else if (Length <= RANGES_MAX_3)
    {
		EntropyLengthsRange3->AddValue(Length);
		EntropyTTLsRange3->AddValue(TTL);
		EntropyIAtimesRange3->AddValue(IAtime);
		EntropyHostPortsRange3->AddValue(HostPort);
    }

    (this->TotalPackets)++;

	this->WriteEntropy(SrcIP,
					   TSsec, TSusec,
					   Length, IAtime, TTL, HostPort,
					   EntropyLengths, EntropyLengthsRange1, EntropyLengthsRange2, EntropyLengthsRange3,
					   EntropyIAtimes, EntropyIAtimesRange1, EntropyIAtimesRange2, EntropyIAtimesRange3,
					   EntropyTTLs, EntropyTTLsRange1, EntropyTTLsRange2, EntropyTTLsRange3,
					   EntropyHostPorts, EntropyHostPortsRange1, EntropyHostPortsRange2, EntropyHostPortsRange3);
}

const double HostAnalyzerExtended::Round(const double &number, const int &precision) const
{
	double Aux, Result;
	int precFactor, intRes;

	if (precision == 0)
		return number;
	else if (precision > 0)
	{
		precFactor = 10;
		for (int i=1; i<precision; i++)
			precFactor *= 10;
	}
	else
		return number;

	Aux = number * (double) precFactor;
	intRes = (int) Aux;

	Result = intRes / (double) precFactor;

    return Result;
}

unsigned long HostAnalyzerExtended::TotHosts() const
{
    return this->TotalHosts;
}

unsigned long HostAnalyzerExtended::TotPackets() const
{
    return this->TotalPackets;
}
