/*
 * \author João V. Gomes
 *
 *  hostanalyzerextended.h
 *
 *  Created on: Apr 12, 2011
 *
 * \brief TODO
 * TODO
 */

#ifndef HOSTANALYZEREXTENDED_H_
#define HOSTANALYZEREXTENDED_H_

#ifdef __cplusplus
extern "C"
{
#endif
#include <time.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include <string.h>
#ifdef __cplusplus
}
#endif

#include <entropycontainer.h>
#include <Folder.h>
#include <slidingwindow.h>

#include <map>
#include <string>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>

#ifndef RANGES_MAX
#define RANGES_MAX
#define RANGES_MAX_1 100
#define RANGES_MAX_2 900
#define RANGES_MAX_3 1600
#endif // RANGES_MAX

#ifndef DEFAULT_PRECISION
#define DEFAULT_PRECISION 6
#endif // DEFAULT_PRECISION

class HostAnalyzerExtended
{
    public:
		HostAnalyzerExtended();
		HostAnalyzerExtended(const int &WindowSize);
		HostAnalyzerExtended(const int &WindowSize, const std::string &LogsPath);
		HostAnalyzerExtended(const int &WindowSize, const int &Precision);
		HostAnalyzerExtended(const int &WindowSize, const int &Precision, const std::string &LogsPath);
        virtual ~HostAnalyzerExtended();
        void AddPacket(const std::string &SrcIP, const std::string &DstIP, const int &DstPort, const std::string &Proto,
					   const int &Length, const int &TTL, const long &TSsec, const long &TSusec);

        unsigned long TotHosts() const;
        unsigned long TotPackets() const;
        void Clear();
    protected:
        virtual void WriteEntropy(const std::string &SrcIP,
								  const long &TSsec, const long &TSusec,
								  const int &Length, const double &IAtime, const int &TTL, const std::string &HostPort,
								  EntropyContainer<int> * EntropyLengths, EntropyContainer<int> * EntropyLengthsRange1, EntropyContainer<int> * EntropyLengthsRange2, EntropyContainer<int> * EntropyLengthsRange3,
								  EntropyContainer<double> * EntropyIAtimes, EntropyContainer<double> * EntropyIAtimesRange1, EntropyContainer<double> * EntropyIAtimesRange2, EntropyContainer<double> * EntropyIAtimesRange3,
								  EntropyContainer<int> * EntropyTTLs, EntropyContainer<int> * EntropyTTLsRange1, EntropyContainer<int> * EntropyTTLsRange2, EntropyContainer<int> * EntropyTTLsRange3,
								  EntropyContainer<std::string> * EntropyHostPorts, EntropyContainer<std::string> * EntropyHostPortsRange1, EntropyContainer<std::string> * EntropyHostPortsRange2, EntropyContainer<std::string> * EntropyHostPortsRange3);
        void init(const int &WindowSize, const int &Precision, const std::string &Path);
        void WriteExistentLog(const std::string &HostIP, const std::string &Msg) const;
    private:
        // Host, Entropy container
        std::map<std::string, EntropyContainer<int> *> Lengths;
        std::map<std::string, EntropyContainer<int> *> LengthsRange1;
        std::map<std::string, EntropyContainer<int> *> LengthsRange2;
        std::map<std::string, EntropyContainer<int> *> LengthsRange3;
        std::map<std::string, EntropyContainer<double> *> IAtimes;
        std::map<std::string, EntropyContainer<double> *> IAtimesRange1;
        std::map<std::string, EntropyContainer<double> *> IAtimesRange2;
        std::map<std::string, EntropyContainer<double> *> IAtimesRange3;
        std::map<std::string, EntropyContainer<int> *> TTLs;
        std::map<std::string, EntropyContainer<int> *> TTLsRange1;
        std::map<std::string, EntropyContainer<int> *> TTLsRange2;
        std::map<std::string, EntropyContainer<int> *> TTLsRange3;
    	std::map<std::string, EntropyContainer<std::string> *> HostPorts;
    	std::map<std::string, EntropyContainer<std::string> *> HostPortsRange1;
    	std::map<std::string, EntropyContainer<std::string> *> HostPortsRange2;
    	std::map<std::string, EntropyContainer<std::string> *> HostPortsRange3;

        std::map<std::string, double> FirstTS;
        std::map<std::string, double> LastTS;
        unsigned long TotalHosts;
        unsigned long TotalPackets;
        int WinSize;
        int Precision;
        Folder LogsFolder;

        void dispose();
        void WriteNewLog(const std::string &HostIP, const std::string &Msg) const;
        const double Round(const double &number, const int &precision) const;
};

#endif /* HOSTANALYZEREXTENDED_H_ */
