#ifndef HOSTPORTANALYZER_H
#define HOSTPORTANALYZER_H

#ifdef __cplusplus
extern "C" {
#endif
#include <time.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include <string.h>
#ifdef __cplusplus
}
#endif

#include <entropycontainer.h>
#include <direxception.h>
#include <Folder.h>
#include <thresholdsset.h>
#include <slidingwindow.h>

#include <map>
#include <string>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>

template<class V>
class HostPortAnalyzer {
public:
	HostPortAnalyzer(const int &WindowSize);
	HostPortAnalyzer(const int &WindowSize, const std::string &LogsPath);
	HostPortAnalyzer(const int &WindowSize, const std::string &LogsPath,
			const bool &Print);
	HostPortAnalyzer(const int &WindowSize, const std::string &LogsPath,
			ThresholdsSet * TSet);
	HostPortAnalyzer(const int &WindowSize, const std::string &LogsPath,
			const bool &Print, ThresholdsSet * TSet);
	virtual ~HostPortAnalyzer();
	double * AddValue(const std::string &HostIP, const int &Port,
			const V &Value);
	double * AddValue(const long &TSsec, const long &TSusec,
			const std::string &HostIP, const int &Port, const V &Value);
	template<class D> double * AddValueDif(const std::string &HostIP,
			const int &Port, const D &Value, int precision);
	template<class D> double * AddValueDif(const long &TSsec,
			const long &TSusec, const std::string &HostIP, const int &Port,
			const D &Value, int precision);
	unsigned long TotHosts() const;
	unsigned long TotValues() const;
	void Clear();
	void ClearHostPort(const std::string &Host, const int &Port);
protected:
private:
	std::map<std::string, EntropyContainer<V> *> Hosts;
	unsigned long TotalHosts;
	unsigned long TotalValues;
	int WinSize;
	Folder LogsFolder;
	ThresholdsSet * Thresholds;
	std::map<std::string, V> LastValue;
	std::map<std::string, double> FirstTS;
	bool PrintOutput;

	void init(const int &WindowSize, const std::string &Path,
			const bool &Print, ThresholdsSet * TSet);
	void dispose();
	void WriteEntropy(const std::string &HostIP, const int &Port,
			const long &TSsec, const long &TSusec, const V &Value,
			EntropyContainer<V>* Entropy);
	void WriteNewLog(const std::string &HostIP, const int &Port,
			const std::string &Msg) const;
	void WriteExistentLog(const std::string &HostIP, const int &Port,
			const std::string &Msg) const;
	template<class D> const D Round(const D &number, const int &precision) const;
};

#endif // HOSTPORTANALYZER_H
