/*
 * \author João V. Gomes
 *
 *  hostportanalyzerextended.cpp
 *
 *  Created on: Apr 12, 2011
 *
 * \brief TODO
 * TODO
 */

#include "hostportanalyzerextended.h"

HostPortAnalyzerExtended::HostPortAnalyzerExtended(const int &WindowSize) {
	this->init(WindowSize, DEFAULT_PRECISION, std::string(), std::string(), true, true);
}

HostPortAnalyzerExtended::HostPortAnalyzerExtended(const int &WindowSize, const std::string &LogsBase, const std::string &LogsComplement) {
	this->init(WindowSize, DEFAULT_PRECISION, LogsBase, LogsComplement, true, true);
}

HostPortAnalyzerExtended::HostPortAnalyzerExtended(const int &WindowSize, const std::string &LogsBase, const std::string &LogsComplement, const bool &PrintSummary, const bool &PrintLogs) {
	this->init(WindowSize, DEFAULT_PRECISION, LogsBase, LogsComplement, PrintSummary, PrintLogs);
}

HostPortAnalyzerExtended::HostPortAnalyzerExtended(const int &WindowSize, const int &precision) {
	this->init(WindowSize, precision, std::string(), std::string(), true, true);
}

HostPortAnalyzerExtended::HostPortAnalyzerExtended(const int &WindowSize, const int &precision, const std::string &LogsBase, const std::string &LogsComplement) {
	this->init(WindowSize, precision, LogsBase, LogsComplement, true, true);
}

HostPortAnalyzerExtended::HostPortAnalyzerExtended(const int &WindowSize, const int &precision, const std::string &LogsBase, const std::string &LogsComplement, const bool &PrintSummary, const bool &PrintLogs) {
	this->init(WindowSize, precision, LogsBase, LogsComplement, PrintSummary, PrintLogs);
}

HostPortAnalyzerExtended::~HostPortAnalyzerExtended() {
	this->WriteSummary();
	this->dispose();
}

void HostPortAnalyzerExtended::Clear() {
	this->dispose();

    this->Lengths.clear();
    this->LengthsRange1.clear();
    this->LengthsRange2.clear();
    this->LengthsRange3.clear();

    this->TTLs.clear();
    this->TTLsRange1.clear();
    this->TTLsRange2.clear();
    this->TTLsRange3.clear();

    this->IAtimes.clear();
    this->IAtimesRange1.clear();
    this->IAtimesRange2.clear();
    this->IAtimesRange3.clear();

    this->HostPorts.clear();
    this->HostPortsRange1.clear();
    this->HostPortsRange2.clear();
    this->HostPortsRange3.clear();

	this->TotalHostPorts = 0;
	this->TotalPackets = 0;
	this->precision = DEFAULT_PRECISION;

	this->WinSize = 0;

    this->TotalBytes.clear();
    this->TotalBytesR1.clear();
    this->TotalBytesR2.clear();
    this->TotalBytesR3.clear();
}

void HostPortAnalyzerExtended::dispose() {
	std::map<std::string, EntropyContainer<int> *>::iterator it;

    for(it = this->Lengths.begin(); it != this->Lengths.end(); it++)
    {
        delete this->IAtimes.at(it->first);
        delete this->IAtimesRange1.at(it->first);
        delete this->IAtimesRange2.at(it->first);
        delete this->IAtimesRange3.at(it->first);

        delete this->TTLs.at(it->first);
        delete this->TTLsRange1.at(it->first);
        delete this->TTLsRange2.at(it->first);
        delete this->TTLsRange3.at(it->first);

        delete this->HostPorts.at(it->first);
        delete this->HostPortsRange1.at(it->first);
        delete this->HostPortsRange2.at(it->first);
        delete this->HostPortsRange3.at(it->first);

		delete this->LengthsRange1.at(it->first);
		delete this->LengthsRange2.at(it->first);
		delete this->LengthsRange3.at(it->first);
        delete (it->second);
    }
}

void HostPortAnalyzerExtended::WriteSummary() {
	if (this->PrintSummary)
	{
		std::stringstream ssHeader(std::stringstream::out);

		time_t rawtime;
		struct tm * timeinfo;

		time(&rawtime);
		timeinfo = localtime(&rawtime);

		ssHeader << "#\n# Created on " << asctime(timeinfo) << "#\n"
			  "# Window of " << this->WinSize << " packets\n#\n" <<
			  "# Total of " << this->TotalHostPorts << " (Host, Port) pairs\n#\n" <<
			  "# Range 1: 0-" << RANGES_MAX_1 << " bytes\n" <<
			  "# Range 2: " << RANGES_MAX_1 << "-"<< RANGES_MAX_2 << " bytes\n" <<
			  "# Range 3: " << RANGES_MAX_2 << "-" << RANGES_MAX_3 << " bytes\n#\n" <<
			  "# <Host, Port>	<Time elapsed>	" <<

			  "<Number of packets>	<Number of packets [Range1]>	<Number of packets [Range2]>	<Number of packets [Range3]>	" <<
			  "<Ratio of packets [Range1]>	<Ratio of packets [Range2]>	<Ratio of packets [Range3]>	" <<
			  "<Ratio over Max of packets [Range1]>	<Ratio over Max of packets [Range2]>	<Ratio over Max of packets [Range3]>	" <<

			  "<Total bytes>	<Total bytes [Range1]>	<Total bytes [Range2]>	<Total bytes [Range3]>	" <<
			  "<Ratio of bytes [Range1]>	<Ratio of bytes [Range2]>	<Ratio of bytes [Range3]>	" <<
			  "<Ratio over Max of bytes [Range1]>	<Ratio over Max of bytes [Range2]>	<Ratio over Max of bytes [Range3]>	" <<

			  "<Mean of the entropy [lengths]>	<Mean of the entropy [lengths Range1]>	<Mean of the entropy [lengths Range2]>	<Mean of the entropy [lengths Range3]>	" <<

			  "<Mean of the entropy * Ratio of packets [lengths Range1]>	<Mean of the entropy * Ratio of packets [lengths Range2]>	<Mean of the entropy * Ratio of packets [lengths Range3]>	<Mean of the entropy * Ratio of packets [lengths MAX]>	" <<
			  "<Mean of the entropy * Ratio over Max of packets [lengths Range1]>	<Mean of the entropy * Ratio over Max of packets [lengths Range2]>	<Mean of the entropy * Ratio over Max of packets [lengths Range3]>	<Mean of the entropy * Ratio over Max of packets [lengths MAX]>	" <<
			  "<Mean of the entropy * Ratio of bytes [lengths Range1]>	<Mean of the entropy * Ratio of bytes [lengths Range2]>	<Mean of the entropy * Ratio of bytes [lengths Range3]>	<Mean of the entropy * Ratio of bytes [lengths MAX]>	" <<
			  "<Mean of the entropy * Ratio over Max of bytes [lengths Range1]>	<Mean of the entropy * Ratio over Max of bytes [lengths Range2]>	<Mean of the entropy * Ratio over Max of bytes [lengths Range3]>	<Mean of the entropy * Ratio over Max of bytes [lengths MAX]>	" <<

			  "<Variance of the entropy [lengths]>	<Variance of the entropy [lengths Range1]>	<Variance of the entropy [lengths Range2]>	<Variance of the entropy [lengths Range3]>	" <<
			  "<MIN of the entropy [lengths]>	<MIN of the entropy [lengths Range1]>	<MIN of the entropy [lengths Range2]>	<MIN of the entropy [lengths Range3]>	" <<
			  "<MAX of the entropy [lengths]>	<MAX of the entropy [lengths Range1]>	<MAX of the entropy [lengths Range2]>	<MAX of the entropy [lengths Range3]>	" <<
			  "<Mean [lengths]>	<Mean [lengths Range1]>	<Mean [lengths Range2]>	<Mean [lengths Range3]>	" <<
			  "<Variance [lengths]>	<Variance [lengths Range1]>	<Variance [lengths Range2]>	<Variance [lengths Range3]>	" <<
			  "<Mean of the MIN [lengths]>	<Mean of the MIN [lengths Range1]>	<Mean of the MIN [lengths Range2]>	<Mean of the MIN [lengths Range3]>	" <<
			  "<Mean of the MAX [lengths]>	<Mean of the MAX [lengths Range1]>	<Mean of the MAX [lengths Range2]>	<Mean of the MAX [lengths Range3]>	" <<
			  "<Variance of the MIN [lengths]>	<Variance of the MIN [lengths Range1]>	<Variance of the MIN [lengths Range2]>	<Variance of the MIN [lengths Range3]>	" <<
			  "<Variance of the MAX [lengths]>	<Variance of the MAX [lengths Range1]>	<Variance of the MAX [lengths Range2]>	<Variance of the MAX [lengths Range3]>	" <<

			  "<Mean of the entropy [IAtimes]>	<Mean of the entropy [IAtimes Range1]>	<Mean of the entropy [IAtimes Range2]>	<Mean of the entropy [IAtimes Range3]>	" <<
			  "<Variance of the entropy [IAtimes]>	<Variance of the entropy [IAtimes Range1]>	<Variance of the entropy [IAtimes Range2]>	<Variance of the entropy [IAtimes Range3]>	" <<
			  "<MIN of the entropy [IAtimes]>	<MIN of the entropy [IAtimes Range1]>	<MIN of the entropy [IAtimes Range2]>	<MIN of the entropy [IAtimes Range3]>	" <<
			  "<MAX of the entropy [IAtimes]>	<MAX of the entropy [IAtimes Range1]>	<MAX of the entropy [IAtimes Range2]>	<MAX of the entropy [IAtimes Range3]>	" <<
			  "<Mean [IAtimes]>	<Mean [IAtimes Range1]>	<Mean [IAtimes Range2]>	<Mean [IAtimes Range3]>	" <<
			  "<Variance [IAtimes]>	<Variance [IAtimes Range1]>	<Variance [IAtimes Range2]>	<Variance [IAtimes Range3]>	" <<
			  "<Mean of the MIN [IAtimes]>	<Mean of the MIN [IAtimes Range1]>	<Mean of the MIN [IAtimes Range2]>	<Mean of the MIN [IAtimes Range3]>	" <<
			  "<Mean of the MAX [IAtimes]>	<Mean of the MAX [IAtimes Range1]>	<Mean of the MAX [IAtimes Range2]>	<Mean of the MAX [IAtimes Range3]>	" <<
			  "<Variance of the MIN [IAtimes]>	<Variance of the MIN [IAtimes Range1]>	<Variance of the MIN [IAtimes Range2]>	<Variance of the MIN [IAtimes Range3]>	" <<
			  "<Variance of the MAX [IAtimes]>	<Variance of the MAX [IAtimes Range1]>	<Variance of the MAX [IAtimes Range2]>	<Variance of the MAX [IAtimes Range3]>	" <<

			  "<Mean of the entropy [TTLs]>	<Mean of the entropy [TTLs Range1]>	<Mean of the entropy [TTLs Range2]>	<Mean of the entropy [TTLs Range3]>	" <<
			  "<Variance of the entropy [TTLs]>	<Variance of the entropy [TTLs Range1]>	<Variance of the entropy [TTLs Range2]>	<Variance of the entropy [TTLs Range3]>	" <<
			  "<MIN of the entropy [TTLs]>	<MIN of the entropy [TTLs Range1]>	<MIN of the entropy [TTLs Range2]>	<MIN of the entropy [TTLs Range3]>	" <<
			  "<MAX of the entropy [TTLs]>	<MAX of the entropy [TTLs Range1]>	<MAX of the entropy [TTLs Range2]>	<MAX of the entropy [TTLs Range3]>	" <<
			  "<Mean [TTLs]>	<Mean [TTLs Range1]>	<Mean [TTLs Range2]>	<Mean [TTLs Range3]>	" <<
			  "<Variance [TTLs]>	<Variance [TTLs Range1]>	<Variance [TTLs Range2]>	<Variance [TTLs Range3]>	" <<
			  "<Mean of the MIN [TTLs]>	<Mean of the MIN [TTLs Range1]>	<Mean of the MIN [TTLs Range2]>	<Mean of the MIN [TTLs Range3]>	" <<
			  "<Mean of the MAX [TTLs]>	<Mean of the MAX [TTLs Range1]>	<Mean of the MAX [TTLs Range2]>	<Mean of the MAX [TTLs Range3]>	" <<
			  "<Variance of the MIN [TTLs]>	<Variance of the MIN [TTLs Range1]>	<Variance of the MIN [TTLs Range2]>	<Variance of the MIN [TTLs Range3]>	" <<
			  "<Variance of the MAX [TTLs]>	<Variance of the MAX [TTLs Range1]>	<Variance of the MAX [TTLs Range2]>	<Variance of the MAX [TTLs Range3]>	" <<

			  "<Mean of the entropy [HostPorts]>	<Mean of the entropy [HostPorts Range1]>	<Mean of the entropy [HostPorts Range2]>	<Mean of the entropy [HostPorts Range3]>	" <<
			  "<Variance of the entropy [HostPorts]>	<Variance of the entropy [HostPorts Range1]>	<Variance of the entropy [HostPorts Range2]>	<Variance of the entropy [HostPorts Range3]>	" <<
			  "<MIN of the entropy [HostPorts]>	<MIN of the entropy [HostPorts Range1]>	<MIN of the entropy [HostPorts Range2]>	<MIN of the entropy [HostPorts Range3]>	" <<
			  "<MAX of the entropy [HostPorts]>	<MAX of the entropy [HostPorts Range1]>	<MAX of the entropy [HostPorts Range2]>	<MAX of the entropy [HostPorts Range3]>	" <<

			  "\n#\n" <<

			  "# Host, Port		Elapsed		Pkts	Pkt[R1]	Pkt[R2]	Pkt[R3]	R Pkt[R1]	R Pkt[R2]	R Pkt[R3]	R M Pkt[R1]	R M Pkt[R2]	R M Pkt[R3]	Bytes	B[R1]	B[R2]	B[R3]	R Bytes[R1]	R Bytes[R2]	R Bytes[R3]	R M Bytes[R1]	R M Bytes[R2]	R M Bytes[R3]	Mean Ent [L]	Mean Ent [LR1]	Mean Ent [LR2]	Mean Ent [LR3]	MEntRTP [LR1]	MEntRTP [LR2]	MEntRTP [LR3]	MEntRTP [LMAX]	MEntRMP [LR1]	MEntRMP [LR2]	MEntRMP [LR3]	MEntRMP [LMAX]	MEntRTB [LR1]	MEntRTB [LR2]	MEntRTB [LR3]	MEntRTB [LMAX]	MEntRMB [LR1]	MEntRMB [LR2]	MEntRMB [LR3]	MEntRMB [LMAX]	Var Ent [L]	Var Ent [LR1]	Var Ent [LR2]	Var Ent [LR3]	MIN Ent [L]	MIN Ent [LR1]	MIN Ent [LR2]	MIN Ent [LR3]	MAX Ent [L]	MAX Ent [LR1]	MAX Ent [LR2]	MAX Ent [LR3]	Mean [L]	Mean [LR1]	Mean [LR2]	Mean [LR3]	Var [L]		Var [LR1]	Var [LR2]	Var [LR3]	Mean of MN [L]	Mean of MN [L1]	Mean of MN [L2]	Mean of MN [L3]	Mean of MX [L]	Mean of MX [L1]	Mean of MX [L2]	Mean of MX [L3]	Var of MN [LR]	Var of MN [LR1]	Var of MN [LR2]	Var of MN [LR3]	Var of MX [LR]	Var of MX [LR1]	Var of MX [LR2]	Var of MX [LR3]	Mean Ent [I]	Mean Ent [IR1]	Mean Ent [IR2]	Mean Ent [IR3]	Var Ent [I]	Var Ent [IR1]	Var Ent [IR2]	Var Ent [IR3]	MIN Ent [I]	MIN Ent [IR1]	MIN Ent [IR2]	MIN Ent [IR3]	MAX Ent [I]	MAX Ent [IR1]	MAX Ent [IR2]	MAX Ent [IR3]	Mean [I]	Mean [IR1]	Mean [IR2]	Mean [IR3]	Var [I]		Var [IR1]	Var [IR2]	Var [IR3]	Mean of MN [I]	Mean of MN [I1]	Mean of MN [I2]	Mean of MN [I3]	Mean of MX [I]	Mean of MX [I1]	Mean of MX [I2]	Mean of MX [I3]	Var of MN [IR]	Var of MN [IR1]	Var of MN [IR2]	Var of MN [IR3]	Var of MX [IR]	Var of MX [IR1]	Var of MX [IR2]	Var of MX [IR3]	Mean Ent [T]	Mean Ent [TR1]	Mean Ent [TR2]	Mean Ent [TR3]	Var Ent [T]	Var Ent [TR1]	Var Ent [TR2]	Var Ent [TR3]	MIN Ent [T]	MIN Ent [TR1]	MIN Ent [TR2]	MIN Ent [TR3]	MAX Ent [T]	MAX Ent [TR1]	MAX Ent [TR2]	MAX Ent [TR3]	Mean [T]	Mean [TR1]	Mean [TR2]	Mean [TR3]	Var [T]		Var [TR1]	Var [TR2]	Var [TR3]	Mean of MN [T]	Mean of MN [T1]	Mean of MN [T2]	Mean of MN [T3]	Mean of MX [T]	Mean of MX [T1]	Mean of MX [T2]	Mean of MX [T3]	Var of MN [TR]	Var of MN [TR1]	Var of MN [TR2]	Var of MN [TR3]	Var of MX [TR]	Var of MX [TR1]	Var of MX [TR2]	Var of MX [TR3]	Mean Ent [HP]	Mean Ent [HPR1]	Mean Ent [HPR2]	Mean Ent [HPR3]	Var Ent [HP]	Var Ent [HPR1]	Var Ent [HPR2]	Var Ent [HPR3]	MIN Ent [HP]	MIN Ent [HPR1]	MIN Ent [HPR2]	MIN Ent [HPR3]	MAX Ent [HP]	MAX Ent [HPR1]	MAX Ent [HPR2]	MAX Ent [HPR3]	\n#\n";

		this->LogsSummary.WriteNewFile(this->SummaryFileName, ssHeader.str());


		std::map<std::string, EntropyContainer<int> * >::iterator it;

		for(it = this->Lengths.begin(); it != this->Lengths.end(); it++)
		{
			std::stringstream ss(std::stringstream::out);

		    long MaxT = this->Max((long long) this->LengthsRange1.at(it->first)->TotalAnalyzedItems(), (long long) this->LengthsRange2.at(it->first)->TotalAnalyzedItems(), (long long) this->LengthsRange3.at(it->first)->TotalAnalyzedItems());
		    long long MaxB = this->Max(this->TotalBytesR1.at(it->first), this->TotalBytesR2.at(it->first), this->TotalBytesR3.at(it->first));

			ss << it->first << "\t" << std::fixed << (this->LastTS.at(it->first) - this->FirstTS.at(it->first)) << "\t" <<

				// Total packets and total packets for each range
				it->second->TotalAnalyzedItems() << "\t" << this->LengthsRange1.at(it->first)->TotalAnalyzedItems() << "\t" << this->LengthsRange2.at(it->first)->TotalAnalyzedItems() << "\t" << this->LengthsRange3.at(it->first)->TotalAnalyzedItems() << "\t" <<
				// Ratio of packets for each range in relation to the total packets
				((double) this->LengthsRange1.at(it->first)->TotalAnalyzedItems() / (double) it->second->TotalAnalyzedItems()) << "\t" << ((double) this->LengthsRange2.at(it->first)->TotalAnalyzedItems() / (double) it->second->TotalAnalyzedItems()) << "\t" << ((double) this->LengthsRange3.at(it->first)->TotalAnalyzedItems() / (double) it->second->TotalAnalyzedItems()) << "\t" <<
				// Ratio of packets for each range in relation to the maximum number of packets from the three ranges
				((double) this->LengthsRange1.at(it->first)->TotalAnalyzedItems() / (double) MaxT) << "\t" << ((double) this->LengthsRange2.at(it->first)->TotalAnalyzedItems() / (double) MaxT) << "\t" << ((double) this->LengthsRange3.at(it->first)->TotalAnalyzedItems() / (double) MaxT) << "\t" <<

				// Total bytes and total bytes for each range
				this->TotalBytes.at(it->first) << "\t" << this->TotalBytesR1.at(it->first) << "\t" << this->TotalBytesR2.at(it->first) << "\t" << this->TotalBytesR3.at(it->first) << "\t" <<
				// Ratio of bytes for each range in relation to the total bytes
				((double) this->TotalBytesR1.at(it->first) / (double) this->TotalBytes.at(it->first)) << "\t" << ((double) this->TotalBytesR2.at(it->first) / (double) this->TotalBytes.at(it->first)) << "\t" << ((double) this->TotalBytesR3.at(it->first) / (double) this->TotalBytes.at(it->first)) << "\t" <<
				// Ratio of bytes for each range in relation to the maximum number of bytes from the three ranges
				((double) this->TotalBytesR1.at(it->first) / (double) MaxB) << "\t" << ((double) this->TotalBytesR2.at(it->first) / (double) MaxB) << "\t" << ((double) this->TotalBytesR3.at(it->first) / (double) MaxB) << "\t" <<

				// Mean of the entropy of the length for all packets and for each range of packets
				it->second->EntropyMean() << "\t" << this->LengthsRange1.at(it->first)->EntropyMean() << "\t" << this->LengthsRange2.at(it->first)->EntropyMean() << "\t" << this->LengthsRange3.at(it->first)->EntropyMean() << "\t" <<

				// Mean of the entropy of the length for each range of packets, with weight of the ratio of packets in relation to the total packets
				(((double) this->LengthsRange1.at(it->first)->TotalAnalyzedItems() / (double) it->second->TotalAnalyzedItems()) * this->LengthsRange1.at(it->first)->EntropyMean()) << "\t" <<
				(((double) this->LengthsRange2.at(it->first)->TotalAnalyzedItems() / (double) it->second->TotalAnalyzedItems()) * this->LengthsRange2.at(it->first)->EntropyMean()) << "\t" <<
				(((double) this->LengthsRange3.at(it->first)->TotalAnalyzedItems() / (double) it->second->TotalAnalyzedItems()) * this->LengthsRange3.at(it->first)->EntropyMean()) << "\t" <<

				this->Max((((double) this->LengthsRange1.at(it->first)->TotalAnalyzedItems() / (double) it->second->TotalAnalyzedItems()) * this->LengthsRange1.at(it->first)->EntropyMean()),
						  (((double) this->LengthsRange2.at(it->first)->TotalAnalyzedItems() / (double) it->second->TotalAnalyzedItems()) * this->LengthsRange2.at(it->first)->EntropyMean()),
						  (((double) this->LengthsRange3.at(it->first)->TotalAnalyzedItems() / (double) it->second->TotalAnalyzedItems()) * this->LengthsRange3.at(it->first)->EntropyMean())) << "\t" <<

				// Mean of the entropy of the length for each range of packets, with weight of the ratio of packets in relation to the maximum number of packets from the three ranges
				(((double) this->LengthsRange1.at(it->first)->TotalAnalyzedItems() / (double) MaxT) * this->LengthsRange1.at(it->first)->EntropyMean()) << "\t" <<
				(((double) this->LengthsRange2.at(it->first)->TotalAnalyzedItems() / (double) MaxT) * this->LengthsRange2.at(it->first)->EntropyMean()) << "\t" <<
				(((double) this->LengthsRange3.at(it->first)->TotalAnalyzedItems() / (double) MaxT) * this->LengthsRange3.at(it->first)->EntropyMean()) << "\t" <<

				this->Max((((double) this->LengthsRange1.at(it->first)->TotalAnalyzedItems() / (double) MaxT) * this->LengthsRange1.at(it->first)->EntropyMean()),
						  (((double) this->LengthsRange2.at(it->first)->TotalAnalyzedItems() / (double) MaxT) * this->LengthsRange2.at(it->first)->EntropyMean()),
						  (((double) this->LengthsRange3.at(it->first)->TotalAnalyzedItems() / (double) MaxT) * this->LengthsRange3.at(it->first)->EntropyMean())) << "\t" <<

				// Mean of the entropy of the length for each range of packets, with weight of the ratio of bytes for each range in relation to the total bytes
				(((double) this->TotalBytesR1.at(it->first) / (double) this->TotalBytes.at(it->first)) * this->LengthsRange1.at(it->first)->EntropyMean()) << "\t" <<
				(((double) this->TotalBytesR2.at(it->first) / (double) this->TotalBytes.at(it->first)) * this->LengthsRange2.at(it->first)->EntropyMean()) << "\t" <<
				(((double) this->TotalBytesR3.at(it->first) / (double) this->TotalBytes.at(it->first)) * this->LengthsRange3.at(it->first)->EntropyMean()) << "\t" <<

				this->Max((((double) this->TotalBytesR1.at(it->first) / (double) this->TotalBytes.at(it->first)) * this->LengthsRange1.at(it->first)->EntropyMean()),
						  (((double) this->TotalBytesR2.at(it->first) / (double) this->TotalBytes.at(it->first)) * this->LengthsRange2.at(it->first)->EntropyMean()),
						  (((double) this->TotalBytesR3.at(it->first) / (double) this->TotalBytes.at(it->first)) * this->LengthsRange3.at(it->first)->EntropyMean())) << "\t" <<

				// Mean of the entropy of the length for each range of packets, with weight of the ratio of bytes for each range in relation to the maximum number of bytes from the three ranges
				(((double) this->TotalBytesR1.at(it->first) / (double) MaxB) * this->LengthsRange1.at(it->first)->EntropyMean()) << "\t" <<
				(((double) this->TotalBytesR2.at(it->first) / (double) MaxB) * this->LengthsRange2.at(it->first)->EntropyMean()) << "\t" <<
				(((double) this->TotalBytesR3.at(it->first) / (double) MaxB) * this->LengthsRange3.at(it->first)->EntropyMean()) << "\t" <<

				this->Max((((double) this->TotalBytesR1.at(it->first) / (double) MaxB) * this->LengthsRange1.at(it->first)->EntropyMean()),
						  (((double) this->TotalBytesR2.at(it->first) / (double) MaxB) * this->LengthsRange2.at(it->first)->EntropyMean()),
						  (((double) this->TotalBytesR3.at(it->first) / (double) MaxB) * this->LengthsRange3.at(it->first)->EntropyMean())) << "\t" <<

				it->second->EntropyVariance() << "\t" << this->LengthsRange1.at(it->first)->EntropyVariance() << "\t" << this->LengthsRange2.at(it->first)->EntropyVariance() << "\t" << this->LengthsRange3.at(it->first)->EntropyVariance() << "\t" <<
				it->second->EntropyMin() << "\t" << this->LengthsRange1.at(it->first)->EntropyMin() << "\t" << this->LengthsRange2.at(it->first)->EntropyMin() << "\t" << this->LengthsRange3.at(it->first)->EntropyMin() << "\t" <<
				it->second->EntropyMax() << "\t" << this->LengthsRange1.at(it->first)->EntropyMax() << "\t" << this->LengthsRange2.at(it->first)->EntropyMax() << "\t" << this->LengthsRange3.at(it->first)->EntropyMax() << "\t" <<
				it->second->ValueMean() << "\t" << this->LengthsRange1.at(it->first)->ValueMean() << "\t" << this->LengthsRange2.at(it->first)->ValueMean() << "\t" << this->LengthsRange3.at(it->first)->ValueMean() << "\t" <<
				it->second->ValueVariance() << "\t" << this->LengthsRange1.at(it->first)->ValueVariance() << "\t" << this->LengthsRange2.at(it->first)->ValueVariance() << "\t" << this->LengthsRange3.at(it->first)->ValueVariance() << "\t" <<
				it->second->MinMean() << "\t" << this->LengthsRange1.at(it->first)->MinMean() << "\t" << this->LengthsRange2.at(it->first)->MinMean() << "\t" << this->LengthsRange3.at(it->first)->MinMean() << "\t" <<
				it->second->MaxMean() << "\t" << this->LengthsRange1.at(it->first)->MaxMean() << "\t" << this->LengthsRange2.at(it->first)->MaxMean() << "\t" << this->LengthsRange3.at(it->first)->MaxMean() << "\t" <<
				it->second->MinVariance() << "\t" << this->LengthsRange1.at(it->first)->MinVariance() << "\t" << this->LengthsRange2.at(it->first)->MinVariance() << "\t" << this->LengthsRange3.at(it->first)->MinVariance() << "\t" <<
				it->second->MaxVariance() << "\t" << this->LengthsRange1.at(it->first)->MaxVariance() << "\t" << this->LengthsRange2.at(it->first)->MaxVariance() << "\t" << this->LengthsRange3.at(it->first)->MaxVariance() << "\t" <<

				this->IAtimes.at(it->first)->EntropyMean() << "\t" << this->IAtimesRange1.at(it->first)->EntropyMean() << "\t" << this->IAtimesRange2.at(it->first)->EntropyMean() << "\t" << this->IAtimesRange3.at(it->first)->EntropyMean() << "\t" <<
				this->IAtimes.at(it->first)->EntropyVariance() << "\t" << this->IAtimesRange1.at(it->first)->EntropyVariance() << "\t" << this->IAtimesRange2.at(it->first)->EntropyVariance() << "\t" << this->IAtimesRange3.at(it->first)->EntropyVariance() << "\t" <<
				this->IAtimes.at(it->first)->EntropyMin() << "\t" << this->IAtimesRange1.at(it->first)->EntropyMin() << "\t" << this->IAtimesRange2.at(it->first)->EntropyMin() << "\t" << this->IAtimesRange3.at(it->first)->EntropyMin() << "\t" <<
				this->IAtimes.at(it->first)->EntropyMax() << "\t" << this->IAtimesRange1.at(it->first)->EntropyMax() << "\t" << this->IAtimesRange2.at(it->first)->EntropyMax() << "\t" << this->IAtimesRange3.at(it->first)->EntropyMax() << "\t" <<
				this->IAtimes.at(it->first)->ValueMean() << "\t" << this->IAtimesRange1.at(it->first)->ValueMean() << "\t" << this->IAtimesRange2.at(it->first)->ValueMean() << "\t" << this->IAtimesRange3.at(it->first)->ValueMean() << "\t" <<
				this->IAtimes.at(it->first)->ValueVariance() << "\t" << this->IAtimesRange1.at(it->first)->ValueVariance() << "\t" << this->IAtimesRange2.at(it->first)->ValueVariance() << "\t" << this->IAtimesRange3.at(it->first)->ValueVariance() << "\t" <<
				this->IAtimes.at(it->first)->MinMean() << "\t" << this->IAtimesRange1.at(it->first)->MinMean() << "\t" << this->IAtimesRange2.at(it->first)->MinMean() << "\t" << this->IAtimesRange3.at(it->first)->MinMean() << "\t" <<
				this->IAtimes.at(it->first)->MaxMean() << "\t" << this->IAtimesRange1.at(it->first)->MaxMean() << "\t" << this->IAtimesRange2.at(it->first)->MaxMean() << "\t" << this->IAtimesRange3.at(it->first)->MaxMean() << "\t" <<
				this->IAtimes.at(it->first)->MinVariance() << "\t" << this->IAtimesRange1.at(it->first)->MinVariance() << "\t" << this->IAtimesRange2.at(it->first)->MinVariance() << "\t" << this->IAtimesRange3.at(it->first)->MinVariance() << "\t" <<
				this->IAtimes.at(it->first)->MaxVariance() << "\t" << this->IAtimesRange1.at(it->first)->MaxVariance() << "\t" << this->IAtimesRange2.at(it->first)->MaxVariance() << "\t" << this->IAtimesRange3.at(it->first)->MaxVariance() << "\t" <<

				this->TTLs.at(it->first)->EntropyMean() << "\t" << this->TTLsRange1.at(it->first)->EntropyMean() << "\t" << this->TTLsRange2.at(it->first)->EntropyMean() << "\t" << this->TTLsRange3.at(it->first)->EntropyMean() << "\t" <<
				this->TTLs.at(it->first)->EntropyVariance() << "\t" << this->TTLsRange1.at(it->first)->EntropyVariance() << "\t" << this->TTLsRange2.at(it->first)->EntropyVariance() << "\t" << this->TTLsRange3.at(it->first)->EntropyVariance() << "\t" <<
				this->TTLs.at(it->first)->EntropyMin() << "\t" << this->TTLsRange1.at(it->first)->EntropyMin() << "\t" << this->TTLsRange2.at(it->first)->EntropyMin() << "\t" << this->TTLsRange3.at(it->first)->EntropyMin() << "\t" <<
				this->TTLs.at(it->first)->EntropyMax() << "\t" << this->TTLsRange1.at(it->first)->EntropyMax() << "\t" << this->TTLsRange2.at(it->first)->EntropyMax() << "\t" << this->TTLsRange3.at(it->first)->EntropyMax() << "\t" <<
				this->TTLs.at(it->first)->ValueMean() << "\t" << this->TTLsRange1.at(it->first)->ValueMean() << "\t" << this->TTLsRange2.at(it->first)->ValueMean() << "\t" << this->TTLsRange3.at(it->first)->ValueMean() << "\t" <<
				this->TTLs.at(it->first)->ValueVariance() << "\t" << this->TTLsRange1.at(it->first)->ValueVariance() << "\t" << this->TTLsRange2.at(it->first)->ValueVariance() << "\t" << this->TTLsRange3.at(it->first)->ValueVariance() << "\t" <<
				this->TTLs.at(it->first)->MinMean() << "\t" << this->TTLsRange1.at(it->first)->MinMean() << "\t" << this->TTLsRange2.at(it->first)->MinMean() << "\t" << this->TTLsRange3.at(it->first)->MinMean() << "\t" <<
				this->TTLs.at(it->first)->MaxMean() << "\t" << this->TTLsRange1.at(it->first)->MaxMean() << "\t" << this->TTLsRange2.at(it->first)->MaxMean() << "\t" << this->TTLsRange3.at(it->first)->MaxMean() << "\t" <<
				this->TTLs.at(it->first)->MinVariance() << "\t" << this->TTLsRange1.at(it->first)->MinVariance() << "\t" << this->TTLsRange2.at(it->first)->MinVariance() << "\t" << this->TTLsRange3.at(it->first)->MinVariance() << "\t" <<
				this->TTLs.at(it->first)->MaxVariance() << "\t" << this->TTLsRange1.at(it->first)->MaxVariance() << "\t" << this->TTLsRange2.at(it->first)->MaxVariance() << "\t" << this->TTLsRange3.at(it->first)->MaxVariance() << "\t" <<

				this->HostPorts.at(it->first)->EntropyMean() << "\t" << this->HostPortsRange1.at(it->first)->EntropyMean() << "\t" << this->HostPortsRange2.at(it->first)->EntropyMean() << "\t" << this->HostPortsRange3.at(it->first)->EntropyMean() << "\t" <<
				this->HostPorts.at(it->first)->EntropyVariance() << "\t" << this->HostPortsRange1.at(it->first)->EntropyVariance() << "\t" << this->HostPortsRange2.at(it->first)->EntropyVariance() << "\t" << this->HostPortsRange3.at(it->first)->EntropyVariance() << "\t" <<
				this->HostPorts.at(it->first)->EntropyMin() << "\t" << this->HostPortsRange1.at(it->first)->EntropyMin() << "\t" << this->HostPortsRange2.at(it->first)->EntropyMin() << "\t" << this->HostPortsRange3.at(it->first)->EntropyMin() << "\t" <<
				this->HostPorts.at(it->first)->EntropyMax() << "\t" << this->HostPortsRange1.at(it->first)->EntropyMax() << "\t" << this->HostPortsRange2.at(it->first)->EntropyMax() << "\t" << this->HostPortsRange3.at(it->first)->EntropyMax() << "\t" <<

				std::endl;

			this->LogsSummary.WriteExistentFile(this->SummaryFileName, ss.str());
		}
	}
}

void HostPortAnalyzerExtended::init(const int &WindowSize, const int &prec, const std::string &LogsBase, const std::string &LogsComplement, const bool &PrintSum, const bool &PrintLogFiles) {
	this->TotalHostPorts = 0;
	this->TotalPackets = 0;
	this->WinSize = WindowSize;

	this->PrintSummary = PrintSum;
	this->PrintLogs = PrintLogFiles;

	this->precision = prec;

	if (PrintLogFiles)
		this->LogsFolder.InitFolder(LogsBase+LogsComplement);

	if (PrintSum)
	{
		this->LogsSummary.InitFolder(LogsBase);
		this->SummaryFileName = LogsComplement + ".log";
		replace(this->SummaryFileName.begin(), this->SummaryFileName.end(), '/', '_');
	}
}

void HostPortAnalyzerExtended::WriteEntropy(const std::string &SrcIP, const int &SrcPort,
										    const long &TSsec, const long &TSusec,
											const int &Length, const double &IAtime, const int &TTL, const std::string &HostPort,
											EntropyContainer<int> * EntropyLengths, EntropyContainer<int> * EntropyLengthsRange1, EntropyContainer<int> * EntropyLengthsRange2, EntropyContainer<int> * EntropyLengthsRange3,
											EntropyContainer<double> * EntropyIAtimes, EntropyContainer<double> * EntropyIAtimesRange1, EntropyContainer<double> * EntropyIAtimesRange2, EntropyContainer<double> * EntropyIAtimesRange3,
											EntropyContainer<int> * EntropyTTLs, EntropyContainer<int> * EntropyTTLsRange1, EntropyContainer<int> * EntropyTTLsRange2, EntropyContainer<int> * EntropyTTLsRange3,
											EntropyContainer<std::string> * EntropyHostPorts, EntropyContainer<std::string> * EntropyHostPortsRange1, EntropyContainer<std::string> * EntropyHostPortsRange2, EntropyContainer<std::string> * EntropyHostPortsRange3)
{
	if (this->PrintLogs) {
		std::stringstream ssKey(std::stringstream::out);
		ssKey << SrcIP << ":" << SrcPort;
		std::string Key = ssKey.str();

		std::stringstream ss(std::stringstream::out);

		const double dTS = TSsec + (TSusec * 0.000001);
		double dTimeElapsed;

		if (this->FirstTS.count(Key) == 0)
			this->FirstTS.insert(std::pair<std::string, double>(Key, dTS));

	    dTimeElapsed = dTS - (this->FirstTS[Key]);
	    long MaxT = this->Max((long long) EntropyLengthsRange1->TotalAnalyzedItems(), (long long) EntropyLengthsRange2->TotalAnalyzedItems(), (long long) EntropyLengthsRange3->TotalAnalyzedItems());
	    long long MaxB = this->Max(this->TotalBytesR1.at(Key), this->TotalBytesR2.at(Key), this->TotalBytesR3.at(Key));

		ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << dTimeElapsed << "\t" <<

			// Total packets and total packets for each range
			EntropyLengths->TotalAnalyzedItems() << "\t" << EntropyLengthsRange1->TotalAnalyzedItems() << "\t" << EntropyLengthsRange2->TotalAnalyzedItems() << "\t" << EntropyLengthsRange3->TotalAnalyzedItems() << "\t" <<
			// Ratio of packets for each range in relation to the total packets
			((double) EntropyLengthsRange1->TotalAnalyzedItems() / (double) EntropyLengths->TotalAnalyzedItems()) << "\t" << ((double) EntropyLengthsRange2->TotalAnalyzedItems() / (double) EntropyLengths->TotalAnalyzedItems()) << "\t" << ((double) EntropyLengthsRange3->TotalAnalyzedItems() / (double) EntropyLengths->TotalAnalyzedItems()) << "\t" <<
			// Ratio of packets for each range in relation to the maximum number of packets from the three ranges
			((double) EntropyLengthsRange1->TotalAnalyzedItems() / (double) MaxT) << "\t" << ((double) EntropyLengthsRange2->TotalAnalyzedItems() / (double) MaxT) << "\t" << ((double) EntropyLengthsRange3->TotalAnalyzedItems() / (double) MaxT) << "\t" <<

			// Total bytes and total bytes for each range
			this->TotalBytes.at(Key) << "\t" << this->TotalBytesR1.at(Key) << "\t" << this->TotalBytesR2.at(Key) << "\t" << this->TotalBytesR3.at(Key) << "\t" <<
			// Ratio of bytes for each range in relation to the total bytes
			((double) this->TotalBytesR1.at(Key) / (double) this->TotalBytes.at(Key)) << "\t" << ((double) this->TotalBytesR2.at(Key) / (double) this->TotalBytes.at(Key)) << "\t" << ((double) this->TotalBytesR3.at(Key) / (double) this->TotalBytes.at(Key)) << "\t" <<
			// Ratio of bytes for each range in relation to the maximum number of bytes from the three ranges
			((double) this->TotalBytesR1.at(Key) / (double) MaxB) << "\t" << ((double) this->TotalBytesR2.at(Key) / (double) MaxB) << "\t" << ((double) this->TotalBytesR3.at(Key) / MaxB) << "\t" <<

			Length << "\t" <<

			// Entropy of the length for all packets and for each range of packets
			EntropyLengths->EntropyVal() << "\t" << EntropyLengthsRange1->EntropyVal() << "\t" << EntropyLengthsRange2->EntropyVal() << "\t" << EntropyLengthsRange3->EntropyVal() << "\t" <<

			// Mean of the entropy of the length for all packets and for each range of packets
			EntropyLengths->EntropyMean() << "\t" << EntropyLengthsRange1->EntropyMean() << "\t" << EntropyLengthsRange2->EntropyMean() << "\t" << EntropyLengthsRange3->EntropyMean() << "\t" <<

			// Mean of the entropy of the length for each range of packets, with weight of the ratio of packets in relation to the total packets
			(((double) EntropyLengthsRange1->TotalAnalyzedItems() / (double) EntropyLengths->TotalAnalyzedItems()) * EntropyLengthsRange1->EntropyMean()) << "\t" <<
			(((double) EntropyLengthsRange2->TotalAnalyzedItems() / (double) EntropyLengths->TotalAnalyzedItems()) * EntropyLengthsRange2->EntropyMean()) << "\t" <<
			(((double) EntropyLengthsRange3->TotalAnalyzedItems() / (double) EntropyLengths->TotalAnalyzedItems()) * EntropyLengthsRange3->EntropyMean()) << "\t" <<

			this->Max((((double) EntropyLengthsRange1->TotalAnalyzedItems() / (double) EntropyLengths->TotalAnalyzedItems()) * EntropyLengthsRange1->EntropyMean()),
					  (((double) EntropyLengthsRange2->TotalAnalyzedItems() / (double) EntropyLengths->TotalAnalyzedItems()) * EntropyLengthsRange2->EntropyMean()),
					  (((double) EntropyLengthsRange3->TotalAnalyzedItems() / (double) EntropyLengths->TotalAnalyzedItems()) * EntropyLengthsRange3->EntropyMean())) << "\t" <<

			// Mean of the entropy of the length for each range of packets, with weight of the ratio of packets in relation to the maximum number of packets from the three ranges
			(((double) EntropyLengthsRange1->TotalAnalyzedItems() / (double) MaxT) * EntropyLengthsRange1->EntropyMean()) << "\t" <<
			(((double) EntropyLengthsRange2->TotalAnalyzedItems() / (double) MaxT) * EntropyLengthsRange2->EntropyMean()) << "\t" <<
			(((double) EntropyLengthsRange3->TotalAnalyzedItems() / (double) MaxT) * EntropyLengthsRange3->EntropyMean()) << "\t" <<

			this->Max((((double) EntropyLengthsRange1->TotalAnalyzedItems() / (double) MaxT) * EntropyLengthsRange1->EntropyMean()),
					  (((double) EntropyLengthsRange2->TotalAnalyzedItems() / (double) MaxT) * EntropyLengthsRange2->EntropyMean()),
					  (((double) EntropyLengthsRange3->TotalAnalyzedItems() / (double) MaxT) * EntropyLengthsRange3->EntropyMean())) << "\t" <<

			// Mean of the entropy of the length for each range of packets, with weight of the ratio of bytes for each range in relation to the total bytes
			(((double) this->TotalBytesR1.at(Key) / (double) this->TotalBytes.at(Key)) * EntropyLengthsRange1->EntropyMean()) << "\t" <<
			(((double) this->TotalBytesR2.at(Key) / (double) this->TotalBytes.at(Key)) * EntropyLengthsRange2->EntropyMean()) << "\t" <<
			(((double) this->TotalBytesR3.at(Key) / (double) this->TotalBytes.at(Key)) * EntropyLengthsRange3->EntropyMean()) << "\t" <<

			this->Max((((double) this->TotalBytesR1.at(Key) / (double) this->TotalBytes.at(Key)) * EntropyLengthsRange1->EntropyMean()),
					  (((double) this->TotalBytesR2.at(Key) / (double) this->TotalBytes.at(Key)) * EntropyLengthsRange2->EntropyMean()),
					  (((double) this->TotalBytesR3.at(Key) / (double) this->TotalBytes.at(Key)) * EntropyLengthsRange3->EntropyMean())) << "\t" <<

			// Mean of the entropy of the length for each range of packets, with weight of the ratio of bytes for each range in relation to the maximum number of bytes from the three ranges
			(((double) this->TotalBytesR1.at(Key) / (double) MaxB) * EntropyLengthsRange1->EntropyMean()) << "\t" <<
			(((double) this->TotalBytesR2.at(Key) / (double) MaxB) * EntropyLengthsRange2->EntropyMean()) << "\t" <<
			(((double) this->TotalBytesR3.at(Key) / (double) MaxB) * EntropyLengthsRange3->EntropyMean()) << "\t" <<

			this->Max((((double) this->TotalBytesR1.at(Key) / (double) MaxB) * EntropyLengthsRange1->EntropyMean()),
					  (((double) this->TotalBytesR2.at(Key) / (double) MaxB) * EntropyLengthsRange2->EntropyMean()),
					  (((double) this->TotalBytesR3.at(Key) / (double) MaxB) * EntropyLengthsRange3->EntropyMean())) << "\t" <<


			EntropyLengths->EntropyVariance() << "\t" << EntropyLengthsRange1->EntropyVariance() << "\t" << EntropyLengthsRange2->EntropyVariance() << "\t" << EntropyLengthsRange3->EntropyVariance() << "\t" <<
			EntropyLengths->EntropyMin() << "\t" << EntropyLengthsRange1->EntropyMin() << "\t" << EntropyLengthsRange2->EntropyMin() << "\t" << EntropyLengthsRange3->EntropyMin() << "\t" <<
			EntropyLengths->EntropyMax() << "\t" << EntropyLengthsRange1->EntropyMax() << "\t" << EntropyLengthsRange2->EntropyMax() << "\t" << EntropyLengthsRange3->EntropyMax() << "\t" <<
			EntropyLengths->ValueMean() << "\t" << EntropyLengthsRange1->ValueMean() << "\t" << EntropyLengthsRange2->ValueMean() << "\t" << EntropyLengthsRange3->ValueMean() << "\t" <<
			EntropyLengths->ValueVariance() << "\t" << EntropyLengthsRange1->ValueVariance() << "\t" << EntropyLengthsRange2->ValueVariance() << "\t" << EntropyLengthsRange3->ValueVariance() << "\t" <<
			EntropyLengths->Min() << "\t" << EntropyLengthsRange1->Min() << "\t" << EntropyLengthsRange2->Min() << "\t" << EntropyLengthsRange3->Min() << "\t" <<
			EntropyLengths->Max() << "\t" << EntropyLengthsRange1->Max() << "\t" << EntropyLengthsRange2->Max() << "\t" << EntropyLengthsRange3->Max() << "\t" <<
			EntropyLengths->MinMean() << "\t" << EntropyLengthsRange1->MinMean() << "\t" << EntropyLengthsRange2->MinMean() << "\t" << EntropyLengthsRange3->MinMean() << "\t" <<
			EntropyLengths->MaxMean() << "\t" << EntropyLengthsRange1->MaxMean() << "\t" << EntropyLengthsRange2->MaxMean() << "\t" << EntropyLengthsRange3->MaxMean() << "\t" <<
			EntropyLengths->MinVariance() << "\t" << EntropyLengthsRange1->MinVariance() << "\t" << EntropyLengthsRange2->MinVariance() << "\t" << EntropyLengthsRange3->MinVariance() << "\t" <<
			EntropyLengths->MaxVariance() << "\t" << EntropyLengthsRange1->MaxVariance() << "\t" << EntropyLengthsRange2->MaxVariance() << "\t" << EntropyLengthsRange3->MaxVariance() << "\t" <<

			IAtime << "\t" <<
			EntropyIAtimes->EntropyVal() << "\t" << EntropyIAtimesRange1->EntropyVal() << "\t" << EntropyIAtimesRange2->EntropyVal() << "\t" << EntropyIAtimesRange3->EntropyVal() << "\t" <<
			EntropyIAtimes->EntropyMean() << "\t" << EntropyIAtimesRange1->EntropyMean() << "\t" << EntropyIAtimesRange2->EntropyMean() << "\t" << EntropyIAtimesRange3->EntropyMean() << "\t" <<
			EntropyIAtimes->EntropyVariance() << "\t" << EntropyIAtimesRange1->EntropyVariance() << "\t" << EntropyIAtimesRange2->EntropyVariance() << "\t" << EntropyIAtimesRange3->EntropyVariance() << "\t" <<
			EntropyIAtimes->EntropyMin() << "\t" << EntropyIAtimesRange1->EntropyMin() << "\t" << EntropyIAtimesRange2->EntropyMin() << "\t" << EntropyIAtimesRange3->EntropyMin() << "\t" <<
			EntropyIAtimes->EntropyMax() << "\t" << EntropyIAtimesRange1->EntropyMax() << "\t" << EntropyIAtimesRange2->EntropyMax() << "\t" << EntropyIAtimesRange3->EntropyMax() << "\t" <<
			EntropyIAtimes->ValueMean() << "\t" << EntropyIAtimesRange1->ValueMean() << "\t" << EntropyIAtimesRange2->ValueMean() << "\t" << EntropyIAtimesRange3->ValueMean() << "\t" <<
			EntropyIAtimes->ValueVariance() << "\t" << EntropyIAtimesRange1->ValueVariance() << "\t" << EntropyIAtimesRange2->ValueVariance() << "\t" << EntropyIAtimesRange3->ValueVariance() << "\t" <<
			EntropyIAtimes->Min() << "\t" << EntropyIAtimesRange1->Min() << "\t" << EntropyIAtimesRange2->Min() << "\t" << EntropyIAtimesRange3->Min() << "\t" <<
			EntropyIAtimes->Max() << "\t" << EntropyIAtimesRange1->Max() << "\t" << EntropyIAtimesRange2->Max() << "\t" << EntropyIAtimesRange3->Max() << "\t" <<
			EntropyIAtimes->MinMean() << "\t" << EntropyIAtimesRange1->MinMean() << "\t" << EntropyIAtimesRange2->MinMean() << "\t" << EntropyIAtimesRange3->MinMean() << "\t" <<
			EntropyIAtimes->MaxMean() << "\t" << EntropyIAtimesRange1->MaxMean() << "\t" << EntropyIAtimesRange2->MaxMean() << "\t" << EntropyIAtimesRange3->MaxMean() << "\t" <<
			EntropyIAtimes->MinVariance() << "\t" << EntropyIAtimesRange1->MinVariance() << "\t" << EntropyIAtimesRange2->MinVariance() << "\t" << EntropyIAtimesRange3->MinVariance() << "\t" <<
			EntropyIAtimes->MaxVariance() << "\t" << EntropyIAtimesRange1->MaxVariance() << "\t" << EntropyIAtimesRange2->MaxVariance() << "\t" << EntropyIAtimesRange3->MaxVariance() << "\t" <<

			TTL << "\t" <<
			EntropyTTLs->EntropyVal() << "\t" << EntropyTTLsRange1->EntropyVal() << "\t" << EntropyTTLsRange2->EntropyVal() << "\t" << EntropyTTLsRange3->EntropyVal() << "\t" <<
			EntropyTTLs->EntropyMean() << "\t" << EntropyTTLsRange1->EntropyMean() << "\t" << EntropyTTLsRange2->EntropyMean() << "\t" << EntropyTTLsRange3->EntropyMean() << "\t" <<
			EntropyTTLs->EntropyVariance() << "\t" << EntropyTTLsRange1->EntropyVariance() << "\t" << EntropyTTLsRange2->EntropyVariance() << "\t" << EntropyTTLsRange3->EntropyVariance() << "\t" <<
			EntropyTTLs->EntropyMin() << "\t" << EntropyTTLsRange1->EntropyMin() << "\t" << EntropyTTLsRange2->EntropyMin() << "\t" << EntropyTTLsRange3->EntropyMin() << "\t" <<
			EntropyTTLs->EntropyMax() << "\t" << EntropyTTLsRange1->EntropyMax() << "\t" << EntropyTTLsRange2->EntropyMax() << "\t" << EntropyTTLsRange3->EntropyMax() << "\t" <<
			EntropyTTLs->ValueMean() << "\t" << EntropyTTLsRange1->ValueMean() << "\t" << EntropyTTLsRange2->ValueMean() << "\t" << EntropyTTLsRange3->ValueMean() << "\t" <<
			EntropyTTLs->ValueVariance() << "\t" << EntropyTTLsRange1->ValueVariance() << "\t" << EntropyTTLsRange2->ValueVariance() << "\t" << EntropyTTLsRange3->ValueVariance() << "\t" <<
			EntropyTTLs->Min() << "\t" << EntropyTTLsRange1->Min() << "\t" << EntropyTTLsRange2->Min() << "\t" << EntropyTTLsRange3->Min() << "\t" <<
			EntropyTTLs->Max() << "\t" << EntropyTTLsRange1->Max() << "\t" << EntropyTTLsRange2->Max() << "\t" << EntropyTTLsRange3->Max() << "\t" <<
			EntropyTTLs->MinMean() << "\t" << EntropyTTLsRange1->MinMean() << "\t" << EntropyTTLsRange2->MinMean() << "\t" << EntropyTTLsRange3->MinMean() << "\t" <<
			EntropyTTLs->MaxMean() << "\t" << EntropyTTLsRange1->MaxMean() << "\t" << EntropyTTLsRange2->MaxMean() << "\t" << EntropyTTLsRange3->MaxMean() << "\t" <<
			EntropyTTLs->MinVariance() << "\t" << EntropyTTLsRange1->MinVariance() << "\t" << EntropyTTLsRange2->MinVariance() << "\t" << EntropyTTLsRange3->MinVariance() << "\t" <<
			EntropyTTLs->MaxVariance() << "\t" << EntropyTTLsRange1->MaxVariance() << "\t" << EntropyTTLsRange2->MaxVariance() << "\t" << EntropyTTLsRange3->MaxVariance() << "\t" <<

			HostPort << "\t" <<
			EntropyHostPorts->EntropyVal() << "\t" << EntropyHostPortsRange1->EntropyVal() << "\t" << EntropyHostPortsRange2->EntropyVal() << "\t" << EntropyHostPortsRange3->EntropyVal() << "\t" <<
			EntropyHostPorts->EntropyMean() << "\t" << EntropyHostPortsRange1->EntropyMean() << "\t" << EntropyHostPortsRange2->EntropyMean() << "\t" << EntropyHostPortsRange3->EntropyMean() << "\t" <<
			EntropyHostPorts->EntropyVariance() << "\t" << EntropyHostPortsRange1->EntropyVariance() << "\t" << EntropyHostPortsRange2->EntropyVariance() << "\t" << EntropyHostPortsRange3->EntropyVariance() << "\t" <<
			EntropyHostPorts->EntropyMin() << "\t" << EntropyHostPortsRange1->EntropyMin() << "\t" << EntropyHostPortsRange2->EntropyMin() << "\t" << EntropyHostPortsRange3->EntropyMin() << "\t" <<
			EntropyHostPorts->EntropyMax() << "\t" << EntropyHostPortsRange1->EntropyMax() << "\t" << EntropyHostPortsRange2->EntropyMax() << "\t" << EntropyHostPortsRange3->EntropyMax() << "\t" <<

			std::endl;

		this->WriteExistentLog(SrcIP, SrcPort, ss.str());
	}
}

void HostPortAnalyzerExtended::WriteNewLog(const std::string &HostIP, const int &Port, const std::string &Msg) const {
	if (this->PrintLogs) {
		std::stringstream fileName(std::stringstream::out);

		fileName << HostIP << "." << Port << ".log";

		this->LogsFolder.WriteNewFile(fileName.str().c_str(), Msg);
	}
}

void HostPortAnalyzerExtended::WriteExistentLog(const std::string &HostIP, const int &Port, const std::string &Msg) const {
	if (this->PrintLogs) {
		std::stringstream fileName(std::stringstream::out);

		fileName << HostIP << "." << Port << ".log";

		this->LogsFolder.WriteExistentFile(fileName.str().c_str(), Msg);
	}
}

void HostPortAnalyzerExtended::AddPacket(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto,
										 const int &Length, const int &TTL, const long &TSsec, const long &TSusec)
{
	std::stringstream ssKey(std::stringstream::out);
	ssKey << SrcIP << ":" << SrcPort;
	const std::string Key = ssKey.str();
    const double dTS = TSsec + ( TSusec * 0.000001 );

	if (this->Lengths.count(Key) == 0) {
        this->Lengths.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->LengthsRange1.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->LengthsRange2.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->LengthsRange3.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->IAtimes.insert(std::pair<std::string, EntropyContainer<double> *>(Key, new EntropyContainer<double>(this->WinSize)));
        this->IAtimesRange1.insert(std::pair<std::string, EntropyContainer<double> *>(Key, new EntropyContainer<double>(this->WinSize)));
        this->IAtimesRange2.insert(std::pair<std::string, EntropyContainer<double> *>(Key, new EntropyContainer<double>(this->WinSize)));
        this->IAtimesRange3.insert(std::pair<std::string, EntropyContainer<double> *>(Key, new EntropyContainer<double>(this->WinSize)));
        this->TTLs.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->TTLsRange1.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->TTLsRange2.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->TTLsRange3.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
		this->HostPorts.insert(std::pair<std::string, EntropyContainer<std::string> *>(Key, new EntropyContainer<std::string>(this->WinSize)));
		this->HostPortsRange1.insert(std::pair<std::string, EntropyContainer<std::string> *>(Key, new EntropyContainer<std::string>(this->WinSize)));
		this->HostPortsRange2.insert(std::pair<std::string, EntropyContainer<std::string> *>(Key, new EntropyContainer<std::string>(this->WinSize)));
		this->HostPortsRange3.insert(std::pair<std::string, EntropyContainer<std::string> *>(Key, new EntropyContainer<std::string>(this->WinSize)));
		(this->TotalHostPorts)++;

	    this->TotalBytes.insert(std::pair<std::string, long>(Key, 0));
	    this->TotalBytesR1.insert(std::pair<std::string, long>(Key, 0));
	    this->TotalBytesR2.insert(std::pair<std::string, long>(Key, 0));
	    this->TotalBytesR3.insert(std::pair<std::string, long>(Key, 0));

		time_t rawtime;
		struct tm * timeinfo;

		time(&rawtime);
		timeinfo = localtime(&rawtime);

		std::stringstream ss(std::stringstream::out);
		ss << "#\n# Log for host " << SrcIP << " on Port " << SrcPort	<<
			  "\n#\n# Created on " << asctime(timeinfo) << "#\n"
        	  "# Window of " << this->WinSize << " packets\n#\n" <<
        	  "# Range 1: 0-" << RANGES_MAX_1 << " bytes\n" <<
        	  "# Range 2: " << RANGES_MAX_1 << "-"<< RANGES_MAX_2 << " bytes\n" <<
        	  "# Range 3: " << RANGES_MAX_2 << "-" << RANGES_MAX_3 << " bytes\n#\n" <<
        	  "# <Timestamp>	<Time elapsed>	" <<

    		  "<Number of packets>	<Number of packets [Range1]>	<Number of packets [Range2]>	<Number of packets [Range3]>	" <<
    		  "<Ratio of packets [Range1]>	<Ratio of packets [Range2]>	<Ratio of packets [Range3]>	" <<
    		  "<Ratio over Max of packets [Range1]>	<Ratio over Max of packets [Range2]>	<Ratio over Max of packets [Range3]>	" <<

    		  "<Total bytes>	<Total bytes [Range1]>	<Total bytes [Range2]>	<Total bytes [Range3]>	" <<
    		  "<Ratio of bytes [Range1]>	<Ratio of bytes [Range2]>	<Ratio of bytes [Range3]>	" <<
    		  "<Ratio over Max of bytes [Range1]>	<Ratio over Max of bytes [Range2]>	<Ratio over Max of bytes [Range3]>	" <<

         	  "<Length>	" <<
          	  "<Entropy [lengths]>	<Entropy [lengths Range1]>	<Entropy [lengths Range2]>	<Entropy [lengths Range3]>	" <<
          	  "<Mean of the entropy [lengths]>	<Mean of the entropy [lengths Range1]>	<Mean of the entropy [lengths Range2]>	<Mean of the entropy [lengths Range3]>	" <<

    		  "<Mean of the entropy * Ratio of packets [lengths Range1]>	<Mean of the entropy * Ratio of packets [lengths Range2]>	<Mean of the entropy * Ratio of packets [lengths Range3]>	<Mean of the entropy * Ratio of packets [lengths MAX]>	" <<
    		  "<Mean of the entropy * Ratio over Max of packets [lengths Range1]>	<Mean of the entropy * Ratio over Max of packets [lengths Range2]>	<Mean of the entropy * Ratio over Max of packets [lengths MAX]>	" <<
    		  "<Mean of the entropy * Ratio of bytes [lengths Range1]>	<Mean of the entropy * Ratio of bytes [lengths Range2]>	<Mean of the entropy * Ratio of bytes [lengths MAX]>	" <<
    		  "<Mean of the entropy * Ratio over Max of bytes [lengths Range1]>	<Mean of the entropy * Ratio over Max of bytes [lengths Range2]>	<Mean of the entropy * Ratio over Max of bytes [lengths MAX]>	" <<

          	  "<Variance of the entropy [lengths]>	<Variance of the entropy [lengths Range1]>	<Variance of the entropy [lengths Range2]>	<Variance of the entropy [lengths Range3]>	" <<
          	  "<MIN of the entropy [lengths]>	<MIN of the entropy [lengths Range1]>	<MIN of the entropy [lengths Range2]>	<MIN of the entropy [lengths Range3]>	" <<
          	  "<MAX of the entropy [lengths]>	<MAX of the entropy [lengths Range1]>	<MAX of the entropy [lengths Range2]>	<MAX of the entropy [lengths Range3]>	" <<
          	  "<Mean [lengths]>	<Mean [lengths Range1]>	<Mean [lengths Range2]>	<Mean [lengths Range3]>	" <<
          	  "<Variance [lengths]>	<Variance [lengths Range1]>	<Variance [lengths Range2]>	<Variance [lengths Range3]>	" <<
          	  "<MIN in the window [lengths]>	<MIN in the window [lengths Range1]>	<MIN in the window [lengths Range2]>	<MIN in the window [lengths Range3]>	" <<
          	  "<MAX in the window [lengths]>	<MAX in the window [lengths Range1]>	<MAX in the window [lengths Range2]>	<MAX in the window [lengths Range3]>	" <<
          	  "<Mean of the MIN [lengths]>	<Mean of the MIN [lengths Range1]>	<Mean of the MIN [lengths Range2]>	<Mean of the MIN [lengths Range3]>	" <<
          	  "<Mean of the MAX [lengths]>	<Mean of the MAX [lengths Range1]>	<Mean of the MAX [lengths Range2]>	<Mean of the MAX [lengths Range3]>	" <<
          	  "<Variance of the MIN [lengths]>	<Variance of the MIN [lengths Range1]>	<Variance of the MIN [lengths Range2]>	<Variance of the MIN [lengths Range3]>	" <<
          	  "<Variance of the MAX [lengths]>	<Variance of the MAX [lengths Range1]>	<Variance of the MAX [lengths Range2]>	<Variance of the MAX [lengths Range3]>	" <<

          	  "<IATime>	" <<
          	  "<Entropy [IAtimes]>	<Entropy [IAtimes Range1]>	<Entropy [IAtimes Range2]>	<Entropy [IAtimes Range3]>	" <<
          	  "<Mean of the entropy [IAtimes]>	<Mean of the entropy [IAtimes Range1]>	<Mean of the entropy [IAtimes Range2]>	<Mean of the entropy [IAtimes Range3]>	" <<
          	  "<Variance of the entropy [IAtimes]>	<Variance of the entropy [IAtimes Range1]>	<Variance of the entropy [IAtimes Range2]>	<Variance of the entropy [IAtimes Range3]>	" <<
          	  "<MIN of the entropy [IAtimes]>	<MIN of the entropy [IAtimes Range1]>	<MIN of the entropy [IAtimes Range2]>	<MIN of the entropy [IAtimes Range3]>	" <<
          	  "<MAX of the entropy [IAtimes]>	<MAX of the entropy [IAtimes Range1]>	<MAX of the entropy [IAtimes Range2]>	<MAX of the entropy [IAtimes Range3]>	" <<
          	  "<Mean [IAtimes]>	<Mean [IAtimes Range1]>	<Mean [IAtimes Range2]>	<Mean [IAtimes Range3]>	" <<
          	  "<Variance [IAtimes]>	<Variance [IAtimes Range1]>	<Variance [IAtimes Range2]>	<Variance [IAtimes Range3]>	" <<
          	  "<MIN in the window [IAtimes]>	<MIN in the window [IAtimes Range1]>	<MIN in the window [IAtimes Range2]>	<MIN in the window [IAtimes Range3]>	" <<
          	  "<MAX in the window [IAtimes]>	<MAX in the window [IAtimes Range1]>	<MAX in the window [IAtimes Range2]>	<MAX in the window [IAtimes Range3]>	" <<
          	  "<Mean of the MIN [IAtimes]>	<Mean of the MIN [IAtimes Range1]>	<Mean of the MIN [IAtimes Range2]>	<Mean of the MIN [IAtimes Range3]>	" <<
          	  "<Mean of the MAX [IAtimes]>	<Mean of the MAX [IAtimes Range1]>	<Mean of the MAX [IAtimes Range2]>	<Mean of the MAX [IAtimes Range3]>	" <<
          	  "<Variance of the MIN [IAtimes]>	<Variance of the MIN [IAtimes Range1]>	<Variance of the MIN [IAtimes Range2]>	<Variance of the MIN [IAtimes Range3]>	" <<
          	  "<Variance of the MAX [IAtimes]>	<Variance of the MAX [IAtimes Range1]>	<Variance of the MAX [IAtimes Range2]>	<Variance of the MAX [IAtimes Range3]>	" <<

          	  "<TTL>	" <<
          	  "<Entropy [TTLs]>	<Entropy [TTLs Range1]>	<Entropy [TTLs Range2]>	<Entropy [TTLs Range3]>	" <<
          	  "<Mean of the entropy [TTLs]>	<Mean of the entropy [TTLs Range1]>	<Mean of the entropy [TTLs Range2]>	<Mean of the entropy [TTLs Range3]>	" <<
          	  "<Variance of the entropy [TTLs]>	<Variance of the entropy [TTLs Range1]>	<Variance of the entropy [TTLs Range2]>	<Variance of the entropy [TTLs Range3]>	" <<
          	  "<MIN of the entropy [TTLs]>	<MIN of the entropy [TTLs Range1]>	<MIN of the entropy [TTLs Range2]>	<MIN of the entropy [TTLs Range3]>	" <<
          	  "<MAX of the entropy [TTLs]>	<MAX of the entropy [TTLs Range1]>	<MAX of the entropy [TTLs Range2]>	<MAX of the entropy [TTLs Range3]>	" <<
          	  "<Mean [TTLs]>	<Mean [TTLs Range1]>	<Mean [TTLs Range2]>	<Mean [TTLs Range3]>	" <<
          	  "<Variance [TTLs]>	<Variance [TTLs Range1]>	<Variance [TTLs Range2]>	<Variance [TTLs Range3]>	" <<
          	  "<MIN in the window [TTLs]>	<MIN in the window [TTLs Range1]>	<MIN in the window [TTLs Range2]>	<MIN in the window [TTLs Range3]>	" <<
          	  "<MAX in the window [TTLs]>	<MAX in the window [TTLs Range1]>	<MAX in the window [TTLs Range2]>	<MAX in the window [TTLs Range3]>	" <<
          	  "<Mean of the MIN [TTLs]>	<Mean of the MIN [TTLs Range1]>	<Mean of the MIN [TTLs Range2]>	<Mean of the MIN [TTLs Range3]>	" <<
          	  "<Mean of the MAX [TTLs]>	<Mean of the MAX [TTLs Range1]>	<Mean of the MAX [TTLs Range2]>	<Mean of the MAX [TTLs Range3]>	" <<
          	  "<Variance of the MIN [TTLs]>	<Variance of the MIN [TTLs Range1]>	<Variance of the MIN [TTLs Range2]>	<Variance of the MIN [TTLs Range3]>	" <<
          	  "<Variance of the MAX [TTLs]>	<Variance of the MAX [TTLs Range1]>	<Variance of the MAX [TTLs Range2]>	<Variance of the MAX [TTLs Range3]>	" <<

        	  "<HostPort>	" <<
        	  "<Entropy [HostPorts]>	<Entropy [HostPorts Range1]>	<Entropy [HostPorts Range2]>	<Entropy [HostPorts Range3]>	" <<
        	  "<Mean of the entropy [HostPorts]>	<Mean of the entropy [HostPorts Range1]>	<Mean of the entropy [HostPorts Range2]>	<Mean of the entropy [HostPorts Range3]>	" <<
        	  "<Variance of the entropy [HostPorts]>	<Variance of the entropy [HostPorts Range1]>	<Variance of the entropy [HostPorts Range2]>	<Variance of the entropy [HostPorts Range3]>	" <<
        	  "<MIN of the entropy [HostPorts]>	<MIN of the entropy [HostPorts Range1]>	<MIN of the entropy [HostPorts Range2]>	<MIN of the entropy [HostPorts Range3]>	" <<
        	  "<MAX of the entropy [HostPorts]>	<MAX of the entropy [HostPorts Range1]>	<MAX of the entropy [HostPorts Range2]>	<MAX of the entropy [HostPorts Range3]>	" <<

        	  "\n#\n" <<

          	  "# Timestamp		Elapsed		Pkts	Pkt[R1]	Pkt[R2]	Pkt[R3]	R Pkt[R1]	R Pkt[R2]	R Pkt[R3]	R M Pkt[R1]	R M Pkt[R2]	R M Pkt[R3]	Bytes	B[R1]	B[R2]	B[R3]	R Bytes[R1]	R Bytes[R2]	R Bytes[R3]	R M Bytes[R1]	R M Bytes[R2]	R M Bytes[R3]	Length	Entropy [L]	Entropy [LR1]	Entropy [LR2]	Entropy [LR3]	Mean Ent [L]	Mean Ent [LR1]	Mean Ent [LR2]	Mean Ent [LR3]	MEntRTP [LR1]	MEntRTP [LR2]	MEntRTP [LR3]	MEntRTP [LMAX]	MEntRMP [LR1]	MEntRMP [LR2]	MEntRMP [LR3]	MEntRMP [LMAX]	MEntRTB [LR1]	MEntRTB [LR2]	MEntRTB [LR3]	MEntRTB [LMAX]	MEntRMB [LR1]	MEntRMB [LR2]	MEntRMB [LR3]	MEntRMB [LMAX]	Var Ent [L]	Var Ent [LR1]	Var Ent [LR2]	Var Ent [LR3]	MIN Ent [L]	MIN Ent [LR1]	MIN Ent [LR2]	MIN Ent [LR3]	MAX Ent [L]	MAX Ent [LR1]	MAX Ent [LR2]	MAX Ent [LR3]	Mean [L]	Mean [LR1]	Mean [LR2]	Mean [LR3]	Var [L]		Var [LR1]	Var [LR2]	Var [LR3]	MN W[L]	MN W[L1]MN W[L2]MN W[L3]MX W[L]	MX W[L1]MX W[L2]MX W[L3]Mean of MN [L]	Mean of MN [L1]	Mean of MN [L2]	Mean of MN [L3]	Mean of MX [L]	Mean of MX [L1]	Mean of MX [L2]	Mean of MX [L3]	Var of MN [LR]	Var of MN [LR1]	Var of MN [LR2]	Var of MN [LR3]	Var of MX [LR]	Var of MX [LR1]	Var of MX [LR2]	Var of MX [LR3]	IATime		Entropy [I]	Entropy [IR1]	Entropy [IR2]	Entropy [IR3]	Mean Ent [I]	Mean Ent [IR1]	Mean Ent [IR2]	Mean Ent [IR3]	Var Ent [I]	Var Ent [IR1]	Var Ent [IR2]	Var Ent [IR3]	MIN Ent [I]	MIN Ent [IR1]	MIN Ent [IR2]	MIN Ent [IR3]	MAX Ent [I]	MAX Ent [IR1]	MAX Ent [IR2]	MAX Ent [IR3]	Mean [I]	Mean [IR1]	Mean [IR2]	Mean [IR3]	Var [I]		Var [IR1]	Var [IR2]	Var [IR3]	MN W[I]		MN W[I1]	MN W[I2]	MN W[I3]	MX W[I]		MX W[I1]	MX W[I2]	MX W[I3]	Mean of MN [I]	Mean of MN [I1]	Mean of MN [I2]	Mean of MN [I3]	Mean of MX [I]	Mean of MX [I1]	Mean of MX [I2]	Mean of MX [I3]	Var of MN [IR]	Var of MN [IR1]	Var of MN [IR2]	Var of MN [IR3]	Var of MX [IR]	Var of MX [IR1]	Var of MX [IR2]	Var of MX [IR3]	TTL	Entropy [T]	Entropy [TR1]	Entropy [TR2]	Entropy [TR3]	Mean Ent [T]	Mean Ent [TR1]	Mean Ent [TR2]	Mean Ent [TR3]	Var Ent [T]	Var Ent [TR1]	Var Ent [TR2]	Var Ent [TR3]	MIN Ent [T]	MIN Ent [TR1]	MIN Ent [TR2]	MIN Ent [TR3]	MAX Ent [T]	MAX Ent [TR1]	MAX Ent [TR2]	MAX Ent [TR3]	Mean [T]	Mean [TR1]	Mean [TR2]	Mean [TR3]	Var [T]		Var [TR1]	Var [TR2]	Var [TR3]	MN W[T]	MN W[T1]MN W[T2]MN W[T3]MX W[T]	MX W[T1]MX W[T2]MX W[T3]Mean of MN [T]	Mean of MN [T1]	Mean of MN [T2]	Mean of MN [T3]	Mean of MX [T]	Mean of MX [T1]	Mean of MX [T2]	Mean of MX [T3]	Var of MN [TR]	Var of MN [TR1]	Var of MN [TR2]	Var of MN [TR3]	Var of MX [TR]	Var of MX [TR1]	Var of MX [TR2]	Var of MX [TR3]	HostPort		Entropy [HP]	Entropy [HPR1]	Entropy [HPR2]	Entropy [HPR3]	Mean Ent [HP]	Mean Ent [HPR1]	Mean Ent [HPR2]	Mean Ent [HPR3]	Var Ent [HP]	Var Ent [HPR1]	Var Ent [HPR2]	Var Ent [HPR3]	MIN Ent [HP]	MIN Ent [HPR1]	MIN Ent [HPR2]	MIN Ent [HPR3]	MAX Ent [HP]	MAX Ent [HPR1]	MAX Ent [HPR2]	MAX Ent [HPR3]	\n#\n";

		this->WriteNewLog(SrcIP, SrcPort, ss.str());

    	this->LastTS.insert(std::pair<std::string, double>(Key,dTS));
    	this->FirstTS.insert(std::pair<std::string, double>(Key,dTS));
	}

    EntropyContainer<int> * EntropyLengths = this->Lengths.at(Key);
    EntropyContainer<int> * EntropyLengthsRange1 = this->LengthsRange1.at(Key);
    EntropyContainer<int> * EntropyLengthsRange2 = this->LengthsRange2.at(Key);
    EntropyContainer<int> * EntropyLengthsRange3 = this->LengthsRange3.at(Key);
    EntropyContainer<double> * EntropyIAtimes = this->IAtimes.at(Key);
    EntropyContainer<double> * EntropyIAtimesRange1 = this->IAtimesRange1.at(Key);
    EntropyContainer<double> * EntropyIAtimesRange2 = this->IAtimesRange2.at(Key);
    EntropyContainer<double> * EntropyIAtimesRange3 = this->IAtimesRange3.at(Key);
    EntropyContainer<int> * EntropyTTLs = this->TTLs.at(Key);
    EntropyContainer<int> * EntropyTTLsRange1 = this->TTLsRange1.at(Key);
    EntropyContainer<int> * EntropyTTLsRange2 = this->TTLsRange2.at(Key);
    EntropyContainer<int> * EntropyTTLsRange3 = this->TTLsRange3.at(Key);
    EntropyContainer<std::string> * EntropyHostPorts = this->HostPorts.at(Key);
    EntropyContainer<std::string> * EntropyHostPortsRange1 = this->HostPortsRange1.at(Key);
    EntropyContainer<std::string> * EntropyHostPortsRange2 = this->HostPortsRange2.at(Key);
    EntropyContainer<std::string> * EntropyHostPortsRange3 = this->HostPortsRange3.at(Key);

	std::stringstream ssHP(std::stringstream::out);
	ssHP << DstIP << ":" << DstPort << ":" << Proto;
    const std::string HostPort = ssHP.str();

    double IAtime = this->Round(dTS - this->LastTS.at(Key), this->precision);
    this->LastTS.at(Key) = dTS;

    EntropyLengths->AddValue(Length);
	EntropyTTLs->AddValue(TTL);
	EntropyIAtimes->AddValue(IAtime);
	EntropyHostPorts->AddValue(HostPort);

    if (Length <= RANGES_MAX_1)
    {
		EntropyLengthsRange1->AddValue(Length);
		EntropyTTLsRange1->AddValue(TTL);
		EntropyIAtimesRange1->AddValue(IAtime);
		EntropyHostPortsRange1->AddValue(HostPort);
	    (this->TotalBytesR1[Key])+=Length;
    }
    else if (Length <= RANGES_MAX_2)
    {
		EntropyLengthsRange2->AddValue(Length);
		EntropyTTLsRange2->AddValue(TTL);
		EntropyIAtimesRange2->AddValue(IAtime);
		EntropyHostPortsRange2->AddValue(HostPort);
	    (this->TotalBytesR2[Key])+=Length;
    }
    else if (Length <= RANGES_MAX_3)
    {
		EntropyLengthsRange3->AddValue(Length);
		EntropyTTLsRange3->AddValue(TTL);
		EntropyIAtimesRange3->AddValue(IAtime);
		EntropyHostPortsRange3->AddValue(HostPort);
	    (this->TotalBytesR3[Key])+=Length;
    }

	(this->TotalPackets)++;
    (this->TotalBytes[Key])+=Length;

	this->WriteEntropy(SrcIP, SrcPort,
					   TSsec, TSusec,
					   Length, IAtime, TTL, HostPort,
					   EntropyLengths, EntropyLengthsRange1, EntropyLengthsRange2, EntropyLengthsRange3,
					   EntropyIAtimes, EntropyIAtimesRange1, EntropyIAtimesRange2, EntropyIAtimesRange3,
					   EntropyTTLs, EntropyTTLsRange1, EntropyTTLsRange2, EntropyTTLsRange3,
					   EntropyHostPorts, EntropyHostPortsRange1, EntropyHostPortsRange2, EntropyHostPortsRange3);
}

unsigned long HostPortAnalyzerExtended::TotHosts() const {
	return this->TotalHostPorts;
}

unsigned long HostPortAnalyzerExtended::TotPackets() const {
	return this->TotalPackets;
}

const double HostPortAnalyzerExtended::Round(const double &number, const int &precision) const {
	double Aux, Result;
	int precFactor, intRes;

	if (precision == 0)
		return number;
	else if (precision > 0) {
		precFactor = 10;
		for (int i = 1; i < precision; i++)
			precFactor *= 10;
	} else
		return number;

	Aux = number * (double) precFactor;
	intRes = (int) Aux;

	Result = intRes / (double) precFactor;

	return Result;
}

const double HostPortAnalyzerExtended::Max(const double &val1, const double &val2, const double &val3) const {

	if (val1 > val2)
	{
		if (val1 > val3)
			return val1;
		else
			return val3;
	}
	else
	{
		if (val2 > val3)
			return val2;
		else
			return val3;
	}
}
