/*
 * \author João V. Gomes
 *
 *  slidingwindow.cpp
 *
 *  Created on: Oct 21, 2010
 *
 * \brief TODO
 * TODO
 */

#include "slidingwindow.h"

template<class V> SlidingWindow<V>::SlidingWindow() {

}

template<class V> SlidingWindow<V>::SlidingWindow(const int &WinSize) {
	this->init(WinSize);
}

template<class V> SlidingWindow<V>::~SlidingWindow() {
	this->dispose();
}

template<class V> void SlidingWindow<V>::init(const int &WinSize) {
	this->max = 0;
	this->min = 0;
	this->ValMean = 0;
	this->WindowSize = WinSize;
}

template<> void SlidingWindow<std::string>::init(const int &WinSize) {
	this->max = "";
	this->min = "";
	this->WindowSize = WinSize;
}

template<class V> void SlidingWindow<V>::Clear()
{
	this->max = 0;
	this->min = 0;
	this->ValMean = 0;
	this->WindowSize = 0;
	this->Window.clear();
	this->Occurrences.clear();
	this->dispose();
}

template<> void SlidingWindow<std::string>::Clear()
{
	this->max = "";
	this->min = "";
	this->WindowSize = 0;
	this->Window.clear();
	this->Occurrences.clear();
	this->dispose();
}

template<class V> void SlidingWindow<V>::dispose()
{

}

template<class V> const V SlidingWindow<V>::Min() const
{
	if (this->IsFilled())
		return this->min;
	else
		return 0;
}

template<class V> const V SlidingWindow<V>::Max() const
{
	if (this->IsFilled())
		return this->max;
	else
		return 0;
}

template<class V> const double SlidingWindow<V>::Mean() const
{
	if (this->IsFilled())
		return this->ValMean;
	else
		return 0;
}

template<class V> const int SlidingWindow<V>::WinSize() const
{
    return this->WindowSize;
}

template<class V> const int SlidingWindow<V>::WinCount() const
{
    return ((int) this->Window.size());
}

template<class V> const bool SlidingWindow<V>::IsFilled() const {
	return ( this->WindowSize <= ((int) this->Window.size()) );
}

template<class V> const V SlidingWindow<V>::NewestElem() const
{
    return this->Window.front();
}

template<class V> const V SlidingWindow<V>::OldestElem() const
{
    return this->Window.back();
}

template<class V> const typename std::deque<V>::iterator SlidingWindow<V>::itToBegin()
{
    return this->Window.begin();
}

template<class V> const typename std::deque<V>::iterator SlidingWindow<V>::itToEnd()
{
    return this->Window.end();
}

template<class V> const int SlidingWindow<V>::Count(const V &Elem) {
	if (this->Occurrences.count(Elem) > 0)
		return this->Occurrences[Elem];
	else
		return 0;
}

template<class V> V SlidingWindow<V>::FindMax()
{
	if (this->WinSize() == 0)
		return 0;
	else if (this->WinSize() == 1)
		return this->NewestElem();
	else
	{
		V max = this->NewestElem();

		for(typename std::deque<V>::iterator it = SlidingWindow<V>::itToBegin() + 1; it != SlidingWindow<V>::itToEnd(); it++)
			if (max < *it)
				max = *it;

		return max;
	}
}

template<class V> V SlidingWindow<V>::FindMin()
{
	if (this->WinSize() == 0)
		return 0;
	else if (this->WinSize() == 1)
		return this->NewestElem();
	else
	{
		V min = this->NewestElem();

		for(typename std::deque<V>::iterator it = SlidingWindow<V>::itToBegin() + 1; it != SlidingWindow<V>::itToEnd(); it ++)
			if (min > *it)
				min = *it;

		return min;
	}
}

template<class V> void SlidingWindow<V>::UpdateLimitsAdding(const V &Elem)
{
	if (this->min > Elem)
		this->min = Elem;
	if (this->max < Elem)
		this->max = Elem;
}

template<class V> void SlidingWindow<V>::UpdateLimitsRemoving(const V &Elem)
{
	if (this->min == Elem)
		this->min = this->FindMin();
	if (this->max == Elem)
		this->max = this->FindMax();
}


template<class V> void SlidingWindow<V>::AddElement(const V &Elem) {
	if (this->Window.size() == 0)
	{
		this->max = Elem;
		this->min = Elem;
	}

	if (this->IsFilled()) {
		this->RemoveElement();
	}

	this->InsertElement(Elem);

	this->ValMean += (Elem / ((double) this->WindowSize));

	if (this->Count(Elem) > 0)
		this->UpdateLimitsAdding(Elem);
}

template<> void SlidingWindow<std::string>::AddElement(const std::string &Elem) {
	if (this->IsFilled()) {
		const std::string Old = this->Window.back();

		this->Window.pop_back();

		if (this->Count(Old) > 1)
			(this->Occurrences[Old])--;
	    else
	        this->Occurrences.erase(Old);
	}

	this->InsertElement(Elem);
}

template<class V> void SlidingWindow<V>::RemoveElement() {
	const V Elem = this->Window.back();

	this->Window.pop_back();

	if (this->Count(Elem) > 1)
		(this->Occurrences[Elem])--;
    else
        this->Occurrences.erase(Elem);

	if (this->Count(Elem) <= 1)
        this->UpdateLimitsRemoving(Elem);

	this->ValMean -= (Elem / ((double) this->WindowSize));
}

template<> void SlidingWindow<std::string>::RemoveElement() {
	const std::string Elem = this->Window.back();

	this->Window.pop_back();

	if (this->Count(Elem) > 1)
		(this->Occurrences[Elem])--;
    else
        this->Occurrences.erase(Elem);
}

template<class V> void SlidingWindow<V>::InsertElement(const V &Elem) {
	this->Window.push_front(Elem);

	if (this->Count(Elem) > 0)
		(this->Occurrences[Elem])++;
	else
		this->Occurrences.insert(std::pair<V,int>(Elem,1));
}

template class SlidingWindow<int>;
template class SlidingWindow<double>;
template class SlidingWindow<std::string>;
