/*
 * \author João V. Gomes
 *
 *  tupleanalyzer.cpp
 *
 *  Created on: Feb 7, 2011
 *
 * \brief TODO
 * TODO
 */

#include "tupleanalyzer.h"

template<class V> TupleAnalyzer<V>::TupleAnalyzer(const int &WindowSize)
{
    init(WindowSize, std::string(), true, 0);
}

template<class V> TupleAnalyzer<V>::TupleAnalyzer(const int &WindowSize, const std::string &LogsPath)
{
    init(WindowSize, LogsPath, true, 0);
}

template<class V> TupleAnalyzer<V>::TupleAnalyzer(const int &WindowSize, const std::string &LogsPath, const bool &Print)
{
    init(WindowSize, LogsPath, Print, 0);
}

template<class V> TupleAnalyzer<V>::TupleAnalyzer(const int &WindowSize, const std::string &LogsPath, ThresholdsSet * TSet)
{
    init(WindowSize, LogsPath, true, TSet);
}

template<class V> TupleAnalyzer<V>::TupleAnalyzer(const int &WindowSize, const std::string &LogsPath, const bool &Print, ThresholdsSet * TSet)
{
    init(WindowSize, LogsPath, Print, TSet);
}

template<class V> TupleAnalyzer<V>::~TupleAnalyzer()
{
    this->dispose();
}

template<class V> void TupleAnalyzer<V>::Clear()
{
    this->dispose();

    this->Tuples.clear();
    this->LastTS.clear();
    this->LastValue.clear();
    this->FirstTS.clear();

    this->TotalTuples = 0;
    this->TotalValues = 0;
    this->Thresholds = 0;
    this->WinSize = 0;
    this->lastClean = 0;
}

template<class V> void TupleAnalyzer<V>::dispose()
{
    typename std::map<std::string,EntropyContainer<V> *>::iterator it;

    for(it = this->Tuples.begin(); it != this->Tuples.end(); it++)
        delete (it->second);
}

template<class V> void TupleAnalyzer<V>::init(const int &WindowSize, const std::string &Path, const bool &Print, ThresholdsSet * TSet)
{
    this->TotalTuples = 0;
    this->TotalValues = 0;
    this->lastClean = 0;
    this->cleanIn = false;
    this->WinSize = WindowSize;
    this->PrintOutput = Print;

    this->Thresholds = TSet;

    if (this->PrintOutput)
    	this->LogsFolder.InitFolder(Path);
}

template<class V> void TupleAnalyzer<V>::WriteEntropy(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const long &TSsec, const long &TSusec, const V &Value, EntropyContainer<V>* Entropy)
{
    if (this->PrintOutput)
	{
		std::stringstream ssKey (std::stringstream::out);
	    ssKey << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto;
	    std::string Key = ssKey.str();

		std::stringstream ss (std::stringstream::out);

		const double dTS = TSsec + ( TSusec * 0.000001 );
		double dTimeElapsed;

		if (this->FirstTS.count(Key) == 0)
			this->FirstTS.insert(std::pair<std::string, double>(Key, dTS));

		dTimeElapsed = dTS - (this->FirstTS[Key]);

		if (this->Thresholds != 0)
			ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << std::setprecision(6) << dTimeElapsed << "\t" << Value << "\t" << Entropy->EntropyVal() << "\t" << Entropy->EntropyMean() << "\t" << Entropy->EntropyVariance() << "\t" << Entropy->EntropyMin() << "\t" << Entropy->EntropyMax() << "\t" << std::setprecision(2) << Entropy->ValueMean() << "\t" << Entropy->ValueVariance() << "\t" << Entropy->Min() << "\t" << Entropy->Max() << "\t" << Entropy->MinMean() << "\t" << Entropy->MaxMean() << "\t" << Entropy->MinVariance() << "\t" << Entropy->MaxVariance() << "\t" << this->Thresholds->GetDescription(Entropy->EntropyVal()) << std::endl;
		else
			ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << std::setprecision(6) << dTimeElapsed << "\t" << Value << "\t" << Entropy->EntropyVal() << "\t" << Entropy->EntropyMean() << "\t" << Entropy->EntropyVariance() << "\t" << Entropy->EntropyMin() << "\t" << Entropy->EntropyMax() << "\t" << std::setprecision(2) << Entropy->ValueMean() << "\t" << Entropy->ValueVariance() << "\t" << Entropy->Min() << "\t" << Entropy->Max() << "\t" << Entropy->MinMean() << "\t" << Entropy->MaxMean() << "\t" << Entropy->MinVariance() << "\t" << Entropy->MaxVariance() << std::endl;

		this->WriteExistentLog(SrcIP, SrcPort, DstIP, DstPort, Proto, ss.str());
	}
}

template<> void TupleAnalyzer<std::string>::WriteEntropy(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const long &TSsec, const long &TSusec, const std::string &Value, EntropyContainer<std::string>* Entropy)
{
    if (this->PrintOutput)
	{
		std::stringstream ssKey (std::stringstream::out);
	    ssKey << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto;
	    std::string Key = ssKey.str();

		std::stringstream ss (std::stringstream::out);

		const double dTS = TSsec + ( TSusec * 0.000001 );
		double dTimeElapsed;

		if (this->FirstTS.count(Key) == 0)
			this->FirstTS.insert(std::pair<std::string, double>(Key, dTS));

		dTimeElapsed = dTS - (this->FirstTS[Key]);

		if (this->Thresholds != 0)
			ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << std::setprecision(6) << dTimeElapsed << "\t" << Value << "\t" << Entropy->EntropyVal() << "\t" << Entropy->EntropyMean() << "\t" << Entropy->EntropyVariance() << "\t" << Entropy->EntropyMin() << "\t" << Entropy->EntropyMax() << "\t" << this->Thresholds->GetDescription(Entropy->EntropyVal()) << std::endl;
		else
			ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << std::setprecision(6) << dTimeElapsed << "\t" << Value << "\t" << Entropy->EntropyVal() << "\t" << Entropy->EntropyMean() << "\t" << Entropy->EntropyVariance() << "\t" << Entropy->EntropyMin() << "\t" << Entropy->EntropyMax() << std::endl;

		this->WriteExistentLog(SrcIP, SrcPort, DstIP, DstPort, Proto, ss.str());
	}
}

template<class V> void TupleAnalyzer<V>::WriteNewLog(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const std::string &Msg) const
{
    if (this->PrintOutput)
	{
		std::stringstream fileName (std::stringstream::out);

		fileName << SrcIP << "." << SrcPort << "." << DstIP << "." << DstPort << "." << Proto << ".log";

		this->LogsFolder.WriteNewFile(fileName.str().c_str(), Msg);
	}
}

template<class V> void TupleAnalyzer<V>::WriteExistentLog(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const std::string &Msg) const
{
    if (this->PrintOutput)
	{
		std::stringstream fileName (std::stringstream::out);

		fileName << SrcIP << "." << SrcPort << "." << DstIP << "." << DstPort << "." << Proto << ".log";

		this->LogsFolder.WriteExistentFile(fileName.str().c_str(), Msg);
	}
}

template<class V> template<class D> double * TupleAnalyzer<V>::AddValueDif(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const D &Value, int precision)
{
	return this->AddValueDif(0, 0, SrcIP, SrcPort, DstIP, DstPort, Proto, Value, precision);
}

template<class V> template<class D> double * TupleAnalyzer<V>::AddValueDif(const long &TSsec, const long &TSusec, const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const D &Value, int precision)
{
    std::stringstream ssKey (std::stringstream::out);
    ssKey << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto;
    std::string Key = ssKey.str();

	if (this->LastValue.count(Key) == 0)
	{
		this->LastValue.insert(std::pair<std::string, D>(Key, Value));
		return 0;
	}

	D difVal = Value - (this->LastValue[Key]);
	D RoundDif = this->Round<D>(difVal, precision);

	this->LastValue[Key] = Value;

	return this->AddValue(TSsec, TSusec, SrcIP, SrcPort, DstIP, DstPort, Proto, RoundDif);
}

template<class V> double * TupleAnalyzer<V>::AddValue(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const V &Value)
{
    return this->AddValue(0, 0, SrcIP, SrcPort, DstIP, DstPort, Proto, Value);
}

template<class V> double * TupleAnalyzer<V>::AddValue(const long &TSsec, const long &TSusec, const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const V &Value)
{
    std::stringstream ssKey (std::stringstream::out);
    ssKey << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto;
    std::string Key = ssKey.str();
    const double dTS = TSsec + ( TSusec * 0.000001 );

    if (this->Tuples.count(Key) == 0)
    {
        this->Tuples.insert(std::pair<std::string,EntropyContainer<V> *>(Key,new EntropyContainer<V>(this->WinSize)));
        (this->TotalTuples)++;

        time_t rawtime;
        struct tm * timeinfo;

        time ( &rawtime );
        timeinfo = localtime ( &rawtime );

        std::stringstream ss (std::stringstream::out);
        ss << "#\n# Log for tuple " << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto <<
              "\n#\n# Created on " << asctime (timeinfo) << "#\n" <<
              "# <Timestamp>	<Time elapsed>   <Analyzed value>   <Entropy for a window of " << this->WinSize << " packets>	<Mean of the entropy values>	<Variance of the entropy values>	<MIN of the entropy values>		<MAX of the entropy values>			<Mean of all the values>		<Variance of all the values>	<MIN value in the window>	<MAX value in the window>	<Mean of the MIN values>	<Mean of the MAX values>	<Variance of the MIN values>	<Variance of the MAX values>\n#\n";

        this->WriteNewLog(SrcIP, SrcPort, DstIP, DstPort, Proto, ss.str());

    	this->LastTS.insert(std::pair<std::string, double>(Key,dTS));
    }
    else
    	this->LastTS.at(Key) = dTS;

    EntropyContainer<V> * Entropy = this->Tuples[Key];
    Entropy->AddValue(Value);

    (this->TotalValues)++;
    this->cleanInactive(dTS);

    this->WriteEntropy(SrcIP, SrcPort, DstIP, DstPort, Proto, TSsec, TSusec, Value, Entropy);

    return Entropy->GetStatsClassification();
}

template<> double * TupleAnalyzer<std::string>::AddValue(const long &TSsec, const long &TSusec, const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const std::string &Value)
{
    std::stringstream ssKey (std::stringstream::out);
    ssKey << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto;
    std::string Key = ssKey.str();
    const double dTS = TSsec + ( TSusec * 0.000001 );

    if (this->Tuples.count(Key) == 0)
    {
        this->Tuples.insert(std::pair<std::string,EntropyContainer<std::string> *>(Key,new EntropyContainer<std::string>(this->WinSize)));
        (this->TotalTuples)++;

        time_t rawtime;
        struct tm * timeinfo;

        time ( &rawtime );
        timeinfo = localtime ( &rawtime );

        std::stringstream ss (std::stringstream::out);
        ss << "#\n# Log for tuple " << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto <<
              "\n#\n# Created on " << asctime (timeinfo) << "#\n" <<
              "# <Timestamp>	<Time elapsed>   <Analyzed value>   <Entropy for a window of " << this->WinSize << " packets>	<Mean of the entropy values>	<Variance of the entropy values>	<MIN of the entropy values>		<MAX of the entropy values>\n#\n";

        this->WriteNewLog(SrcIP, SrcPort, DstIP, DstPort, Proto, ss.str());

    	this->LastTS.insert(std::pair<std::string, double>(Key,dTS));
    }
    else
    	this->LastTS.at(Key) = dTS;

    EntropyContainer<std::string> * Entropy = this->Tuples[Key];
    Entropy->AddValue(Value);

    (this->TotalValues)++;
    this->cleanInactive(dTS);

    this->WriteEntropy(SrcIP, SrcPort, DstIP, DstPort, Proto, TSsec, TSusec, Value, Entropy);

    return Entropy->GetStatsClassification();
}

template<class V> unsigned long TupleAnalyzer<V>::TotHosts() const
{
    return this->TotalTuples;
}

template<class V> unsigned long TupleAnalyzer<V>::TotValues() const
{
    return this->TotalValues;
}

template<class V> template<class D> const D TupleAnalyzer<V>::Round(const D &number, const int &precision) const
{
	D Aux, Result;
	int precFactor, intRes;

	if (precision == 0)
		return number;
	else if (precision > 0)
	{
		precFactor = 10;
		for (int i=1; i<precision; i++)
			precFactor *= 10;
	}
	else
		return number;

	Aux = number * (D) precFactor;
	intRes = (int) Aux;

	Result = intRes / (D) precFactor;

    return Result;
}

template<class V> void TupleAnalyzer<V>::setCleanInactive(const bool &clean) {
	this->cleanIn = clean;
}

template<class V> void TupleAnalyzer<V>::cleanInactive(const double &lastTS) {
	if (this->cleanIn)
	{
		if ((lastTS - this->lastClean) >= ENTROPY_CLEAN_INTERVAL)
		{
			std::map<std::string, double>::iterator it;

			for(it = this->LastTS.begin(); it != this->LastTS.end(); it++)
			{
				if ((lastTS - it->second) > ENTROPY_INACTIVE_TIMEOUT)
				{
					delete this->Tuples.at(it->first);
					this->Tuples.erase(it->first);
					this->LastValue.erase(it->first);
					this->FirstTS.erase(it->first);
					this->LastTS.erase(it->first);
				}
			}
			this->lastClean = lastTS;
		}
	}
}

template class TupleAnalyzer<int>;
template class TupleAnalyzer<double>;
template class TupleAnalyzer<std::string>;
template const int TupleAnalyzer<int>::Round<int>(const int &number, const int &precision) const;
template const double TupleAnalyzer<double>::Round<double>(const double &number, const int &precision) const;
template double * TupleAnalyzer<int>::AddValueDif<int>(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const int &Value, int precision);
template double * TupleAnalyzer<double>::AddValueDif<double>(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const double &Value, int precision);
template double * TupleAnalyzer<int>::AddValueDif<int>(const long &TSsec, const long &TSusec, const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const int &Value, int precision);
template double * TupleAnalyzer<double>::AddValueDif<double>(const long &TSsec, const long &TSusec, const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const double &Value, int precision);

