/*
 * \author João V. Gomes
 *
 *  tupleanalyzer.h
 *
 *  Created on: Feb 7, 2011
 *
 * \brief TODO
 * TODO
 */

#ifndef TUPPLEANALYZER_H_
#define TUPPLEANALYZER_H_

#ifdef __cplusplus
extern "C"
{
#endif
#include <time.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include <string.h>
#ifdef __cplusplus
}
#endif

#include <entropycontainer.h>
#include <direxception.h>
#include <Folder.h>
#include <thresholdsset.h>
#include <slidingwindow.h>

#include <map>
#include <string>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>

#ifndef ENTROPY_TIMEOUTS
#define ENTROPY_TIMEOUTS
#define ENTROPY_INACTIVE_TIMEOUT 64
#define ENTROPY_CLEAN_INTERVAL 300
#endif // TIMEOUTS

template<class V>
class TupleAnalyzer
{
    public:
        TupleAnalyzer(const int &WindowSize);
        TupleAnalyzer(const int &WindowSize, const std::string &LogsPath);
        TupleAnalyzer(const int &WindowSize, const std::string &LogsPath, const bool &Print);
        TupleAnalyzer(const int &WindowSize, const std::string &LogsPath, ThresholdsSet * TSet);
        TupleAnalyzer(const int &WindowSize, const std::string &LogsPath, const bool &Print, ThresholdsSet * TSet);
        virtual ~TupleAnalyzer();
        double * AddValue(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const V &Value);
        double * AddValue(const long &TSsec, const long &TSusec, const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const V &Value);
        template<class D> double * AddValueDif(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const D &Value, int precision);
        template<class D> double * AddValueDif(const long &TSsec, const long &TSusec, const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const D &Value, int precision);
        unsigned long TotHosts() const;
        unsigned long TotValues() const;
        void Clear();
        void setCleanInactive(const bool &clean);
    protected:
    private:
        std::map<std::string, EntropyContainer<V> *> Tuples;
        std::map<std::string, double> LastTS;
        unsigned long TotalTuples;
        unsigned long TotalValues;
        int WinSize;
        Folder LogsFolder;
        ThresholdsSet * Thresholds;
        std::map<std::string, V> LastValue;
        std::map<std::string, double> FirstTS;
        bool PrintOutput;
        bool cleanIn;
        double lastClean;

        void init(const int &WindowSize, const std::string &Path, const bool &Print, ThresholdsSet * TSet);
        void cleanInactive(const double &lastTS);
        void dispose();
        void WriteEntropy(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const long &TSsec, const long &TSusec, const V &Value, EntropyContainer<V>* Entropy);
        void WriteNewLog(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const std::string &Msg) const;
        void WriteExistentLog(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const std::string &Msg) const;
        template<class D> const D Round(const D &number, const int &precision) const;
};

#endif /* TUPPLEANALYZER_H_ */
