/*
 * \author João V. Gomes
 *
 *  tupleanalyzerextended.cpp
 *
 *  Created on: Apr 12, 2011
 *
 * \brief TODO
 * TODO
 */

#include "tupleanalyzerextended.h"

TupleAnalyzerExtended::TupleAnalyzerExtended(const int &WindowSize)
{
    init(WindowSize, DEFAULT_PRECISION, std::string(), true);
}

TupleAnalyzerExtended::TupleAnalyzerExtended(const int &WindowSize, const std::string &LogsPath)
{
    init(WindowSize, DEFAULT_PRECISION, LogsPath, true);
}

TupleAnalyzerExtended::TupleAnalyzerExtended(const int &WindowSize, const std::string &LogsPath, const bool &Print)
{
    init(WindowSize, DEFAULT_PRECISION, LogsPath, Print);
}

TupleAnalyzerExtended::TupleAnalyzerExtended(const int &WindowSize, const int &precision)
{
    init(WindowSize, precision, std::string(), true);
}

TupleAnalyzerExtended::TupleAnalyzerExtended(const int &WindowSize, const int &precision, const std::string &LogsPath)
{
    init(WindowSize, precision, LogsPath, true);
}

TupleAnalyzerExtended::TupleAnalyzerExtended(const int &WindowSize, const int &precision, const std::string &LogsPath, const bool &Print)
{
    init(WindowSize, precision, LogsPath, Print);
}

TupleAnalyzerExtended::~TupleAnalyzerExtended()
{
    this->dispose();
}

void TupleAnalyzerExtended::Clear()
{
    this->dispose();

    this->Lengths.clear();
    this->LengthsRange1.clear();
    this->LengthsRange2.clear();
    this->LengthsRange3.clear();

    this->TTLs.clear();
    this->TTLsRange1.clear();
    this->TTLsRange2.clear();
    this->TTLsRange3.clear();

    this->IAtimes.clear();
    this->IAtimesRange1.clear();
    this->IAtimesRange2.clear();
    this->IAtimesRange3.clear();

//    this->HostPorts.clear();
//    this->HostPortsRange1.clear();
//    this->HostPortsRange2.clear();
//    this->HostPortsRange3.clear();

    this->LastTS.clear();
    this->FirstTS.clear();

    this->TotalTuples = 0;
    this->TotalPackets = 0;
    this->WinSize = 0;
    this->lastClean = 0;
	this->precision = DEFAULT_PRECISION;
}

void TupleAnalyzerExtended::dispose()
{
    std::map<std::string,EntropyContainer<int> *>::iterator it;

    for(it = this->Lengths.begin(); it != this->Lengths.end(); it++)
    {
        delete (it->second);
		delete this->LengthsRange1.at(it->first);
		delete this->LengthsRange2.at(it->first);
		delete this->LengthsRange3.at(it->first);

        delete this->IAtimes.at(it->first);
        delete this->IAtimesRange1.at(it->first);
        delete this->IAtimesRange2.at(it->first);
        delete this->IAtimesRange3.at(it->first);

        delete this->TTLs.at(it->first);
        delete this->TTLsRange1.at(it->first);
        delete this->TTLsRange2.at(it->first);
        delete this->TTLsRange3.at(it->first);

//        delete this->HostPorts.at(it->first);
//        delete this->HostPortsRange1.at(it->first);
//        delete this->HostPortsRange2.at(it->first);
//        delete this->HostPortsRange3.at(it->first);
    }
}

void TupleAnalyzerExtended::init(const int &WindowSize, const int &prec, const std::string &Path, const bool &Print)
{
    this->TotalTuples = 0;
    this->TotalPackets = 0;
    this->lastClean = 0;
    this->cleanIn = false;
    this->WinSize = WindowSize;
    this->PrintOutput = Print;
    this->precision = prec;

    if (this->PrintOutput)
    	this->LogsFolder.InitFolder(Path);
}

void TupleAnalyzerExtended::WriteEntropy(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto,
										 const long &TSsec, const long &TSusec,
										 const int &Length, const double &IAtime, const int &TTL,
										 EntropyContainer<int> * EntropyLengths, EntropyContainer<int> * EntropyLengthsRange1, EntropyContainer<int> * EntropyLengthsRange2, EntropyContainer<int> * EntropyLengthsRange3,
										 EntropyContainer<double> * EntropyIAtimes, EntropyContainer<double> * EntropyIAtimesRange1, EntropyContainer<double> * EntropyIAtimesRange2, EntropyContainer<double> * EntropyIAtimesRange3,
										 EntropyContainer<int> * EntropyTTLs, EntropyContainer<int> * EntropyTTLsRange1, EntropyContainer<int> * EntropyTTLsRange2, EntropyContainer<int> * EntropyTTLsRange3)
{
    if (this->PrintOutput)
	{
    	std::stringstream ssKey (std::stringstream::out);
	    ssKey << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto;
	    std::string Key = ssKey.str();

		std::stringstream ss (std::stringstream::out);

		const double dTS = TSsec + ( TSusec * 0.000001 );
		double dTimeElapsed;

		if (this->FirstTS.count(Key) == 0)
			this->FirstTS.insert(std::pair<std::string, double>(Key, dTS));

		dTimeElapsed = dTS - (this->FirstTS[Key]);

		ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << dTimeElapsed << "\t" <<

			Length << "\t" <<
			EntropyLengths->EntropyVal() << "\t" << EntropyLengthsRange1->EntropyVal() << "\t" << EntropyLengthsRange2->EntropyVal() << "\t" << EntropyLengthsRange3->EntropyVal() << "\t" <<
			EntropyLengths->EntropyMean() << "\t" << EntropyLengthsRange1->EntropyMean() << "\t" << EntropyLengthsRange2->EntropyMean() << "\t" << EntropyLengthsRange3->EntropyMean() << "\t" <<
			EntropyLengths->EntropyVariance() << "\t" << EntropyLengthsRange1->EntropyVariance() << "\t" << EntropyLengthsRange2->EntropyVariance() << "\t" << EntropyLengthsRange3->EntropyVariance() << "\t" <<
			EntropyLengths->EntropyMin() << "\t" << EntropyLengthsRange1->EntropyMin() << "\t" << EntropyLengthsRange2->EntropyMin() << "\t" << EntropyLengthsRange3->EntropyMin() << "\t" <<
			EntropyLengths->EntropyMax() << "\t" << EntropyLengthsRange1->EntropyMax() << "\t" << EntropyLengthsRange2->EntropyMax() << "\t" << EntropyLengthsRange3->EntropyMax() << "\t" <<
			EntropyLengths->ValueMean() << "\t" << EntropyLengthsRange1->ValueMean() << "\t" << EntropyLengthsRange2->ValueMean() << "\t" << EntropyLengthsRange3->ValueMean() << "\t" <<
			EntropyLengths->ValueVariance() << "\t" << EntropyLengthsRange1->ValueVariance() << "\t" << EntropyLengthsRange2->ValueVariance() << "\t" << EntropyLengthsRange3->ValueVariance() << "\t" <<
			EntropyLengths->Min() << "\t" << EntropyLengthsRange1->Min() << "\t" << EntropyLengthsRange2->Min() << "\t" << EntropyLengthsRange3->Min() << "\t" <<
			EntropyLengths->Max() << "\t" << EntropyLengthsRange1->Max() << "\t" << EntropyLengthsRange2->Max() << "\t" << EntropyLengthsRange3->Max() << "\t" <<
			EntropyLengths->MinMean() << "\t" << EntropyLengthsRange1->MinMean() << "\t" << EntropyLengthsRange2->MinMean() << "\t" << EntropyLengthsRange3->MinMean() << "\t" <<
			EntropyLengths->MaxMean() << "\t" << EntropyLengthsRange1->MaxMean() << "\t" << EntropyLengthsRange2->MaxMean() << "\t" << EntropyLengthsRange3->MaxMean() << "\t" <<
			EntropyLengths->MinVariance() << "\t" << EntropyLengthsRange1->MinVariance() << "\t" << EntropyLengthsRange2->MinVariance() << "\t" << EntropyLengthsRange3->MinVariance() << "\t" <<
			EntropyLengths->MaxVariance() << "\t" << EntropyLengthsRange1->MaxVariance() << "\t" << EntropyLengthsRange2->MaxVariance() << "\t" << EntropyLengthsRange3->MaxVariance() << "\t" <<

			IAtime << "\t" <<
			EntropyIAtimes->EntropyVal() << "\t" << EntropyIAtimesRange1->EntropyVal() << "\t" << EntropyIAtimesRange2->EntropyVal() << "\t" << EntropyIAtimesRange3->EntropyVal() << "\t" <<
			EntropyIAtimes->EntropyMean() << "\t" << EntropyIAtimesRange1->EntropyMean() << "\t" << EntropyIAtimesRange2->EntropyMean() << "\t" << EntropyIAtimesRange3->EntropyMean() << "\t" <<
			EntropyIAtimes->EntropyVariance() << "\t" << EntropyIAtimesRange1->EntropyVariance() << "\t" << EntropyIAtimesRange2->EntropyVariance() << "\t" << EntropyIAtimesRange3->EntropyVariance() << "\t" <<
			EntropyIAtimes->EntropyMin() << "\t" << EntropyIAtimesRange1->EntropyMin() << "\t" << EntropyIAtimesRange2->EntropyMin() << "\t" << EntropyIAtimesRange3->EntropyMin() << "\t" <<
			EntropyIAtimes->EntropyMax() << "\t" << EntropyIAtimesRange1->EntropyMax() << "\t" << EntropyIAtimesRange2->EntropyMax() << "\t" << EntropyIAtimesRange3->EntropyMax() << "\t" <<
			EntropyIAtimes->ValueMean() << "\t" << EntropyIAtimesRange1->ValueMean() << "\t" << EntropyIAtimesRange2->ValueMean() << "\t" << EntropyIAtimesRange3->ValueMean() << "\t" <<
			EntropyIAtimes->ValueVariance() << "\t" << EntropyIAtimesRange1->ValueVariance() << "\t" << EntropyIAtimesRange2->ValueVariance() << "\t" << EntropyIAtimesRange3->ValueVariance() << "\t" <<
			EntropyIAtimes->Min() << "\t" << EntropyIAtimesRange1->Min() << "\t" << EntropyIAtimesRange2->Min() << "\t" << EntropyIAtimesRange3->Min() << "\t" <<
			EntropyIAtimes->Max() << "\t" << EntropyIAtimesRange1->Max() << "\t" << EntropyIAtimesRange2->Max() << "\t" << EntropyIAtimesRange3->Max() << "\t" <<
			EntropyIAtimes->MinMean() << "\t" << EntropyIAtimesRange1->MinMean() << "\t" << EntropyIAtimesRange2->MinMean() << "\t" << EntropyIAtimesRange3->MinMean() << "\t" <<
			EntropyIAtimes->MaxMean() << "\t" << EntropyIAtimesRange1->MaxMean() << "\t" << EntropyIAtimesRange2->MaxMean() << "\t" << EntropyIAtimesRange3->MaxMean() << "\t" <<
			EntropyIAtimes->MinVariance() << "\t" << EntropyIAtimesRange1->MinVariance() << "\t" << EntropyIAtimesRange2->MinVariance() << "\t" << EntropyIAtimesRange3->MinVariance() << "\t" <<
			EntropyIAtimes->MaxVariance() << "\t" << EntropyIAtimesRange1->MaxVariance() << "\t" << EntropyIAtimesRange2->MaxVariance() << "\t" << EntropyIAtimesRange3->MaxVariance() << "\t" <<

			TTL << "\t" <<
			EntropyTTLs->EntropyVal() << "\t" << EntropyTTLsRange1->EntropyVal() << "\t" << EntropyTTLsRange2->EntropyVal() << "\t" << EntropyTTLsRange3->EntropyVal() << "\t" <<
			EntropyTTLs->EntropyMean() << "\t" << EntropyTTLsRange1->EntropyMean() << "\t" << EntropyTTLsRange2->EntropyMean() << "\t" << EntropyTTLsRange3->EntropyMean() << "\t" <<
			EntropyTTLs->EntropyVariance() << "\t" << EntropyTTLsRange1->EntropyVariance() << "\t" << EntropyTTLsRange2->EntropyVariance() << "\t" << EntropyTTLsRange3->EntropyVariance() << "\t" <<
			EntropyTTLs->EntropyMin() << "\t" << EntropyTTLsRange1->EntropyMin() << "\t" << EntropyTTLsRange2->EntropyMin() << "\t" << EntropyTTLsRange3->EntropyMin() << "\t" <<
			EntropyTTLs->EntropyMax() << "\t" << EntropyTTLsRange1->EntropyMax() << "\t" << EntropyTTLsRange2->EntropyMax() << "\t" << EntropyTTLsRange3->EntropyMax() << "\t" <<
			EntropyTTLs->ValueMean() << "\t" << EntropyTTLsRange1->ValueMean() << "\t" << EntropyTTLsRange2->ValueMean() << "\t" << EntropyTTLsRange3->ValueMean() << "\t" <<
			EntropyTTLs->ValueVariance() << "\t" << EntropyTTLsRange1->ValueVariance() << "\t" << EntropyTTLsRange2->ValueVariance() << "\t" << EntropyTTLsRange3->ValueVariance() << "\t" <<
			EntropyTTLs->Min() << "\t" << EntropyTTLsRange1->Min() << "\t" << EntropyTTLsRange2->Min() << "\t" << EntropyTTLsRange3->Min() << "\t" <<
			EntropyTTLs->Max() << "\t" << EntropyTTLsRange1->Max() << "\t" << EntropyTTLsRange2->Max() << "\t" << EntropyTTLsRange3->Max() << "\t" <<
			EntropyTTLs->MinMean() << "\t" << EntropyTTLsRange1->MinMean() << "\t" << EntropyTTLsRange2->MinMean() << "\t" << EntropyTTLsRange3->MinMean() << "\t" <<
			EntropyTTLs->MaxMean() << "\t" << EntropyTTLsRange1->MaxMean() << "\t" << EntropyTTLsRange2->MaxMean() << "\t" << EntropyTTLsRange3->MaxMean() << "\t" <<
			EntropyTTLs->MinVariance() << "\t" << EntropyTTLsRange1->MinVariance() << "\t" << EntropyTTLsRange2->MinVariance() << "\t" << EntropyTTLsRange3->MinVariance() << "\t" <<
			EntropyTTLs->MaxVariance() << "\t" << EntropyTTLsRange1->MaxVariance() << "\t" << EntropyTTLsRange2->MaxVariance() << "\t" << EntropyTTLsRange3->MaxVariance() << "\t" <<

			std::endl;


//		ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << dTimeElapsed << "\t" <<
//
//			Length << "\t" << EntropyLengths->EntropyVal() << "\t" << EntropyLengths->EntropyMean() << "\t" << EntropyLengths->EntropyVariance() << "\t" << EntropyLengths->EntropyMin() << "\t" << EntropyLengths->EntropyMax() << "\t" << EntropyLengths->ValueMean() << "\t" << EntropyLengths->ValueVariance() << "\t" << EntropyLengths->Min() << "\t" << EntropyLengths->Max() << "\t" << EntropyLengths->MinMean() << "\t" << EntropyLengths->MaxMean() << "\t" << EntropyLengths->MinVariance() << "\t" << EntropyLengths->MaxVariance() << "\t" <<
//			EntropyLengthsRange1->EntropyVal() << "\t" << EntropyLengthsRange1->EntropyMean() << "\t" << EntropyLengthsRange1->EntropyVariance() << "\t" << EntropyLengthsRange1->EntropyMin() << "\t" << EntropyLengthsRange1->EntropyMax() << "\t" << EntropyLengthsRange1->ValueMean() << "\t" << EntropyLengthsRange1->ValueVariance() << "\t" << EntropyLengthsRange1->Min() << "\t" << EntropyLengthsRange1->Max() << "\t" << EntropyLengthsRange1->MinMean() << "\t" << EntropyLengthsRange1->MaxMean() << "\t" << EntropyLengthsRange1->MinVariance() << "\t" << EntropyLengthsRange1->MaxVariance() << "\t" <<
//			EntropyLengthsRange2->EntropyVal() << "\t" << EntropyLengthsRange2->EntropyMean() << "\t" << EntropyLengthsRange2->EntropyVariance() << "\t" << EntropyLengthsRange2->EntropyMin() << "\t" << EntropyLengthsRange2->EntropyMax() << "\t" << EntropyLengthsRange2->ValueMean() << "\t" << EntropyLengthsRange2->ValueVariance() << "\t" << EntropyLengthsRange2->Min() << "\t" << EntropyLengthsRange2->Max() << "\t" << EntropyLengthsRange2->MinMean() << "\t" << EntropyLengthsRange2->MaxMean() << "\t" << EntropyLengthsRange2->MinVariance() << "\t" << EntropyLengthsRange2->MaxVariance() << "\t" <<
//			EntropyLengthsRange3->EntropyVal() << "\t" << EntropyLengthsRange3->EntropyMean() << "\t" << EntropyLengthsRange3->EntropyVariance() << "\t" << EntropyLengthsRange3->EntropyMin() << "\t" << EntropyLengthsRange3->EntropyMax() << "\t" << EntropyLengthsRange3->ValueMean() << "\t" << EntropyLengthsRange3->ValueVariance() << "\t" << EntropyLengthsRange3->Min() << "\t" << EntropyLengthsRange3->Max() << "\t" << EntropyLengthsRange3->MinMean() << "\t" << EntropyLengthsRange3->MaxMean() << "\t" << EntropyLengthsRange3->MinVariance() << "\t" << EntropyLengthsRange3->MaxVariance() << "\t" <<
//
//			TTL << "\t" << EntropyTTLs->EntropyVal() << "\t" << EntropyTTLs->EntropyMean() << "\t" << EntropyTTLs->EntropyVariance() << "\t" << EntropyTTLs->EntropyMin() << "\t" << EntropyTTLs->EntropyMax() << "\t" << EntropyTTLs->ValueMean() << "\t" << EntropyTTLs->ValueVariance() << "\t" << EntropyTTLs->Min() << "\t" << EntropyTTLs->Max() << "\t" << EntropyTTLs->MinMean() << "\t" << EntropyTTLs->MaxMean() << "\t" << EntropyTTLs->MinVariance() << "\t" << EntropyTTLs->MaxVariance() << "\t" <<
//			EntropyTTLsRange1->EntropyVal() << "\t" << EntropyTTLsRange1->EntropyMean() << "\t" << EntropyTTLsRange1->EntropyVariance() << "\t" << EntropyTTLsRange1->EntropyMin() << "\t" << EntropyTTLsRange1->EntropyMax() << "\t" << EntropyTTLsRange1->ValueMean() << "\t" << EntropyTTLsRange1->ValueVariance() << "\t" << EntropyTTLsRange1->Min() << "\t" << EntropyTTLsRange1->Max() << "\t" << EntropyTTLsRange1->MinMean() << "\t" << EntropyTTLsRange1->MaxMean() << "\t" << EntropyTTLsRange1->MinVariance() << "\t" << EntropyTTLsRange1->MaxVariance() << "\t" <<
//			EntropyTTLsRange2->EntropyVal() << "\t" << EntropyTTLsRange2->EntropyMean() << "\t" << EntropyTTLsRange2->EntropyVariance() << "\t" << EntropyTTLsRange2->EntropyMin() << "\t" << EntropyTTLsRange2->EntropyMax() << "\t" << EntropyTTLsRange2->ValueMean() << "\t" << EntropyTTLsRange2->ValueVariance() << "\t" << EntropyTTLsRange2->Min() << "\t" << EntropyTTLsRange2->Max() << "\t" << EntropyTTLsRange2->MinMean() << "\t" << EntropyTTLsRange2->MaxMean() << "\t" << EntropyTTLsRange2->MinVariance() << "\t" << EntropyTTLsRange2->MaxVariance() << "\t" <<
//			EntropyTTLsRange3->EntropyVal() << "\t" << EntropyTTLsRange3->EntropyMean() << "\t" << EntropyTTLsRange3->EntropyVariance() << "\t" << EntropyTTLsRange3->EntropyMin() << "\t" << EntropyTTLsRange3->EntropyMax() << "\t" << EntropyTTLsRange3->ValueMean() << "\t" << EntropyTTLsRange3->ValueVariance() << "\t" << EntropyTTLsRange3->Min() << "\t" << EntropyTTLsRange3->Max() << "\t" << EntropyTTLsRange3->MinMean() << "\t" << EntropyTTLsRange3->MaxMean() << "\t" << EntropyTTLsRange3->MinVariance() << "\t" << EntropyTTLsRange3->MaxVariance() << "\t" <<
//
//			IAtime << "\t" << EntropyIAtimes->EntropyVal() << "\t" << EntropyIAtimes->EntropyMean() << "\t" << EntropyIAtimes->EntropyVariance() << "\t" << EntropyIAtimes->EntropyMin() << "\t" << EntropyIAtimes->EntropyMax() << "\t" << EntropyIAtimes->ValueMean() << "\t" << EntropyIAtimes->ValueVariance() << "\t" << EntropyIAtimes->Min() << "\t" << EntropyIAtimes->Max() << "\t" << EntropyIAtimes->MinMean() << "\t" << EntropyIAtimes->MaxMean() << "\t" << EntropyIAtimes->MinVariance() << "\t" << EntropyIAtimes->MaxVariance() << "\t" <<
//			EntropyIAtimesRange1->EntropyVal() << "\t" << EntropyIAtimesRange1->EntropyMean() << "\t" << EntropyIAtimesRange1->EntropyVariance() << "\t" << EntropyIAtimesRange1->EntropyMin() << "\t" << EntropyIAtimesRange1->EntropyMax() << "\t" << EntropyIAtimesRange1->ValueMean() << "\t" << EntropyIAtimesRange1->ValueVariance() << "\t" << EntropyIAtimesRange1->Min() << "\t" << EntropyIAtimesRange1->Max() << "\t" << EntropyIAtimesRange1->MinMean() << "\t" << EntropyIAtimesRange1->MaxMean() << "\t" << EntropyIAtimesRange1->MinVariance() << "\t" << EntropyIAtimesRange1->MaxVariance() << "\t" <<
//			EntropyIAtimesRange2->EntropyVal() << "\t" << EntropyIAtimesRange2->EntropyMean() << "\t" << EntropyIAtimesRange2->EntropyVariance() << "\t" << EntropyIAtimesRange2->EntropyMin() << "\t" << EntropyIAtimesRange2->EntropyMax() << "\t" << EntropyIAtimesRange2->ValueMean() << "\t" << EntropyIAtimesRange2->ValueVariance() << "\t" << EntropyIAtimesRange2->Min() << "\t" << EntropyIAtimesRange2->Max() << "\t" << EntropyIAtimesRange2->MinMean() << "\t" << EntropyIAtimesRange2->MaxMean() << "\t" << EntropyIAtimesRange2->MinVariance() << "\t" << EntropyIAtimesRange2->MaxVariance() << "\t" <<
//			EntropyIAtimesRange3->EntropyVal() << "\t" << EntropyIAtimesRange3->EntropyMean() << "\t" << EntropyIAtimesRange3->EntropyVariance() << "\t" << EntropyIAtimesRange3->EntropyMin() << "\t" << EntropyIAtimesRange3->EntropyMax() << "\t" << EntropyIAtimesRange3->ValueMean() << "\t" << EntropyIAtimesRange3->ValueVariance() << "\t" << EntropyIAtimesRange3->Min() << "\t" << EntropyIAtimesRange3->Max() << "\t" << EntropyIAtimesRange3->MinMean() << "\t" << EntropyIAtimesRange3->MaxMean() << "\t" << EntropyIAtimesRange3->MinVariance() << "\t" << EntropyIAtimesRange3->MaxVariance() << "\t" <<
//
//			std::endl;


//		ss << TSsec << "." << std::setfill('0') << std::setw(6) << TSusec << "\t" << std::fixed << std::setprecision(6) << dTimeElapsed << "\t" <<
//
//			Length << "\t" << EntropyLengths->EntropyVal() << "\t" << EntropyLengths->EntropyMean() << "\t" << EntropyLengths->EntropyVariance() << "\t" << EntropyLengths->EntropyMin() << "\t" << EntropyLengths->EntropyMax() << "\t" << std::setprecision(2) << EntropyLengths->ValueMean() << "\t" << EntropyLengths->ValueVariance() << "\t" << EntropyLengths->Min() << "\t" << EntropyLengths->Max() << "\t" << EntropyLengths->MinMean() << "\t" << EntropyLengths->MaxMean() << "\t" << EntropyLengths->MinVariance() << "\t" << EntropyLengths->MaxVariance() << "\t" <<
//			EntropyLengthsRange1->EntropyVal() << "\t" << EntropyLengthsRange1->EntropyMean() << "\t" << EntropyLengthsRange1->EntropyVariance() << "\t" << EntropyLengthsRange1->EntropyMin() << "\t" << EntropyLengthsRange1->EntropyMax() << "\t" << std::setprecision(2) << EntropyLengthsRange1->ValueMean() << "\t" << EntropyLengthsRange1->ValueVariance() << "\t" << EntropyLengthsRange1->Min() << "\t" << EntropyLengthsRange1->Max() << "\t" << EntropyLengthsRange1->MinMean() << "\t" << EntropyLengthsRange1->MaxMean() << "\t" << EntropyLengthsRange1->MinVariance() << "\t" << EntropyLengthsRange1->MaxVariance() << "\t" <<
//			EntropyLengthsRange2->EntropyVal() << "\t" << EntropyLengthsRange2->EntropyMean() << "\t" << EntropyLengthsRange2->EntropyVariance() << "\t" << EntropyLengthsRange2->EntropyMin() << "\t" << EntropyLengthsRange2->EntropyMax() << "\t" << std::setprecision(2) << EntropyLengthsRange2->ValueMean() << "\t" << EntropyLengthsRange2->ValueVariance() << "\t" << EntropyLengthsRange2->Min() << "\t" << EntropyLengthsRange2->Max() << "\t" << EntropyLengthsRange2->MinMean() << "\t" << EntropyLengthsRange2->MaxMean() << "\t" << EntropyLengthsRange2->MinVariance() << "\t" << EntropyLengthsRange2->MaxVariance() << "\t" <<
//			EntropyLengthsRange3->EntropyVal() << "\t" << EntropyLengthsRange3->EntropyMean() << "\t" << EntropyLengthsRange3->EntropyVariance() << "\t" << EntropyLengthsRange3->EntropyMin() << "\t" << EntropyLengthsRange3->EntropyMax() << "\t" << std::setprecision(2) << EntropyLengthsRange3->ValueMean() << "\t" << EntropyLengthsRange3->ValueVariance() << "\t" << EntropyLengthsRange3->Min() << "\t" << EntropyLengthsRange3->Max() << "\t" << EntropyLengthsRange3->MinMean() << "\t" << EntropyLengthsRange3->MaxMean() << "\t" << EntropyLengthsRange3->MinVariance() << "\t" << EntropyLengthsRange3->MaxVariance() << "\t" <<
//
//			TTL << "\t" << EntropyTTLs->EntropyVal() << "\t" << EntropyTTLs->EntropyMean() << "\t" << EntropyTTLs->EntropyVariance() << "\t" << EntropyTTLs->EntropyMin() << "\t" << EntropyTTLs->EntropyMax() << "\t" << std::setprecision(2) << EntropyTTLs->ValueMean() << "\t" << EntropyTTLs->ValueVariance() << "\t" << EntropyTTLs->Min() << "\t" << EntropyTTLs->Max() << "\t" << EntropyTTLs->MinMean() << "\t" << EntropyTTLs->MaxMean() << "\t" << EntropyTTLs->MinVariance() << "\t" << EntropyTTLs->MaxVariance() << "\t" <<
//			EntropyTTLsRange1->EntropyVal() << "\t" << EntropyTTLsRange1->EntropyMean() << "\t" << EntropyTTLsRange1->EntropyVariance() << "\t" << EntropyTTLsRange1->EntropyMin() << "\t" << EntropyTTLsRange1->EntropyMax() << "\t" << std::setprecision(2) << EntropyTTLsRange1->ValueMean() << "\t" << EntropyTTLsRange1->ValueVariance() << "\t" << EntropyTTLsRange1->Min() << "\t" << EntropyTTLsRange1->Max() << "\t" << EntropyTTLsRange1->MinMean() << "\t" << EntropyTTLsRange1->MaxMean() << "\t" << EntropyTTLsRange1->MinVariance() << "\t" << EntropyTTLsRange1->MaxVariance() << "\t" <<
//			EntropyTTLsRange2->EntropyVal() << "\t" << EntropyTTLsRange2->EntropyMean() << "\t" << EntropyTTLsRange2->EntropyVariance() << "\t" << EntropyTTLsRange2->EntropyMin() << "\t" << EntropyTTLsRange2->EntropyMax() << "\t" << std::setprecision(2) << EntropyTTLsRange2->ValueMean() << "\t" << EntropyTTLsRange2->ValueVariance() << "\t" << EntropyTTLsRange2->Min() << "\t" << EntropyTTLsRange2->Max() << "\t" << EntropyTTLsRange2->MinMean() << "\t" << EntropyTTLsRange2->MaxMean() << "\t" << EntropyTTLsRange2->MinVariance() << "\t" << EntropyTTLsRange2->MaxVariance() << "\t" <<
//			EntropyTTLsRange3->EntropyVal() << "\t" << EntropyTTLsRange3->EntropyMean() << "\t" << EntropyTTLsRange3->EntropyVariance() << "\t" << EntropyTTLsRange3->EntropyMin() << "\t" << EntropyTTLsRange3->EntropyMax() << "\t" << std::setprecision(2) << EntropyTTLsRange3->ValueMean() << "\t" << EntropyTTLsRange3->ValueVariance() << "\t" << EntropyTTLsRange3->Min() << "\t" << EntropyTTLsRange3->Max() << "\t" << EntropyTTLsRange3->MinMean() << "\t" << EntropyTTLsRange3->MaxMean() << "\t" << EntropyTTLsRange3->MinVariance() << "\t" << EntropyTTLsRange3->MaxVariance() << "\t" <<
//
//			std::setprecision(6) << IAtime << "\t" << EntropyIAtimes->EntropyVal() << "\t" << EntropyIAtimes->EntropyMean() << "\t" << EntropyIAtimes->EntropyVariance() << "\t" << EntropyIAtimes->EntropyMin() << "\t" << EntropyIAtimes->EntropyMax() << "\t" << std::setprecision(2) << EntropyIAtimes->ValueMean() << "\t" << EntropyIAtimes->ValueVariance() << "\t" << std::setprecision(6) << EntropyIAtimes->Min() << "\t" << std::setprecision(6) << EntropyIAtimes->Max() << "\t" << std::setprecision(6) << EntropyIAtimes->MinMean() << "\t" << std::setprecision(6) << EntropyIAtimes->MaxMean() << "\t" << EntropyIAtimes->MinVariance() << "\t" << EntropyIAtimes->MaxVariance() << "\t" <<
//			EntropyIAtimesRange1->EntropyVal() << "\t" << EntropyIAtimesRange1->EntropyMean() << "\t" << EntropyIAtimesRange1->EntropyVariance() << "\t" << EntropyIAtimesRange1->EntropyMin() << "\t" << EntropyIAtimesRange1->EntropyMax() << "\t" << std::setprecision(2) << EntropyIAtimesRange1->ValueMean() << "\t" << EntropyIAtimesRange1->ValueVariance() << "\t" << EntropyIAtimesRange1->Min() << "\t" << EntropyIAtimesRange1->Max() << "\t" << EntropyIAtimesRange1->MinMean() << "\t" << EntropyIAtimesRange1->MaxMean() << "\t" << EntropyIAtimesRange1->MinVariance() << "\t" << EntropyIAtimesRange1->MaxVariance() << "\t" <<
//			EntropyIAtimesRange2->EntropyVal() << "\t" << EntropyIAtimesRange2->EntropyMean() << "\t" << EntropyIAtimesRange2->EntropyVariance() << "\t" << EntropyIAtimesRange2->EntropyMin() << "\t" << EntropyIAtimesRange2->EntropyMax() << "\t" << std::setprecision(2) << EntropyIAtimesRange2->ValueMean() << "\t" << EntropyIAtimesRange2->ValueVariance() << "\t" << EntropyIAtimesRange2->Min() << "\t" << EntropyIAtimesRange2->Max() << "\t" << EntropyIAtimesRange2->MinMean() << "\t" << EntropyIAtimesRange2->MaxMean() << "\t" << EntropyIAtimesRange2->MinVariance() << "\t" << EntropyIAtimesRange2->MaxVariance() << "\t" <<
//			EntropyIAtimesRange3->EntropyVal() << "\t" << EntropyIAtimesRange3->EntropyMean() << "\t" << EntropyIAtimesRange3->EntropyVariance() << "\t" << EntropyIAtimesRange3->EntropyMin() << "\t" << EntropyIAtimesRange3->EntropyMax() << "\t" << std::setprecision(2) << EntropyIAtimesRange3->ValueMean() << "\t" << EntropyIAtimesRange3->ValueVariance() << "\t" << EntropyIAtimesRange3->Min() << "\t" << EntropyIAtimesRange3->Max() << "\t" << EntropyIAtimesRange3->MinMean() << "\t" << EntropyIAtimesRange3->MaxMean() << "\t" << EntropyIAtimesRange3->MinVariance() << "\t" << EntropyIAtimesRange3->MaxVariance() << "\t" <<
//
//			std::endl;

		this->WriteExistentLog(SrcIP, SrcPort, DstIP, DstPort, Proto, ss.str());
	}
}

void TupleAnalyzerExtended::WriteNewLog(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const std::string &Msg) const
{
    if (this->PrintOutput)
	{
		std::stringstream fileName (std::stringstream::out);

		fileName << SrcIP << "." << SrcPort << "." << DstIP << "." << DstPort << "." << Proto << ".log";

		this->LogsFolder.WriteNewFile(fileName.str().c_str(), Msg);
	}
}

void TupleAnalyzerExtended::WriteExistentLog(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const std::string &Msg) const
{
    if (this->PrintOutput)
	{
		std::stringstream fileName (std::stringstream::out);

		fileName << SrcIP << "." << SrcPort << "." << DstIP << "." << DstPort << "." << Proto << ".log";

		this->LogsFolder.WriteExistentFile(fileName.str().c_str(), Msg);
	}
}

void TupleAnalyzerExtended::AddPacket(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto,
									  const int &Length, const int &TTL, const long &TSsec, const long &TSusec)
{
    std::stringstream ssKey (std::stringstream::out);
    ssKey << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto;
    std::string Key = ssKey.str();
    const double dTS = TSsec + ( TSusec * 0.000001 );

    if (this->Lengths.count(Key) == 0)
    {
        this->Lengths.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->LengthsRange1.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->LengthsRange2.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->LengthsRange3.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->IAtimes.insert(std::pair<std::string, EntropyContainer<double> *>(Key, new EntropyContainer<double>(this->WinSize)));
        this->IAtimesRange1.insert(std::pair<std::string, EntropyContainer<double> *>(Key, new EntropyContainer<double>(this->WinSize)));
        this->IAtimesRange2.insert(std::pair<std::string, EntropyContainer<double> *>(Key, new EntropyContainer<double>(this->WinSize)));
        this->IAtimesRange3.insert(std::pair<std::string, EntropyContainer<double> *>(Key, new EntropyContainer<double>(this->WinSize)));
        this->TTLs.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->TTLsRange1.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->TTLsRange2.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        this->TTLsRange3.insert(std::pair<std::string, EntropyContainer<int> *>(Key, new EntropyContainer<int>(this->WinSize)));
        (this->TotalTuples)++;

        time_t rawtime;
        struct tm * timeinfo;

        time ( &rawtime );
        timeinfo = localtime ( &rawtime );

        std::stringstream ss (std::stringstream::out);
        ss << "#\n# Log for tuple " << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto <<
        	  "\n#\n# Created on " << asctime (timeinfo) << "#\n" <<
        	  "# Window of " << this->WinSize << " packets\n#\n" <<
        	  "# Range 1: 0-" << RANGES_MAX_1 << " bytes\n" <<
        	  "# Range 2: " << RANGES_MAX_1 << "-"<< RANGES_MAX_2 << " bytes\n" <<
        	  "# Range 3: " << RANGES_MAX_2 << "-" << RANGES_MAX_3 << " bytes\n#\n" <<
        	  "# <Timestamp>	<Time elapsed>	" <<

        	  "<Length>	" <<
        	  "<Entropy [lengths]>	<Entropy [lengths Range1]>	<Entropy [lengths Range2]>	<Entropy [lengths Range3]>	" <<
        	  "<Mean of the entropy [lengths]>	<Mean of the entropy [lengths Range1]>	<Mean of the entropy [lengths Range2]>	<Mean of the entropy [lengths Range3]>	" <<
        	  "<Variance of the entropy [lengths]>	<Variance of the entropy [lengths Range1]>	<Variance of the entropy [lengths Range2]>	<Variance of the entropy [lengths Range3]>	" <<
        	  "<MIN of the entropy [lengths]>	<MIN of the entropy [lengths Range1]>	<MIN of the entropy [lengths Range2]>	<MIN of the entropy [lengths Range3]>	" <<
        	  "<MAX of the entropy [lengths]>	<MAX of the entropy [lengths Range1]>	<MAX of the entropy [lengths Range2]>	<MAX of the entropy [lengths Range3]>	" <<
        	  "<Mean [lengths]>	<Mean [lengths Range1]>	<Mean [lengths Range2]>	<Mean [lengths Range3]>	" <<
        	  "<Variance [lengths]>	<Variance [lengths Range1]>	<Variance [lengths Range2]>	<Variance [lengths Range3]>	" <<
        	  "<MIN in the window [lengths]>	<MIN in the window [lengths Range1]>	<MIN in the window [lengths Range2]>	<MIN in the window [lengths Range3]>	" <<
        	  "<MAX in the window [lengths]>	<MAX in the window [lengths Range1]>	<MAX in the window [lengths Range2]>	<MAX in the window [lengths Range3]>	" <<
        	  "<Mean of the MIN [lengths]>	<Mean of the MIN [lengths Range1]>	<Mean of the MIN [lengths Range2]>	<Mean of the MIN [lengths Range3]>	" <<
        	  "<Mean of the MAX [lengths]>	<Mean of the MAX [lengths Range1]>	<Mean of the MAX [lengths Range2]>	<Mean of the MAX [lengths Range3]>	" <<
        	  "<Variance of the MIN [lengths]>	<Variance of the MIN [lengths Range1]>	<Variance of the MIN [lengths Range2]>	<Variance of the MIN [lengths Range3]>	" <<
        	  "<Variance of the MAX [lengths]>	<Variance of the MAX [lengths Range1]>	<Variance of the MAX [lengths Range2]>	<Variance of the MAX [lengths Range3]>	" <<

        	  "<IATime>	" <<
        	  "<Entropy [IAtimes]>	<Entropy [IAtimes Range1]>	<Entropy [IAtimes Range2]>	<Entropy [IAtimes Range3]>	" <<
        	  "<Mean of the entropy [IAtimes]>	<Mean of the entropy [IAtimes Range1]>	<Mean of the entropy [IAtimes Range2]>	<Mean of the entropy [IAtimes Range3]>	" <<
        	  "<Variance of the entropy [IAtimes]>	<Variance of the entropy [IAtimes Range1]>	<Variance of the entropy [IAtimes Range2]>	<Variance of the entropy [IAtimes Range3]>	" <<
        	  "<MIN of the entropy [IAtimes]>	<MIN of the entropy [IAtimes Range1]>	<MIN of the entropy [IAtimes Range2]>	<MIN of the entropy [IAtimes Range3]>	" <<
        	  "<MAX of the entropy [IAtimes]>	<MAX of the entropy [IAtimes Range1]>	<MAX of the entropy [IAtimes Range2]>	<MAX of the entropy [IAtimes Range3]>	" <<
        	  "<Mean [IAtimes]>	<Mean [IAtimes Range1]>	<Mean [IAtimes Range2]>	<Mean [IAtimes Range3]>	" <<
        	  "<Variance [IAtimes]>	<Variance [IAtimes Range1]>	<Variance [IAtimes Range2]>	<Variance [IAtimes Range3]>	" <<
        	  "<MIN in the window [IAtimes]>	<MIN in the window [IAtimes Range1]>	<MIN in the window [IAtimes Range2]>	<MIN in the window [IAtimes Range3]>	" <<
        	  "<MAX in the window [IAtimes]>	<MAX in the window [IAtimes Range1]>	<MAX in the window [IAtimes Range2]>	<MAX in the window [IAtimes Range3]>	" <<
        	  "<Mean of the MIN [IAtimes]>	<Mean of the MIN [IAtimes Range1]>	<Mean of the MIN [IAtimes Range2]>	<Mean of the MIN [IAtimes Range3]>	" <<
        	  "<Mean of the MAX [IAtimes]>	<Mean of the MAX [IAtimes Range1]>	<Mean of the MAX [IAtimes Range2]>	<Mean of the MAX [IAtimes Range3]>	" <<
        	  "<Variance of the MIN [IAtimes]>	<Variance of the MIN [IAtimes Range1]>	<Variance of the MIN [IAtimes Range2]>	<Variance of the MIN [IAtimes Range3]>	" <<
        	  "<Variance of the MAX [IAtimes]>	<Variance of the MAX [IAtimes Range1]>	<Variance of the MAX [IAtimes Range2]>	<Variance of the MAX [IAtimes Range3]>	" <<

        	  "<TTL>	" <<
        	  "<Entropy [TTLs]>	<Entropy [TTLs Range1]>	<Entropy [TTLs Range2]>	<Entropy [TTLs Range3]>	" <<
        	  "<Mean of the entropy [TTLs]>	<Mean of the entropy [TTLs Range1]>	<Mean of the entropy [TTLs Range2]>	<Mean of the entropy [TTLs Range3]>	" <<
        	  "<Variance of the entropy [TTLs]>	<Variance of the entropy [TTLs Range1]>	<Variance of the entropy [TTLs Range2]>	<Variance of the entropy [TTLs Range3]>	" <<
        	  "<MIN of the entropy [TTLs]>	<MIN of the entropy [TTLs Range1]>	<MIN of the entropy [TTLs Range2]>	<MIN of the entropy [TTLs Range3]>	" <<
        	  "<MAX of the entropy [TTLs]>	<MAX of the entropy [TTLs Range1]>	<MAX of the entropy [TTLs Range2]>	<MAX of the entropy [TTLs Range3]>	" <<
        	  "<Mean [TTLs]>	<Mean [TTLs Range1]>	<Mean [TTLs Range2]>	<Mean [TTLs Range3]>	" <<
        	  "<Variance [TTLs]>	<Variance [TTLs Range1]>	<Variance [TTLs Range2]>	<Variance [TTLs Range3]>	" <<
        	  "<MIN in the window [TTLs]>	<MIN in the window [TTLs Range1]>	<MIN in the window [TTLs Range2]>	<MIN in the window [TTLs Range3]>	" <<
        	  "<MAX in the window [TTLs]>	<MAX in the window [TTLs Range1]>	<MAX in the window [TTLs Range2]>	<MAX in the window [TTLs Range3]>	" <<
        	  "<Mean of the MIN [TTLs]>	<Mean of the MIN [TTLs Range1]>	<Mean of the MIN [TTLs Range2]>	<Mean of the MIN [TTLs Range3]>	" <<
        	  "<Mean of the MAX [TTLs]>	<Mean of the MAX [TTLs Range1]>	<Mean of the MAX [TTLs Range2]>	<Mean of the MAX [TTLs Range3]>	" <<
        	  "<Variance of the MIN [TTLs]>	<Variance of the MIN [TTLs Range1]>	<Variance of the MIN [TTLs Range2]>	<Variance of the MIN [TTLs Range3]>	" <<
        	  "<Variance of the MAX [TTLs]>	<Variance of the MAX [TTLs Range1]>	<Variance of the MAX [TTLs Range2]>	<Variance of the MAX [TTLs Range3]>	" <<

        	  "\n#\n" <<

          	  "# Timestamp		Elapsed		Length	Entropy [L]	Entropy [LR1]	Entropy [LR2]	Entropy [LR3]	Mean Ent [L]	Mean Ent [LR1]	Mean Ent [LR2]	Mean Ent [LR3]	Var Ent [L]	Var Ent [LR1]	Var Ent [LR2]	Var Ent [LR3]	MIN Ent [L]	MIN Ent [LR1]	MIN Ent [LR2]	MIN Ent [LR3]	MAX Ent [L]	MAX Ent [LR1]	MAX Ent [LR2]	MAX Ent [LR3]	Mean [L]	Mean [LR1]	Mean [LR2]	Mean [LR3]	Var [L]		Var [LR1]	Var [LR2]	Var [LR3]	MN W[L]	MN W[L1]MN W[L2]MN W[L3]MX W[L]	MX W[L1]MX W[L2]MX W[L3]Mean of MN [L]	Mean of MN [L1]	Mean of MN [L2]	Mean of MN [L3]	Mean of MX [L]	Mean of MX [L1]	Mean of MX [L2]	Mean of MX [L3]	Var of MN [LR]	Var of MN [LR1]	Var of MN [LR2]	Var of MN [LR3]	Var of MX [LR]	Var of MX [LR1]	Var of MX [LR2]	Var of MX [LR3]	IATime		Entropy [I]	Entropy [IR1]	Entropy [IR2]	Entropy [IR3]	Mean Ent [I]	Mean Ent [IR1]	Mean Ent [IR2]	Mean Ent [IR3]	Var Ent [I]	Var Ent [IR1]	Var Ent [IR2]	Var Ent [IR3]	MIN Ent [I]	MIN Ent [IR1]	MIN Ent [IR2]	MIN Ent [IR3]	MAX Ent [I]	MAX Ent [IR1]	MAX Ent [IR2]	MAX Ent [IR3]	Mean [I]	Mean [IR1]	Mean [IR2]	Mean [IR3]	Var [I]		Var [IR1]	Var [IR2]	Var [IR3]	MN W[I]		MN W[I1]	MN W[I2]	MN W[I3]	MX W[I]		MX W[I1]	MX W[I2]	MX W[I3]	Mean of MN [I]	Mean of MN [I1]	Mean of MN [I2]	Mean of MN [I3]	Mean of MX [I]	Mean of MX [I1]	Mean of MX [I2]	Mean of MX [I3]	Var of MN [IR]	Var of MN [IR1]	Var of MN [IR2]	Var of MN [IR3]	Var of MX [IR]	Var of MX [IR1]	Var of MX [IR2]	Var of MX [IR3]	TTL	Entropy [T]	Entropy [TR1]	Entropy [TR2]	Entropy [TR3]	Mean Ent [T]	Mean Ent [TR1]	Mean Ent [TR2]	Mean Ent [TR3]	Var Ent [T]	Var Ent [TR1]	Var Ent [TR2]	Var Ent [TR3]	MIN Ent [T]	MIN Ent [TR1]	MIN Ent [TR2]	MIN Ent [TR3]	MAX Ent [T]	MAX Ent [TR1]	MAX Ent [TR2]	MAX Ent [TR3]	Mean [T]	Mean [TR1]	Mean [TR2]	Mean [TR3]	Var [T]		Var [TR1]	Var [TR2]	Var [TR3]	MN W[T]	MN W[T1]MN W[T2]MN W[T3]MX W[T]	MX W[T1]MX W[T2]MX W[T3]Mean of MN [T]	Mean of MN [T1]	Mean of MN [T2]	Mean of MN [T3]	Mean of MX [T]	Mean of MX [T1]	Mean of MX [T2]	Mean of MX [T3]	Var of MN [TR]	Var of MN [TR1]	Var of MN [TR2]	Var of MN [TR3]	Var of MX [TR]	Var of MX [TR1]	Var of MX [TR2]	Var of MX [TR3]	\n#\n";


//        ss << "#\n# Log for tuple " << SrcIP << ":" << SrcPort << ":" << DstIP << ":" << DstPort << ":" << Proto <<
//        	  "\n#\n# Created on " << asctime (timeinfo) << "#\n" <<
//        	  "# Window of " << this->WinSize << " packets\n#\n" <<
//        	  "# <Timestamp>	<Time elapsed>	" <<
//        	  "<Length>	<Entropy [lengths]>	<Mean of the entropy [lengths]>	<Variance of the entropy [lengths]>	<MIN of the entropy [lengths]>	<MAX of the entropy [lengths]>	<Mean [lengths]>	<Variance [lengths]>	<MIN in the window [lengths]>	<MAX in the window [lengths]>	<Mean of the MIN [lengths]>	<Mean of the MAX [lengths]>	<Variance of the MIN [lengths]>	<Variance of the MAX [lengths]>	" <<
//        	  "<Entropy [lengths Range1]>	<Mean of the entropy [lengths Range1]>	<Variance of the entropy [lengths Range1]>	<MIN of the entropy [lengths Range1]>	<MAX of the entropy [lengths Range1]>	<Mean [lengths Range1]>	<Variance [lengths Range1]>	<MIN in the window [lengths Range1]>	<MAX in the window [lengths Range1]>	<Mean of the MIN [lengths Range1]>	<Mean of the MAX [lengths Range1]>	<Variance of the MIN [lengths Range1]>	<Variance of the MAX [lengths Range1]>	" <<
//        	  "<Entropy [lengths Range2]>	<Mean of the entropy [lengths Range2]>	<Variance of the entropy [lengths Range2]>	<MIN of the entropy [lengths Range2]>	<MAX of the entropy [lengths Range2]>	<Mean [lengths Range2]>	<Variance [lengths Range2]>	<MIN in the window [lengths Range2]>	<MAX in the window [lengths Range2]>	<Mean of the MIN [lengths Range2]>	<Mean of the MAX [lengths Range2]>	<Variance of the MIN [lengths Range2]>	<Variance of the MAX [lengths Range2]>	" <<
//        	  "<Entropy [lengths Range3]>	<Mean of the entropy [lengths Range3]>	<Variance of the entropy [lengths Range3]>	<MIN of the entropy [lengths Range3]>	<MAX of the entropy [lengths Range3]>	<Mean [lengths Range3]>	<Variance [lengths Range3]>	<MIN in the window [lengths Range3]>	<MAX in the window [lengths Range3]>	<Mean of the MIN [lengths Range3]>	<Mean of the MAX [lengths Range3]>	<Variance of the MIN [lengths Range3]>	<Variance of the MAX [lengths Range3]>	" <<
//        	  "<TTL>	<Entropy [TTLs]>	<Mean of the entropy [TTLs]>	<Variance of the entropy [TTLs]>	<MIN of the entropy [TTLs]>	<MAX of the entropy [TTLs]>	<Mean [TTLs]>	<Variance [TTLs]>	<MIN in the window [TTLs]>	<MAX in the window [TTLs]>	<Mean of the MIN [TTLs]>	<Mean of the MAX [TTLs]>	<Variance of the MIN [TTLs]>	<Variance of the MAX [TTLs]>	" <<
//        	  "<Entropy [TTLs Range1]>	<Mean of the entropy [TTLs Range1]>	<Variance of the entropy [TTLs Range1]>	<MIN of the entropy [TTLs Range1]>	<MAX of the entropy [TTLs Range1]>	<Mean [TTLs Range1]>	<Variance [TTLs Range1]>	<MIN in the window [TTLs Range1]>	<MAX in the window [TTLs Range1]>	<Mean of the MIN [TTLs Range1]>	<Mean of the MAX [TTLs Range1]>	<Variance of the MIN [TTLs Range1]>	<Variance of the MAX [TTLs Range1]>	" <<
//        	  "<Entropy [TTLs Range2]>	<Mean of the entropy [TTLs Range2]>	<Variance of the entropy [TTLs Range2]>	<MIN of the entropy [TTLs Range2]>	<MAX of the entropy [TTLs Range2]>	<Mean [TTLs Range2]>	<Variance [TTLs Range2]>	<MIN in the window [TTLs Range2]>	<MAX in the window [TTLs Range2]>	<Mean of the MIN [TTLs Range2]>	<Mean of the MAX [TTLs Range2]>	<Variance of the MIN [TTLs Range2]>	<Variance of the MAX [TTLs Range2]>	" <<
//        	  "<Entropy [TTLs Range3]>	<Mean of the entropy [TTLs Range3]>	<Variance of the entropy [TTLs Range3]>	<MIN of the entropy [TTLs Range3]>	<MAX of the entropy [TTLs Range3]>	<Mean [TTLs Range3]>	<Variance [TTLs Range3]>	<MIN in the window [TTLs Range3]>	<MAX in the window [TTLs Range3]>	<Mean of the MIN [TTLs Range3]>	<Mean of the MAX [TTLs Range3]>	<Variance of the MIN [TTLs Range3]>	<Variance of the MAX [TTLs Range3]>	" <<
//        	  "<IATime>	<Entropy [IAtimes]>	<Mean of the entropy [IAtimes]>	<Variance of the entropy [IAtimes]>	<MIN of the entropy [IAtimes]>	<MAX of the entropy [IAtimes]>	<Mean [IAtimes]>	<Variance [IAtimes]>	<MIN in the window [IAtimes]>	<MAX in the window [IAtimes]>	<Mean of the MIN [IAtimes]>	<Mean of the MAX [IAtimes]>	<Variance of the MIN [IAtimes]>	<Variance of the MAX [IAtimes]>	" <<
//        	  "<Entropy [IAtimes Range1]>	<Mean of the entropy [IAtimes Range1]>	<Variance of the entropy [IAtimes Range1]>	<MIN of the entropy [IAtimes Range1]>	<MAX of the entropy [IAtimes Range1]>	<Mean [IAtimes Range1]>	<Variance [IAtimes Range1]>	<MIN in the window [IAtimes Range1]>	<MAX in the window [IAtimes Range1]>	<Mean of the MIN [IAtimes Range1]>	<Mean of the MAX [IAtimes Range1]>	<Variance of the MIN [IAtimes Range1]>	<Variance of the MAX [IAtimes Range1]>	" <<
//        	  "<Entropy [IAtimes Range2]>	<Mean of the entropy [IAtimes Range2]>	<Variance of the entropy [IAtimes Range2]>	<MIN of the entropy [IAtimes Range2]>	<MAX of the entropy [IAtimes Range2]>	<Mean [IAtimes Range2]>	<Variance [IAtimes Range2]>	<MIN in the window [IAtimes Range2]>	<MAX in the window [IAtimes Range2]>	<Mean of the MIN [IAtimes Range2]>	<Mean of the MAX [IAtimes Range2]>	<Variance of the MIN [IAtimes Range2]>	<Variance of the MAX [IAtimes Range2]>	" <<
//        	  "<Entropy [IAtimes Range3]>	<Mean of the entropy [IAtimes Range3]>	<Variance of the entropy [IAtimes Range3]>	<MIN of the entropy [IAtimes Range3]>	<MAX of the entropy [IAtimes Range3]>	<Mean [IAtimes Range3]>	<Variance [IAtimes Range3]>	<MIN in the window [IAtimes Range3]>	<MAX in the window [IAtimes Range3]>	<Mean of the MIN [IAtimes Range3]>	<Mean of the MAX [IAtimes Range3]>	<Variance of the MIN [IAtimes Range3]>	<Variance of the MAX [IAtimes Range3]>	" <<
//        	  "\n#\n" <<
//
//			  "# Timestamp		Elapsed		Length	Entropy [L]	Mean Ent [L]	Var Ent [L]	MIN Ent [L]	MAX Ent [L]	Mean [L]	Var [L] 	MN W[L]	MX W[L]	Mean of MN [L]	Mean of MX [L]	Var of MN [L]	Var of MX [L]	Entropy [LR1]	Mean Ent [LR1]	Var Ent [LR1]	MIN Ent [LR1]	MAX Ent [LR1]	Mean [LR1]	Var [LR1] 	MN W[L1]MX W[L1]Mean of MN [L1]	Mean of MX [L1]	Var of MN [LR1]	Var of MX [LR1]	Entropy [LR2]	Mean Ent [LR2]	Var Ent [LR2]	MIN Ent [LR2]	MAX Ent [LR2]	Mean [LR2]	Var [LR2] 	MN W[L2]MX W[L2]Mean of MN [L2]	Mean of MX [L2]	Var of MN [LR2]	Var of MX [LR2]	Entropy [LR3]	Mean Ent [LR3]	Var Ent [LR3]	MIN Ent [LR3]	MAX Ent [LR3]	Mean [LR3]	Var [LR3] 	MN W[L3]MX W[L3]Mean of MN [L3]	Mean of MX [L3]	Var of MN [LR3]	Var of MX [LR3]	TTL	Entropy [T]	Mean Ent [T]	Var Ent [T]	MIN Ent [T]	MAX Ent [T]	Mean [T]	Var [T] 	MN W[T]	MX W[T]	Mean of MN [T]	Mean of MX [T]	Var of MN [T]	Var of MX [T]	Entropy [TR1]	Mean Ent [TR1]	Var Ent [TR1]	MIN Ent [TR1]	MAX Ent [TR1]	Mean [TR1]	Var [TR1] 	MN W[T1]MX W[T1]Mean of MN [T1]	Mean of MX [T1]	Var of MN [TR1]	Var of MX [TR1]	Entropy [TR2]	Mean Ent [TR2]	Var Ent [TR2]	MIN Ent [TR2]	MAX Ent [TR2]	Mean [TR2]	Var [TR2] 	MN W[T2]MX W[T2]Mean of MN [T2]	Mean of MX [T2]	Var of MN [TR2]	Var of MX [TR2]	Entropy [TR3]	Mean Ent [TR3]	Var Ent [TR3]	MIN Ent [TR3]	MAX Ent [TR3]	Mean [TR3]	Var [TR3] 	MN W[T3]MX W[T3]Mean of MN [T3]	Mean of MX [T3]	Var of MN [TR3]	Var of MX [TR3]	IATime  	Entropy [I]	Mean Ent [I]	Var Ent [I]	MIN Ent [I]	MAX Ent [I]	Mean [I]	Var [I] 	MN W[I]		MX W[I]		Mean of MN [I]	Mean of MX [I]	Var of MN [I]	Var of MX [I]	Entropy [IR1]	Mean Ent [IR1]	Var Ent [IR1]	MIN Ent [IR1]	MAX Ent [IR1]	Mean [IR1]	Var [IR1] 	MN W[I1]	MX W[I1]	Mean of MN [I1]	Mean of MX [I1]	Var of MN [IR1]	Var of MX [IR1]	Entropy [IR2]	Mean Ent [IR2]	Var Ent [IR2]	MIN Ent [IR2]	MAX Ent [IR2]	Mean [IR2]	Var [IR2] 	MN W[I2]	MX W[I2]	Mean of MN [I2]	Mean of MX [I2]	Var of MN [IR2]	Var of MX [IR2]	Entropy [IR3]	Mean Ent [IR3]	Var Ent [IR3]	MIN Ent [IR3]	MAX Ent [IR3]	Mean [IR3]	Var [IR3] 	MN W[I3]	MX W[I3]	Mean of MN [I3]	Mean of MX [I3]	Var of MN [IR3]	Var of MX [IR3]	\n#\n";


        this->WriteNewLog(SrcIP, SrcPort, DstIP, DstPort, Proto, ss.str());

    	this->LastTS.insert(std::pair<std::string, double>(Key,dTS));
    }

    EntropyContainer<int> * EntropyLengths = this->Lengths.at(Key);
    EntropyContainer<int> * EntropyLengthsRange1 = this->LengthsRange1.at(Key);
    EntropyContainer<int> * EntropyLengthsRange2 = this->LengthsRange2.at(Key);
    EntropyContainer<int> * EntropyLengthsRange3 = this->LengthsRange3.at(Key);
    EntropyContainer<double> * EntropyIAtimes = this->IAtimes.at(Key);
    EntropyContainer<double> * EntropyIAtimesRange1 = this->IAtimesRange1.at(Key);
    EntropyContainer<double> * EntropyIAtimesRange2 = this->IAtimesRange2.at(Key);
    EntropyContainer<double> * EntropyIAtimesRange3 = this->IAtimesRange3.at(Key);
    EntropyContainer<int> * EntropyTTLs = this->TTLs.at(Key);
    EntropyContainer<int> * EntropyTTLsRange1 = this->TTLsRange1.at(Key);
    EntropyContainer<int> * EntropyTTLsRange2 = this->TTLsRange2.at(Key);
    EntropyContainer<int> * EntropyTTLsRange3 = this->TTLsRange3.at(Key);

    double IAtime = this->Round(dTS - this->LastTS.at(Key), this->precision);
    this->LastTS.at(Key) = dTS;

    EntropyLengths->AddValue(Length);
	EntropyTTLs->AddValue(TTL);
	EntropyIAtimes->AddValue(IAtime);

    if (Length <= RANGES_MAX_1)
    {
		EntropyLengthsRange1->AddValue(Length);
		EntropyTTLsRange1->AddValue(TTL);
		EntropyIAtimesRange1->AddValue(IAtime);
    }
    else if (Length <= RANGES_MAX_2)
    {
		EntropyLengthsRange2->AddValue(Length);
		EntropyTTLsRange2->AddValue(TTL);
		EntropyIAtimesRange2->AddValue(IAtime);
    }
    else if (Length <= RANGES_MAX_3)
    {
		EntropyLengthsRange3->AddValue(Length);
		EntropyTTLsRange3->AddValue(TTL);
		EntropyIAtimesRange3->AddValue(IAtime);
    }

    (this->TotalPackets)++;
    this->cleanInactive(dTS);

    this->WriteEntropy(SrcIP, SrcPort, DstIP, DstPort, Proto,
    		           TSsec, TSusec,
    		           Length, IAtime, TTL,
    		           EntropyLengths, EntropyLengthsRange1, EntropyLengthsRange2, EntropyLengthsRange3,
    		           EntropyIAtimes, EntropyIAtimesRange1, EntropyIAtimesRange2, EntropyIAtimesRange3,
    		           EntropyTTLs, EntropyTTLsRange1, EntropyTTLsRange2, EntropyTTLsRange3);

}

unsigned long TupleAnalyzerExtended::TotHosts() const
{
    return this->TotalTuples;
}

unsigned long TupleAnalyzerExtended::TotPackets() const
{
    return this->TotalPackets;
}

const double TupleAnalyzerExtended::Round(const double &number, const int &precision) const
{
	double Aux, Result;
	int precFactor, intRes;

	if (precision == 0)
		return number;
	else if (precision > 0)
	{
		precFactor = 10;
		for (int i=1; i<precision; i++)
			precFactor *= 10;
	}
	else
		return number;

	Aux = number * (double) precFactor;
	intRes = (int) Aux;

	Result = intRes / (double) precFactor;

    return Result;
}

void TupleAnalyzerExtended::setCleanInactive(const bool &clean) {
	this->cleanIn = clean;
}

void TupleAnalyzerExtended::cleanInactive(const double &lastTS) {
	if (this->cleanIn)
	{
		if ((lastTS - this->lastClean) >= ENTROPY_CLEAN_INTERVAL)
		{
			std::map<std::string, double>::iterator it;

			for(it = this->LastTS.begin(); it != this->LastTS.end(); it++)
			{
				if ((lastTS - it->second) > ENTROPY_INACTIVE_TIMEOUT)
				{
					delete this->Lengths.at(it->first);
					delete this->LengthsRange1.at(it->first);
					delete this->LengthsRange2.at(it->first);
					delete this->LengthsRange3.at(it->first);
					delete this->IAtimes.at(it->first);
					delete this->IAtimesRange1.at(it->first);
					delete this->IAtimesRange2.at(it->first);
					delete this->IAtimesRange3.at(it->first);
					delete this->TTLs.at(it->first);
					delete this->TTLsRange1.at(it->first);
					delete this->TTLsRange2.at(it->first);
					delete this->TTLsRange3.at(it->first);
//					delete this->HostPorts.at(it->first);
//					delete this->HostPortsRange1.at(it->first);
//					delete this->HostPortsRange2.at(it->first);
//					delete this->HostPortsRange3.at(it->first);

					this->Lengths.erase(it->first);
					this->LengthsRange1.erase(it->first);
					this->LengthsRange2.erase(it->first);
					this->LengthsRange3.erase(it->first);
					this->IAtimes.erase(it->first);
					this->IAtimesRange1.erase(it->first);
					this->IAtimesRange2.erase(it->first);
					this->IAtimesRange3.erase(it->first);
					this->TTLs.erase(it->first);
					this->TTLsRange1.erase(it->first);
					this->TTLsRange2.erase(it->first);
					this->TTLsRange3.erase(it->first);
//					this->HostPorts.erase(it->first);
//					this->HostPortsRange1.erase(it->first);
//					this->HostPortsRange2.erase(it->first);
//					this->HostPortsRange3.erase(it->first);

					this->FirstTS.erase(it->first);
					this->LastTS.erase(it->first);
				}
			}
			this->lastClean = lastTS;
		}
	}
}

