/*
 * \author João V. Gomes
 *
 *  tupleanalyzerextended.h
 *
 *  Created on: Apr 12, 2011
 *
 * \brief TODO
 * TODO
 */

#ifndef TUPLEANALYZEREXTENDED_H_
#define TUPLEANALYZEREXTENDED_H_

#ifdef __cplusplus
extern "C"
{
#endif
#include <time.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include <string.h>
#ifdef __cplusplus
}
#endif

#include <entropycontainer.h>
#include <direxception.h>
#include <Folder.h>
#include <slidingwindow.h>

#include <map>
#include <string>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>

#ifndef ENTROPY_TIMEOUTS
#define ENTROPY_TIMEOUTS
#define ENTROPY_INACTIVE_TIMEOUT 64
#define ENTROPY_CLEAN_INTERVAL 300
#endif // TIMEOUTS

#ifndef RANGES_MAX
#define RANGES_MAX
#define RANGES_MAX_1 100
#define RANGES_MAX_2 900
#define RANGES_MAX_3 1600
#endif // RANGES_MAX

#ifndef DEFAULT_PRECISION
#define DEFAULT_PRECISION 6
#endif // DEFAULT_PRECISION

class TupleAnalyzerExtended
{
    public:
		TupleAnalyzerExtended(const int &WindowSize);
		TupleAnalyzerExtended(const int &WindowSize, const std::string &LogsPath);
		TupleAnalyzerExtended(const int &WindowSize, const std::string &LogsPath, const bool &Print);
		TupleAnalyzerExtended(const int &WindowSize, const int &precision);
		TupleAnalyzerExtended(const int &WindowSize, const int &precision, const std::string &LogsPath);
		TupleAnalyzerExtended(const int &WindowSize, const int &precision, const std::string &LogsPath, const bool &Print);
        virtual ~TupleAnalyzerExtended();
        void AddPacket(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto,
				      const int &Length, const int &TTL, const long &TSsec, const long &TSusec);
        unsigned long TotHosts() const;
        unsigned long TotPackets() const;
        void Clear();
        void setCleanInactive(const bool &clean);
    protected:
    private:
        // Tuple, Entropy container
        std::map<std::string, EntropyContainer<int> *> Lengths;
        std::map<std::string, EntropyContainer<int> *> LengthsRange1;
        std::map<std::string, EntropyContainer<int> *> LengthsRange2;
        std::map<std::string, EntropyContainer<int> *> LengthsRange3;
        std::map<std::string, EntropyContainer<double> *> IAtimes;
        std::map<std::string, EntropyContainer<double> *> IAtimesRange1;
        std::map<std::string, EntropyContainer<double> *> IAtimesRange2;
        std::map<std::string, EntropyContainer<double> *> IAtimesRange3;
        std::map<std::string, EntropyContainer<int> *> TTLs;
        std::map<std::string, EntropyContainer<int> *> TTLsRange1;
        std::map<std::string, EntropyContainer<int> *> TTLsRange2;
        std::map<std::string, EntropyContainer<int> *> TTLsRange3;
//        std::map<std::string, EntropyContainer<int> *> HostPorts;
//        std::map<std::string, EntropyContainer<int> *> HostPortsRange1;
//        std::map<std::string, EntropyContainer<int> *> HostPortsRange2;
//        std::map<std::string, EntropyContainer<int> *> HostPortsRange3;

        std::map<std::string, double> FirstTS;
        std::map<std::string, double> LastTS;
        unsigned long TotalTuples;
        unsigned long TotalPackets;
        int WinSize;
        Folder LogsFolder;
        bool PrintOutput;
        bool cleanIn;
        double lastClean;
        int precision;

        void init(const int &WindowSize, const int &precision, const std::string &Path, const bool &Print);
        void cleanInactive(const double &lastTS);
        void dispose();
        void WriteEntropy(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto,
						  const long &TSsec, const long &TSusec,
						  const int &Length, const double &IAtime, const int &TTL,
						  EntropyContainer<int> * EntropyLengths, EntropyContainer<int> * EntropyLengthsRange1, EntropyContainer<int> * EntropyLengthsRange2, EntropyContainer<int> * EntropyLengthsRange3,
						  EntropyContainer<double> * EntropyIAtimes, EntropyContainer<double> * EntropyIAtimesRange1, EntropyContainer<double> * EntropyIAtimesRange2, EntropyContainer<double> * EntropyIAtimesRange3,
						  EntropyContainer<int> * EntropyTTLs, EntropyContainer<int> * EntropyTTLsRange1, EntropyContainer<int> * EntropyTTLsRange2, EntropyContainer<int> * EntropyTTLsRange3);
        void WriteNewLog(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const std::string &Msg) const;
        void WriteExistentLog(const std::string &SrcIP, const int &SrcPort, const std::string &DstIP, const int &DstPort, const std::string &Proto, const std::string &Msg) const;
        const double Round(const double &number, const int &precision) const;
};

#endif /* TUPLEANALYZEREXTENDED_H_ */
