#include "packetprocessor.h"

PacketProcessor::PacketProcessor(PacketSource *PacketSrc)
{
    this->init(PacketSrc, false, "255.255.255.255", "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength)
{
    this->init(PacketSrc, OnlyPayloadLength, "255.255.255.255", "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const std::string &IPaddr)
{
    this->init(PacketSrc, false, IPaddr, "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddr)
{
    this->init(PacketSrc, OnlyPayloadLength, IPaddr, "255.255.255.255");
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const std::string &IPaddr, const std::string &Mask)
{
    this->init(PacketSrc, false, IPaddr, Mask);
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddr, const std::string &Mask)
{
    this->init(PacketSrc, OnlyPayloadLength, IPaddr, Mask);
}

PacketProcessor::~PacketProcessor()
{
	this->dispose();
}

void PacketProcessor::dispose() {

}

void PacketProcessor::init(PacketSource *PacketSrc, const bool &OnlyPayloadLength, const std::string &IPaddr, const std::string &Mask)
{
    this->ResetPacketSource(PacketSrc);

    this->PacketSizeProp = false;
    this->InterArrivalTimeProp = false;
    this->IPPortProp = false;

    this->AllTraffic = false;
    this->IncomingTraffic = false;
    this->OutgoingTraffic = false;

    this->FilterAck = false;
    this->FilterSyn = false;
    this->FilterFull = false;
    this->FilterTCPnoPayload = false;
    this->FilterNoTCP = false;
    this->FilterNoLess60 = false;
    this->FilterNoLess65 = false;
    this->FilterTCPPayloadLess5 = false;
    this->FilterPayloadLess5 = false;

    this->IPaddress = inet_addr(IPaddr.c_str());
    this->NetMask = inet_addr(Mask.c_str());

    this->OnlyPayloadLen = OnlyPayloadLength;
    this->Segmentation = false;
    this->SegmentationBase = 1;
}

void PacketProcessor::Clear()
{
    this->dispose();


}

void PacketProcessor::SetSegmentation(const bool &Seg, const int &Base)
{
	this->Segmentation = Seg;
	this->SegmentationBase = Base;
}

void PacketProcessor::SetPropertiesToAnalyze(const int &Prop)
{
	if (Prop == PACKET_SIZE)
		this->PacketSizeProp = true;
	else if (Prop == INTER_ARRIVAL_TIME)
		this->InterArrivalTimeProp = true;
	else if (Prop == IP_PORT)
		this->IPPortProp = true;
}

void PacketProcessor::UnsetPropertiesToAnalyze(const int &Prop)
{
	if (Prop == PACKET_SIZE)
		this->PacketSizeProp = false;
	else if (Prop == INTER_ARRIVAL_TIME)
		this->InterArrivalTimeProp = false;
	else if (Prop == IP_PORT)
		this->IPPortProp = false;
}

void PacketProcessor::SetDirections(const int &Direction)
{
	if (Direction == DIR_OUT)
		this->OutgoingTraffic = true;
	else if (Direction == DIR_IN)
		this->IncomingTraffic = true;
	else if (Direction == DIR_ALL)
		this->AllTraffic = true;
}

void PacketProcessor::UnsetDirections(const int &Direction)
{
	if (Direction == DIR_OUT)
		this->OutgoingTraffic = false;
	else if (Direction == DIR_IN)
		this->IncomingTraffic = false;
	else if (Direction == DIR_ALL)
		this->AllTraffic = false;
}

void PacketProcessor::SetFilters(const int &Filter)
{
	if (Filter == ACK_PCKT)
		this->FilterAck = true;
	else if (Filter == SYN_PCKT)
		this->FilterSyn = true;
	else if (Filter == FULL_PCKT)
		this->FilterFull = true;
	else if (Filter == NO_PAYLOAD)
		this->FilterTCPnoPayload = true;
	else if (Filter == NO_TCP)
		this->FilterNoTCP = true;
	else if (Filter == NO_LESS_60)
		this->FilterNoLess60 = true;
	else if (Filter == NO_LESS_65)
		this->FilterNoLess65 = true;
	else if (Filter == NO_TCP_PL_LESS_5)
		this->FilterTCPPayloadLess5 = true;
	else if (Filter == NO_PL_LESS_5)
		this->FilterPayloadLess5 = true;
}

void PacketProcessor::UnsetFilters(const int &Filter)
{
	if (Filter == ACK_PCKT)
		this->FilterAck = false;
	else if (Filter == SYN_PCKT)
		this->FilterSyn = false;
	else if (Filter == FULL_PCKT)
		this->FilterFull = false;
	else if (Filter == NO_PAYLOAD)
		this->FilterTCPnoPayload = false;
	else if (Filter == NO_TCP)
		this->FilterNoTCP = false;
	else if (Filter == NO_LESS_60)
		this->FilterNoLess60 = false;
	else if (Filter == NO_LESS_65)
		this->FilterNoLess65 = false;
	else if (Filter == NO_TCP_PL_LESS_5)
		this->FilterTCPPayloadLess5 = false;
	else if (Filter == NO_PL_LESS_5)
		this->FilterPayloadLess5 = false;
}

void PacketProcessor::ResetPacketSource(PacketSource *PacketSrc)
{
    if (PacketSrc->IsOpen())
        this->PcktSrc = PacketSrc;
    else
        throw PSrc_Not_Open();
}

const bool PacketProcessor::SelectHost(const std::string &IPaddr) const
{
	if (inet_addr("255.255.255.255") == this->IPaddress)
		return true;

	if ((this->NetMask & inet_addr(IPaddr.c_str())) == this->IPaddress)
		return true;

	return false;
}

bool const PacketProcessor::FilterPacket(const Packet * Pckt) const
{
	if (this->FilterNoTCP)
		if (Pckt->Protocol() == PKT_PTC_TCP)
			return true;

	if ( this->FilterFull )
		if (Pckt->Len() == 1514)
			return true;

	if ( this->FilterNoLess60 )
		if (Pckt->Len() <= 60)
			return true;

	if ( this->FilterNoLess65 )
		if (Pckt->Len() <= 65)
			return true;

	if ( Pckt->Protocol() == PKT_PTC_UDP )
	{
		const UDPPacket * UDPP = reinterpret_cast<UDPPacket const *>(Pckt);

		if ( this->FilterPayloadLess5 && (UDPP->UdpPayloadLen() <= 5) )
			return true;
	}

	if ( Pckt->Protocol() == PKT_PTC_TCP )
	{
		const TCPPacket * TCPP = reinterpret_cast<TCPPacket const *>(Pckt);

		if ( (this->FilterTCPPayloadLess5 || this->FilterPayloadLess5) &&
			 (TCPP->TcpPayloadLen() <= 5) )
			return true;

		if ( this->FilterSyn && TCPP->Syn() )
			return true;

		if ( this->FilterTCPnoPayload && (TCPP->TcpPayloadLen() == 0) )
			return true;

		if ( this->FilterAck &&
			 TCPP->Ack()     &&
			 (TCPP->TcpPayloadLen() == 0) )
//					if ( !(TCPP->Syn()) 	  &&
//						 !(TCPP->Rst())       &&
//						 !(TCPP->Fin())       &&
//						 !(TCPP->Urg())       &&
//						 !(TCPP->Psh())          )
			return true;
	}

	return false;
}

void PacketProcessor::AnalyzeHosts(const int &WinSize, const std::string &LogsPath)
{
	std::string Path = this->CleanLastBar(LogsPath);

	HostAnalyzer<int>         * psHAnOut  = 0, * psHAnIn  = 0, * psHAnAll  = 0;
	HostAnalyzer<double> 	  * iaHAnOut  = 0, * iaHAnIn  = 0, * iaHAnAll  = 0;
	HostAnalyzer<std::string> * ippHAnOut = 0, * ippHAnIn = 0, * ippHAnAll = 0;

    if (this->PacketSizeProp)
    {
    	if ( this->AllTraffic )
			psHAnAll = new HostAnalyzer<int>(WinSize, Path + PS_FOLDER + ALL_FOLDER);
    	if ( this->OutgoingTraffic )
    		psHAnOut = new HostAnalyzer<int>(WinSize, Path + PS_FOLDER + OUT_FOLDER);
    	if ( this->IncomingTraffic )
    		psHAnIn  = new HostAnalyzer<int>(WinSize, Path + PS_FOLDER +  IN_FOLDER);
    }

    if (this->InterArrivalTimeProp)
    {
    	if ( this->AllTraffic )
    		iaHAnAll = new HostAnalyzer<double>(WinSize, Path + IA_FOLDER + ALL_FOLDER);
    	if ( this->OutgoingTraffic )
    		iaHAnOut = new HostAnalyzer<double>(WinSize, Path + IA_FOLDER + OUT_FOLDER);
    	if ( this->IncomingTraffic )
    		iaHAnIn  = new HostAnalyzer<double>(WinSize, Path + IA_FOLDER +  IN_FOLDER);
    }

    if (this->IPPortProp)
    {
    	if ( this->AllTraffic )
    		ippHAnAll = new HostAnalyzer<std::string>(WinSize, Path + IPP_FOLDER + ALL_FOLDER);
    	if ( this->OutgoingTraffic )
    		ippHAnOut = new HostAnalyzer<std::string>(WinSize, Path + IPP_FOLDER + OUT_FOLDER);
    	if ( this->IncomingTraffic )
    		ippHAnIn  = new HostAnalyzer<std::string>(WinSize, Path + IPP_FOLDER +  IN_FOLDER);
    }

	this->ProcessHosts(WinSize, psHAnOut,  psHAnIn,  psHAnAll,
								iaHAnOut,  iaHAnIn,  iaHAnAll,
								ippHAnOut, ippHAnIn, ippHAnAll);

	delete psHAnOut;  delete psHAnIn;  delete psHAnAll;
	delete iaHAnOut;  delete iaHAnIn;  delete iaHAnAll;
	delete ippHAnOut; delete ippHAnIn; delete ippHAnAll;
}

void PacketProcessor::ClassifyHosts(const int &WinSize, const std::string &LogsPath, ThresholdsSet * TSet)
{
	std::string Path = this->CleanLastBar(LogsPath);

	HostAnalyzer<int> * psHAnOut  = 0, * psHAnIn = 0, * psHAnAll = 0;

    if (this->PacketSizeProp)
    {
    	if ( this->AllTraffic )
			psHAnAll = new HostAnalyzer<int>(WinSize, Path + PS_FOLDER + ALL_FOLDER, TSet);
    	if ( this->OutgoingTraffic )
    		psHAnOut = new HostAnalyzer<int>(WinSize, Path + PS_FOLDER + OUT_FOLDER, TSet);
    	if ( this->IncomingTraffic )
    		psHAnIn  = new HostAnalyzer<int>(WinSize, Path + PS_FOLDER +  IN_FOLDER, TSet);
    }

	this->ProcessHosts(WinSize, psHAnOut, psHAnIn, psHAnAll,
								0, 		  0, 	   0,
								0, 		  0, 	   0);

	delete psHAnOut; delete psHAnIn; delete psHAnAll;
}

void PacketProcessor::ProcessHosts(const int &WinSize, HostAnalyzer<int> * psHAnOut,
													   HostAnalyzer<int> * psHAnIn,
													   HostAnalyzer<int> * psHAnAll,
													   HostAnalyzer<double> * iaHAnOut,
													   HostAnalyzer<double> * iaHAnIn,
													   HostAnalyzer<double> * iaHAnAll,
													   HostAnalyzer<std::string> * ippHAnOut,
													   HostAnalyzer<std::string> * ippHAnIn,
													   HostAnalyzer<std::string> * ippHAnAll)
{
    bool end = false;

    while(!end)
        try
        {
            Packet *Pckt;

            Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();

            // Analyze only TCP and UDP packets
//            if ( (Proto != PKT_PTC_ETHERNET) && (Proto != PKT_PTC_NON_ETHERNET) && !(this->FilterPacket(Pckt)) )
            if ( ((Proto == PKT_PTC_TCP) || (Proto == PKT_PTC_UDP)) && !(this->FilterPacket(Pckt)) )
            {
                IPPacket *IPP = static_cast<IPPacket *>(Pckt);

                std::string const *src = IPP->SrcIPaddr(), *dst = IPP->DstIPaddr();
                long TSsec = IPP->TSseconds(), TSusec = IPP->TSmicrosec();

                if (this->PacketSizeProp)
                {
                    int len = IPP->Len();

                    if (this->OnlyPayloadLen)
					{
						if (Proto == PKT_PTC_UDP)
						{
							UDPPacket const *UDPP = reinterpret_cast<UDPPacket const *>(Pckt);
							len = UDPP->UdpPayloadLen();
						}
						else if (Proto == PKT_PTC_TCP)
						{
							TCPPacket const *TCPP = reinterpret_cast<TCPPacket const *>(Pckt);
							len = TCPP->TcpPayloadLen();
						}
					}

                    if (this->Segmentation)
                    	len = len / this->SegmentationBase;

                	if ( this->AllTraffic && (psHAnAll != 0) )
					{
						if (this->SelectHost(*src)) psHAnAll->AddValue(TSsec, TSusec, *src, len);
						if (this->SelectHost(*dst)) psHAnAll->AddValue(TSsec, TSusec, *dst, len);
					}
                	if ( this->OutgoingTraffic && (psHAnOut != 0) )
                		if (this->SelectHost(*src)) psHAnOut->AddValue(TSsec, TSusec, *src, len);
                	if ( this->IncomingTraffic && (psHAnIn != 0) )
                		if (this->SelectHost(*dst)) psHAnIn->AddValue(TSsec, TSusec, *dst, len);
                }

                if (this->InterArrivalTimeProp)
                {
                	double TStamp = TSsec + ( TSusec * 0.000001 );

                	int precision = 3;

                	if ( this->AllTraffic && (iaHAnAll != 0) )
					{
                		if (this->SelectHost(*src)) iaHAnAll->AddValueDif<double>(TSsec, TSusec, *src, TStamp, precision);
                		if (this->SelectHost(*dst)) iaHAnAll->AddValueDif<double>(TSsec, TSusec, *dst, TStamp, precision);
					}
                	if ( this->OutgoingTraffic && (iaHAnOut != 0) )
                		if (this->SelectHost(*src)) iaHAnOut->AddValueDif<double>(TSsec, TSusec, *src, TStamp, precision);
                	if ( this->IncomingTraffic && (iaHAnIn != 0) )
                		if (this->SelectHost(*dst)) iaHAnIn->AddValueDif<double>(TSsec, TSusec, *dst, TStamp, precision);
                }

                if (this->IPPortProp && ((Proto == PKT_PTC_TCP) || (Proto == PKT_PTC_UDP)) && ( this->SelectHost(*src)  || this->SelectHost(*dst) ) )
                {
                	int sPort, dPort;

                	if (Proto == PKT_PTC_TCP)
                	{
						TCPPacket * TCPP = static_cast<TCPPacket *>(Pckt);
						sPort = TCPP->SrcPort();
						dPort = TCPP->DstPort();
                	}
                	else if (Proto == PKT_PTC_UDP)
                	{
                		UDPPacket * UDPP = static_cast<UDPPacket *>(Pckt);
						sPort = UDPP->SrcPort();
						dPort = UDPP->DstPort();
                	}

                	std::stringstream ssSrc (std::stringstream::out), ssDst (std::stringstream::out);
					ssSrc << (*dst) << ":" << dPort;
					ssDst << (*src) << ":" << sPort;

                	if ( this->AllTraffic && (ippHAnAll != 0) )
					{
                		if (this->SelectHost(*src)) ippHAnAll->AddValue(TSsec, TSusec, *src, ssSrc.str());
                		if (this->SelectHost(*dst)) ippHAnAll->AddValue(TSsec, TSusec, *dst, ssDst.str());
					}
                	if ( this->OutgoingTraffic && (ippHAnOut != 0) )
                		if (this->SelectHost(*src)) ippHAnOut->AddValue(TSsec, TSusec, *src, ssSrc.str());
                	if ( this->IncomingTraffic && (ippHAnIn != 0) )
                		if (this->SelectHost(*dst)) ippHAnIn->AddValue(TSsec, TSusec, *dst, ssDst.str());

                }
            }

            delete Pckt;
        }
        catch (No_More_Packets e1)
        {
            end = true;
        }
        catch (Timeout_Expired e2)
        {
            std::cout << "Timeout: " << e2.what() << "\n";
            end = true;
        }
        catch (Packet_Reading_Error e3)
        {
            std::cout << "Failed: " << e3.what() << "\n";
            end = true;
        }
}

void PacketProcessor::AnalyzeHostsPorts(const int &WinSize, const std::string &LogsPath)
{
	std::string Path = this->CleanLastBar(LogsPath);

	HostPortAnalyzer<int>         * psHPAnOut  = 0, * psHPAnIn  = 0, * psHPAnAll  = 0;
	HostPortAnalyzer<double> 	  * iaHPAnOut  = 0, * iaHPAnIn  = 0, * iaHPAnAll  = 0;
	HostPortAnalyzer<std::string> * ippHPAnOut = 0, * ippHPAnIn = 0, * ippHPAnAll = 0;

    if (this->PacketSizeProp)
    {
    	if ( this->AllTraffic )
			psHPAnAll = new HostPortAnalyzer<int>(WinSize, Path + PS_FOLDER + ALL_FOLDER);
    	if ( this->OutgoingTraffic )
    		psHPAnOut = new HostPortAnalyzer<int>(WinSize, Path + PS_FOLDER + OUT_FOLDER);
    	if ( this->IncomingTraffic )
    		psHPAnIn  = new HostPortAnalyzer<int>(WinSize, Path + PS_FOLDER +  IN_FOLDER);
    }

    if (this->InterArrivalTimeProp)
    {
    	if ( this->AllTraffic )
    		iaHPAnAll = new HostPortAnalyzer<double>(WinSize, Path + IA_FOLDER + ALL_FOLDER);
    	if ( this->OutgoingTraffic )
    		iaHPAnOut = new HostPortAnalyzer<double>(WinSize, Path + IA_FOLDER + OUT_FOLDER);
    	if ( this->IncomingTraffic )
    		iaHPAnIn  = new HostPortAnalyzer<double>(WinSize, Path + IA_FOLDER +  IN_FOLDER);
    }

    if (this->IPPortProp)
    {
    	if ( this->AllTraffic )
    		ippHPAnAll = new HostPortAnalyzer<std::string>(WinSize, Path + IPP_FOLDER + ALL_FOLDER);
    	if ( this->OutgoingTraffic )
    		ippHPAnOut = new HostPortAnalyzer<std::string>(WinSize, Path + IPP_FOLDER + OUT_FOLDER);
    	if ( this->IncomingTraffic )
    		ippHPAnIn  = new HostPortAnalyzer<std::string>(WinSize, Path + IPP_FOLDER +  IN_FOLDER);
    }

	this->ProcessHostsPorts(WinSize, psHPAnOut,  psHPAnIn,  psHPAnAll,
									 iaHPAnOut,  iaHPAnIn,  iaHPAnAll,
									 ippHPAnOut, ippHPAnIn, ippHPAnAll);

	delete psHPAnOut;  delete psHPAnIn;  delete psHPAnAll;
	delete iaHPAnOut;  delete iaHPAnIn;  delete iaHPAnAll;
	delete ippHPAnOut; delete ippHPAnIn; delete ippHPAnAll;
}

void PacketProcessor::ClassifyHostsPorts(const int &WinSize, const std::string &LogsPath, ThresholdsSet * TSet)
{
	std::string Path = this->CleanLastBar(LogsPath);
	HostPortAnalyzer<int> * psHPAnOut  = 0, * psHPAnIn = 0, * psHPAnAll = 0;

    if (this->PacketSizeProp)
    {
    	if ( this->AllTraffic )
			psHPAnAll = new HostPortAnalyzer<int>(WinSize, Path + PS_FOLDER + ALL_FOLDER, TSet);
    	if ( this->OutgoingTraffic )
    		psHPAnOut = new HostPortAnalyzer<int>(WinSize, Path + PS_FOLDER + OUT_FOLDER, TSet);
    	if ( this->IncomingTraffic )
    		psHPAnIn  = new HostPortAnalyzer<int>(WinSize, Path + PS_FOLDER +  IN_FOLDER, TSet);
    }

	this->ProcessHostsPorts(WinSize, psHPAnOut, psHPAnIn, psHPAnAll,
									 0, 	    0,		  0,
									 0, 	    0,		  0);

	delete psHPAnOut; delete psHPAnIn; delete psHPAnAll;
}

void PacketProcessor::ProcessHostsPorts(const int &WinSize, HostPortAnalyzer<int> * psHPAnOut,
															HostPortAnalyzer<int> * psHPAnIn,
															HostPortAnalyzer<int> * psHPAnAll,
															HostPortAnalyzer<double> * iaHPAnOut,
															HostPortAnalyzer<double> * iaHPAnIn,
															HostPortAnalyzer<double> * iaHPAnAll,
															HostPortAnalyzer<std::string> * ippHPAnOut,
															HostPortAnalyzer<std::string> * ippHPAnIn,
															HostPortAnalyzer<std::string> * ippHPAnAll)
{
    bool end = false;

    while(!end)
        try
        {
            Packet *Pckt;

            Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();
            int srcP, dstP;
            std::string src, dst;

            if ( ( (Proto == PKT_PTC_UDP) || (Proto == PKT_PTC_TCP) ) && !(this->FilterPacket(Pckt)) )
            {
				int len = Pckt->Len();

				if (Proto == PKT_PTC_UDP)
				{
					UDPPacket const *UDPP = reinterpret_cast<UDPPacket const *>(Pckt);
					src = *(UDPP->SrcIPaddr());
					dst = *(UDPP->DstIPaddr());
					srcP = UDPP->SrcPort();
					dstP = UDPP->DstPort();

					if (this->OnlyPayloadLen)
						len = UDPP->UdpPayloadLen();

                    if (this->Segmentation)
                    	len = len / this->SegmentationBase;
				}
				else if (Proto == PKT_PTC_TCP)
				{
					TCPPacket *TCPP = reinterpret_cast<TCPPacket * const>(Pckt);
					src = *(TCPP->SrcIPaddr());
					dst = *(TCPP->DstIPaddr());
					srcP = TCPP->SrcPort();
					dstP = TCPP->DstPort();

					if (this->OnlyPayloadLen)
						len = TCPP->TcpPayloadLen();

                    if (this->Segmentation)
                    	len = len / this->SegmentationBase;
				}

				long TSsec = Pckt->TSseconds(), TSusec = Pckt->TSmicrosec();

				if (this->PacketSizeProp && ( this->SelectHost(src) || this->SelectHost(dst) ) )
				{
					if ( this->AllTraffic && (psHPAnAll != 0) )
					{
						psHPAnAll->AddValue(TSsec, TSusec, src, srcP, len);
						psHPAnAll->AddValue(TSsec, TSusec, dst, dstP, len);
					}
					if ( this->OutgoingTraffic && (psHPAnOut != 0) )
						psHPAnOut->AddValue(TSsec, TSusec, src, srcP, len);
					if ( this->IncomingTraffic && (psHPAnIn != 0) )
						psHPAnIn->AddValue(TSsec, TSusec, dst, dstP, len);
				}

				if (this->InterArrivalTimeProp && ( this->SelectHost(src) || this->SelectHost(dst) ) )
				{
					double TStamp = TSsec + ( TSusec * 0.000001 );

					int precision = 3;

					if ( this->AllTraffic && (iaHPAnAll != 0) )
					{
						iaHPAnAll->AddValueDif<double>(TSsec, TSusec, src, srcP, TStamp, precision);
						iaHPAnAll->AddValueDif<double>(TSsec, TSusec, dst, dstP, TStamp, precision);
					}
					if ( this->OutgoingTraffic && (iaHPAnOut != 0) )
						iaHPAnOut->AddValueDif<double>(TSsec, TSusec, src, srcP, TStamp, precision);
					if ( this->IncomingTraffic && (iaHPAnIn != 0) )
						iaHPAnIn->AddValueDif<double>(TSsec, TSusec, dst, dstP, TStamp, precision);
				}

				if (this->IPPortProp && ( this->SelectHost(src) || this->SelectHost(dst) ) )
				{
					std::stringstream ssSrc (std::stringstream::out), ssDst (std::stringstream::out);
					ssSrc << dst << ":" << dstP;
					ssDst << src << ":" << srcP;

					if ( this->AllTraffic && (ippHPAnAll != 0) )
					{
						ippHPAnAll->AddValue(TSsec, TSusec, src, srcP, ssSrc.str());
						ippHPAnAll->AddValue(TSsec, TSusec, dst, dstP, ssDst.str());
					}
					if ( this->OutgoingTraffic && (ippHPAnOut != 0) )
						ippHPAnOut->AddValue(TSsec, TSusec, src, srcP, ssSrc.str());
					if ( this->IncomingTraffic && (ippHPAnIn != 0) )
						ippHPAnIn->AddValue(TSsec, TSusec, dst, dstP, ssDst.str());
				}
            }

            delete Pckt;
        }
        catch (No_More_Packets e1)
        {
            end = true;
        }
        catch (Timeout_Expired e2)
        {
            std::cout << "Timeout: " << e2.what() << "\n";
            end = true;
        }
        catch (Packet_Reading_Error e3)
        {
            std::cout << "Failed: " << e3.what() << "\n";
            end = true;
        }
}

void PacketProcessor::AnalyzeFlows(const int &WinSize, const std::string &LogsPath)
{
	std::string Path = this->CleanLastBar(LogsPath);

	FlowAnalyzer<int>    * psFAnAll = 0;
	FlowAnalyzer<double> * iaFAnAll = 0;

    if (this->PacketSizeProp)
    	psFAnAll = new FlowAnalyzer<int>(WinSize, Path + PS_FOLDER + ALL_FOLDER);

    if (this->InterArrivalTimeProp)
    	iaFAnAll = new FlowAnalyzer<double>(WinSize, Path + IA_FOLDER + ALL_FOLDER);

	this->ProcessFlows(WinSize, psFAnAll,  iaFAnAll);

	delete psFAnAll;
	delete iaFAnAll;
}

void PacketProcessor::ClassifyFlows(const int &WinSize, const std::string &LogsPath, ThresholdsSet * TSet)
{
	std::string Path = this->CleanLastBar(LogsPath);
	FlowAnalyzer<int> * psFAnAll = 0;

    if (this->PacketSizeProp)
    	psFAnAll = new FlowAnalyzer<int>(WinSize, Path + PS_FOLDER + ALL_FOLDER);

	this->ProcessFlows(WinSize, psFAnAll, 0);

	delete psFAnAll;
}

void PacketProcessor::ProcessFlows(const int &WinSize, FlowAnalyzer<int> * psFAnAll, FlowAnalyzer<double> * iaFAnAll)
{
    bool end = false;

    while(!end)
        try
        {
            const Packet *Pckt;

            Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();

            if ( ((Proto == PKT_PTC_UDP) || (Proto == PKT_PTC_TCP)) && !(this->FilterPacket(Pckt)) )
            {
				if ( this->PacketSizeProp && (psFAnAll != 0)) // && ( this->SelectHost(*src)  || this->SelectHost(*dst) ) )
				{
                    int len = Pckt->Len();

                    if (this->OnlyPayloadLen)
					{
						if (Proto == PKT_PTC_UDP)
						{
							UDPPacket const *UDPP = reinterpret_cast<UDPPacket const *>(Pckt);
							len = UDPP->UdpPayloadLen();
						}
						else if (Proto == PKT_PTC_TCP)
						{
							TCPPacket const *TCPP = reinterpret_cast<TCPPacket const *>(Pckt);
							len = TCPP->TcpPayloadLen();
						}
					}

                    if (this->Segmentation)
                    	len = len / this->SegmentationBase;

					psFAnAll->AddValue(Pckt, len);
				}

				if ( this->InterArrivalTimeProp && (iaFAnAll != 0)) // && ( this->SelectHost(*src)  || this->SelectHost(*dst) ) )
				{
					double TStamp = Pckt->TSseconds() + ( Pckt->TSmicrosec() * 0.000001 );
					int const precision = 4;

					iaFAnAll->AddValueDif(Pckt, TStamp, precision);
				}
            }

            delete Pckt;
        }
        catch (No_More_Packets e1)
        {
            end = true;
        }
        catch (Timeout_Expired e2)
        {
            std::cout << "Timeout: " << e2.what() << "\n";
            end = true;
        }
        catch (Packet_Reading_Error e3)
        {
            std::cout << "Failed: " << e3.what() << "\n";
            end = true;
        }
}

void PacketProcessor::AnalyzeHostsExtended(const int &WinSize, const std::string &LogsPath)
{
	this->AnalyzeHostsExtended(WinSize, PP_IAT_PRECISION, LogsPath);
}

void PacketProcessor::AnalyzeHostsExtended(const int &WinSize, const int &Precision, const std::string &LogsPath)
{
    bool end = false;

	std::string Path = this->CleanLastBar(LogsPath);

	HostAnalyzerExtended HAnALL(WinSize, Precision, Path + EX_FOLDER + ALL_FOLDER);
	HostAnalyzerExtended HAnIN (WinSize, Precision, Path + EX_FOLDER + IN_FOLDER );
	HostAnalyzerExtended HAnOUT(WinSize, Precision, Path + EX_FOLDER + OUT_FOLDER);

    while(!end)
        try
        {
            Packet *Pckt;

            Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();
            int srcP, dstP;
            std::string src, dst, strProto;

            if ( ( (Proto == PKT_PTC_UDP) || (Proto == PKT_PTC_TCP) ) && !(this->FilterPacket(Pckt)) )
            {
				int len = Pckt->Len();
				long TSsec = Pckt->TSseconds(), TSusec = Pckt->TSmicrosec();

				if (Proto == PKT_PTC_UDP)
				{
					UDPPacket const *UDPP = reinterpret_cast<UDPPacket const *>(Pckt);
					src = *(UDPP->SrcIPaddr());
					dst = *(UDPP->DstIPaddr());
					srcP = UDPP->SrcPort();
					dstP = UDPP->DstPort();

					int TTL = UDPP->TTL();

					if (this->OnlyPayloadLen)
						len = UDPP->UdpPayloadLen();

                    if (this->Segmentation)
                    	len = len / this->SegmentationBase;

                	if ( this->AllTraffic )
					{
                		HAnALL.AddPacket(src, dst, dstP, "UDP", len, TTL, TSsec, TSusec);
                		HAnALL.AddPacket(dst, src, srcP, "UDP", len, TTL, TSsec, TSusec);
					}
                	if ( this->OutgoingTraffic )
                		HAnOUT.AddPacket(src, dst, dstP, "UDP", len, TTL, TSsec, TSusec);
                	if ( this->IncomingTraffic )
                		HAnIN.AddPacket(dst, src, srcP, "UDP", len, TTL, TSsec, TSusec);

				}
				else if (Proto == PKT_PTC_TCP)
				{
					TCPPacket *TCPP = reinterpret_cast<TCPPacket * const>(Pckt);
					src = *(TCPP->SrcIPaddr());
					dst = *(TCPP->DstIPaddr());
					srcP = TCPP->SrcPort();
					dstP = TCPP->DstPort();

					int TTL = TCPP->TTL();

					if (this->OnlyPayloadLen)
						len = TCPP->TcpPayloadLen();

                    if (this->Segmentation)
                    	len = len / this->SegmentationBase;

                	if ( this->AllTraffic )
					{
                		HAnALL.AddPacket(src, dst, dstP, "TCP", len, TTL, TSsec, TSusec);
                		HAnALL.AddPacket(dst, src, srcP, "TCP", len, TTL, TSsec, TSusec);
					}
                	if ( this->OutgoingTraffic )
                		HAnOUT.AddPacket(src, dst, dstP, "TCP", len, TTL, TSsec, TSusec);
                	if ( this->IncomingTraffic )
                		HAnIN.AddPacket(dst, src, srcP, "TCP", len, TTL, TSsec, TSusec);
				}
            }

            delete Pckt;
        }
        catch (No_More_Packets e1)
        {
            end = true;
        }
        catch (Timeout_Expired e2)
        {
            std::cout << "Timeout: " << e2.what() << "\n";
            end = true;
        }
        catch (Packet_Reading_Error e3)
        {
            std::cout << "Failed: " << e3.what() << "\n";
            end = true;
        }
}

void PacketProcessor::AnalyzeHostPortsExtended(const int &WinSize, const std::string &LogsBase, const std::string &LogsComplement)
{
	this->AnalyzeHostPortsExtended(WinSize, PP_IAT_PRECISION, LogsBase, LogsComplement, true, true);
}

void PacketProcessor::AnalyzeHostPortsExtended(const int &WinSize, const int &Precision,
											   const std::string &LogsBase, const std::string &LogsComplement,
											   const bool &PrintSummary, const bool &PrintLogs)
{
    bool end = false;

	std::string LogsComp = this->CleanLastBar(LogsComplement);

	HostPortAnalyzerExtended HPAnALL(WinSize, Precision, LogsBase, LogsComp + EX_FOLDER + ALL_FOLDER, PrintSummary, PrintLogs);
	HostPortAnalyzerExtended HPAnIN (WinSize, Precision, LogsBase, LogsComp + EX_FOLDER + IN_FOLDER , PrintSummary, PrintLogs);
	HostPortAnalyzerExtended HPAnOUT(WinSize, Precision, LogsBase, LogsComp + EX_FOLDER + OUT_FOLDER, PrintSummary, PrintLogs);

    while(!end)
        try
        {
            Packet *Pckt;

            Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();
            int srcP, dstP;
            std::string src, dst, strProto;

            if ( ( (Proto == PKT_PTC_UDP) || (Proto == PKT_PTC_TCP) ) && !(this->FilterPacket(Pckt)) )
            {
				int len = Pckt->Len();
				long TSsec = Pckt->TSseconds(), TSusec = Pckt->TSmicrosec();

				if (Proto == PKT_PTC_UDP)
				{
					UDPPacket const *UDPP = reinterpret_cast<UDPPacket const *>(Pckt);
					src = *(UDPP->SrcIPaddr());
					dst = *(UDPP->DstIPaddr());
					srcP = UDPP->SrcPort();
					dstP = UDPP->DstPort();

					int TTL = UDPP->TTL();

					if (this->OnlyPayloadLen)
						len = UDPP->UdpPayloadLen();

                    if (this->Segmentation)
                    	len = len / this->SegmentationBase;

                	if ( this->AllTraffic )
					{
                		if (this->SelectHost(src)) HPAnALL.AddPacket(src, srcP, dst, dstP, "UDP", len, TTL, TSsec, TSusec);
                		if (this->SelectHost(dst)) HPAnALL.AddPacket(dst, dstP, src, srcP, "UDP", len, TTL, TSsec, TSusec);
					}
                	if ( this->OutgoingTraffic )
                		if (this->SelectHost(src)) HPAnOUT.AddPacket(src, srcP, dst, dstP, "UDP", len, TTL, TSsec, TSusec);
                	if ( this->IncomingTraffic )
                		if (this->SelectHost(dst)) HPAnIN.AddPacket(dst, dstP, src, srcP, "UDP", len, TTL, TSsec, TSusec);
				}
				else if (Proto == PKT_PTC_TCP)
				{
					TCPPacket *TCPP = reinterpret_cast<TCPPacket * const>(Pckt);
					src = *(TCPP->SrcIPaddr());
					dst = *(TCPP->DstIPaddr());
					srcP = TCPP->SrcPort();
					dstP = TCPP->DstPort();

					int TTL = TCPP->TTL();

					if (this->OnlyPayloadLen)
						len = TCPP->TcpPayloadLen();

                    if (this->Segmentation)
                    	len = len / this->SegmentationBase;

                	if ( this->AllTraffic )
					{
                		if (this->SelectHost(src)) HPAnALL.AddPacket(src, srcP, dst, dstP, "TCP", len, TTL, TSsec, TSusec);
                		if (this->SelectHost(dst)) HPAnALL.AddPacket(dst, dstP, src, srcP, "TCP", len, TTL, TSsec, TSusec);
					}
                	if ( this->OutgoingTraffic )
                		if (this->SelectHost(src)) HPAnOUT.AddPacket(src, srcP, dst, dstP, "TCP", len, TTL, TSsec, TSusec);
                	if ( this->IncomingTraffic )
                		if (this->SelectHost(dst)) HPAnIN.AddPacket(dst, dstP, src, srcP, "TCP", len, TTL, TSsec, TSusec);
				}
            }

            delete Pckt;
        }
        catch (No_More_Packets e1)
        {
            end = true;
        }
        catch (Timeout_Expired e2)
        {
            std::cout << "Timeout: " << e2.what() << "\n";
            end = true;
        }
        catch (Packet_Reading_Error e3)
        {
            std::cout << "Failed: " << e3.what() << "\n";
            end = true;
        }
        catch (std::runtime_error e4)
        {
        	std::cout << "Error: " << e4.what() << "\n";
            end = true;
        }
}

void PacketProcessor::AnalyzeTuplesExtended(const int &WinSize, const std::string &LogsPath)
{
	this->AnalyzeTuplesExtended(WinSize, PP_IAT_PRECISION, LogsPath);
}

void PacketProcessor::AnalyzeTuplesExtended(const int &WinSize, const int &Precision, const std::string &LogsPath)
{
    bool end = false;

	std::string Path = this->CleanLastBar(LogsPath);

	TupleAnalyzerExtended TAnALL(WinSize, Precision, Path + EX_FOLDER + ALL_FOLDER);
	TupleAnalyzerExtended TAnIN (WinSize, Precision, Path + EX_FOLDER + IN_FOLDER );
	TupleAnalyzerExtended TAnOUT(WinSize, Precision, Path + EX_FOLDER + OUT_FOLDER);

    while(!end)
        try
        {
            Packet *Pckt;

            Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();
            int srcP, dstP;
            std::string src, dst, strProto;

            if ( ( (Proto == PKT_PTC_UDP) || (Proto == PKT_PTC_TCP) ) && !(this->FilterPacket(Pckt)) )
            {
				int len = Pckt->Len();
				long TSsec = Pckt->TSseconds(), TSusec = Pckt->TSmicrosec();

				if (Proto == PKT_PTC_UDP)
				{
					UDPPacket const *UDPP = reinterpret_cast<UDPPacket const *>(Pckt);
					src = *(UDPP->SrcIPaddr());
					dst = *(UDPP->DstIPaddr());
					srcP = UDPP->SrcPort();
					dstP = UDPP->DstPort();

					int TTL = UDPP->TTL();

					if (this->OnlyPayloadLen)
						len = UDPP->UdpPayloadLen();

                    if (this->Segmentation)
                    	len = len / this->SegmentationBase;

                	if ( this->AllTraffic )
					{
                		TAnALL.AddPacket(src, srcP, dst, dstP, "UDP", len, TTL, TSsec, TSusec);
                		TAnALL.AddPacket(dst, dstP, src, srcP, "UDP", len, TTL, TSsec, TSusec);
					}
                	if ( this->OutgoingTraffic )
                		TAnOUT.AddPacket(src, srcP, dst, dstP, "UDP", len, TTL, TSsec, TSusec);
                	if ( this->IncomingTraffic )
                		TAnIN.AddPacket(dst, dstP, src, srcP, "UDP", len, TTL, TSsec, TSusec);

				}
				else if (Proto == PKT_PTC_TCP)
				{
					TCPPacket *TCPP = reinterpret_cast<TCPPacket * const>(Pckt);
					src = *(TCPP->SrcIPaddr());
					dst = *(TCPP->DstIPaddr());
					srcP = TCPP->SrcPort();
					dstP = TCPP->DstPort();

					int TTL = TCPP->TTL();

					if (this->OnlyPayloadLen)
						len = TCPP->TcpPayloadLen();

                    if (this->Segmentation)
                    	len = len / this->SegmentationBase;

                	if ( this->AllTraffic )
					{
                		TAnALL.AddPacket(src, srcP, dst, dstP, "TCP", len, TTL, TSsec, TSusec);
                		TAnALL.AddPacket(dst, dstP, src, srcP, "TCP", len, TTL, TSsec, TSusec);
					}
                	if ( this->OutgoingTraffic )
                		TAnOUT.AddPacket(src, srcP, dst, dstP, "TCP", len, TTL, TSsec, TSusec);
                	if ( this->IncomingTraffic )
                		TAnIN.AddPacket(dst, dstP, src, srcP, "TCP", len, TTL, TSsec, TSusec);
				}
            }

            delete Pckt;
        }
        catch (No_More_Packets e1)
        {
            end = true;
        }
        catch (Timeout_Expired e2)
        {
            std::cout << "Timeout: " << e2.what() << "\n";
            end = true;
        }
        catch (Packet_Reading_Error e3)
        {
            std::cout << "Failed: " << e3.what() << "\n";
            end = true;
        }
}

void PacketProcessor::AnalyzeFlowsExtended(const int &WinSize, const std::string &LogsBase, const std::string &LogsComplement)
{
	this->AnalyzeFlowsExtended(WinSize, PP_IAT_PRECISION, LogsBase, LogsComplement, true, true);
}

void PacketProcessor::AnalyzeFlowsExtended(const int &WinSize, const int &Precision,
										   const std::string &LogsBase, const std::string &LogsComplement,
										   const bool &PrintSummary, const bool &PrintLogs)
{
    bool end = false;

	std::string LogsComp = this->CleanLastBar(LogsComplement);

	FlowAnalyzerExtended FAnALL(WinSize, Precision, LogsBase, LogsComp + EX_FOLDER + ALL_FOLDER, PrintSummary, PrintLogs);
	FlowAnalyzerExtended FAnIN (WinSize, Precision, LogsBase, LogsComp + EX_FOLDER + IN_FOLDER , PrintSummary, PrintLogs);
	FlowAnalyzerExtended FAnOUT(WinSize, Precision, LogsBase, LogsComp + EX_FOLDER + OUT_FOLDER, PrintSummary, PrintLogs);

    while(!end)
        try
        {
            Packet *Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();

            IPPacket const *IPP = reinterpret_cast<IPPacket const *>(Pckt);

            const std::string *src = IPP->SrcIPaddr();
            const std::string *dst = IPP->DstIPaddr();

            if ( ( (Proto == PKT_PTC_UDP) || (Proto == PKT_PTC_TCP) ) && !(this->FilterPacket(Pckt)) )
            {
            	if ( this->AllTraffic      && (    this->SelectHost(*src)  ||    this->SelectHost(*dst)  ) )
            		FAnALL.AddPacket(Pckt, this->OnlyPayloadLen);

            	if ( this->OutgoingTraffic && (    this->SelectHost(*src)  || (! this->SelectHost(*dst)) ) )
            		FAnOUT.AddPacket(Pckt, this->OnlyPayloadLen);

            	if ( this->IncomingTraffic && ( (! this->SelectHost(*src)) ||    this->SelectHost(*dst)  ) )
            		FAnIN.AddPacket(Pckt, this->OnlyPayloadLen);
            }

            delete Pckt;
        }
        catch (No_More_Packets e1)
        {
            end = true;
        }
        catch (Timeout_Expired e2)
        {
            std::cout << "Timeout: " << e2.what() << "\n";
            end = true;
        }
        catch (Packet_Reading_Error e3)
        {
            std::cout << "Failed: " << e3.what() << "\n";
            end = true;
        }
}

const std::string PacketProcessor::CleanLastBar(const std::string &Path) const
{
	std::string PathStr = Path;

	while ( *(PathStr.end() - 1) == ('/') )
		PathStr.erase(PathStr.end() - 1);

	return PathStr;
}



//void PacketProcessor::SetPropertyInterArrivalTime()
//{
//	this->InterArrivalTimeProp = true;
//}
//
//void PacketProcessor::SetPropertyPacketSize()
//{
//	this->PacketSizeProp = true;
//}
//
//void PacketProcessor::SetPropertyIPPort()
//{
//	this->IPPortProp = true;
//}
//
//void PacketProcessor::UnsetPropertyInterArrivalTime()
//{
//	this->InterArrivalTimeProp = false;
//}
//
//void PacketProcessor::UnsetPropertyPacketSize()
//{
//	this->PacketSizeProp = false;
//}
//
//void PacketProcessor::UnsetPropertyIPPort()
//{
//	this->IPPortProp = false;
//}


//void SetPropertyInterArrivalTime();
//void SetPropertyPacketSize();
//void SetPropertyIPPort();
//void UnsetPropertyInterArrivalTime();
//void UnsetPropertyPacketSize();
//void UnsetPropertyIPPort();


//void PacketProcessor::SetDirectionAll()
//{
//	this->AllTraffic = true;
//}
//
//void PacketProcessor::SetDirectionIncoming()
//{
//	this->IncomingTraffic = true;
//}
//
//void PacketProcessor::SetDirectionOutgoing()
//{
//	this->OutgoingTraffic = true;
//}
//
//void PacketProcessor::UnsetDirectionAll()
//{
//	this->AllTraffic = false;
//}
//
//void PacketProcessor::UnsetDirectionIncoming()
//{
//	this->IncomingTraffic = false;
//}
//
//void PacketProcessor::UnsetDirectionOutgoing()
//{
//	this->OutgoingTraffic = false;
//}


//void SetDirectionOutgoing();
//void SetDirectionIncoming();
//void SetDirectionAll();
//void UnsetDirectionOutgoing();
//void UnsetDirectionIncoming();
//void UnsetDirectionAll();
