/*
 * \author João V. Gomes
 *
 *  alarmraiser.cpp
 *
 *  Created on: Dec 28, 2010
 *
 * \brief TODO
 * TODO
 */

#include "alarmraiser.h"

AlarmRaiser::AlarmRaiser() {

}

AlarmRaiser::AlarmRaiser(const int &maxOccurences, const int &limit, const int &Action) {
	this->init(maxOccurences, limit, Action, VERB_NORMAL, false);
}

AlarmRaiser::AlarmRaiser(const int &maxOccurences, const int &limit, const int &Action, const int &VerboseLevel) {
	this->init(maxOccurences, limit, Action, VerboseLevel, false);
}

AlarmRaiser::AlarmRaiser(const int &maxOccurences, const int &limit, const int &Action, const bool &CleanInactive) {
	this->init(maxOccurences, limit, Action, VERB_NORMAL, CleanInactive);
}

AlarmRaiser::AlarmRaiser(const int &maxOccurences, const int &limit, const int &Action, const int &VerboseLevel, const bool &CleanInactive) {
	this->init(maxOccurences, limit, Action, VerboseLevel, CleanInactive);
}

AlarmRaiser::~AlarmRaiser() {
	this->dispose();
}

void AlarmRaiser::init(const int &maxOccurences, const int &limit, const int &Action, const int &VerboseLevel, const bool &CleanInactive) {
	this->maxOccur = maxOccurences;
	this->lim = limit;
	this->VerbLevel = VerboseLevel;
	this->CleanIn = CleanInactive;
	this->AlarmAction = Action;
	this->LastLength = -1;
	this->LastEntropy = 0;
	this->LastClean = 0;
}

void AlarmRaiser::dispose() {
    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator it;
    std::map<VoIPPattern *, std::set<std::string> *>::iterator itAlarm;

    for(it = this->VoIPPatternsIN.begin(); it != this->VoIPPatternsIN.end(); it++)
        delete (it->second);

    for(it = this->VoIPPatternsOUT.begin(); it != this->VoIPPatternsOUT.end(); it++)
        delete (it->second);

    for(itAlarm = this->AlarmsON.begin(); itAlarm != this->AlarmsON.end(); itAlarm++)
    {
        delete (itAlarm->second);
        delete (itAlarm->first);
    }
}

void AlarmRaiser::CleanOccurrences() {
    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator it;
    std::map<VoIPPattern *, std::set<std::string> *>::iterator itAlarm;

    for(it = this->VoIPPatternsIN.begin(); it != this->VoIPPatternsIN.end(); it++)
    	it->second->clear();

    for(it = this->VoIPPatternsOUT.begin(); it != this->VoIPPatternsOUT.end(); it++)
    	it->second->clear();

    for(itAlarm = this->AlarmsON.begin(); itAlarm != this->AlarmsON.end(); itAlarm++)
    	itAlarm->second->clear();

	this->LastTS.clear();
}

void AlarmRaiser::Clear() {
	this->dispose();

	this->LastLength = -1;
	this->LastEntropy = 0;
	this->LastClean = 0;

	this->VoIPPatternsIN.clear();
	this->VoIPPatternsOUT.clear();
	this->AlarmsON.clear();
	this->LastTS.clear();
}

void AlarmRaiser::AddPattern(const int &minLength, const int &maxLength, const double &minEnt, const double &maxEnt, const std::string &sDesc) {

	VoIPPattern * Pat = new VoIPPattern(minLength, maxLength, minEnt, maxEnt, sDesc);
	std::map<std::string, int> * OccurIN = new std::map<std::string, int>();
	std::map<std::string, int> * OccurOUT = new std::map<std::string, int>();
	std::set<std::string> * Alarms = new std::set<std::string>();

	this->VoIPPatternsIN.insert(std::pair<VoIPPattern *, std::map<std::string, int> *> (Pat, OccurIN) );
	this->VoIPPatternsOUT.insert(std::pair<VoIPPattern *, std::map<std::string, int> *> (Pat, OccurOUT) );
	this->AlarmsON.insert(std::pair<VoIPPattern *, std::set<std::string> *> (Pat, Alarms) );
}

void AlarmRaiser::ProcessIncomingPacket(const double &TimeStamp, const std::string &HostPort, const int &Length,
										const double &Entropy, const std::string &Peer, const int &TransProto) {

    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator it;

    this->LastLength = Length;
    this->LastEntropy = Entropy;

    for(it = this->VoIPPatternsIN.begin(); it != this->VoIPPatternsIN.end(); it++)
    {
        bool match = it->first->Match(Length, Entropy);

    	if (it->second->count(HostPort) > 0)
        {
    		if ( ((*(it->second))[HostPort]) > 1)
    		{
				if (match)
				{
					if ( ((*(it->second))[HostPort]) < this->maxOccur )
						((*(it->second))[HostPort])++;
				}
				else
					((*(it->second))[HostPort])--;
    		}
			else if ( ((*(it->second))[HostPort]) == 1)
			{
				if (match)
				{
					if ( ((*(it->second))[HostPort]) < this->maxOccur )
						((*(it->second))[HostPort])++;
				}
				else
					it->second->erase(HostPort);
			}
        }
        else
        	if (match)
        		it->second->insert(std::pair<std::string, int> (HostPort, 1));

        this->ProcessAlarms(TimeStamp, HostPort, it->first, Peer, TransProto);
    }

    if (this->CleanIn)
    {
    	std::cout << "ENTROOOOOOOU\n";
    	Tupple Tup(HostPort, Peer, TransProto);
    	this->UpdateTimeStamp(Tup.ToString(), TimeStamp);
    	this->AutoClean(TimeStamp);
    }
}

void AlarmRaiser::ProcessOutgoingPacket(const double &TimeStamp, const std::string &HostPort, const int &Length,
										const double &Entropy, const std::string &Peer, const int &TransProto) {

    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator it;

    this->LastLength = Length;
    this->LastEntropy = Entropy;

    for(it = this->VoIPPatternsOUT.begin(); it != this->VoIPPatternsOUT.end(); it++)
    {
        bool match = it->first->Match(Length, Entropy);

    	if (it->second->count(HostPort) > 0)
        {
    		if ( ((*(it->second))[HostPort]) > 1)
    		{
				if (match)
				{
					if ( ((*(it->second))[HostPort]) < this->maxOccur )
						((*(it->second))[HostPort])++;
				}
				else
					((*(it->second))[HostPort])--;
    		}
			else if ( ((*(it->second))[HostPort]) == 1)
			{
				if (match)
				{
					if ( ((*(it->second))[HostPort]) < this->maxOccur )
						((*(it->second))[HostPort])++;
				}
				else
					it->second->erase(HostPort);
			}
        }
        else
        	if (match)
        		it->second->insert(std::pair<std::string, int> (HostPort, 1));

        this->ProcessAlarms(TimeStamp, HostPort, it->first, Peer, TransProto);
    }

    if (this->CleanIn)
    {
    	std::cout << "ENTROOOOOOOU\n";
    	Tupple Tup(HostPort, Peer, TransProto);
    	this->UpdateTimeStamp(Tup.ToString(), TimeStamp);
    	this->AutoClean(TimeStamp);
    }
}

void AlarmRaiser::ProcessAlarms(const double &TimeStamp, const std::string &HostPort, VoIPPattern * Pat, const std::string &Peer, const int &TransProto) {

	bool RaiseAlarm = false;

	if (  this->VoIPPatternsIN.at(Pat)->count(HostPort) > 0 &&
    	 this->VoIPPatternsOUT.at(Pat)->count(HostPort) > 0    )

    	if (  this->VoIPPatternsIN.at(Pat)->at(HostPort) >= this->lim &&
    		 this->VoIPPatternsOUT.at(Pat)->at(HostPort) >= this->lim    )

    	{

			//std::cout << HostPort << " ---- " << this->VoIPPatternsIN.at(Pat)->at(HostPort) << std::endl;
    		RaiseAlarm = true;
    	}

	Tupple Tup(HostPort, Peer, TransProto);

	if (RaiseAlarm)
	{
		if (this->AlarmsON.at(Pat)->count(Tup.ToString()) == 0 &&
			this->AlarmsON.at(Pat)->count(Tup.InvTupple()) == 0 )
		{
			this->AlarmsON.at(Pat)->insert(Tup.ToString());

			std::cout << HostPort << " ---- " << this->VoIPPatternsIN.at(Pat)->at(HostPort) << std::endl;
			std::cout << Tup.ToString() << " - " << this->AlarmsON.at(Pat)->count(Tup.ToString()) << std::endl;

			this->PrintMsg("Length: " + this->InttoString(this->LastLength) + " Entropy: " +  this->DoubletoString(this->LastEntropy, 4) + "\n", VERB_ENTROPY);
			this->TriggerAction(TimeStamp, &Tup, Pat, AL_RAISE, this->AlarmAction);
		}
	}
	else
	{
		if (this->AlarmsON.at(Pat)->count(Tup.ToString()) > 0)
		{
			std::cout << HostPort << " ---- " << this->VoIPPatternsIN.at(Pat)->at(HostPort) << std::endl;
			this->AlarmsON.at(Pat)->erase(Tup.ToString());
			this->PrintMsg("Length: " + this->InttoString(this->LastLength) + " Entropy: " +  this->DoubletoString(this->LastEntropy, 4) + "\n", VERB_ENTROPY);
			this->TriggerAction(TimeStamp, &Tup, Pat, AL_STOP, this->AlarmAction);
		}
//		else if (this->AlarmsON.at(Pat)->count(Tup.InvTupple()) > 0)
//		{
//			std::cout << HostPort << " ---- " << this->VoIPPatternsIN.at(Pat)->at(HostPort) << std::endl;
//			this->AlarmsON.at(Pat)->erase(Tup.InvTupple());
//			Tupple InvTup(Tup.InvTupple());
//			this->PrintMsg("Length: " + this->InttoString(this->LastLength) + " Entropy: " +  this->DoubletoString(this->LastEntropy, 4) + "\n", VERB_ENTROPY);
//			this->TriggerAction(TimeStamp, &InvTup, Pat, AL_STOP, this->AlarmAction);
//		}
	}
}

void AlarmRaiser::TriggerAction(const double &TimeStamp, const Tupple *Tup, const VoIPPattern * Pat, const int &Alarm, const int &Action) const {

	const std::string Host1 = Tup->SrcHost();
	const std::string Port1 = Tup->StrSrcPort();
	const std::string Host2 = Tup->DstHost();
	const std::string Port2 = Tup->StrDstPort();
	const std::string Proto = Tup->StrProto();

	std::cout << Tup->ToString() << " :::: " << Host1 << ":" << Port1 << ":" << Host2 << ":" << Port2 << ":" << Proto << std::endl;

	if (Action == ACT_PRINT_MSG)
	{
		if (Alarm == AL_RAISE)
			this->PrintMsg(this->TStoString(TimeStamp) + "\nA VoIP call was detected:\nProcess running on host " + Host1 + " port " + Port1 + "\n" + Pat->Description() + "\n\n", VERB_NORMAL);
		else if (Alarm == AL_STOP)
			this->PrintMsg(this->TStoString(TimeStamp) + "\nA VoIP call was finished:\nProcess running on host " + Host1 + " port " + Port1 + "\n" + Pat->Description() + "\n\n", VERB_NORMAL);
	}
}

void AlarmRaiser::AutoClean(const double &TimeStamp) {
	if ( TimeStamp > (this->LastClean + CLEAN_INTERVAL) )
		this->CleanInactiveConnections(TimeStamp);
}

void AlarmRaiser::CleanInactiveConnections(const double &TimeStamp) {

	std::map<std::string, double>::iterator it;

	for(it = this->LastTS.begin(); it != this->LastTS.end(); it++)
    	if ( (it->second + INACTIVITY_TIMEOUT) < TimeStamp )
    	{
    	    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator itPat;
    	    std::map<VoIPPattern *, std::set<std::string> *>::iterator itAlarm;

    	    Tupple Tup(it->first);
    	    const std::string SrcHostPort = Tup.SrcHostPort();

    	    for(itPat = this->VoIPPatternsIN.begin(); itPat != this->VoIPPatternsIN.end(); itPat++)
    	        itPat->second->erase(SrcHostPort);

    	    for(itPat = this->VoIPPatternsOUT.begin(); itPat != this->VoIPPatternsOUT.end(); itPat++)
    	    	itPat->second->erase(SrcHostPort);

    	    for(itAlarm = this->AlarmsON.begin(); itAlarm != this->AlarmsON.end(); itAlarm++)
				if (itAlarm->second->count(it->first) > 0)
				{
					itAlarm->second->erase(it->first);
					this->TriggerAction(TimeStamp, &Tup, itAlarm->first, AL_STOP, this->AlarmAction);
				}

        	this->LastTS.erase(it->first);
    	}

    this->LastClean = TimeStamp;
}

void AlarmRaiser::UpdateTimeStamp(const std::string &Tup, const double &TimeStamp) {

	if (this->LastTS.count(Tup) > 0)
    	this->LastTS.at(Tup) = TimeStamp;
	else
	{
		Tupple InvTup(Tup);
		if (this->LastTS.count(InvTup.InvTupple()) > 0)
			this->LastTS.at(InvTup.InvTupple()) = TimeStamp;
	    else
	    	this->LastTS.insert(std::pair<std::string, double> (Tup, TimeStamp));
	}
}

void AlarmRaiser::PrintMsg(const std::string &Msg, const int &VerboseLevel) const {

	if (this->VerbLevel >= VerboseLevel)
		std::cout << Msg;
}

const std::string AlarmRaiser::TStoString(const double &TS) const {
	std::stringstream ss (std::stringstream::out);
	ss << std::fixed << std::setprecision(6) << TS;
	return ss.str();
}

const std::string AlarmRaiser::InttoString(const int &Value) const {
	std::stringstream ss (std::stringstream::out);
	ss << Value;
	return ss.str();
}

const std::string AlarmRaiser::DoubletoString(const double &Value, const int &Precision) const {
	std::stringstream ss (std::stringstream::out);
	ss << std::fixed << std::setprecision(Precision) << Value;
	return ss.str();
}

