/*
 * \author João V. Gomes
 *
 *  alarmraiser.h
 *
 *  Created on: Dec 28, 2010
 *
 * \brief TODO
 * TODO
 */

#ifndef ALARMRAISER_H_
#define ALARMRAISER_H_

#include <voippattern.h>
#include <pcapwrap.h>
#include <tupple.h>
#include <map>
#include <set>
#include <string>
#include <iostream>
#include <sstream>
#include <iomanip>

#ifndef ACTIONS
#define ACTIONS
#define ACT_PRINT_MSG 0
#define ACT_OUTPUT 1
#endif

#ifndef ALARMS
#define ALARMS
#define AL_RAISE 0
#define AL_STOP 1
#endif

#ifndef VERBOSE_LEVELS
#define VERBOSE_LEVELS
#define VERB_NORMAL 0
#define VERB_ENTROPY 1
#endif // VERBOSE_LEVELS

#ifndef CLEAN
#define CLEAN
#define CLEAN_INTERVAL 10
#define INACTIVITY_TIMEOUT 10
#endif // CLEAN

class AlarmRaiser {
public:
	AlarmRaiser();
	AlarmRaiser(const int &maxOccurences, const int &limit, const int &Action);
	AlarmRaiser(const int &maxOccurences, const int &limit, const int &Action, const int &VerboseLevel);
	AlarmRaiser(const int &maxOccurences, const int &limit, const int &Action, const bool &CleanInactive);
	AlarmRaiser(const int &maxOccurences, const int &limit, const int &Action, const int &VerboseLevel, const bool &CleanInactive);
	virtual ~AlarmRaiser();
	void AddPattern(const int &minLength, const int &maxLength, const double &minEnt, const double &maxEnt, const std::string &sDesc);
	void ProcessIncomingPacket(const double &TimeStamp, const std::string &HostPort, const int &Length, const double &Entropy, const std::string &Peer, const int &TransProto);
	void ProcessOutgoingPacket(const double &TimeStamp, const std::string &HostPort, const int &Length, const double &Entropy, const std::string &Peer, const int &TransProto);
	void Clear();
	void CleanOccurrences();
	void CleanInactiveConnections(const double &TimeStamp);

private:
	void init(const int &maxOccurences, const int &limit, const int &Action, const int &VerboseLevel, const bool &CleanInactive);
	void dispose();
	void ProcessAlarms(const double &TimeStamp, const std::string &HostPort, VoIPPattern * Pat, const std::string &Peer, const int &TransProto);
	void TriggerAction(const double &TimeStamp, const Tupple *Tup, const VoIPPattern * Pat, const int &Alarm, const int &Action) const;
	void AutoClean(const double &TimeStamp);
	void UpdateTimeStamp(const std::string &HostPort, const double &TimeStamp);
	void PrintMsg(const std::string &Msg, const int &VerboseLevel) const;
	const std::string TStoString(const double &TS) const;
	const std::string InttoString(const int &Value) const;
	const std::string DoubletoString(const double &Value, const int &Precision) const;

	std::map<VoIPPattern *, std::map<std::string, int> *> VoIPPatternsIN;
	std::map<VoIPPattern *, std::map<std::string, int> *> VoIPPatternsOUT;
	std::map<VoIPPattern *, std::set<std::string> *> AlarmsON;
	std::map<std::string, double> LastTS;
	int maxOccur;
	int lim;
	int LastLength;
	double LastEntropy;
	int VerbLevel;
	bool CleanIn;
	double LastClean;
	int AlarmAction;
};

#endif /* ALARMRAISER_H_ */
