/*
 * \author João V. Gomes
 *
 *  packetprocessor.cpp
 *
 *  Created on: Dec 28, 2010
 *
 * \brief TODO
 * TODO
 */

#include "packetprocessor.h"

PacketProcessor::PacketProcessor() {
	this->init();
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc) {
	this->PcktSrc = PacketSrc;
	this->init();
}

PacketProcessor::PacketProcessor(AlarmRaiser *AR) {
	this->ARaiser = AR;
	this->init();
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, AlarmRaiser *AR) {
	this->PcktSrc = PacketSrc;
	this->ARaiser = AR;
	this->init();
}

PacketProcessor::~PacketProcessor() {
	this->dispose();
}

void PacketProcessor::init() {
    this->FilterAck = false;
    this->FilterSyn = false;
    this->FilterFull = false;
    this->FilterTCPnoPayload = false;
    this->FilterNoTCP = false;
}

void PacketProcessor::dispose() {

}

void PacketProcessor::SetFilters(const int &Filter)
{
	if (Filter == ACK_PCKT)
		this->FilterAck = true;
	else if (Filter == SYN_PCKT)
		this->FilterSyn = true;
	else if (Filter == FULL_PCKT)
		this->FilterFull = true;
	else if (Filter == NO_PAYLOAD)
		this->FilterTCPnoPayload = true;
	else if (Filter == NO_TCP)
		this->FilterNoTCP = true;
}

void PacketProcessor::UnsetFilters(const int &Filter)
{
	if (Filter == ACK_PCKT)
		this->FilterAck = false;
	else if (Filter == SYN_PCKT)
		this->FilterSyn = false;
	else if (Filter == FULL_PCKT)
		this->FilterFull = false;
	else if (Filter == NO_PAYLOAD)
		this->FilterTCPnoPayload = false;
	else if (Filter == NO_TCP)
		this->FilterNoTCP = false;
}

void PacketProcessor::ResetPacketSource(PacketSource *PacketSrc) {

	if (PacketSrc->IsOpen())
		this->PcktSrc = PacketSrc;
    else
        throw PSrc_Not_Open();
}

bool const PacketProcessor::FilterPacket(const Packet * Pckt) const {

	if (this->FilterNoTCP)
		if (Pckt->Protocol() == PKT_PTC_TCP)
			return true;

	if ( this->FilterFull )
		if (Pckt->Len() == 1514)
			return true;

	if ( Pckt->Protocol() == PKT_PTC_TCP )
	{
		const TCPPacket * TCPP = reinterpret_cast<TCPPacket const *>(Pckt);

		if ( this->FilterSyn )
			if ( TCPP->Syn() )
				return true;

		if ( this->FilterTCPnoPayload )
		{
			int iLen = ETHER_HDRLEN + TCPP->IPhdrSize() + TCPP->TcpHdrSize();

			if (Pckt->Len() == iLen)
					return true;
		}

		if ( this->FilterAck )
			if ( TCPP->Ack() )
			{
				int iLen = ETHER_HDRLEN + TCPP->IPhdrSize() + TCPP->TcpHdrSize();

				if (Pckt->Len() == iLen)
//					if ( !(TCPP->Syn()) 	  &&
//						 !(TCPP->Rst())       &&
//						 !(TCPP->Fin())       &&
//						 !(TCPP->Urg())       &&
//						 !(TCPP->Psh())          )
						return true;
			}
	}

	return false;
}

void PacketProcessor::FindVoIP(const int &WinSize) {

	HostPortAnalyzer<int> HPAnOut(WinSize, "", false, 0);
	HostPortAnalyzer<int> HPAnIn(WinSize, "", false, 0);

    bool end = false;

    double FirstTS = 0;
    bool first = true;

    while(!end)
        try
        {
            Packet *Pckt;

            Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();
            int srcP, dstP;
            std::string src, dst;

            if ( ( (Proto == PKT_PTC_UDP) || (Proto == PKT_PTC_TCP) ) && !(this->FilterPacket(Pckt)) )
            {
				if (Proto == PKT_PTC_UDP)
				{
					UDPPacket const *UDPP = reinterpret_cast<UDPPacket const *>(Pckt);
					src = *(UDPP->SrcIPaddr());
					dst = *(UDPP->DstIPaddr());
					srcP = UDPP->SrcPort();
					dstP = UDPP->DstPort();
				}
				else if (Proto == PKT_PTC_TCP)
				{
					TCPPacket *TCPP = reinterpret_cast<TCPPacket * const>(Pckt);
					src = *(TCPP->SrcIPaddr());
					dst = *(TCPP->DstIPaddr());
					srcP = TCPP->SrcPort();
					dstP = TCPP->DstPort();
				}

				std::stringstream ssKeyIN (std::stringstream::out), ssKeyOUT (std::stringstream::out);
				ssKeyIN  << dst << ":" << dstP;
				ssKeyOUT << src << ":" << srcP;
				std::string HostPortIN = ssKeyIN.str(), HostPortOUT = ssKeyOUT.str();

				long TSsec = Pckt->TSseconds(), TSusec = Pckt->TSmicrosec();
				int len = Pckt->Len();

				double EntropyOUT = HPAnOut.AddValue(TSsec, TSusec, src, srcP, len);
				double EntropyIN  = HPAnIn.AddValue (TSsec, TSusec, dst, dstP, len);

				const double dTS = TSsec + ( TSusec * 0.000001 );
			    if (first)
			    {
			    	FirstTS = dTS;
			    	first = false;
			    }
			    double dElapsedTime = dTS - FirstTS;

				this->ARaiser->ProcessIncomingPacket(dElapsedTime, HostPortIN, len, EntropyIN, HostPortOUT, Proto);
				this->ARaiser->ProcessOutgoingPacket(dElapsedTime, HostPortOUT, len, EntropyOUT, HostPortIN, Proto);
            }
		}
		catch (No_More_Packets e1)
		{
			end = true;
		}
		catch (Timeout_Expired e2)
		{
			std::cout << "Timeout: " << e2.what() << "\n";
			end = true;
		}
		catch (Packet_Reading_Error e3)
		{
			std::cout << "Failed: " << e3.what() << "\n";
			end = true;
		}

}
