/*
 * \author João V. Gomes
 *
 *  alarmraiser.cpp
 *
 *  Created on: Dec 28, 2010
 *
 * \brief TODO
 * TODO
 */

#include "alarmraiser.h"

AlarmRaiser::AlarmRaiser() {

}

AlarmRaiser::AlarmRaiser(const int &maxOccurences, const int &Action, const int &MaxGap, const int &AlarmLevel) {
	this->init(maxOccurences, Action, MaxGap, VERB_NORMAL, false, AlarmLevel);
}

AlarmRaiser::AlarmRaiser(const int &maxOccurences, const int &Action, const int &MaxGap, const int &VerboseLevel, const int &AlarmLevel) {
	this->init(maxOccurences, Action, MaxGap, VerboseLevel, false, AlarmLevel);
}

AlarmRaiser::AlarmRaiser(const int &maxOccurences, const int &Action, const int &MaxGap, const bool &CleanInactive, const int &AlarmLevel) {
	this->init(maxOccurences, Action, MaxGap, VERB_NORMAL, CleanInactive, AlarmLevel);
}

AlarmRaiser::AlarmRaiser(const int &maxOccurences, const int &Action, const int &MaxGap, const int &VerboseLevel, const bool &CleanInactive, const int &AlarmLevel) {
	this->init(maxOccurences, Action, MaxGap, VerboseLevel, CleanInactive, AlarmLevel);
}

AlarmRaiser::~AlarmRaiser() {
	this->dispose();
}

void AlarmRaiser::init(const int &maxOccurences, const int &Action, const int &MaxGap, const int &VerboseLevel, const bool &CleanInactive, const int &AlarmLevel) {
	this->maxOccur = maxOccurences;
	this->verbLevel = VerboseLevel;
	this->cleanIn = CleanInactive;
	this->alarmAction = Action;
	this->lastLength = -1;
	this->lastEntropy = 0;
	this->lastClean = 0;
	this->maxGap = MaxGap;
	this->firstP = true;
	this->alarmLevel = AlarmLevel;
}

void AlarmRaiser::dispose() {
    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator it;
    std::map<std::string, std::map<std::string, double> *>::iterator itAlarm;
    std::map<std::string, std::set<VoIPPattern *> *>::iterator itCodecs;

    for(it = this->VoIPPatternsIN.begin(); it != this->VoIPPatternsIN.end(); it++)
        delete (it->second);

    for(it = this->VoIPPatternsOUT.begin(); it != this->VoIPPatternsOUT.end(); it++)
        delete (it->second);

    for(it = this->VoIPPatterns.begin(); it != this->VoIPPatterns.end(); it++)
    {
    	delete (it->second);
    	delete (it->first);
    }

    for(itCodecs = this->Codecs.begin(); itCodecs != this->Codecs.end(); itCodecs++)
        delete (itCodecs->second);

    for(itAlarm = this->AlarmsON.begin(); itAlarm != this->AlarmsON.end(); itAlarm++)
        delete (itAlarm->second);
}

void AlarmRaiser::CleanOccurrences() {
    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator it;
    std::map<std::string, std::map<std::string, double> *>::iterator itAlarm;
    std::map<std::string, std::set<VoIPPattern *> *>::iterator itCodecs;

    for(it = this->VoIPPatternsIN.begin(); it != this->VoIPPatternsIN.end(); it++)
    	it->second->clear();

    for(it = this->VoIPPatternsOUT.begin(); it != this->VoIPPatternsOUT.end(); it++)
    	it->second->clear();

    for(it = this->VoIPPatterns.begin(); it != this->VoIPPatterns.end(); it++)
    	it->second->clear();

    for(itCodecs = this->Codecs.begin(); itCodecs != this->Codecs.end(); itCodecs++)
        itCodecs->second->clear();

    for(itAlarm = this->AlarmsON.begin(); itAlarm != this->AlarmsON.end(); itAlarm++)
    	itAlarm->second->clear();

	this->LastTS.clear();
}

void AlarmRaiser::Clear() {
	this->dispose();

	this->lastLength = -1;
	this->lastEntropy = 0;
	this->lastClean = 0;
	this->firstP = true;

	this->VoIPPatternsIN.clear();
	this->VoIPPatternsOUT.clear();
	this->VoIPPatterns.clear();
	this->AlarmsON.clear();
	this->LastTS.clear();
}

void AlarmRaiser::AddPattern(const int &minLength, const int &maxLength, const double &minEnt, const double &maxEnt, const int &limit, const std::string &sDesc, const std::string &Codec, const int &Level) {

	VoIPPattern * Pat = new VoIPPattern(minLength, maxLength, minEnt, maxEnt, limit, sDesc, Codec, Level);

	this->StartVectors(Pat);
}

void AlarmRaiser::AddPattern(const double &minLengthMean, const double &maxLengthMean, const double &minEnt, const double &maxEnt, const int &limit, const std::string &sDesc, const std::string &Codec, const int &Level) {

	VoIPPattern * Pat = new VoIPPattern(minLengthMean, maxLengthMean, minEnt, maxEnt, limit, sDesc, Codec, Level);

	this->StartVectors(Pat);
}

void AlarmRaiser::AddPattern(const int &minLength, const int &maxLength, const double &minLengthMean, const double &maxLengthMean, const double &minEnt, const double &maxEnt, const int &limit, const std::string &sDesc, const std::string &Codec, const int &Level) {

	VoIPPattern * Pat = new VoIPPattern(minLength, maxLength, minLengthMean, maxLengthMean, minEnt, maxEnt, limit, sDesc, Codec, Level);

	this->StartVectors(Pat);
}

void AlarmRaiser::StartVectors(VoIPPattern * Pat) {
	std::map<std::string, int> * OccurIN = new std::map<std::string, int>();
	std::map<std::string, int> * OccurOUT = new std::map<std::string, int>();
	std::map<std::string, int> * Occur = new std::map<std::string, int>();
	std::map<std::string, double> * Alarms = new std::map<std::string, double>();

	this->VoIPPatternsIN.insert(std::pair<VoIPPattern *, std::map<std::string, int> *> (Pat, OccurIN) );
	this->VoIPPatternsOUT.insert(std::pair<VoIPPattern *, std::map<std::string, int> *> (Pat, OccurOUT) );
	this->VoIPPatterns.insert(std::pair<VoIPPattern *, std::map<std::string, int> *> (Pat, Occur) );

	if (this->Codecs.count(Pat->Description()) > 0)
		this->Codecs.at(Pat->Description())->insert(Pat);
	else
	{
		std::set<VoIPPattern *> * Pats = new std::set<VoIPPattern *>();
		Pats->insert(Pat);
		this->Codecs.insert(std::pair<std::string, std::set<VoIPPattern *> *> (Pat->Description(), Pats) );
		this->AlarmsON.insert(std::pair<std::string, std::map<std::string, double> *> (Pat->Description(), Alarms) );
	}
}

void AlarmRaiser::ProcessIncomingPacket(const double &TimeStamp, const std::string &HostPort, const int &Length,
										const double &LengthMean, const double &Entropy, const std::string &Peer,
										const int &TransProto) {

    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator it;

    this->lastLength = Length;
    this->lastLengthMean = LengthMean;
    this->lastEntropy = Entropy;

	Tuple Tup(HostPort, Peer, TransProto);

	this->SetFirst(TimeStamp);

    for(it = this->VoIPPatternsIN.begin(); it != this->VoIPPatternsIN.end(); it++)
    	if (it->first->Level() == this->alarmLevel)
    	{
			bool match = it->first->Match(Length, LengthMean, Entropy);

			if (it->second->count(HostPort) > 0)
			{
				if ( ((*(it->second))[HostPort]) > 1)
				{
					if (match)
					{
						if ( ((*(it->second))[HostPort]) < this->maxOccur )
							((*(it->second))[HostPort])++;
					}
					else
						((*(it->second))[HostPort])--;
				}
				else if ( ((*(it->second))[HostPort]) == 1)
				{
					if (match)
					{
						if ( ((*(it->second))[HostPort]) < this->maxOccur )
							((*(it->second))[HostPort])++;
					}
					else
						it->second->erase(HostPort);
				}
			}
			else
				if (match)
					it->second->insert(std::pair<std::string, int> (HostPort, 1));

			this->ProcessAlarms(TimeStamp, it->first, &Tup);
		}

    if (this->cleanIn)
    {
    	Tuple Tup(HostPort, Peer, TransProto);
    	this->UpdateTimeStamp(Tup.ToString(), TimeStamp);
    	this->AutoClean(TimeStamp);
    }
}

void AlarmRaiser::ProcessOutgoingPacket(const double &TimeStamp, const std::string &HostPort, const int &Length,
										const double &LengthMean, const double &Entropy, const std::string &Peer,
										const int &TransProto) {

    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator it;

    this->lastLength = Length;
    this->lastLengthMean = LengthMean;
    this->lastEntropy = Entropy;

	Tuple Tup(HostPort, Peer, TransProto);

	this->SetFirst(TimeStamp);

    for(it = this->VoIPPatternsOUT.begin(); it != this->VoIPPatternsOUT.end(); it++)
    	if (it->first->Level() == this->alarmLevel)
    	{
			bool match = it->first->Match(Length, LengthMean, Entropy);

			if (it->second->count(HostPort) > 0)
			{
				if ( ((*(it->second))[HostPort]) > 1)
				{
					if (match)
					{
						if ( ((*(it->second))[HostPort]) < this->maxOccur )
							((*(it->second))[HostPort])++;
					}
					else
						((*(it->second))[HostPort])--;
				}
				else if ( ((*(it->second))[HostPort]) == 1)
				{
					if (match)
					{
						if ( ((*(it->second))[HostPort]) < this->maxOccur )
							((*(it->second))[HostPort])++;
					}
					else
						it->second->erase(HostPort);
				}
			}
			else
				if (match)
					it->second->insert(std::pair<std::string, int> (HostPort, 1));

			this->ProcessAlarms(TimeStamp, it->first, &Tup);
		}

    if (this->cleanIn)
    {
    	this->UpdateTimeStamp(Tup.SrcHostPort(), TimeStamp);
    	this->AutoClean(TimeStamp);
    }
}

void AlarmRaiser::ProcessPacket(const double &TimeStamp, const std::string &HostPort, const int &Length,
								const double &LengthMean, const double &Entropy, const std::string &Peer,
								const int &TransProto) {

    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator it;

    this->lastLength = Length;
    this->lastLengthMean = LengthMean;
    this->lastEntropy = Entropy;

	Tuple Tup(HostPort, Peer, TransProto);

	const std::string strTup = Tup.ToString();

	this->SetFirst(TimeStamp);

    for(it = this->VoIPPatterns.begin(); it != this->VoIPPatterns.end(); it++)
    	if (it->first->Level() == this->alarmLevel)
    	{
			bool match = it->first->Match(Length, LengthMean, Entropy);

			if (it->second->count(strTup) > 0)
			{
				if ( ((*(it->second))[strTup]) > 1)
				{
					if (match)
					{
						if ( ((*(it->second))[strTup]) < this->maxOccur )
							((*(it->second))[strTup])++;
					}
					else
						((*(it->second))[strTup])--;
				}
				else if ( ((*(it->second))[strTup]) == 1)
				{
					if (match)
					{
						if ( ((*(it->second))[strTup]) < this->maxOccur )
							((*(it->second))[strTup])++;
					}
					else
						it->second->erase(strTup);
				}
			}
			else
				if (match)
					it->second->insert(std::pair<std::string, int> (strTup, 1));

			this->ProcessAlarmsTuple(TimeStamp, it->first, &Tup);
		}

    if (this->cleanIn)
    {
    	this->UpdateTimeStampTuple(&Tup, TimeStamp);
    	this->AutoCleanTuple(TimeStamp);
    }
}

bool AlarmRaiser::CheckCodec(const std::string &CodecID, const std::string &Key) {

	std::set<VoIPPattern *>::iterator it;

	bool keyCheck = false, invKeyCheck = false;

	for (it = this->Codecs.at(CodecID)->begin(); it != this->Codecs.at(CodecID)->end(); it++)
	{
		if ( this->VoIPPatternsOUT.at(*it)->count(Key) > 0 )
	    	if ( (*it)->HasTheMin(this->VoIPPatternsOUT.at(*it)->at(Key)) )
	    		keyCheck = true;

		if ( this->VoIPPatternsIN.at(*it)->count(Key) > 0 )
	    	if ( (*it)->HasTheMin(this->VoIPPatternsIN.at(*it)->at(Key)) )
	    		invKeyCheck = true;
	}

	return (keyCheck && invKeyCheck);
}

bool AlarmRaiser::CheckCodec(const std::string &CodecID, const std::string &Key, const std::string &invKey) {

	std::set<VoIPPattern *>::iterator it;

	bool keyCheck = false, invKeyCheck = false;

	//std::cout << "AAA" << std::endl;

	for (it = this->Codecs.at(CodecID)->begin(); it != this->Codecs.at(CodecID)->end(); it++)
	{
		if ( this->VoIPPatterns.at(*it)->count(Key) > 0 )
	    	if ( (*it)->HasTheMin(this->VoIPPatterns.at(*it)->at(Key)) )
	    	{
//	    		std::cout << "11111 " << (*it)->Description() << std::endl;

	    		keyCheck = true;
	    	}

		if ( this->VoIPPatterns.at(*it)->count(invKey) > 0 )
	    	if ( (*it)->HasTheMin(this->VoIPPatterns.at(*it)->at(invKey)) )
	    	{
	    	//	std::cout << "22222 " << (*it)->Description() << std::endl;

	    		invKeyCheck = true;
	    	}
	}

	//std::cout << "BBB" << std::endl;
	return (keyCheck && invKeyCheck);
}

void AlarmRaiser::UpdateAtLeastOneClassification(const std::string &Key) {
	if (this->AtLeastOneClassification.count(Key) == 0)
		this->AtLeastOneClassification.insert(Key);
}

void AlarmRaiser::UpdateAtLeastOneClassification(const std::string &Key, const std::string &invKey) {
	if (this->AtLeastOneClassification.count(Key) == 0 &&
		this->AtLeastOneClassification.count(invKey) == 0 )

		this->AtLeastOneClassification.insert(Key);
}

void AlarmRaiser::ProcessAlarmsTuple(const double &TimeStamp, VoIPPattern *Pat, const Tuple *Tup) {

	const std::string strTup = Tup->ToString();
	const std::string strInvTup = Tup->InvTuple();

	if (this->CheckCodec(Pat->Description(), strTup, strInvTup))
	{
		if (   this->AlarmsON.at(Pat->Description())->count(strTup)    == 0
			&& this->AlarmsON.at(Pat->Description())->count(strInvTup) == 0
			&& this->Classified.count(strTup) == 0 )
		{
			this->UpdateAtLeastOneClassification(strTup,strInvTup);

			this->AlarmsON.at(Pat->Description())->insert(std::pair<std::string, double> (strTup, TimeStamp));
			this->PrintMsg("Length: " + this->InttoString(this->lastLength) + " Length Mean: " + this->DoubletoString(this->lastLengthMean, 6) + " Entropy: " +  this->DoubletoString(this->lastEntropy, 4) + "\n", VERB_ENTROPY);
			this->TriggerActionTuple(TimeStamp, Tup, Pat, AL_RAISE, this->alarmAction);

			this->Classified.insert(strTup);
			this->Classified.insert(strInvTup);
		}
		else
		{
			if (this->AlarmsON.at(Pat->Description())->count(strTup) > 0)
				this->AlarmsON.at(Pat->Description())->at(strTup) = TimeStamp;
			else if (this->AlarmsON.at(Pat->Description())->count(strInvTup) > 0)
				this->AlarmsON.at(Pat->Description())->at(strInvTup) = TimeStamp;
		}
	}
	else
	{
		if (this->AlarmsON.at(Pat->Description())->count(strTup) > 0)
		{
			if ( (TimeStamp - this->AlarmsON.at(Pat->Description())->at(strTup)) > this->maxGap )
			{
				this->AlarmsON.at(Pat->Description())->erase(strTup);
				this->PrintMsg("Length: " + this->InttoString(this->lastLength) + " Length Mean: " + this->DoubletoString(this->lastLengthMean, 6) + " Entropy: " +  this->DoubletoString(this->lastEntropy, 4) + "\n", VERB_ENTROPY);
				this->TriggerActionTuple(TimeStamp, Tup, Pat, AL_STOP, this->alarmAction);

				this->Classified.erase(strTup);
				this->Classified.erase(strInvTup);
			}
		}
		else if (this->AlarmsON.at(Pat->Description())->count(strInvTup) > 0)
		{
			if ( (TimeStamp - this->AlarmsON.at(Pat->Description())->at(strInvTup)) > this->maxGap )
			{
				this->AlarmsON.at(Pat->Description())->erase(strInvTup);
				this->PrintMsg("Length: " + this->InttoString(this->lastLength) + " Length Mean: " + this->DoubletoString(this->lastLengthMean, 6) + " Entropy: " +  this->DoubletoString(this->lastEntropy, 4) + "\n", VERB_ENTROPY);
				this->TriggerActionTuple(TimeStamp, Tup, Pat, AL_STOP, this->alarmAction);

				this->Classified.erase(strTup);
				this->Classified.erase(strInvTup);
			}
		}

	}
}

void AlarmRaiser::ProcessAlarms(const double &TimeStamp, VoIPPattern *Pat, const Tuple *Tup) {

	const std::string HostPort = Tup->SrcHostPort();

	if (this->CheckCodec(Pat->Description(), HostPort))
	{

		if (   this->AlarmsON.at(Pat->Description())->count(HostPort) == 0
			&& this->Classified.count(HostPort) == 0 )
		{
			this->UpdateAtLeastOneClassification(HostPort);

			this->AlarmsON.at(Pat->Description())->insert(std::pair<std::string, double> (HostPort, TimeStamp));
			this->PrintMsg("Length: " + this->InttoString(this->lastLength) + " Length Mean: " + this->DoubletoString(this->lastLengthMean, 6) + " Entropy: " +  this->DoubletoString(this->lastEntropy, 4) + "\n", VERB_ENTROPY);
			this->TriggerAction(TimeStamp, Tup, Pat, AL_RAISE, this->alarmAction);

			this->Classified.insert(HostPort);
		}
		else
			this->AlarmsON.at(Pat->Description())->at(HostPort) = TimeStamp;
	}
	else
		if (this->AlarmsON.at(Pat->Description())->count(HostPort) > 0)
		{
			if ( (TimeStamp - this->AlarmsON.at(Pat->Description())->at(HostPort)) > this->maxGap )
			{
				this->AlarmsON.at(Pat->Description())->erase(HostPort);
				this->PrintMsg("Length: " + this->InttoString(this->lastLength) + " Length Mean: " + this->DoubletoString(this->lastLengthMean, 6) + " Entropy: " +  this->DoubletoString(this->lastEntropy, 4) + "\n", VERB_ENTROPY);
				this->TriggerAction(TimeStamp, Tup, Pat, AL_STOP, this->alarmAction);

				this->Classified.erase(HostPort);
			}
		}
}

void AlarmRaiser::TriggerAction(const double &TimeStamp, const Tuple *Tup, const VoIPPattern * Pat, const int &Alarm, const int &Action) const {

	const std::string Host1 = Tup->SrcHost();
	const std::string Port1 = Tup->StrSrcPort();
	const std::string Host2 = Tup->DstHost();
	const std::string Port2 = Tup->StrDstPort();
	const std::string Proto = Tup->StrProto();

	if (Action == ACT_PRINT_MSG)
	{
		if (Alarm == AL_RAISE)
			this->PrintMsg(this->TStoString(this->ElapsedTime(TimeStamp)) + "\nA VoIP call was detected:\nProcess running on host " + Host1 + " port " + Port1 + "\n" + Pat->Description() + "\n\n", VERB_NORMAL);
		else if (Alarm == AL_STOP)
			this->PrintMsg(this->TStoString(this->ElapsedTime(TimeStamp)) + "\nA VoIP call was finished:\nProcess running on host " + Host1 + " port " + Port1 + "\n" + Pat->Description() + "\n\n", VERB_NORMAL);
	}
	else if (Action == ACT_OUTPUT)
	{
		if (Alarm == AL_RAISE)
			this->PrintMsg(this->InttoString(this->AtLeastOneClassification.size()) + "\t" + this->TStoString(TimeStamp) + "\t" + Host1 + ":" + Port1 + "\t" + Pat->Description() + "\n", VERB_NORMAL);
	}
}

void AlarmRaiser::TriggerActionTuple(const double &TimeStamp, const Tuple *Tup, const VoIPPattern * Pat, const int &Alarm, const int &Action) const {

	const std::string Host1 = Tup->SrcHost();
	const std::string Port1 = Tup->StrSrcPort();
	const std::string Host2 = Tup->DstHost();
	const std::string Port2 = Tup->StrDstPort();
	const std::string Proto = Tup->StrProto();

	if (Action == ACT_PRINT_MSG)
	{
		if (Alarm == AL_RAISE)
			this->PrintMsg(this->TStoString(this->ElapsedTime(TimeStamp)) + "\nA VoIP call was detected: " + Tup->SrcHostPort() + " -> " + Tup->DstHostPort() + "\n" + Pat->Description() + "\n\n", VERB_NORMAL);
		else if (Alarm == AL_STOP)
			this->PrintMsg(this->TStoString(this->ElapsedTime(TimeStamp)) + "\nA VoIP call was finished: " + Tup->SrcHostPort() + " -> " + Tup->DstHostPort() + "\n" + Pat->Description() + "\n\n", VERB_NORMAL);
	}
	else if (Action == ACT_OUTPUT)
	{
		if (Alarm == AL_RAISE)
			this->PrintMsg(this->InttoString(this->AtLeastOneClassification.size()) + "\t" + this->TStoString(TimeStamp) + "\t" + Tup->ToString() + "\t" + Pat->Description() + "\n", VERB_NORMAL);
	}
}

void AlarmRaiser::AutoClean(const double &TimeStamp) {
	if ( TimeStamp > (this->lastClean + CLEAN_INTERVAL) )
		this->CleanInactiveConnections(TimeStamp);
}

void AlarmRaiser::AutoCleanTuple(const double &TimeStamp) {
	if ( TimeStamp > (this->lastClean + CLEAN_INTERVAL) )
		this->CleanInactiveConnectionsTuple(TimeStamp);
}

void AlarmRaiser::CleanInactiveConnections(const double &TimeStamp) {

	std::map<std::string, double>::iterator it;

	for(it = this->LastTS.begin(); it != this->LastTS.end(); it++)
    	if ( (it->second + INACTIVITY_TIMEOUT) < TimeStamp )
    	{
    	    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator itPat;
    	    std::map<std::string, std::map<std::string, double> *>::iterator itAlarm;

    	    for(itPat = this->VoIPPatternsIN.begin(); itPat != this->VoIPPatternsIN.end(); itPat++)
    	        itPat->second->erase(it->first);

    	    for(itPat = this->VoIPPatternsOUT.begin(); itPat != this->VoIPPatternsOUT.end(); itPat++)
    	    	itPat->second->erase(it->first);

    	    for(itPat = this->VoIPPatterns.begin(); itPat != this->VoIPPatterns.end(); itPat++)
    	    	itPat->second->erase(it->first);

    	    for(itAlarm = this->AlarmsON.begin(); itAlarm != this->AlarmsON.end(); itAlarm++)
				if (itAlarm->second->count(it->first) > 0)
				{
					itAlarm->second->erase(it->first);
					VoIPPattern * Pat = *(this->Codecs.at(itAlarm->first)->begin());

					//TODO
					Tuple Tup(it->first,"",PKT_PTC_IP);
					this->TriggerAction(TimeStamp, &Tup, Pat, AL_STOP, this->alarmAction);

					this->Classified.erase(it->first);
				}

        	this->LastTS.erase(it->first);
    	}

    this->lastClean = TimeStamp;
}

void AlarmRaiser::CleanInactiveConnectionsTuple(const double &TimeStamp) {

	std::map<std::string, double>::iterator it;

	for(it = this->LastTS.begin(); it != this->LastTS.end(); it++)
    	if ( (it->second + INACTIVITY_TIMEOUT) < TimeStamp )
    	{
    	    std::map<VoIPPattern *, std::map<std::string, int> *>::iterator itPat;
    	    std::map<std::string, std::map<std::string, double> *>::iterator itAlarm;

    	    Tuple Tup(it->first);
    	    for(itPat = this->VoIPPatterns.begin(); itPat != this->VoIPPatterns.end(); itPat++)
    	    {
    	    	itPat->second->erase(Tup.ToString());
    	    	itPat->second->erase(Tup.InvTuple());
    	    }

    	    for(itAlarm = this->AlarmsON.begin(); itAlarm != this->AlarmsON.end(); itAlarm++)
    	    {
    	    	if (itAlarm->second->count(Tup.ToString()) > 0)
				{
					itAlarm->second->erase(Tup.ToString());
					VoIPPattern * Pat = *(this->Codecs.at(itAlarm->first)->begin());
					this->TriggerActionTuple(TimeStamp, &Tup, Pat, AL_STOP, this->alarmAction);

					this->Classified.erase(Tup.ToString());
					this->Classified.erase(Tup.InvTuple());
				}
    	    	else if (itAlarm->second->count(Tup.InvTuple()) > 0)
				{
    	    		Tuple InvTup(Tup.InvTuple());
					itAlarm->second->erase(InvTup.ToString());
					VoIPPattern * Pat = *(this->Codecs.at(itAlarm->first)->begin());
					this->TriggerActionTuple(TimeStamp, &InvTup, Pat, AL_STOP, this->alarmAction);

					this->Classified.erase(Tup.ToString());
					this->Classified.erase(Tup.InvTuple());
				}
    	    }

        	this->LastTS.erase(it->first);
    	}

    this->lastClean = TimeStamp;
}

void AlarmRaiser::UpdateTimeStamp(const std::string &Key, const double &TimeStamp) {

	if (this->LastTS.count(Key) > 0)
    	this->LastTS.at(Key) = TimeStamp;
	else
	    this->LastTS.insert(std::pair<std::string, double> (Key, TimeStamp));
}

void AlarmRaiser::UpdateTimeStampTuple(const Tuple *Tup, const double &TimeStamp) {

	if (this->LastTS.count(Tup->ToString()) > 0)
    	this->LastTS.at(Tup->ToString()) = TimeStamp;
	else if (this->LastTS.count(Tup->InvTuple()) > 0)
    	this->LastTS.at(Tup->InvTuple()) = TimeStamp;
	else
	    this->LastTS.insert(std::pair<std::string, double> (Tup->ToString(), TimeStamp));
}

void AlarmRaiser::PrintMsg(const std::string &Msg, const int &VerboseLevel) const {

	if (this->verbLevel >= VerboseLevel)
		std::cout << Msg;
}

const std::string AlarmRaiser::TStoString(const double &TS) const {
	std::stringstream ss (std::stringstream::out);
	ss << std::fixed << std::setprecision(6) << TS;
	return ss.str();
}

const std::string AlarmRaiser::InttoString(const int &Value) const {
	std::stringstream ss (std::stringstream::out);
	ss << Value;
	return ss.str();
}

const std::string AlarmRaiser::DoubletoString(const double &Value, const int &Precision) const {
	std::stringstream ss (std::stringstream::out);
	ss << std::fixed << std::setprecision(Precision) << Value;
	return ss.str();
}

void AlarmRaiser::SetFirst(const double &TS) {
	if (this->firstP)
	{
		this->firstTS = TS;
		this->firstP = false;
	}
}

const double AlarmRaiser::ElapsedTime(const double &TS) const {
    return TS - this->firstTS;
}
