/*
 * \author João V. Gomes
 *
 *  alarmraiser.h
 *
 *  Created on: Dec 28, 2010
 *
 * \brief TODO
 * TODO
 */

#ifndef ALARMRAISER_H_
#define ALARMRAISER_H_

#include <voippattern.h>
#include <pcapwrap.h>
#include <tuple.h>
#include <map>
#include <set>
#include <string>
#include <iostream>
#include <sstream>
#include <iomanip>

#ifndef ACTIONS
#define ACTIONS
#define ACT_PRINT_MSG 0
#define ACT_OUTPUT 1
#endif

#ifndef ALARMS
#define ALARMS
#define AL_RAISE 0
#define AL_STOP 1
#endif

#ifndef VERBOSE_LEVELS
#define VERBOSE_LEVELS
#define VERB_NORMAL 0
#define VERB_ENTROPY 1
#endif // VERBOSE_LEVELS

#ifndef CLEAN
#define CLEAN
#define CLEAN_INTERVAL 10 //10
#define INACTIVITY_TIMEOUT 10 //10
#endif // CLEAN

class AlarmRaiser {
public:
	AlarmRaiser();
	AlarmRaiser(const int &maxOccurences, const int &Action, const int &MaxGap, const int &AlarmLevel);
	AlarmRaiser(const int &maxOccurences, const int &Action, const int &MaxGap, const int &VerboseLevel, const int &AlarmLevel);
	AlarmRaiser(const int &maxOccurences, const int &Action, const int &MaxGap, const bool &CleanInactive, const int &AlarmLevel);
	AlarmRaiser(const int &maxOccurences, const int &Action, const int &MaxGap, const int &VerboseLevel, const bool &CleanInactive, const int &AlarmLevel);
	virtual ~AlarmRaiser();
	void AddPattern(const int &minLength, const int &maxLength, const double &minEnt, const double &maxEnt, const int &limit, const std::string &sDesc, const std::string &Codec, const int &Level);
	void AddPattern(const double &minLengthMean, const double &maxLengthMean, const double &minEnt, const double &maxEnt, const int &limit, const std::string &sDesc, const std::string &Codec, const int &Level);
	void AddPattern(const int &minLength, const int &maxLength, const double &minLengthMean, const double &maxLengthMean, const double &minEnt, const double &maxEnt, const int &limit, const std::string &sDesc, const std::string &Codec, const int &Level);
	void ProcessIncomingPacket(const double &TimeStamp, const std::string &HostPort, const int &Length, const double &LengthMean, const double &Entropy, const std::string &Peer, const int &TransProto);
	void ProcessOutgoingPacket(const double &TimeStamp, const std::string &HostPort, const int &Length, const double &LengthMean, const double &Entropy, const std::string &Peer, const int &TransProto);
	void ProcessPacket(const double &TimeStamp, const std::string &HostPort, const int &Length, const double &LengthMean, const double &Entropy, const std::string &Peer, const int &TransProto);
	void Clear();
	void CleanOccurrences();

private:
	void init(const int &maxOccurences, const int &Action, const int &MaxGap, const int &VerboseLevel, const bool &CleanInactive, const int &AlarmLevel);
	void dispose();
	void ProcessAlarms(const double &TimeStamp, VoIPPattern * Pat, const Tuple *Tup);
	void ProcessAlarmsTuple(const double &TimeStamp, VoIPPattern * Pat, const Tuple *Tup);
	void TriggerAction(const double &TimeStamp, const Tuple *Tup, const VoIPPattern * Pat, const int &Alarm, const int &Action) const;
	void TriggerActionTuple(const double &TimeStamp, const Tuple *Tup, const VoIPPattern * Pat, const int &Alarm, const int &Action) const;
	void AutoClean(const double &TimeStamp);
	void UpdateTimeStamp(const std::string &Key, const double &TimeStamp);
	void AutoCleanTuple(const double &TimeStamp);
	void UpdateTimeStampTuple(const Tuple *Tup, const double &TimeStamp);
	void PrintMsg(const std::string &Msg, const int &VerboseLevel) const;
	void SetFirst(const double &TS);
	void StartVectors(VoIPPattern * Pat);
	const std::string TStoString(const double &TS) const;
	const std::string InttoString(const int &Value) const;
	const std::string DoubletoString(const double &Value, const int &Precision) const;
	const double ElapsedTime(const double &TS) const;
	bool CheckCodec(const std::string &CodecID, const std::string &Key);
	bool CheckCodec(const std::string &CodecID, const std::string &Key, const std::string &invKey);
	void UpdateAtLeastOneClassification(const std::string &Key);
	void UpdateAtLeastOneClassification(const std::string &Key, const std::string &invKey);
	void CleanInactiveConnections(const double &TimeStamp);
	void CleanInactiveConnectionsTuple(const double &TimeStamp);

	// VoIPPattern, Key (HostPort/Tuple), number of packets
	std::map<VoIPPattern *, std::map<std::string, int> *> VoIPPatternsIN;
	std::map<VoIPPattern *, std::map<std::string, int> *> VoIPPatternsOUT;
	std::map<VoIPPattern *, std::map<std::string, int> *> VoIPPatterns;

	// Codec, VoIPPattern
	std::map<std::string, std::set<VoIPPattern *> *> Codecs;

	// VoIPPattern description, Key (HostPort/Tuple), Last TS
	std::map<std::string, std::map<std::string, double> *> AlarmsON;

	// Key (HostPort/Tuple)
	std::set<std::string> Classified;

	// Key (HostPort/Tuple) // keeps record of the keys that already had at least one classification
	std::set<std::string> AtLeastOneClassification;

	std::map<std::string, double> LastTS;
	int maxOccur;
	int lastLength;
	int alarmAction;
	int verbLevel;
	int maxGap;
	int alarmLevel;
	double lastLengthMean;
	double lastEntropy;
	double lastClean;
	double firstTS;
	bool cleanIn;
	bool firstP;
};

#endif /* ALARMRAISER_H_ */
