/*
 * \author João V. Gomes
 *
 *  main.cpp
 *
 *  Created on: Dec 28, 2010
 *
 * \brief TODO
 * TODO
 */

#include <packetprocessor.h>
#include <alarmraiser.h>
#include <cmath>
#include <vector>

using namespace std;

int main(int argc, char* argv[])
{
	// Window length
	const int WinSize = 500;

	// Probability parameters
	const int maxOccur = 500;

	// Analysis perspective
	const bool OnlyPLlength = true;
	const bool TupleBased = true;
	const bool CleanInactiveCalls = true;
	const int AlarmLevel = PAT_LEVEL_CD;	// PAT_LEVEL_BR, PAT_LEVEL_GR, PAT_LEVEL_CD
	const int VerboseLevel = VERB_NORMAL;	// VERB_NORMAL, VERB_ENTROPY
	const int MsgMode = ACT_PRINT_MSG; 		// ACT_PRINT_MSG, ACT_OUTPUT
	const int MaxTimeGap = 5;

	// Filtering options
	const bool F_TCPNoPayload = false;
	const bool F_NoTCP = false;
	const bool F_NoAcks = false;
	const bool F_NoSyns = false;
	const bool F_NoLess65 = false;
	const bool F_TCPNoPLLess5 = false;
	const bool F_NoPLLess5 = true;

	// Online analysis parameters
	const bool OnlineAn = false;
	const int device = 1;
	const int iMaxSize = 1600;
	const int iTimeOut = 100000;

	// Alarm raiser
	cerr << "Preparing the alarm raiser... ";
	AlarmRaiser AR(maxOccur, MsgMode, MaxTimeGap, VerboseLevel, CleanInactiveCalls, AlarmLevel);


	if (AlarmLevel == PAT_LEVEL_BR)
	{
	// BR level patterns
		AR.AddPattern( 15, 400,               0.00, 1.00, 400, "VoIP, CBR codec",                 "CBR",     PAT_LEVEL_BR);
		AR.AddPattern( 10, 400,               1.25, 3.25, 450, "VoIP, VBR codec - low variation", "VBR_Low", PAT_LEVEL_BR);
		AR.AddPattern( 15, 800,               2.80, 6.00, 450, "VoIP, VBR codec",                 "VBR",     PAT_LEVEL_BR);
	}


	if (AlarmLevel == PAT_LEVEL_GR)
	{
	// GR level patterns
		AR.AddPattern( 75, 700,               3.50, 5.50, 400, "VoIP, GIPS VBR codec",              "GIPS VBR",              PAT_LEVEL_GR);
		AR.AddPattern(160, 190,               0.00, 1.00, 400, "VoIP, PCM based codec",             "PCM based",             PAT_LEVEL_GR);
		AR.AddPattern( 25, 190,               0.10, 1.00, 400, "VoIP, CBR Skype",                   "CBR Skype",             PAT_LEVEL_GR);
		AR.AddPattern( 20, 120,               1.50, 4.50, 400, "VoIP, Skype proprietary VBR codec", "Skype VBR proprietary", PAT_LEVEL_GR);
	}


	if (AlarmLevel == PAT_LEVEL_CD)
	{
		// Low entropy codecs
		AR.AddPattern( 25,  30,               0.10, 0.95, 400, "VoIP, G.729, Skype",      "G.729",  PAT_LEVEL_CD);
		AR.AddPattern( 44,  45,               0.00, 0.10, 450, "VoIP, GSM, SIP",          "GSM",    PAT_LEVEL_CD);
	//	AR.AddPattern(100, 125,               0.00, 0.10, 450, "VoIP, GSM, SIP",          "GSM",    PAT_LEVEL_CD);
		AR.AddPattern( 49,  51,               0.00, 0.10, 450, "VoIP, iLBC, SIP",         "iLBC",   PAT_LEVEL_CD);
	//	AR.AddPattern( 60,  65,               0.00, 0.10, 450, "VoIP, iLBC, SIP",         "iLBC",   PAT_LEVEL_CD);
		AR.AddPattern( 87,  90,               0.00, 0.10, 450, "VoIP, iLBC, SIP",         "iLBC",   PAT_LEVEL_CD);
		AR.AddPattern( 46,  50,               0.05, 1.00, 400, "VoIP, iLBC, Skype",       "iLBC",   PAT_LEVEL_CD);
		AR.AddPattern( 86,  90,               0.05, 1.00, 400, "VoIP, iLBC, Skype",       "iLBC",   PAT_LEVEL_CD);
		AR.AddPattern(165, 185,               0.00, 0.10, 450, "VoIP, PCM, SIP",          "PCM",    PAT_LEVEL_CD);
		AR.AddPattern(160, 171,               0.10, 1.00, 450, "VoIP, PCMA, Skype",       "PCMA",   PAT_LEVEL_CD);
		AR.AddPattern(170, 185,               0.10, 1.00, 450, "VoIP, PCMU, Skype",       "PCMA",   PAT_LEVEL_CD);
		AR.AddPattern(171, 175,               0.00, 0.10, 450, "VoIP, G.722, SIP",        "G.722",  PAT_LEVEL_CD);
		AR.AddPattern(160, 171,               0.10, 1.00, 450, "VoIP, NWC, Skype",        "NWC",    PAT_LEVEL_CD);
		AR.AddPattern( 85,  87,               0.00, 0.10, 450, "VoIP, Speex32, SIP",      "Speex",  PAT_LEVEL_CD);
		AR.AddPattern( 45,  50,               0.00, 0.10, 450, "VoIP, Speex32, SIP",      "Speex",  PAT_LEVEL_CD);
		AR.AddPattern( 80,  85,               0.00, 0.10, 450, "VoIP, Speex16, SIP",      "Speex",  PAT_LEVEL_CD);
		AR.AddPattern( 40,  45,               0.00, 0.10, 450, "VoIP, Speex16, SIP",      "Speex",  PAT_LEVEL_CD);
		AR.AddPattern( 50,  52,               0.00, 0.10, 450, "VoIP, Speex8, SIP",       "Speex",  PAT_LEVEL_CD);
		AR.AddPattern( 30,  35,               0.00, 0.10, 450, "VoIP, Speex8, SIP",       "Speex",  PAT_LEVEL_CD);


		// High entropy codecs
		AR.AddPattern( 45,  80,               0.15, 1.75, 250, "VoIP, AMR-WB, Skype",     "AMR-WB", PAT_LEVEL_CD);    //	10,  80,               0.50, 1.50, 250
		AR.AddPattern(200, 550,               3.00, 5.50, 400, "VoIP, EG711, Skype",      "EG711",  PAT_LEVEL_CD);
		AR.AddPattern( 75, 250,               3.50, 5.50, 400, "VoIP, EG711, Skype",      "EG711",  PAT_LEVEL_CD);
		AR.AddPattern(250, 700,               3.00, 5.50, 400, "VoIP, iPCMwb, Skype",     "iPCMwb", PAT_LEVEL_CD);
		AR.AddPattern(150, 300,               3.50, 5.50, 400, "VoIP, iPCMwb, Skype",     "iPCMwb", PAT_LEVEL_CD);
		AR.AddPattern(100, 300,               3.00, 5.50, 400, "VoIP, ISAC, Skype",       "ISAC",   PAT_LEVEL_CD);
		AR.AddPattern( 60, 200,               3.50, 5.50, 400, "VoIP, ISAC, Skype",       "ISAC",   PAT_LEVEL_CD);
		AR.AddPattern( 40, 120,               2.75, 4.50, 250, "VoIP, SILK, Skype",       "SILK",   PAT_LEVEL_CD);
		AR.AddPattern( 20,  60,			      2.00, 3.50, 400, "VoIP, SILK MB/NB, Skype", "SILK",   PAT_LEVEL_CD);
		AR.AddPattern( 80, 120, 			  1.50, 3.00, 400, "VoIP, SVOPC, Skype",      "SVOPC",  PAT_LEVEL_CD);
		AR.AddPattern( 20, 100,               2.00, 2.50, 400, "VoIP, Speex, SIP",        "Speex",  PAT_LEVEL_CD);
		AR.AddPattern( 20, 100,               1.50, 2.00, 400, "VoIP, Speex, SIP",        "Speex",  PAT_LEVEL_CD);
	}

//	AR.AddPattern(60, 300, 0.01, 1.0, "VoIP, Low entropy codec");
//	AR.AddPattern(60, 300, 2.5, log(WinSize)+10, "VoIP, High entropy codec");
	cerr << "DONE" << endl;


	const string FilePath = argv[1];

	cerr << FilePath << endl;

//	const string FilePath = "/home/joao/Desktop/labredes_21012011_02___1.pcap";
//	const string FilePath = "/home/joao/Desktop/labredes_21012011_03___1.pcap";
//	const string FilePath = "/home/joao/Desktop/labredes_21012011_05___1.pcap";
//	const string FilePath = "/home/joao/Desktop/labredes_21012011_04___1.pcap";
//	const string FilePath = "/home/joao/Desktop/labredes_04032011_03___1.pcap";
//	const string FilePath = "/home/joao/Desktop/trace_voip_6.pcap";
//	const string FilePath = "/home/joao/Desktop/trace_voip_7.pcap";
//	const string FilePath = "/home/joao/Desktop/trace_voip_8.pcap";
//	const string FilePath = "/home/joao/Desktop/trace_voip_9.pcap";
//	const string FilePath = "/home/joao/Desktop/trace_voip_10.pcap";
//	const string FilePath = "/home/joao/Desktop/trace_voip_11.pcap";
//	const string FilePath = "/home/joao/Desktop/trace_voip_12.pcap";
//	const string FilePath = "/home/joao/Desktop/trace_voip_13.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/resources_traces/trace_voip_13.pcap";

//	------


//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/17012011/selected/labredes_17012011_01___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/17012011/selected/labredes_17012011_02___1.pcap";

//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/17012011/selected/negatives/labredes_17012011_01_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/17012011/selected/negatives/labredes_17012011_02_negatives.pcap";


//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_G729_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_PCMA_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_PCMU_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_NWC_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SVOPC_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SVOPC_SB_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SVOPC_SB_UDP_17012011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_V3_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_WB_V3_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_MB_V3_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_NB_V3_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_WB_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_WB_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_NB_V3_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_MB_V3_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_WB_V3_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_V3_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SVOPC_SB_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SVOPC_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_NWC_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_PCMU_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_PCMA_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_G729_TCP_17012011_1.pcap";




//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/19012011/selected/labredes_19012011_01___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/19012011/selected/labredes_19012011_02___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/19012011/selected/labredes_19012011_03___1.pcap";

//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/19012011/selected/negatives/labredes_19012011_01_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/19012011/selected/negatives/labredes_19012011_02_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/19012011/selected/negatives/labredes_19012011_03_negatives.pcap";


//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_ILBC_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_ISAC_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_G729_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_IPCMWB_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_PCMA_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_PCMU_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_EG711A_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_EG711U_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_EG711U_TCP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_EG711A_TCP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_PCMA_TCP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_PCMU_TCP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_IPCMWB_TCP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_G729_TCP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_ISAC_TCP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_ILBC_TCP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_A_PCMA_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_A_G729_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_A_SILK_MB_V3_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_A_SILK_WB_V3_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_A_SILK_NB_V3_UDP_19012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes19012011/skype_A_PCMU_UDP_19012011_1.pcap";




//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/selected/labredes_21012011_01___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/selected/labredes_21012011_02___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/selected/labredes_21012011_03___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/selected/labredes_21012011_04___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/selected/labredes_21012011_05___1.pcap";

//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/selected/negatives/labredes_21012011_01_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/selected/negatives/labredes_21012011_02_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/selected/negatives/labredes_21012011_03_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/selected/negatives/labredes_21012011_04_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/selected/negatives/labredes_21012011_05_negatives.pcap";


//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_L-L_SPEEX32_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_L-L_SPEEX16_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_L-L_SPEEX8_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_L-L_GSM_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_L-L_PCMU_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_L-L_PCMA_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_B-L_SPEEX8_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_B-L_SPEEX16_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_B-L_SPEEX32_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_B-L_GSM_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_B-L_PCMU_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_B-AL_SPEEX16_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_B-AL_SPEEX8_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_L-AL_GSM_UDP_21012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes21012011/sip_L-AL_PCMA_UDP_21012011_1.pcap";




//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/labredes_04032011_01___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/labredes_04032011_02___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/labredes_04032011_03___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/labredes_04032011_04___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/labredes_04032011_05___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/labredes_04032011_06___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/labredes_04032011_07___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/labredes_04032011_08___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/labredes_04032011_09___1.pcap";
//	const string FilePath = "/media/iomega/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/labredes_04032011_10___1.pcap";

//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/negatives/labredes_04032011_01_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/negatives/labredes_04032011_02_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/negatives/labredes_04032011_03_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/negatives/labredes_04032011_04_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/negatives/labredes_04032011_05_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/negatives/labredes_04032011_06_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/negatives/labredes_04032011_07_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/negatives/labredes_04032011_08_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/negatives/labredes_04032011_09_negatives.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/04032011/selected/negatives/labredes_04032011_10_negatives.pcap";



//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_AMRWB_UDP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_AMRWB_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_EG711A_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_EG711U_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_ISAC_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_ILBC_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_AMRWB_TCP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_EG711A_TCP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_EG711U_TCP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_ISAC_TCP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_ILBC_TCP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_AMRWB_UDP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_IPCMWB_UDP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_IPCMWB_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_IPCMWB_TCP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SILK_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SILK_TCP_04032011_3.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SILK_TCP_04032011_4.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SILK_WB_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_PCMA_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_PCMU_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_G729_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_G729_TCP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SILK_WB_TCP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_PCMA_TCP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_PCMU_TCP_04032011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_G729_TCP_04032011_3.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SILK_V3_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SILK_WB_V3_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SILK_MB_V3_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SILK_NB_V3_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SVOPC_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_SVOPC_SB_TCP_04032011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes04032011/skype_NWC_TCP_04032011_1.pcap";


//	--- UNIBS ---

//	const string FilePath = "/home/joao/Research/Documents/Traces/traces_UNIBS/unibs20090930.anon.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/traces_UNIBS/unibs20091001.anon.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/traces_UNIBS/unibs20091002.anon.pcap";


//	--- Polito ---


//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_110108_SVOPC.dump.anonim.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_110108_SVOPC.dump.anonim.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_140606_UDP_EG711A.dump.anonim.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_140606_UDP_EG711A.dump.anonim.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_140606_UDP_EG711U.dump.anonim.pcap";
//	const string FilePath = "/media/iomega/BACKUP/ORI/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_140606_UDP_EG711U.dump.anonim.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_140606_UDP_G729.dump.anonim.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_140606_UDP_iLBC.dump.anonim.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_140606_UDP_IPCMWB.dump.anonim.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_140606_UDP_PCMA.dump.anonim.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_140606_UDP_PCMU.dump.anonim.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_250906_TCP_ISAC.dump.anonim.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/Skype_E2E_Traces/skype_call_260606_UDP_ISAC.dump.anonim.pcap";

//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/SkypeOut_Traces/skypeOut_call_260906_TCP.dump.anonim.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_Polito/SkypeOut_Traces/skypeOut_call_260906_UDP.dump.anonim.pcap";


//	----------------


//	const string FilePath = "/home/joao/Downloads/sip-rtp-g711.cap";
//	const string FilePath = "/home/joao/Downloads/kw_voip_g711_dn_sv_all.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/torrent_23082010_1_CLEAN1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/web_browsing_25082010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/ssh_04092010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/telnet_24082010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/tvu_12092010_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/amule_29082010_1_CLEAN1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/live_audio_streaming_RTSP_12092010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/live_audio_streaming_RTSP_13092010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/live_audio_streaming_RTSP_17082010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/live_audio_streaming_RTSP_rai_06092010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_2010_1/skype_voip_30072010_1_CLEAN1.pcap";


	// 192.168.2.3
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-AL_GSM_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-AL_PCMU_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-AL_SPEEX16_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-AL_SPEEX8_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-B_G722_UDP_13012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-B_GSM_UDP_13012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-B_ILBC_UDP_13012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-B_PCMA_UDP_13012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-B_PCMA_UDP_13012011_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-B_PCMU_UDP_13012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-B_SPEEX16_UDP_13012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-B_SPEEX32_UDP_13012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-C_G722_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-C_GSM_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-C_ILBC_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-C_PCMA_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-C_PCMU_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-C_SPEEX32_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-C_SPEEX8_UDP_13012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-C_SPEEX8_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-X_GSM_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-X_ILBC_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-X_PCMA_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-X_PCMU_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/sip_B-X_SPEEX8_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_A_G729_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_A_SILK_NB_V3_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_A_SILK_WB_V3_UDP_18012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_EG711A_TCP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_EG711A_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_EG711A_UDP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_EG711U_TCP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_EG711U_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_EG711U_UDP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_G729_TCP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_G729_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_G729_TCP_17122010_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_G729_TCP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_G729_UDP_13122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_G729_UDP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_G729_UDP_29112010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_ILBC_TCP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_ILBC_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_ILBC_UDP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_IPCMWB_TCP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_IPCMWB_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_IPCMWB_UDP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_ISAC_TCP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_ISAC_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_ISAC_UDP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_NWC_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_NWC_TCP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_NWC_UDP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_NWC_UDP_29112010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMA_TCP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMA_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMA_TCP_17122010_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMA_TCP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMA_UDP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMA_UDP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMA_UDP_29112010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMU_TCP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMU_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMU_TCP_17122010_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMU_TCP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMU_UDP_16122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMU_UDP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_PCMU_UDP_29112010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_MB_V3_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_MB_V3_TCP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_MB_V3_UDP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_MB_V3_UDP_29112010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_NB_V3_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_NB_V3_TCP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_NB_V3_UDP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_NB_V3_UDP_29112010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_TCP_12012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_UDP_12012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_V3_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_V3_TCP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_V3_UDP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_V3_UDP_29112010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_WB_TCP_12012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_WB_UDP_12012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_WB_V3_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_WB_V3_TCP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_WB_V3_UDP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SILK_WB_V3_UDP_29112010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SVOPC_SB_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SVOPC_SB_TCP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SVOPC_SB_UDP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SVOPC_SB_UDP_29112010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SVOPC_TCP_17122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SVOPC_TCP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SVOPC_UDP_21122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.3/skype_SVOPC_UDP_29112010_1_CLEAN.pcap";


	// 192.168.2.2
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_silence_SPEEX16_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_silence_SPEEX16_UDP_27012011_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_silence_SPEEX32_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_silence_SPEEX32_UDP_27012011_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_silence_SPEEX8_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_silence_SPEEX8_UDP_27012011_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_SPEEX16_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_SPEEX32_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_SPEEX8_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_voice_SPEEX16_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_voice_SPEEX32_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Ll_voice_SPEEX8_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Lw_silence_SPEEX16_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/sip_Lw-Lw_voice_SPEEX32_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_EG711A_UDP_13122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_EG711U_UDP_13122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_G729_UDP_13122010_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_ILBC_UDP_13122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_ILBC_UDP_13122010_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_IPCMWB_UDP_13122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_IPCMWB_UDP_13122010_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_ISAC_UDP_13122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_PCMA_UDP_13122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_PCMU_UDP_13122010_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_PCMU_UDP_13122010_2_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_SILK_TCP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_SILK_UDP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_SILK_WB_TCP_27012011_1_CLEAN.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/192.168.2.2/skype_SILK_WB_UDP_27012011_1_CLEAN.pcap";


	// labredes 17012011
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_G729_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_G729_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_NWC_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_NWC_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_PCMA_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_PCMA_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_PCMU_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_PCMU_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_MB_V3_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_MB_V3_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_NB_V3_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_NB_V3_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_V3_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_V3_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_WB_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_WB_UDP_17012011_1.pcap";//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_MB_V3_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_WB_V3_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SILK_WB_V3_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SVOPC_SB_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SVOPC_SB_UDP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SVOPC_SB_UDP_17012011_2.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SVOPC_TCP_17012011_1.pcap";
//	const string FilePath = "/home/joao/Research/Documents/Traces/Traces_VoIP_2010/labredes17012011/skype_SVOPC_UDP_17012011_1.pcap";


//	const string FilePath = "/media/PENDRIVE/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/17012011/labredes_17012011_1.pcap";
//	const string FilePath = "/media/PENDRIVE/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/17012011/labredes_17012011_2.pcap";
//	const string FilePath = "/media/PENDRIVE/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/19012011/labredes_19012011_1.pcap";
//	const string FilePath = "/media/PENDRIVE/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/19012011/labredes_19012011_2.pcap";
//	const string FilePath = "/media/PENDRIVE/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/19012011/labredes_19012011_3.pcap";
//	const string FilePath = "/media/PENDRIVE/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/labredes_21012011_1.pcap";
//	const string FilePath = "/media/PENDRIVE/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/labredes_21012011_2.pcap";
//	const string FilePath = "/media/PENDRIVE/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/labredes_21012011_3.pcap";
//	const string FilePath = "/media/PENDRIVE/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/labredes_21012011_4.pcap";
//	const string FilePath = "/media/PENDRIVE/ORI/Research/Documents/Traces/Traces_VoIP_2010/labredes/21012011/labredes_21012011_5.pcap";


	PacketSource PS;

	try
	{
		cerr << "Opening the packet source... ";
		if (!OnlineAn)
			PS.OpenOfflineSource(FilePath.c_str());
		else
		{
		    vector<Device> DevL = PS.GetDeviceList();
		    vector<Device>::iterator it;
		    it = DevL.begin();
		    const string DevName = (it+device-1)->Name();
		    DevL.clear();

		    PS.OpenOnlineSource(DevName.c_str(), iMaxSize, iTimeOut);
		}
		cerr << "DONE" << endl;

		cerr << "Preparing the packet processor... ";
		PacketProcessor PP(&PS, &AR, OnlyPLlength, TupleBased);

		// Filters
		if (F_TCPNoPayload)
			PP.SetFilters(NO_PAYLOAD);
		if (F_NoAcks)
			PP.SetFilters(ACK_PCKT);
		if (F_NoSyns)
			PP.SetFilters(SYN_PCKT);
		if (F_NoLess65)
			PP.SetFilters(NO_LESS_65);
		if (F_TCPNoPLLess5)
			PP.SetFilters(NO_TCP_PL_LESS_5);
		if (F_NoPLLess5)
			PP.SetFilters(NO_PL_LESS_5);
		if (F_NoTCP)
			PP.SetFilters(NO_TCP);

		cerr << "DONE" << endl;

		cerr << "Analyzing the traffic... " << endl;
		PP.FindVoIP(WinSize);
	}
	catch (Src_Already_Open e)
	{
		cout << "This Object PacketSource already has an open source. Please close it first.\n";
	}
	catch (Dev_Open_Offline_Fail e1)
	{
		cout << "Failed to open the file: " << e1.what() << "\n";
	}
	catch (Dev_Open_Live_Fail e2)
	{
		cout << "Failed to open the device: " << e2.what() << "\n";
	}
	catch (Dev_Open_Live_Warning e3)
	{
		cout << "Open live warning: " << e3.what() << "\n";
	}
	catch (Timeout_Expired e4)
	{
		cout << "Timeout: " << e4.what() << "\n";
	}
	catch (No_More_Packets e5)
	{
		cout << "No more packets: " << e5.what() << "\n";
	}
	catch (Packet_Reading_Error e6)
	{
		cout << "Failed: " << e6.what() << "\n";
	}
	catch (Src_Not_Open e7)
	{
		cout << "The Packet Source is not open yet. Please provide an open source.\n";
	}
	catch (PSrc_Not_Open e8)
	{
		cout << "Packet Source failed: " << e8.what() << "\n";
	}
	return 0;
}
