/*
 * \author João V. Gomes
 *
 *  packetprocessor.cpp
 *
 *  Created on: Dec 28, 2010
 *
 * \brief TODO
 * TODO
 */

#include "packetprocessor.h"

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, AlarmRaiser *AR) {
	this->init(PacketSrc, AR, false, true);
}

PacketProcessor::PacketProcessor(PacketSource *PacketSrc, AlarmRaiser *AR, const bool &OnlyPayloadLength, const bool &TupleBased) {
	this->init(PacketSrc, AR, OnlyPayloadLength, TupleBased);
}

PacketProcessor::~PacketProcessor() {
	this->dispose();
}

void PacketProcessor::init(PacketSource *PacketSrc, AlarmRaiser *AR, const bool &OnlyPayloadLength, const bool &TupleBased) {
    this->FilterAck = false;
    this->FilterSyn = false;
    this->FilterFull = false;
    this->FilterTCPnoPayload = false;
    this->FilterNoTCP = false;
    this->FilterNoLess60 = false;
    this->FilterNoLess65 = false;
    this->FilterTCPPayloadLess5 = false;
    this->FilterPayloadLess5 = false;

	this->OnlyPayloadLen = OnlyPayloadLength;
	this->TupBased = TupleBased;

	this->PcktSrc = PacketSrc;
	this->ARaiser = AR;
}

void PacketProcessor::dispose() {

}

void PacketProcessor::SetFilters(const int &Filter)
{
	if (Filter == ACK_PCKT)
		this->FilterAck = true;
	else if (Filter == SYN_PCKT)
		this->FilterSyn = true;
	else if (Filter == FULL_PCKT)
		this->FilterFull = true;
	else if (Filter == NO_PAYLOAD)
		this->FilterTCPnoPayload = true;
	else if (Filter == NO_TCP)
		this->FilterNoTCP = true;
	else if (Filter == NO_LESS_60)
		this->FilterNoLess60 = true;
	else if (Filter == NO_LESS_65)
		this->FilterNoLess65 = true;
	else if (Filter == NO_TCP_PL_LESS_5)
		this->FilterTCPPayloadLess5 = true;
	else if (Filter == NO_PL_LESS_5)
		this->FilterPayloadLess5 = true;
}

void PacketProcessor::UnsetFilters(const int &Filter)
{
	if (Filter == ACK_PCKT)
		this->FilterAck = false;
	else if (Filter == SYN_PCKT)
		this->FilterSyn = false;
	else if (Filter == FULL_PCKT)
		this->FilterFull = false;
	else if (Filter == NO_PAYLOAD)
		this->FilterTCPnoPayload = false;
	else if (Filter == NO_TCP)
		this->FilterNoTCP = false;
	else if (Filter == NO_LESS_60)
		this->FilterNoLess60 = false;
	else if (Filter == NO_LESS_65)
		this->FilterNoLess65 = false;
	else if (Filter == NO_TCP_PL_LESS_5)
		this->FilterTCPPayloadLess5 = false;
	else if (Filter == NO_PL_LESS_5)
		this->FilterPayloadLess5 = false;
}

void PacketProcessor::ResetPacketSource(PacketSource *PacketSrc) {

	if (PacketSrc->IsOpen())
		this->PcktSrc = PacketSrc;
    else
        throw PSrc_Not_Open();
}

bool const PacketProcessor::FilterPacket(const Packet * Pckt) const {

	if (this->FilterNoTCP)
		if (Pckt->Protocol() == PKT_PTC_TCP)
			return true;

	if ( this->FilterFull )
		if (Pckt->Len() == 1514)
			return true;

	if ( this->FilterNoLess60 )
		if (Pckt->Len() <= 60)
			return true;

	if ( this->FilterNoLess65 )
		if (Pckt->Len() <= 65)
			return true;

	if ( Pckt->Protocol() == PKT_PTC_UDP )
	{
		const UDPPacket * UDPP = reinterpret_cast<UDPPacket const *>(Pckt);

		if ( this->FilterPayloadLess5 && (UDPP->UdpPayloadLen() <= 5) )
			return true;
	}

	if ( Pckt->Protocol() == PKT_PTC_TCP )
	{
		const TCPPacket * TCPP = reinterpret_cast<TCPPacket const *>(Pckt);

		if ( (this->FilterTCPPayloadLess5 || this->FilterPayloadLess5) &&
			 (TCPP->TcpPayloadLen() <= 5) )
			return true;

		if ( this->FilterSyn && TCPP->Syn() )
			return true;

		if ( this->FilterTCPnoPayload && (TCPP->TcpPayloadLen() == 0) )
			return true;

		if ( this->FilterAck &&
			 TCPP->Ack()     &&
			 (TCPP->TcpPayloadLen() == 0) )
//					if ( !(TCPP->Syn()) 	  &&
//						 !(TCPP->Rst())       &&
//						 !(TCPP->Fin())       &&
//						 !(TCPP->Urg())       &&
//						 !(TCPP->Psh())          )
			return true;
	}

	return false;
}

void PacketProcessor::FindVoIP(const int &WinSize) {

	HostPortAnalyzer<int> HPAnOut(WinSize, "", false, 0);
	HostPortAnalyzer<int> HPAnIn(WinSize, "", false, 0);
	TupleAnalyzer<int> TAn(WinSize, "", false, 0);

	TAn.setCleanInactive(true);

    bool end = false;

    while(!end)
        try
        {
            Packet *Pckt;

            Pckt = this->PcktSrc->NextPacket();
            int const Proto = Pckt->Protocol();
            int srcP, dstP, len;
            std::string src, dst, strProto;

            len = Pckt->Len();

            if ( ( (Proto == PKT_PTC_UDP) || (Proto == PKT_PTC_TCP) ) && !(this->FilterPacket(Pckt)) )
            {
				if (Proto == PKT_PTC_UDP)
				{
					UDPPacket const *UDPP = reinterpret_cast<UDPPacket const *>(Pckt);
					src = *(UDPP->SrcIPaddr());
					dst = *(UDPP->DstIPaddr());
					srcP = UDPP->SrcPort();
					dstP = UDPP->DstPort();
					strProto = "UDP";

					if (this->OnlyPayloadLen)
						len = UDPP->UdpPayloadLen();
				}
				else if (Proto == PKT_PTC_TCP)
				{
					TCPPacket *TCPP = reinterpret_cast<TCPPacket * const>(Pckt);
					src = *(TCPP->SrcIPaddr());
					dst = *(TCPP->DstIPaddr());
					srcP = TCPP->SrcPort();
					dstP = TCPP->DstPort();
					strProto = "TCP";

					if (this->OnlyPayloadLen)
						len = TCPP->TcpPayloadLen();
				}

				if (this->TupBased)
				{
					std::stringstream ssKeySrc (std::stringstream::out), ssKeyDst (std::stringstream::out);
					ssKeyDst  << dst << ":" << dstP;
					ssKeySrc << src << ":" << srcP;
					std::string HostPortSrc = ssKeySrc.str(), HostPortDst = ssKeyDst.str();

					long TSsec = Pckt->TSseconds(), TSusec = Pckt->TSmicrosec();

					double * aStats = TAn.AddValue(TSsec, TSusec, src, srcP, dst, dstP, strProto, len);

					const double dTS = TSsec + ( TSusec * 0.000001 );

//					std::cout << HostPortSrc << ":" << HostPortDst << " -- " << aStats[0] << " - " << len  << " - " << aStats[6] << std::endl;

					if (aStats[5] >= WinSize)
						this->ARaiser->ProcessPacket(dTS, HostPortSrc, len, aStats[6], aStats[0], HostPortDst, Proto);
				}
				else
				{
					std::stringstream ssKeyIN (std::stringstream::out), ssKeyOUT (std::stringstream::out);
					ssKeyIN  << dst << ":" << dstP;
					ssKeyOUT << src << ":" << srcP;
					std::string HostPortIN = ssKeyIN.str(), HostPortOUT = ssKeyOUT.str();

					long TSsec = Pckt->TSseconds(), TSusec = Pckt->TSmicrosec();

					double * aStatsOUT = HPAnOut.AddValue(TSsec, TSusec, src, srcP, len);
					double * aStatsIN  = HPAnIn.AddValue (TSsec, TSusec, dst, dstP, len);

					const double dTS = TSsec + ( TSusec * 0.000001 );

//					if (HostPortIN == "192.168.2.6:35501" && HostPortOUT == "192.168.2.2:10559")
//					{
//						std::cout << ind << "   " << HostPortOUT << ":" << HostPortIN << " -- " << aStatsOUT[0] << ":" << aStatsIN[0] << " - " << len << " - " << aStatsOUT[6] << ":" << aStatsIN[6] << std::endl;
//						ind++;
//					}

					if (aStatsIN[5] >= WinSize)
						this->ARaiser->ProcessIncomingPacket(dTS, HostPortIN, len, aStatsIN[6], aStatsIN[1], HostPortOUT, Proto);
					if (aStatsOUT[5] >= WinSize)
						this->ARaiser->ProcessOutgoingPacket(dTS, HostPortOUT, len, aStatsOUT[6], aStatsOUT[1], HostPortIN, Proto);
				}
            }

            delete Pckt;
		}
		catch (No_More_Packets e1)
		{
			end = true;
		}
		catch (Timeout_Expired e2)
		{
			std::cout << "Timeout: " << e2.what() << "\n";
			end = true;
		}
		catch (Packet_Reading_Error e3)
		{
			std::cout << "Failed: " << e3.what() << "\n";
			end = true;
		}

}
