/*
 * \author João V. Gomes
 *
 *  voippattern.cpp
 *
 *  Created on: Dec 28, 2010
 *
 * \brief TODO
 * TODO
 */

#include "voippattern.h"

VoIPPattern::VoIPPattern() {
	this->init();
}

VoIPPattern::VoIPPattern(const int &minLength, const int &maxLength, const double &minEnt, const double &maxEnt, const int &limit, const std::string &sDesc, const std::string &Codec, const int &Level) {
	this->init(minLength, maxLength, 0.0, 1600.0, minEnt, maxEnt, limit, sDesc, Codec, Level);
}

VoIPPattern::VoIPPattern(const double &minLengthMean, const double &maxLengthMean, const double &minEnt, const double &maxEnt, const int &limit, const std::string &sDesc, const std::string &Codec, const int &Level) {
	this->init(0, 1600, minLengthMean, maxLengthMean, minEnt, maxEnt, limit, sDesc, Codec, Level);
}

VoIPPattern::VoIPPattern(const int &minLength, const int &maxLength, const double &minLengthMean, const double &maxLengthMean, const double &minEnt, const double &maxEnt, const int &limit, const std::string &sDesc, const std::string &Codec, const int &Level) {
	this->init(minLength, maxLength, minLengthMean, maxLengthMean, minEnt, maxEnt, limit, sDesc, Codec, Level);
}

VoIPPattern::~VoIPPattern() {

}

void VoIPPattern::init() {
	this->init(0, 1600, 0, 1600, 0, 100, 450, "NO_DESC", "NO_CODEC", PAT_LEVEL_BR);
}

void VoIPPattern::init(const int &minLength, const int &maxLength, const double &minLengthMean, const double &maxLengthMean, const double &minEnt, const double &maxEnt, const int &limit, const std::string &sDesc, const std::string &Codec, const int &Level) {
	this->minLen = minLength;
	this->maxLen = maxLength;
	this->minLenMean = minLengthMean;
	this->maxLenMean = maxLengthMean;
	this->minEntropy = minEnt;
	this->maxEntropy = maxEnt;
	this->lim = limit;
	this->Desc = sDesc;
	this->CodecID = Codec;
	this->level = Level;
}

void VoIPPattern::dispose() {

}

const bool VoIPPattern::Match(const int &len, const double &entropy) const {
	return (len     >= this->minLen     && len     <= this->maxLen    ) &&
		   (entropy >= this->minEntropy && entropy <= this->maxEntropy);
}

const bool VoIPPattern::Match(const double &lenMean, const double &entropy) const {
	return (lenMean >= this->minLenMean && lenMean <= this->maxLenMean) &&
		   (entropy >= this->minEntropy && entropy <= this->maxEntropy);
}

const bool VoIPPattern::Match(const int &len, const double &lenMean, const double &entropy) const {
	return (len     >= this->minLen     && len     <= this->maxLen    ) &&
		   (lenMean >= this->minLenMean && lenMean <= this->maxLenMean) &&
		   (entropy >= this->minEntropy && entropy <= this->maxEntropy);
}

const std::string VoIPPattern::Description() const {
	return this->Desc;
}

const std::string VoIPPattern::Codec() const {
	return this->CodecID;
}

const int VoIPPattern::Level() const {
	return this->level;
}

const bool VoIPPattern::HasTheMin(const int &Val) const {
	return (Val >= this->lim);
}
